      PROGRAM DOUB2D
C.....2-D program for double-body flow (potential flow without free surface)
      IMPLICIT NONE
      INTEGER I, KS, LP, NP
      REAL AWET, C, U
      PARAMETER (LP=200)
      INTEGER KN(2,LP)
      REAL A(LP), M(LP), N(2,LP), V(2,LP), X(2,LP), XD(2,LP), XQ(2,LP)
      COMMON C(LP,LP+1)
C.....display name and purpose of program
      CALL MELDE
C.....read all input
      CALL INPUT(LP,NP,KN,AWET,U,A,M,N,V,X,XD,XQ)
C.....make linear system of equations from boundary conditions
      CALL LGSBOD(LP,NP,KN,U,A,N,X,XD,XQ,C)
C.....solve system of equations to determine element strengths
      CALL SIMQ2(C,NP,1,LP,KS,1.E-5)
      IF (KS.EQ.4) STOP'>> System of equations singular, fatal error'
      DO I = 1, NP
         M(I) = C(I,NP+1)
      END DO
C.....determine velocities with now known element strengths
      CALL VELBOD(NP,KN,U,A,M,X,XD,XQ,V)
C.....integrate pressure to forces
      CALL KRAFT(NP,AWET,U,A,N,V,X)
C.....write output
      CALL OUTPUT(NP,U,M,V,X,XQ)
      END

      SUBROUTINE INPUT(LP,NP,KN,AWET,U,A,M,N,V,X,XD,XQ)
C.....read all input and initialise all arrays
      IMPLICIT NONE
      CHARACTER FILE*80, TEXT*80
      INTEGER LP, NB, NP
      REAL AWET, U
      INTEGER KN(2,*)
      REAL A(*), M(*), N(2,*), V(2,*), X(2,*), XD(2,*), XQ(2,*)
C.....read control file <CONTRL.CTL>
      CALL INPST(TEXT,U,FILE)
C.....read grid for body 
      CALL INBODY(FILE,LP,NB,NP,KN,AWET,A,N,X,XD,XQ)
      WRITE(*,*) '>> ', NP, ' panels on body'
C.....initialise all arrays (set potential to uniform flow)
      CALL INITVA(NP,U,M,V)
      WRITE(*,*) '>> finished initial preparations'
      END

      SUBROUTINE INPST(TEXT,U,FILE)
C.....read control file <CONTRL.CTL>
      IMPLICIT NONE
      CHARACTER*80 TEXT, FILE
      REAL U
      OPEN(10,FILE='CONTRL.CTL',STATUS='OLD',ERR=100)
         READ(10,'(A)',ERR=200) TEXT
         WRITE(*,'(1X,A)') TEXT
         READ(10,'(A)',ERR=200) FILE
         READ(10,*,ERR=200) U
      CLOSE(10)
      WRITE(*,*) '>> control file <CONTRL.CTL> successfully read'
      RETURN
100   CONTINUE
      WRITE(*,*) '>> control file <CONTRL.CTL> not found'
      STOP
200   CONTINUE
      WRITE(*,*) '>> error reading control file <CONTRL.CTL>'
      STOP
      END

      SUBROUTINE INBODY(FILE,LP,NB,NP,KN,AWET,A,N,X,XD,XQ)
C.....read input file for body
      IMPLICIT NONE
      CHARACTER FILE*80
      INTEGER I, ID, IECK, ILEN, LP, NB, NP
      REAL AWET
      INTEGER KN(2,*)
      REAL A(*), N(2,*), X(2,*), XD(2,*), XQ(2,*)
      OPEN(10,FILE=FILE,STATUS='OLD',ERR=100)
         READ(10,*,ERR=200) NB, NP
         IF (NB.GT.LP) THEN
            WRITE(*,*) '>> Too many points on body'
            GOTO 200
         END IF
         IF (NP.GT.LP) THEN
            WRITE(*,*) '>> Too many panels on body'
            GOTO 200
         END IF
         READ(10,*,ERR=200) AWET
         DO I = 1, NB
            READ(10,*,ERR=200) (XD(ID,I),ID=1,2)
         END DO
         DO I = 1, NP
            READ(10,*,ERR=200) (KN(IECK,I),IECK=1,2)
            READ(10,*,ERR=200) (X(ID,I),ID=1,2),(XQ(ID,I),ID=1,2)
            READ(10,*,ERR=200) A(I),(N(ID,I),ID=1,2)
         END DO
      CLOSE(10)
      RETURN
100   CONTINUE
      WRITE(*,*) '>> File', FILE(1:ILEN(FILE,80)), ' not found'
      STOP
200   CONTINUE
      WRITE(*,*) '>> Error reading file ', FILE(1:ILEN(FILE,80))
      STOP
      END

      INTEGER FUNCTION ILEN(R,ISIZE)
C.....finds last non-blank character of string R of size ISIZE
      IMPLICIT NONE
      INTEGER I, ISIZE
      CHARACTER R*(*)
      DO I = ISIZE, 1, -1
         IF (R(I:I).NE.' ') GOTO 100
      END DO
100   CONTINUE
      ILEN = I
      END

      SUBROUTINE INITVA(NP,U,M,V)
C.....Initialise all arrays
      IMPLICIT NONE
      INTEGER I, NP
      REAL U
      REAL M(*), V(2,*)
      DO I = 1, NP
         M(I)    = 0.
         V(1,I)  = -U
         V(2,I)  = 0.
      END DO
      END

      SUBROUTINE LGSBOD(LP,NP,KN,U,A,N,X,XD,XQ,C)
C     equations for body (Neumann condition)
      IMPLICIT NONE
      INTEGER I, K, LP, NP
      REAL QX,QZ, U
      INTEGER KN(2,*)
      REAL A(*), C(LP,*), N(2,*), X(2,*), XD(2,*), XQ(2,*)
      DO K = 1, NP
         C(K,NP+1) = -N(1,K)*U
         DO I = 1, NP
            CALL QULBD(A(I),XQ(1,I),X(1,K),QX,QZ)
            C(K,I)=N(1,K)*QX+N(2,K)*QZ
         END DO
      END DO
      END

      SUBROUTINE KRAFT(NP,AWET,U,A,N,V,X)
C.....direct pressure integration gives forces
      IMPLICIT NONE
      INTEGER K, NP
      REAL AWET, CW, FORCE, P, U
      REAL A(*), N(2,*), V(2,*), X(2,*)
C     rho is not considered as it cancels out for the drag coefficient CW
      FORCE = 0.
      DO K = 1, NP
         P = (U**2-(V(1,K)**2+V(2,K)**2))/2.
         FORCE = FORCE + P*N(1,K)*A(K)
      END DO
C.....wave resistance coefficient
      CW = -FORCE/(0.5*U**2*AWET)
      WRITE(*,*) '>> CW = ', CW
      END

      SUBROUTINE MELDE
      WRITE(*,1)'******************************************************'
      WRITE(*,1)'*                                                    *'
      WRITE(*,1)'*                      DOUB2D                        *'
      WRITE(*,1)'*                                                    *'
      WRITE(*,1)'*           Double-body 2-D Potential Flow           *'
      WRITE(*,1)'*                                                    *'
      WRITE(*,1)'******************************************************'
      WRITE(*,*)
1     FORMAT(12X,A)
      END

      SUBROUTINE OUTPUT(NP,U,M,V,X,XQ)
C.....output on file 
      IMPLICIT NONE
      INTEGER I, ID, NP
      REAL CP, U
      REAL M(*), V(2,*), X(2,*), XQ(2,*)
      OPEN(12,FILE='OUTPUT.DAT')
         WRITE(12,*) NP
         DO I = 1, NP
            CP = 1.0-(V(1,I)**2+V(2,I)**2)/U**2
            WRITE(12,1000) (X(ID,I),ID=1,2), M(I),
     &                     (V(ID,I),ID=1,2), CP
         END DO
      CLOSE(12)
      WRITE(*,*) '>> File OUTPUT.DAT created'
1000  FORMAT(7(1X,G13.7))
      END

      SUBROUTINE QULBD(M,XQ,XK,PX,PZ)
C.....influence of body element
      IMPLICIT NONE
      INTEGER I
      REAL M, PX,PZ, QX,QZ
      REAL XQ(2), XK(2)
C.....initialise influence functions to 0
      CALL SET0(PX,PZ)
      CALL RANK2D(M,XK(1),XK(2),XQ(1), XQ(2),QX,QZ)
      CALL SUMM(PX,PZ,QX,QZ)
C.....mirror image at still waterplane
      CALL RANK2D(M,XK(1),XK(2),XQ(1),-XQ(2),QX,QZ)
      CALL SUMM(PX,PZ,QX,QZ)
      END

      SUBROUTINE SET0(PX,PZ)
C.....sets potential and derivatives to 0
      IMPLICIT NONE
      INTEGER LEVEL
      REAL PX, PZ
      PX = 0.
      PZ = 0.
      END

      SUBROUTINE SUMM(PX,PZ,QX,QZ)
C.....sums up potential and derivatives
      IMPLICIT NONE
      REAL PX, PZ, QX, QZ
      PX = PX + QX
      PZ = PZ + QZ
      END

      SUBROUTINE VELBOD(NP,KN,U,A,M,X,XD,XQ,V)
C     velocity on body
      IMPLICIT NONE
      INTEGER I, K, NP
      REAL QX,QZ, U
      INTEGER KN(2,*)
      REAL A(*), M(*), V(2,*), X(2,*), XD(2,*), XQ(2,*)
      DO K = 1, NP
         V(1,K) = -U
         V(2,K) = 0.
         DO I = 1, NP
            CALL QULBD(A(I)*M(I),XQ(1,I),X(1,K),QX,QZ)
            V(1,K) = V(1,K) + QX
            V(2,K) = V(2,K) + QZ
         END DO
      END DO
      END

      SUBROUTINE RANK2D(M,X,Z,XQ,ZQ,PX,PZ)
C.....2-D Rankine source of strength M
      IMPLICIT NONE
      INTEGER LEVEL
      REAL AUX, DX, DZ, FACTOR, M, PI, PX,PZ, R2, X, XQ, Z, ZQ
      PI = 3.141592654
      FACTOR = M/(4*PI)
      DX = X-XQ
      DZ = Z-ZQ
      R2 = DX**2+DZ**2
      IF (R2.EQ.0.) THEN
         WRITE(*,*) '>> Fatal error, singular distance in RANK2D'
         STOP
      END IF
      AUX = 2./R2
      PX = FACTOR*AUX*DX
      PZ = FACTOR*AUX*DZ
      END

      SUBROUTINE SIMQ2(A,N,NR,IM,I,S)
C Gauss algorithms to solve (real) system of linear equations with (optional)
C more than 1 right hand sides. The solver is suitable for full, unsymmitric
C matrices.
C A=matrix of coefficients + negative r.h.s; after execution the solution
C   are stored where previously the r.h.s. were. The coefficient matrix
C   is destroyed. 1st index of A = row (1 to N), 2nd index = column (1 to N+NR)
C NR=number of r.h.s.
C IM=maximal size of 1st index of A in the dimension declaraion of the calling
C    routine
C I=0 usually; not equal 0, if pivot element is smaller than S
C recommended value for S is 1.E-5
      INTEGER I, IM, IMAX, J, K, K1, K2, MMR, N, NNR, NR
      REAL BIGA, S, SAVE
      REAL A(*)
      IMAX=-1000000
      NNR=(N+NR)*IM
      MMR=N*IM
      K1=-IM
      DO 10 I=1,N
         K1=K1+IM+1
         BIGA=A(K1)
         IMAX=K1
         DO 20 J=K1+1,K1+N-I
            IF(ABS(BIGA).LT.ABS(A(J)))THEN
               BIGA=A(J)
               IMAX=J
            ENDIF
20       CONTINUE
         IF(ABS(BIGA).LT.S)RETURN
         A(IMAX)=A(K1)
         DO 30 K=K1+IM,NNR,IM
            IMAX=IMAX+IM
            SAVE=-A(IMAX)/BIGA
            A(IMAX)=A(K)
            A(K)=SAVE
            K2=K1
            DO 40 J=K+1,K+N-I
               K2=K2+1
               A(J)=A(J)+A(K2)*SAVE
40          CONTINUE
30       CONTINUE
10    CONTINUE
      DO 50 I=MMR,NNR-1,IM
         K1=MMR+1
         DO 60 J=N,2,-1
            K1=K1-IM
            K2=I
            SAVE=A(I+J)
            DO 70 K=K1,K1+J-2
               K2=K2+1
               A(K2)=A(K2)+A(K)*SAVE
70          CONTINUE
60       CONTINUE
50    CONTINUE
      I=0
      END

