C ---------------------------------------------------
C File fktkq.for
C Berechnung von Propeller-Schub- und Momentenbeiwerten
C Institut fuer Schiffbau, Universitaet Hamburg
C Laemmersieth 90, 2000 HH 60, 040/2984-3122
C (Rueckmeldung ueber notwendige Korrekturen erwuenscht)
C Autoren:
C FKTB,FKQB,FKTR,FKQR: Poulsen, Lee, Isensee, Schmidt
C FKTN,FKQN:           Schmidt, Schumann
C FKTCPP,FKQCPP:       Windmoeller, Schmidt
C FKT,FKQ:             Schmidt
C Aenderungen:
C 8.12.92 D.Schmidt
C    Zusammenstellung der Funktionen in FKTKQ.FOR,
C    globale Aufruf-Funktionen FKT und FKQ,
C    Aufrufmethode fuer Duesenpropeller,
C    Korrektur eines Koeffizienten fuer Ka 4-70/19a,
C    Parameterueberpruefung in FKT und FKQ,
C    Anpassung an Fortran 77,
C    Verwendung gleicher Parameternamen,
C    FKTB und FKQB aus FKTR und FKQR generiert
C 17.12.92 D.Schmidt
C    CHECK wird so gesetzt, dass nicht nur der zuletzt detektierte
C    unzulaessige Parameter durch Werte von 1 bis 6 angezeigt wird,
C    sondern dass alle unzulaessigen Parameter aus einer 6stelligen
C    Zahl, die nur aus Nullen und Einsen besteht, herausgelesen
C    werden koennen
C 4.1.93 D.Schmidt
C    Check ist REAL statt INTEGER
C 5.1.93 D.Schmidt
C    bei Abfragen auf kleine Werte wird 1.E-5 statt 1.E-10 gesetzt,
C    weil mit 1.E-10 in aufrufenden Programmen bei REAL-Variablen
C    'underflow' auftrat
C 21.12.93 D.Schmidt
C    die Fluegelzahl wird als REAL ZREAL uebergeben und mittels IFIX() in
C    INTEGER Z umgewandelt (Kompatibilitaet zu den alten Funktionen; der
C    wichtigste Einsatz dieser Funktionen ist ausserdem in Optimierungen, 
C    die typischerweise reell sind)
C 21.12.93 D.Schmidt
C    B-Serien Propeller mit Z < 3 nicht mehr ausgeschlossen
C 23.12.93 mit FTN-Check kommen viele Warnungen: Konstanten der Prop.-
C Funktinen haben mehr Stellen als gespeichert werden koennen; ev. auf
C DOUBLE PRECISION umstellen.

C enthaelt:
C REAL FUNCTION FKT   (AEA0,PZD0,PZDC,D,VA,N,ZREAL,PROTYP,CHECK)
C REAL FUNCTION FKQ   (AEA0,PZD0,PZDC,D,VA,N,ZREAL,PROTYP,CHECK)
C REAL FUNCTION FKTB  (AEA0,PZD0,     D,VA,N,ZREAL)
C REAL FUNCTION FKQB  (AEA0,PZD0,     D,VA,N,ZREAL)
C REAL FUNCTION FKTR  (AEA0,PZD0,     D,VA,N,ZREAL)
C REAL FUNCTION FKQR  (AEA0,PZD0,     D,VA,N,ZREAL)
C REAL FUNCTION FKTCPP(AEA0,PZD0,PZDC,D,VA,N,ZREAL)
C REAL FUNCTION FKQCPP(AEA0,PZD0,PZDC,D,VA,N,ZREAL)
C REAL FUNCTION FKTN  (     PZD0,     D,VA,N,  CONFIG)
C REAL FUNCTION FKQN  (     PZD0,     D,VA,N,  CONFIG)
C REAL FUNCTION FKTW()   (fuer FKTCPP() erforderlich)
C REAL FUNCTION FKQW()   (fuer FKQCPP() erforderlich)
C REAL FUNCTION LINPOL() (fuer FKTCPP() und FKQCPP() erforderlich)

C FKT()    ueberprueft die Aufrufparameter und ruft eine der Funktionen
C          FKTB,FKTR,FKTCPP,FKTN in Abhaengigkeit des Parameters PROTYP auf
C FKQ()    ueberprueft die Aufrufparameter und ruft eine der Funktionen
C          FKQB,FKQR,FKQCPP,FKQN in Abhaengigkeit des Parameters PROTYP auf
C FKTB()   berechnet Kt fuer Propeller der Wageninger B-Serie ohne Korrek-
C          tur fuer Zaehigkeitseinfluesse
C FKQB()   berechnet Kq fuer Propeller der Wageninger B-Serie ohne Korrek-
C          tur fuer Zaehigkeitseinfluesse
C FKTR()   berechnet Kt fuer Propeller der Wageninger B-Serie mit Korrektur
C          fuer Zaehigkeitseinfluesse (benoetigt FKTB())
C FKQR()   berechnet Kq fuer Propeller der Wageninger B-Serie mit Korrektur
C          fuer Zaehigkeitseinfluesse (benoetigt FKQB())
C FKTCPP() berechnet Kt fuer 3fluegelige Verstellpropeller ohne Korrektur
C          fuer Zaehigkeitseinfluesse (benoetigt FKTW() und LINPOL())
C FKQCPP() berechnet Kq fuer 3fluegelige Verstellpropeller ohne Korrektur
C          fuer Zaehigkeitseinfluesse (benoetigt FKQW() und LINPOL())
C FKTN()   berechnet Kt fuer Kombinationen aus Kaplan-Propeller und Wageninger
C          Duesen ohne Korrektur fuer Zaehigkeitseinfluesse, der Parameter
C          PROTYP (s. Kommentar weiter unten) gibt dabei die Propeller/
C          Duesen-Kombination an und damit auch AEA0 und die Fluegelzahl
C FKQN()   berechnet Kq fuer Kombinationen aus Kaplan-Propeller und Wageninger
C          Duesen ohne Korrektur fuer Zaehigkeitseinfluesse, der Parameter
C          PROTYP (s. Kommentar weiter unten) gibt dabei die Propeller/
C          Duesen-Kombination an und damit auch AEA0 und die Fluegelzahl

C Die Parameter sind:
C REAL AEA0          [-]   Flaechenverhaeltnis
C REAL PZD0          [-]   Nennsteigungsverhaeltnis
C REAL PZDC          [-]   akt.Steigungsverhaeltnis f.Verstellprop.
C REAL D             [m]   Propellerdurchmesser
C REAL VA            [m/s] Anstroemgeschwindigkeit
C REAL N             [1/s] Propellerdrehzahl
C REAL ZREAL         [-]   Fluegelzahl
C CHARACTER*8 PROTYP [-]   Kennwort fuer Propeller/Duesen-Kombination
C              (die zulaessigen Werte fuer PROTYP sind:
C              'B'        -->  B-Serie ohne Re-Korrektur
C              'BR'       -->  B-Serie mit Re-Korrektur
C              'CPP'      -->  Verstellpropeller
C              '3-65/19a' -->  Ka 3-65  in Duese 19a
C              '4-55/19a' -->  Ka 4-55  in Duese 19a
C              '4-70/19a' -->  Ka 4-70  in Duese 19a
C              '5-75/19a' -->  Ka 5-75  in Duese 19a
C              '4-70/37'  -->  Ka 4-70  in Duese 37
C              '4-70/22'  -->  Ka 4-70  in Duese 24
C              '4-70/24'  -->  Ka 4-70  in Duese 22
C              '5-100/33' -->  Kd 5-100 in Duese 33)
C REAL CHECK         [-]   Ergebnis der Parameterpruefung
C             (bei korrekten Parametern CHECK = 0,
C              bei unzulaessigem AEA0   CHECK = CHECK + 1.0E5,
C              bei unzulaessigem PZD0   CHECK = CHECK + 1.0E4,
C              bei unzulaessigem PZDC   CHECK = CHECK + 1.0E3,
C              bei unzulaessigem Z      CHECK = CHECK + 1.0E2,
C              bei unzulaessigem PROTYP CHECK = CHECK + 1.0E1,
C              bei unzulaessigem J      CHECK = CHECK + 1.0,
C              also bei voellig unzulaessigen Parametern CHECK = 111111)

C Bei Aufrufen ueber die Funktionen FKT und FKQ werden die Aufruf-
C parameter auf Zulaessigkeit geprueft und der Parameter CHECK bei
C Fehlerzustaenden auf den entsprechenden Wert (siehe oben) gesetzt;
C teilweise ist der zulaessige Bereich um 10 % extrapoliert; zur
C Bestimmung der maximal zulaessigen Werte der Fortschrittsziffer
C (Variable AC) wird das aktuelle Steigungsverhaeltnis PZD0 ver-
C wendet: fuer B-Serien- und Verstellpropeller ist der Grenzwert von
C AC der PZD0-Wert, fuer Duesenpropeller ist es PZD0 multipliziert
C mit einem Faktor, der aus den Wageninger Diagrammen bestimmt wurde.
C -------------------------------------------------------------------
C Parameter-Ueberpruefung und einheitlicher Aufruf der Funktionen
C FKTB,FKTR,FKTCPP,FKTN in Abhaengigkeit des Parameters PROTYP

      REAL FUNCTION FKT(AEA0,PZD0,PZDC,D,VA,N,ZREAL,PROTYP,CHECK)
      REAL AEA0,PZD0,PZDC,D,VA,N,FKTB,FKTR,FKTCPP,FKTN,AC,EPS,CHECK
      REAL ZREAL
      CHARACTER*8 PROTYP
      INTEGER Z,CONFIG
C      PRINT*,'FKT-Meldung: PROTYP= ',PROTYP
C Fuer alle Festpropeller soll gelten PZDC = PZD0       //j.i. 20.9.93
      IF (.NOT.LLT(PROTYP(1:1),'B').AND.
     &    .NOT.LGT(PROTYP(1:1),'B')) GO TO 1
      IF (.NOT.LLT(PROTYP(2:2),'-').AND.
     &    .NOT.LGT(PROTYP(2:2),'-')) GO TO 1
      GO TO 2
1     PZDC = PZD0
2     CONTINUE
C Initialisierung
      CHECK = 0
      CONFIG = -1
      EPS=1.0E-4
      Z = IFIX(ZREAL+0.5)
C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
      AC= MAX(AC,1.E-5) 
C
      IF (.NOT.LLT(PROTYP(1:1),'B').AND.
     &.NOT.LGT(PROTYP(1:1),'B')) GO TO 10
      IF (.NOT.LLT(PROTYP(1:3),'CPP').AND.
     &.NOT.LGT(PROTYP(1:3),'CPP')) GO TO 20

C Zuweisung der Konfigurationskennziffer in Abhaengigkeit von
C PROTYP fuer Propeller/Duesen-Kombinationen;
C Ueberpruefen der Parameter und Setzen von CHECK fuer FKTN
      IF(PZD0 .LT. 0.45 .OR. PZD0 .GT. 1.76) CHECK = CHECK + 1.0E4

      IF (.NOT.LLT(PROTYP(1:8),'3-65/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'3-65/19a')) THEN
         CONFIG=0
         IF(AC .GT. PZD0*0.81) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'4-55/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'4-55/19a'))  THEN
         CONFIG=1
         IF(AC .GT. PZD0*0.79) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'4-70/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'4-70/19a'))  THEN
         CONFIG=2
         IF(AC .GT. PZD0*0.81) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'5-75/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'5-75/19a'))  THEN
         CONFIG=3
         IF(AC .GT. PZD0*0.81) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:7),'4-70/37').AND.
     &.NOT.LGT(PROTYP(1:7),'4-70/37'))  THEN
         CONFIG=4
         IF(AC .GT. PZD0*0.75) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:7),'4-70/22').AND.
     &.NOT.LGT(PROTYP(1:7),'4-70/22'))  THEN
         CONFIG=6
         IF(AC .GT. PZD0*0.53) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:7),'4-70/24').AND.
     &.NOT.LGT(PROTYP(1:7),'4-70/24'))  THEN
         CONFIG=5
         IF(AC .GT. PZD0*0.706) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'5-100/33').AND.
     &.NOT.LGT(PROTYP(1:8),'5-100/33'))  THEN
         CONFIG=7
         IF(AC .GT. PZD0*0.37) CHECK = CHECK + 1.0
      END IF

      IF(CONFIG.LT.0) CHECK = CHECK + 1.0E1
C und Rechnen ...
      FKT = FKTN(PZD0,D,VA,N,CONFIG)
      RETURN

C Ueberpruefen der Parameter und Setzen von CHECK fuer FKTB
C und fuer FKTR
   10 IF(AEA0 .LT. 0.315 .OR. AEA0 .GT. 1.155) CHECK = CHECK + 1.0E5
      IF(PZD0 .LT. 0.45 .OR. PZD0 .GT. 1.54) CHECK = CHECK + 1.0E4
      IF(Z .LT. 2 .OR. Z .GT. 7) CHECK = CHECK + 1.0E2
      IF(AC .GT. PZD0) CHECK = CHECK + 1.0

      IF (.NOT.LLT(PROTYP(2:2),'R').AND.
     &.NOT.LGT(PROTYP(2:2),'R')) THEN
         FKT = FKTR(AEA0,PZD0,D,VA,N,ZREAL)
      ELSE
         FKT = FKTB(AEA0,PZD0,D,VA,N,ZREAL)
      END IF
      RETURN

C Ueberpruefen der Parameter und Setzen von CHECK fuer FKTCPP
C und Rechnen ...
   20 IF(AEA0 .LT. (0.45-EPS) .OR. AEA0 .GT. (0.7+EPS))
     &CHECK = CHECK + 1.0E5
      IF(PZD0 .LT. (0.4-EPS) .OR. PZD0 .GT. (1.2+EPS))
     &CHECK = CHECK + 1.0E4
      IF(PZDC .LT. (-0.55-EPS) .OR. PZDC .GT. (1.55+EPS))
     &CHECK = CHECK + 1.0E3
      IF(Z .NE. 3) CHECK = CHECK + 1.0E2
      IF(ABS(AC) .GT. (ABS(PZDC)+EPS)) CHECK = CHECK + 1.0
      FKT = FKTCPP(AEA0,PZD0,PZDC,D,VA,N,ZREAL)
      RETURN
      END
C ---------------------------------------------------
C Parameter-Ueberpruefung und einheitlicher Aufruf der Funktionen
C FKQB,FKQR,FKQCPP,FKQN in Abhaengigkeit des Parameters PROTYP

      REAL FUNCTION FKQ(AEA0,PZD0,PZDC,D,VA,N,ZREAL,PROTYP,CHECK)
      REAL AEA0,PZD0,PZDC,D,VA,N,FKQB,FKQR,FKQCPP,FKQN,AC,EPS,CHECK
      REAL ZREAL
      CHARACTER*8 PROTYP
      INTEGER Z,CONFIG
C      PRINT*,'FKQ-Meldung: PROTYP= ',PROTYP
C Fuer alle Festpropeller soll gelten PZDC = PZD0       //j.i. 20.9.93
      IF (.NOT.LLT(PROTYP(1:1),'B').AND.
     &    .NOT.LGT(PROTYP(1:1),'B')) GO TO 1
      IF (.NOT.LLT(PROTYP(2:2),'-').AND.
     &    .NOT.LGT(PROTYP(2:2),'-')) GO TO 1
      GO TO 2
1     PZDC = PZD0
2     CONTINUE
C Initialisierung
      CHECK = 0
      CONFIG = -1
      EPS=1.0E-4
      Z = IFIX(ZREAL+0.5)
C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
      AC= MAX(AC,1.E-5) 
C
      IF (.NOT.LLT(PROTYP(1:1),'B').AND.
     &.NOT.LGT(PROTYP(1:1),'B')) GO TO 10
      IF (.NOT.LLT(PROTYP(1:3),'CPP').AND.
     &.NOT.LGT(PROTYP(1:3),'CPP')) GO TO 20

C Zuweisung der Konfigurationskennziffer in Abhaengigkeit von
C PROTYP fuer Propeller/Duesen-Kombinationen;
C Ueberpruefen der Parameter und Setzen von CHECK fuer FKTN
      IF(PZD0 .LT. 0.45 .OR. PZD0 .GT. 1.76) CHECK = CHECK + 1.0E4

      IF (.NOT.LLT(PROTYP(1:8),'3-65/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'3-65/19a')) THEN
         CONFIG=0
         IF(AC .GT. PZD0*0.81) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'4-55/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'4-55/19a'))  THEN
         CONFIG=1
         IF(AC .GT. PZD0*0.79) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'4-70/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'4-70/19a'))  THEN
         CONFIG=2
         IF(AC .GT. PZD0*0.81) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'5-75/19a').AND.
     &.NOT.LGT(PROTYP(1:8),'5-75/19a'))  THEN
         CONFIG=3
         IF(AC .GT. PZD0*0.81) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:7),'4-70/37').AND.
     &.NOT.LGT(PROTYP(1:7),'4-70/37'))  THEN
         CONFIG=4
         IF(AC .GT. PZD0*0.75) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:7),'4-70/22').AND.
     &.NOT.LGT(PROTYP(1:7),'4-70/22'))  THEN
         CONFIG=6
         IF(AC .GT. PZD0*0.53) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:7),'4-70/24').AND.
     &.NOT.LGT(PROTYP(1:7),'4-70/24'))  THEN
         CONFIG=5
         IF(AC .GT. PZD0*0.706) CHECK = CHECK + 1.0
      END IF

      IF (.NOT.LLT(PROTYP(1:8),'5-100/33').AND.
     &.NOT.LGT(PROTYP(1:8),'5-100/33'))  THEN
         CONFIG=7
         IF(AC .GT. PZD0*0.37) CHECK = CHECK + 1.0
      END IF

      IF(CONFIG.LT.0) CHECK = CHECK + 1.0E1
C und Rechnen ...
      FKQ = FKQN(PZD0,D,VA,N,CONFIG)
      RETURN

C Ueberpruefen der Parameter und Setzen von CHECK fuer FKQB
C und fuer FKQR
   10 IF(AC .GT. PZD0) CHECK = CHECK + 1.0
      IF(AEA0 .LT. 0.315 .OR. AEA0 .GT. 1.155) CHECK = CHECK + 1.0E5
      IF(PZD0 .LT. 0.45 .OR. PZD0 .GT. 1.54) CHECK = CHECK + 1.0E4
      IF(Z .LT. 3 .OR. Z .GT. 7) CHECK = CHECK + 1.0E2

      IF (.NOT.LLT(PROTYP(2:2),'R').AND.
     &.NOT.LGT(PROTYP(2:2),'R')) THEN
         FKQ = FKQR(AEA0,PZD0,D,VA,N,ZREAL)
      ELSE
         FKQ = FKQB(AEA0,PZD0,D,VA,N,ZREAL)
      END IF
      RETURN

C Ueberpruefen der Parameter und Setzen von CHECK fuer FKQCPP
C und Rechnen ...
   20 IF(ABS(AC) .GT. (ABS(PZDC)+EPS)) CHECK = CHECK + 1.0
      IF(PZD0 .LT. (0.4-EPS) .OR. PZD0 .GT. (1.2+EPS))
     &CHECK = CHECK + 1.0E4
      IF(PZDC .LT. (-0.55-EPS) .OR. PZDC .GT. (1.55+EPS))
     &CHECK = CHECK + 1.0E3
      IF(AEA0 .LT. (0.45-EPS) .OR. AEA0 .GT. (0.7+EPS))
     &CHECK = CHECK + 1.0E5
      IF(Z .NE. 3) CHECK = CHECK + 1.0E2
      FKQ = FKQCPP(AEA0,PZD0,PZDC,D,VA,N,ZREAL)
      RETURN
      END
C ---------------------------------------------------
C Berechnung von Kt fuer Propeller der B-Serie ohne Korrektur
C fuer Zaehigkeit
C Literatur: Oosterveld u. Oossanen, Further Computer-analysed
C Data of the Wageningen B-Screw Series, ISP 1975 S.251 ff

      REAL FUNCTION FKTB(AEA0,PZD,D,VA,N,ZREAL)
      REAL AEA0,PZD,D,VA,N,AC,A,ZREAL
      INTEGER Z,I,II,III,K
      DIMENSION A(39),K(156)

C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
      Z = IFIX(ZREAL+0.5)

      DATA(A(I),I=1,39)/
     1.00880496,-.204554,.166351,.158114,-.147581,-.481497,.415437,
     2.0144043,-.0530054,.0143481,.0606826,-.0125894,.0109689,-.133698,
     3.00638407,-.00132718,.168496,-.0507214,.0854559,-.0504475,
     4.010465,-.00648272,-.00841728,.0168424,-.00102296,-.0317791,
     5.018604,-.00410798,-.000606848,-.0049819,.0025983,-.000560528,
     6-.00163652,-.000328787,.000116502,.000690904,.00421749,.0000565229
     7,-.00146564/
      DATA (K(I),I=1,32)/
     10,0,0,0,1,0,0,0,0,1,0,0,0,2,0,0,2,0,1,0,1,1,1,0,0,2,1,0,0,0,0,1/
      DATA (K(I),I=33,64)/
     22,0,0,1,0,1,0,1,1,1,0,1,0,0,1,1,1,0,1,1,0,3,0,0,0,6,0,0,2,6,0,0/
      DATA (K(I),I=65,96)/
     33,0,1,0,0,0,2,0,2,0,2,0,3,0,2,0,1,6,2,0,2,6,2,0,0,3,0,1,1,3,0,1/
      DATA (K(I),I=97,128)/
     43,3,0,1,0,3,1,1,1,0,2,1,0,2,2,1,0,0,0,2,1,0,0,2,2,0,0,2,3,0,0,2/
      DATA (K(I),I=129,156)/
     51,2,0,2,1,6,0,2,2,6,0,2,0,0,1,2,0,3,1,2,3,6,1,2,0,3,2,2/

      FKTB=0.0
      DO 10 II=1,39
         III=4*II
         FKTB=FKTB+
     &   A(II)*AC**K(III-3)*PZD**K(III-2)*AEA0**K(III-1)*Z**K(III)
   10 CONTINUE
      RETURN
      END
C ---------------------------------------------------
C Berechnung von Kq fuer Propeller der B-Serie ohne Korrektur
C fuer Zaehigkeit
C Literatur: Oosterveld u. Oossanen, Further Computer-analysed
C Data of the Wageningen B-Screw Series, ISP 1975 S.251 ff

      REAL FUNCTION FKQB (AEA0,PZD,D,VA,N,ZREAL)
      REAL AEA0,PZD,D,VA,N,AC,A,ZREAL
      INTEGER Z,I,II,III,K
      DIMENSION A(47),K(188)

C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
      Z = IFIX(ZREAL+0.5)

      DATA(A(I),I=1,47)/
     1.00379368,.00886523,-.032241,.00344778,-.0408811,-.108009,
     2-.0885381,.188561,-.00370871,.00513696,.0209449,.00474319,
     3-.00723408,.00438388,-.0269403,.0558082,.0161886,.00318086,
     4.015896,.0471729,.0196283,-.0502782,-.030055,.0417122,
     5-.0397722,-.00350024,-.0106854,.00110903,-.000313912,.0035985,
     6-.00142121,-.00383637,.0126803,-.00318278,.00334268,-.00183491,
     7.000112451,-.0000297228,.000269551,.00083265,.00155334,
     8.000302683,-.0001843,-.000425399,.0000869243,-.0004659,
     9.0000554194/
      DATA (K(I),I=1,32)/
     10,0,0,0,2,0,0,0,1,1,0,0,0,2,0,0,0,1,1,0,1,1,1,0,2,1,1,0,0,2,1,0/
      DATA (K(I),I=33,64)/
     21,0,0,1,0,1,0,1,1,1,0,1,2,1,0,1,2,0,1,1,1,1,1,1,0,2,1,1,3,0,1,0/
      DATA (K(I),I=65,96)/
     30,3,1,0,1,3,1,0,0,0,2,0,1,0,2,0,3,0,2,0,0,1,2,0,3,1,2,0,2,2,2,0/
      DATA (K(I),I=97,128)/
     40,3,2,0,0,6,2,0,3,0,0,1,3,3,0,1,0,6,0,1,3,0,1,1,0,6,1,1,1,0,2,1/
      DATA (K(I),I=129,160)/
     50,2,2,1,2,3,2,1,0,6,2,1,1,1,0,2,3,2,0,2,3,6,0,2,1,0,1,2,2,0,1,2/
      DATA (K(I),I=161,188)/
     60,2,1,2,0,6,1,2,0,0,2,2,0,3,2,2,3,3,2,2,0,6,2,2,1,6,2,2/

      FKQB=0.0
      DO 10 II=1,47
         III=4*II
         FKQB=FKQB+
     &   A(II)*AC**K(III-3)*PZD**K(III-2)*AEA0**K(III-1)*Z**K(III)
   10 CONTINUE
      RETURN
      END
C ---------------------------------------------------
C Berechnung von Kt fuer Propeller der B-Serie mit Korrektur
C fuer Zaehigkeit
C Literatur: Oosterveld u. Oossanen, Further Computer-analysed
C Data of the Wageningen B-Screw Series, ISP 1975 S.251 ff
C und Holtrop u. Mennen, ISP 1982, S.166 ff (KP=0.00003);
C benoetigt REAL FUNCTION FKTB()

      REAL FUNCTION FKTR(AEA0,PZD,D,VA,N,ZREAL)
      REAL AEA0,PZD,D,VA,N,FKTB,KTB,C075,TZUC75,DELCD,ZREAL
      INTEGER Z
      D = MAX(D,1.E-5)
      Z = IFIX(ZREAL+0.5)

      KTB = FKTB(AEA0,PZD,D,VA,N,ZREAL)
C ... und die Reibungskorrektur
      C075=2.073*AEA0*D/ZREAL
      TZUC75=(0.0185-0.00125*ZREAL)*D/C075
      DELCD=(2.+4.*TZUC75)*(0.003605-(1.89+1.62*
     &ALOG10(C075/0.00003))**(-2.5))
      FKTR=KTB+DELCD*0.3*PZD*C075*ZREAL/D
      RETURN
      END
C ---------------------------------------------------
C Berechnung von Kq fuer Propeller der B-Serie mit Korrektur
C fuer Zaehigkeit
C Literatur: Oosterveld u. Oossanen, Further Computer-analysed
C Data of the Wageningen B-Screw Series   ISP 1975 S.251
C und Holtrop u. Mennen, ISP 1982, S.166 ff (KP=0.00003) 
C benoetigt REAL FUNCTION FKQB()

      REAL FUNCTION FKQR(AEA0,PZD,D,VA,N,ZREAL)
      REAL AEA0,PZD,D,VA,N,FKQB,KQB,C075,TZUC75,DELCD,ZREAL
      INTEGER Z
      D = MAX(D,1.E-5)
      Z = IFIX(ZREAL+0.5)

      KQB = FKQB(AEA0,PZD,D,VA,N,ZREAL)
C ... und die Reibungskorrektur
      C075=2.073*AEA0*D/ZREAL
      TZUC75=(0.0185-0.00125*ZREAL)*D/C075
      DELCD=(2.+4.*TZUC75)*(0.003605-(1.89+1.62*
     &ALOG10(C075/0.00003))**(-2.5))
      FKQR=KQB-DELCD*0.25*C075*ZREAL/D
      RETURN
      END
C ---------------------------------------------------
C Berechnung von Kt fuer 3-fluegelige Verstellpopeller ohne
C Korrektur fuer Zaehigkeit
C Literatur: Chu,C.,Chan,Z.L.,She,Y.S. und Yuan,V.Z., The 3-Bladed
C JD-CPP Series, 4th Lips Propeller Symposium, Okt. 1979, S. 53 ff
C benoetigt REAL FUNCTION FKTW() und REAL FUNCTION LINPOL()
C (Z ist z.Z. ein dummy-Argument)

      REAL FUNCTION FKTCPP(AEA0,PZD0,PZDC,D,VA,N,ZREAL)
      INTEGER I,I1,I2,I3,I4,II,II1,II2,II3,II4,J,K,Z
      REAL AC,AEA0,AEA01,AEA02,AEA03,BT(2,5,0:7,0:4),D,EPS,
     &         FKT1,FKT2,FKT3,FKT4,FKT5,FKT6,PZD0,PZD01,PZD02,PZD03,
     &         PZD04,PZDC,N,VA,ZREAL
      REAL FKTW,LINPOL

      EPS=1.0E-4
C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
      Z = IFIX(ZREAL+0.5)

      DATA ((BT(1,1,J,K),K=0,4),J=0,7)/.01197601716,-.1406611942,
     &  -0.03736515298,0.0,0.0,
     &  .2028848898,-.1825221368,-.56310286360,.4949167396,.307665702,
     &  0.16806816660,-0.10690754960,0.0,0.0,-0.80299674400,
     &  0.29950899400,0.21835107640,0.04618970034,0.0,0.35698596400,
     &  -0.27177599060,0.0,0.0,0.0,0.0,
     &  0.0,-0.01558323856,0.0,0.0,0.0,
     &  0.0,0.0,0.0,-0.02040035822,0.0,
     &  0.01592364002,0.0,0.0,0.0,0.0/
      DATA ((BT(1,2,J,K),K=0,4),J=0,7)/
     &  0.01916939436,-.165422553,-.12086777,.1167534252,0.0,
     &  0.24440279180,-0.14948128740,-0.49765717140,0.0,0.0,
     &  0.17268001280,0.0,0.62051824800,0.0,0.0,
     &  0.13468651760,0.0,-0.26119134120,0.0,0.0,
     &  -0.24830947680,0.0,0.0,0.0,0.0,
     &  0.15033845220,0.0,0.0,0.0,0.0,
     &  -0.04314463446,0.01962958410,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0/
      DATA ((BT(1,3,J,K),K=0,4),J=0,7) /
     &  0.01898715000,-0.16223346160,0.0,0.0,0.0,
     &  0.25858205420,-0.31550357900,-0.38487439180,0.25561925920,0.0,
     &  0.22721500020,0.0,0.0,0.0,-0.04170169232,
     &  0.13919529140,0.44033047020,0.0,0.0,0.0,
     &  -0.41807538800,-0.23280818460,0.0,0.0,0.0,
     &  0.23220967140,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  -0.02449106210,0.01284252262,0.0,0.0,0.0/
      DATA ((BT(1,4,J,K),K=0,4),J=0,7) /
     &  0.02754158792,-0.11549594660,-0.17282285960,-0.52938543200,0.0,
     &  0.25419162280,-0.20206578940,0.0,1.32321933200,0.81615147940,
     &  .1412952742,-.2110476494,-.20955070120,-0.457283483,-1.56233617,
     &  0.20080624700,0.24379293460,0.0,0.0,0.52770007100,
     &  -0.22667671340,0.0,0.0,0.0,0.0,
     &  0.04744317258,0.0,0.0,0.0,0.17128959200,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,-0.05199881898/
      DATA ((BT(1,5,J,K),K=0,4),J=0,7) /
     &  .03616524162,-.1370718544,-.190924828,-.2451265222,-.6104741048,
     &  0.28592376600,-0.182485061,-.31202776980,1.835285332,.354896023,
     &  0.13104205120,0.0,0.0,0.0,0.0,
     &  0.13794755740,0.0,-0.27738873620,-1.50179816200,0.0,
     &  -0.15040405940,0.06590532436,0.0,0.58111936320,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.01445627894,0.0,0.42137991320,0.0,0.0,
     &  0.0,0.0,-0.22417063860,0.0,0.0/
      DATA ((BT(2,1,J,K),K=0,4),J=0,7) /
     &  0.00613061908,-0.14317154380,0.0,0.0,0.0,
     &  0.16400348680,-0.30414649360,-0.30425436500,0.19658392440,0.0,
     &  0.32459299120,-0.25285147540,0.0,0.0,0.0,
     &  0.36060217500,0.43930972560,0.0,-0.03938713702,0.0,
     &  -0.64602514120,-0.09708446200,0.0,0.0,0.0,
     &  0.28514972100,0.0,0.0,0.0,0.0,
     &  -0.03807185579,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0/
      DATA ((BT(2,2,J,K),K=0,4),J=0,7) /
     &  0.01411112042,-0.15885234940,0.0,0.0,-0.18351206000,
     &  0.21570472440,-0.28880447620,-0.27027083660,0.44196601560,0.0,
     &  0.23949257620,-0.19512398940,0.0,0.0,0.0,
     &  0.25844928560,0.28718812920,-0.13978641940,0.0,0.0,
     &  -0.41682618460,0.0,0.0,0.0,0.0,
     &  0.18787879960,0.0,0.0,0.0,0.0,
     &  -0.03546795432,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0/
      DATA ((BT(2,3,J,K),K=0,4),J=0,7) /
     &  0.02789609542,-0.24930143400,0.0,0.0,0.0,
     &  0.32277231560,-0.18721864900,-0.34219140560,0.12724498820,0.0,
     &  0.10489783920,0.17988476400,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  -0.00810292049,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0/
      DATA ((BT(2,4,J,K),K=0,4),J=0,7) /
     &  0.02441619960,-0.05849223606,0.0,-1.80731612800,0.0,
     &  0.257266212,-.4807569308,1.16829361,2.260144096,3.973380742,
     &  0.3451112054,-.87101227160,-1.51224247000,0.0,-5.43563010600,
     &  0.14035153660,1.20896085000,0.0,0.0,-5.41422546800,
     &  -0.60346310720,0.0,0.0,-0.16626950280,9.92602540600,
     &  0.42328550380,-0.18378203180,0.0,0.0,0.0,
     &  -0.10178256460,0.0,0.0,0.0,-4.84120344000,
     &  0.0,0.0,0.04156952068,0.0,1.71050473200/
      DATA ((BT(2,5,J,K),K=0,4),J=0,7) /.03557392216,
     &  -.1441859672,-.2182747626,-.27239204116,-.4013441052,
     &  0.32348806420,-0.25444302640,0.0,1.54437423000,0.20743875440,
     &  .318733058,-0.29943934060,-0.19070612300,-0.24861973660,0.0,
     &  0.0,0.31504870060,0.0,-1.06351915600,0.0,
     &  -0.48829613200,0.0,0.0,0.56262802980,0.0,
     &  0.37154683200,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  -0.05436761042,0.0,0.0,-0.01908083062,0.0/

C Abfrage von AEA0,PZD0 auf Uebereinstimmung mit einer AEA0,
C PZD0-Kombination, fuer die ein Berechnungspolynom vorliegt
C Gegebenenfalls Besetzen der Variablen I,II
      IF(ABS(AEA0-0.50).LT.EPS)I=1
      IF(ABS(AEA0-0.65).LT.EPS)I=2
      IF(ABS(PZD0-0.40).LT.EPS)II=1
      IF(ABS(PZD0-0.60).LT.EPS)II=2
      IF(ABS(PZD0-0.80).LT.EPS)II=3
      IF(ABS(PZD0-1.00).LT.EPS)II=4
      IF(ABS(PZD0-1.20).LT.EPS)II=5

C Suche nach den benachbarten AEA0,PZD0-Kombinationen und
C Berechnung des Kt-Wertes durch Aufruf von FKTW, LINPOL
      IF((I.EQ.1).OR.(I.EQ.2))THEN
           IF((II.EQ.1).OR.(II.EQ.2).OR.(II.EQ.3).OR.(II.EQ.4)
     &          .OR.(II.EQ.5))THEN
                FKTCPP=FKTW(I,II,PZDC,AC,BT)
           ELSE 
                IF(PZD0.LT.0.60)THEN
                     PZD01=0.40
                     II1=1
                     PZD02=0.60
                     II2=2
                ELSE IF(PZD0.GT.0.60.AND.PZD0.LT.0.80)THEN
                     PZD01=0.60
                     II1=2
                     PZD02=0.80
                     II2=3  
                ELSE IF(PZD0.GT.0.80.AND.PZD0.LT.1.00)THEN
                     PZD01=0.80
                     II1=3
                     PZD02=1.00
                     II2=4 
                ELSE IF(PZD0.GT.1.00)THEN
                     PZD01=1.00
                     II1=4
                     PZD02=1.20
                     II2=5
                END IF
                FKT1=FKTW(I,II1,PZDC,AC,BT)
                FKT2=FKTW(I,II2,PZDC,AC,BT)
                FKTCPP=LINPOL(PZD0,PZD01,PZD02,FKT1,FKT2)
           END IF
      ELSE
           IF((II.EQ.1).OR.(II.EQ.2).OR.(II.EQ.3).OR.(II.EQ.4)
     &        .OR.(II.EQ.5))THEN
                AEA01=0.50
                I1=1
                AEA02=0.65
                I2=2
                FKT1=FKTW(I1,II,PZDC,AC,BT)
                FKT2=FKTW(I2,II,PZDC,AC,BT)
                FKTCPP=LINPOL(AEA0,AEA01,AEA02,FKT1,FKT2)
           ELSE
                AEA01=0.50
                I1=1
                AEA02=0.50
                I2=1
                AEA03=0.65
                I3=2
                I4=2
                IF(PZD0.LT.0.60)THEN
                      PZD01=0.40
                      II1=1
                      PZD02=0.60
                      II2=2
                      PZD03=0.40
                      II3=1
                      PZD04=0.60
                      II4=2
                ELSE IF(PZD0.GT.0.60.AND.PZD0.LT.0.80)THEN
                      PZD01=0.60
                      II1=2
                      PZD02=0.80
                      II2=3
                      PZD03=0.60
                      II3=2 
                      PZD04=0.80
                      II4=3
                ELSE IF(PZD0.GT.0.80.AND.PZD0.LT.1.00)THEN
                      PZD01=0.80
                      II1=3
                      PZD02=1.00
                      II2=4
                      PZD03=0.80
                      II3=3
                      PZD04=1.00
                      II4=4
                ELSE IF(PZD0.GT.1.00)THEN
                      PZD01=1.00
                      II1=4
                      PZD02=1.20
                      II2=5
                      PZD03=1.00
                      II3=4
                      PZD04=1.20
                      II4=5
                END IF
                FKT1=FKTW(I1,II1,PZDC,AC,BT)
                FKT2=FKTW(I2,II2,PZDC,AC,BT)
                FKT3=FKTW(I3,II3,PZDC,AC,BT)
                FKT4=FKTW(I4,II4,PZDC,AC,BT)
                FKT5=LINPOL(PZD0,PZD01,PZD02,FKT1,FKT2)
                FKT6=LINPOL(PZD0,PZD03,PZD04,FKT3,FKT4)
                FKTCPP=LINPOL(AEA0,AEA01,AEA03,FKT5,FKT6)
           END IF
      END IF
      RETURN
      END
C ----------------------------------------------------------
C Unterfunktion von FKTCPP()
C I = Kennnummer fuer AEA0 (I=1 fuer AEA0=0.50, I=2 fuer AEA0=0.65)
C II = Kennummer fuer PZD0 (II=1 fuer PZD0 =0.40,II=2 fuer PZD0 =0.60,
C                          II=3 fuer PZD0 =0.80,II=4 fuer PZD0 =1.00
C                          II=5 fuer PZD0 =1.20)

      REAL FUNCTION FKTW(I,II,PZDC,AC,BT)
      INTEGER I,II,J,K
      REAL AC,BT(2,5,0:7,0:4),EPS,PZDC,SUM,SUM1,SUM2
      EPS=1.0E-6

C Nullabfrage und Neubesetzen von AC, PZDC
      IF(ABS(PZDC).LT.EPS)PZDC=EPS
      IF(ABS(AC).LT.EPS)AC=EPS

C Berechnung des Kt-Wertes
      SUM2=0.0
      DO 100 J=0,7
           SUM=0.0
           SUM1=0.0     
                DO 105 K=0,4
                SUM=BT(I,II,J,K)*PZDC**J*AC**K
                SUM1=SUM1+SUM
  105           CONTINUE
           SUM2=SUM2+SUM1
  100 CONTINUE
      FKTW=SUM2
      RETURN
      END
C ----------------------------------------------------------
C Berechnung von Kq fuer 3-fluegelige Verstellpopeller ohne
C Korrektur fuer Zaehigkeit
C Literatur: Chu,C.,Chan,Z.L.,She,Y.S. und Yuan,V.Z., The 3-Bladed
C JD-CPP Series, 4th Lips Propeller Symposium, Okt. 1979, S. 53 ff
C benoetigt REAL FUNCTION FKQW() und REAL FUNCTION LINPOL()
C (Z ist z.Z. ein dummy-Argument)

      REAL FUNCTION FKQCPP(AEA0,PZD0,PZDC,D,VA,N,ZREAL)
      INTEGER I,I1,I2,I3,I4,II,II1,II2,II3,II4,J,K,Z
      REAL AC,AEA0,AEA01,AEA02,AEA03,BQ(2,5,0:7,0:4),D,EPS,
     &     FKQ1,FKQ2,FKQ3,FKQ4,FKQ5,FKQ6,PZD0,PZD01,PZD02,PZD03,
     &     PZD04,PZDC,N,VA,ZREAL
      REAL FKQW,LINPOL

      EPS=1.0E-4
C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
      Z = IFIX(ZREAL+0.5)

      DATA((BQ(1,1,J,K),K=0,4),J=0,7)/
     &  0.00289707384,0.0,-0.00651349564,0.04001401856,-0.04053042328,
     &  0.0,-0.01444489874,-0.06324379550,0.05135543506,0.01766703302,
     &  0.03228587698,0.0,0.0,-0.02494194872,0.0,
     &  0.01818520720,-0.11487579380,0.0,0.0,0.0,
     &  0.05817731264,0.09567320636,0.0,0.0,0.0,
     &  -0.05472713468,0.05338337958,0.0,0.0,0.0,
     &  0.0,-0.07749127542,0.0,0.0,0.0,
     &  0.00707992994,0.02162540994,0.0,0.0,0.0/
      DATA((BQ(1,2,J,K),K=0,4),J=0,7)/
     &  0.00327521615,0.0,0.0,0.01679157352,0.0,
     &  0.0,-0.04439501818,-0.02946078850,0.0,0.0,
     &  0.05110837322,0.0,-0.00471239956,0.0,0.0,
     &  0.00686336698,0.0,0.0,0.0,0.0,
     &  0.00442565986,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  -0.00130649558,0.00095246426,0.0,0.0,0.0/
      DATA((BQ(1,3,J,K),K=0,4),J=0,7)/
     &  0.00365019369,0.0,0.0,0.02241450956,-0.01137951774,
     &  0.0,-0.04319586730,-0.03573530630,0.0,0.0,
     &  0.05760661496,0.0,0.0,0.01265620702,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,-0.00605111885,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.00850769761,0.0,0.0,0.0,0.0,
     &  -0.00532457834,0.00134801596,0.0,0.0,0.0/
      DATA((BQ(1,4,J,K),K=0,4),J=0,7)/
     &  0.00419633067,0.0,0.0,0.0,-0.01721485458,
     &  0.0,-0.03318743636,-0.01610859632,0.05916452114,0.0,
     &  0.05529437768,0.0,-0.07761759662,0.0,0.0,
     &  -0.00603007315,0.0,0.02997211412,0.0,0.0,
     &  0.01334556336,0.0,0.0,-0.00407664374,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  -0.00212093089,0.00066072591,0.0,0.0,0.0/
      DATA((BQ(1,5,J,K),K=0,4),J=0,7)/
     &  0.00502538281,0.0,0.0,0.0,0.0,
     &  0.0,-.04038462476,-.02395320296,-.00966943344,.0308376148,
     &  .06325008338,0.0,0.0,0.0,-.02130608978,
     &  -.00128461116,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,.00635265679,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  .00024973977,0.0,-.00041851515,0.0,0.0/
      DATA((BQ(2,1,J,K),K=0,4),J=0,7)/
     &  0.00295972854,0.0,0.0,0.03107874694,0.0,
     &  0.0,-0.05182530670,-0.05004250574,0.0,0.0,
     &  0.06006180402,0.0,0.0,0.0,0.0,
     &  0.00816402238,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  -0.00100518569,0.00178120106,0.0,-0.00028166244,0.0,
     &  0.0,0.0,0.0,0.0,0.0/
      DATA((BQ(2,2,J,K),K=0,4),J=0,7)/
     &  0.00407693500,0.0,0.00798798955,0.02997118166,-0.02639842278,
     &  0.0,-0.04618927856,-0.05455452050,0.0,0.0,
     &  0.05353643350,0.0,0.0,0.03182924150,0.0,
     &  0.0,-0.00816417752,0.0,0.0,0.0,
     &  0.01855129846,0.0,0.0,0.0,0.0,
     &  0.0,0.0,-0.00846595735,0.0,0.0,
     &  -0.00577905807,0.0,0.0,0.0,0.0,
     &  0.0,0.00332976648,0.0,0.0,0.0/
      DATA((BQ(2,3,J,K),K=0,4),J=0,7)/
     &  0.00498969686,0.0,0.0,0.0,0.0,
     &  0.0,-0.04575255584,-0.02682791900,0.0,0.0,
     &  0.06478150430,0.0,0.0,0.00441860953,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0,
     &  0.0,0.0,0.0,0.0,0.0/
      DATA((BQ(2,4,J,K),K=0,4),J=0,7)/
     &  0.00611885201,0.0,-0.00929600966,-0.02001179468,0.0,
     &  0.00158380930,-0.00966232209,0.0,0.0,0.0,
     &  0.05691190800,-0.03930398727,0.0,0.06857260544,-0.01653036734,
     &  0.0,-0.08310789026,0.0,0.0,0.0,
     &  0.02470706210,0.05550845308,0.0,0.0,0.0,
     &  -0.01423216502,0.02865617212,-0.04734982966,0.0,0.0,
     &  -0.00024078082,0.0,0.0,0.0,0.0,
     &  0.0,-0.00843962756,0.01175304622,0.0,0.0/
      DATA((BQ(2,5,J,K),K=0,4),J=0,7)/
     &  .00776994429,-0.00262822728,0.0,-.00710113626,-0.03422350708,
     &  0.00368693486,-0.030085797,0.0,.05486974932,0.0,.06781325374,
     &  -.04525018202,-.05326624574,.02870820566,.00557373124,
     &  -0.00453490946,0.0,0.0,0.0,0.0,
     &  0.0,0.06262054692,0.0,-0.01060604826,0.0,
     &  0.0,-0.04903303392,0.0,0.0,0.0,
     &  0.01494716424,0.0,0.0,0.0,0.0,
     &  -0.01009339672,0.00956065317,0.0,0.0,0.0/

C Abfrage von AEA0,PZD0 auf Uebereinstimmung mit einer AEA0,
C PZD0-Kombination, fuer die ein Berechnungspolynom vorliegt
C Gegebenenfalls Besetzen der Variablen I,II
      IF(ABS(AEA0-0.50).LT.EPS)I=1
      IF(ABS(AEA0-0.65).LT.EPS)I=2
      IF(ABS(PZD0-0.40).LT.EPS)II=1
      IF(ABS(PZD0-0.60).LT.EPS)II=2
      IF(ABS(PZD0-0.80).LT.EPS)II=3
      IF(ABS(PZD0-1.00).LT.EPS)II=4
      IF(ABS(PZD0-1.20).LT.EPS)II=5

C Suche nach den benachbarten AEA0,PZD0-Kombinationen und
C Berechnung des Kq-Wertes durch Aufruf von FKQW, LINPOL
      IF((I.EQ.1).OR.(I.EQ.2))THEN
           IF((II.EQ.1).OR.(II.EQ.2).OR.(II.EQ.3).OR.(II.EQ.4)
     &        .OR.(II.EQ.5))THEN
                FKQCPP=FKQW(I,II,PZDC,AC,BQ)
           ELSE 
                IF(PZD0.LT.0.60)THEN
                     PZD01=0.40
                     II1=1
                     PZD02=0.60
                     II2=2
                ELSE IF(PZD0.GT.0.60.AND.PZD0.LT.0.80)THEN
                     PZD01=0.60
                     II1=2
                     PZD02=0.80
                     II2=3  
                ELSE IF(PZD0.GT.0.80.AND.PZD0.LT.1.00)THEN
                     PZD01=0.80
                     II1=3
                     PZD02=1.00
                     II2=4 
               ELSE IF(PZD0.GT.1.00)THEN
                     PZD01=1.00
                     II1=4
                     PZD02=1.20
                     II2=5
                END IF
                FKQ1=FKQW(I,II1,PZDC,AC,BQ)
                FKQ2=FKQW(I,II2,PZDC,AC,BQ)
                FKQCPP=LINPOL(PZD0,PZD01,PZD02,FKQ1,FKQ2)
           END IF
      ELSE
           IF((II.EQ.1).OR.(II.EQ.2).OR.(II.EQ.3).OR.(II.EQ.4)
     &        .OR.(II.EQ.5))THEN
                AEA01=0.50
                I1=1
                AEA02=0.65
                I2=2
                FKQ1=FKQW(I1,II,PZDC,AC,BQ)
                FKQ2=FKQW(I2,II,PZDC,AC,BQ)
                FKQCPP=LINPOL(AEA0,AEA01,AEA02,FKQ1,FKQ2)
           ELSE
                AEA01=0.50
                I1=1
                AEA02=0.50
                I2=1
                AEA03=0.65
                I3=2
                I4=2
                IF(PZD0.LT.0.60)THEN
                      PZD01=0.40
                      II1=1
                      PZD02=0.60
                      II2=2
                      PZD03=0.40
                      II3=1
                      PZD04=0.60
                      II4=2
                ELSE IF(PZD0.GT.0.60.AND.PZD0.LT.0.80)THEN
                      PZD01=0.60
                      II1=2
                      PZD02=0.80
                      II2=3
                      PZD03=0.60
                      II3=2 
                      PZD04=0.80
                      II4=3
                ELSE IF(PZD0.GT.0.80.AND.PZD0.LT.1.00)THEN
                      PZD01=0.80
                      II1=3
                      PZD02=1.00
                      II2=4
                      PZD03=0.80
                      II3=3
                      PZD04=1.00
                      II4=4
                ELSE IF(PZD0.GT.1.00)THEN
                      PZD01=1.00
                      II1=4
                      PZD02=1.20
                      II2=5
                      PZD03=1.00
                      II3=4
                      PZD04=1.20
                      II4=5
                END IF
                FKQ1=FKQW(I1,II1,PZDC,AC,BQ)
                FKQ2=FKQW(I2,II2,PZDC,AC,BQ)
                FKQ3=FKQW(I3,II3,PZDC,AC,BQ)
                FKQ4=FKQW(I4,II4,PZDC,AC,BQ)
                FKQ5=LINPOL(PZD0,PZD01,PZD02,FKQ1,FKQ2)
                FKQ6=LINPOL(PZD0,PZD03,PZD04,FKQ3,FKQ4)
                FKQCPP=LINPOL(AEA0,AEA01,AEA03,FKQ5,FKQ6)
           END IF
      END IF
      RETURN
      END
C ---------------------------------------------------------
C Unterfunktion von FKQCPP()
C I = Kennummer fuer AEA0 (I=1 fuer AEA0=0.50, I=2 fuer AEA0=0.65)
C II = Kennummer fuer PZD0 (II=1 fuer PZD0 =0.40, II=2 fuer PZD0 =0.60,
C                          II=3 fuer PZD0 =0.80, II=4 fuer PZD0 =1.00,
C                          II=5 fuer PZD0 =1.20)

      REAL FUNCTION FKQW(I,II,PZDC,AC,BQ)
      INTEGER I,II,J,K
      REAL    AC,BQ(2,5,0:7,0:4),EPS,PZDC,SUM,SUM1,SUM2
      EPS=1.0E-6

C Nullabfrage und Neubesetzen von AC, PZDC
      IF(ABS(PZDC).LT.EPS)PZDC=EPS
      IF(ABS(AC).LT.EPS)AC=EPS

C Berechnung des Kq-Wertes
      SUM2=0.0
      DO 100 J=0,7
           SUM=0.0
           SUM1=0.0     
                DO 105 K=0,4
                SUM=BQ(I,II,J,K)*PZDC**J*AC**K
                SUM1=SUM1+SUM
  105           CONTINUE
           SUM2=SUM2+SUM1
  100 CONTINUE
      FKQW=SUM2
      END
C ---------------------------------------------------------
C Unterfunktion von FKTCPP() und FKQCPP()
C Lineare Interpolation

      REAL FUNCTION LINPOL(X,X1,X2,Y1,Y2)
      REAL X,X1,X2,Y1,Y2
      LINPOL=(Y2-Y1)/(X2-X1)*(X-X1)+Y1
      END
C ---------------------------------------------------------
C Berechnung von Kt fuer Propeller/Duesen-Kombinationen ohne Korrektur
C fuer Zaehigkeit
C Folgende Propeller/Duesen-Kombinationen koennen ueber den Parameter
C PROTYP ausgewaehlt werden:
C 0 -->  Ka 3-65  in Duese 19a
C 1 -->  Ka 4-55  in Duese 19a
C 2 -->  Ka 4-70  in Duese 19a
C 3 -->  Ka 5-75  in Duese 19a
C 4 -->  Ka 4-70  in Duese 37
C 5 -->  Ka 4-70  in Duese 24
C 6 -->  Ka 4-70  in Duese 22
C 7 -->  Kd 5-100 in Duese 33
C Literatur: Oosterveld und van Oortmessen, Thruster systems for Improving
C the Manueverability and Position-Keeping Capability Of Floating Objects,
C Offshore Technology Conference 1972, Paper Nr. 1625

      REAL FUNCTION FKTN(PZD,D,VA,N,CONFIG)
      REAL PZD,D,VA,N,AC,A
      INTEGER I,J,CONFIG
      DIMENSION A(0:800)

C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
C      PRINT*,'FKTN-Meldung:' 
C      PRINT*,' PZD= ',PZD,' CONFIG= ',CONFIG
C      PRINT*,'D= ',D,' VA= ',VA,' N= ',N

C Ka 3-65 in Duese 19a
      DATA (A(I),I=0,55)/
     &+0.028100,-0.143910,+0.000000,-0.383783,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.429709,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.671268,+0.000000,+0.286926,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.182294,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.017378,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-55 in Duese 19a
      DATA (A(I),I=100,155)/
     &-0.375000,-0.203050,+0.830306,-2.746930,+0.000000,+0.0,
     &                                                 0.0067548,0.0,
     &+2.030070,-0.392301,-0.611743,+4.319840,-0.341290,+0.0,0.0,0.0,
     &-3.031670,+0.000000,+0.000000,-2.007860,+0.000000,+0.0,0.0,0.0,
     &+2.836970,+0.000000,+0.000000,+0.391304,+0.000000,+0.0,0.0,0.0,
     &-0.994962,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.015742,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.043782,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-70 in Duese 19a
      DATA (A(I),I=200,255)/
     &+0.030550,-0.148687,+0.000000,-0.391137,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.432612,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.667657,+0.000000,+0.285076,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.172529,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.017293,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 5-75 in Duese 19a
      DATA (A(I),I=300,355)/
     &+0.033000,-0.153463,+0.000000,-0.398491,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.435515,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.664045,+0.000000,+0.283225,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.162764,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.017208,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-70 in Duese 37
      DATA (A(I),I=400,455)/
     &-0.162557,+0.000000,+0.000000,+0.000000,-0.077387,+0.0,0.0,
     &                                        +0.036998,
     &+0.598107,-1.009030,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.085087,+0.425585,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,-0.021044,
     &                                                       0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.038383,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.014992,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-70 in Duese 24
      DATA (A(I),I=500,555)/
     &-0.056541,+0.001197,+0.000000,-1.234240,+0.000000,+0.0,0.0,
     &                                        +0.452712,
     &+0.235791,-0.705042,+0.000000,-0.159436,+0.000000,+0.908131, 
     &                                        -0.840424,+0.000000,
     &+0.454377,+0.000000,+1.254499,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.110537,+0.000000,-0.392827,-0.205174,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.022669,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.011161,+0.000000,+0.023147,+0.000000,+0.0,0.0,0.0/
C Ka 4-70 in Duese 22
      DATA (A(I),I=600,655)/
     &+0.008043,+0.000000,-0.208843,-0.902650,+0.000000,+0.369317,
     &                                        +0.000000,+0.000000,
     &+0.000000,-0.661804,+0.752246,+0.000000,+0.000000,-0.159272, 
     &                                        +0.000000,+0.000000,
     &+0.720632,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.202075,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/
C Kd 5-100 in Duese 33
      DATA (A(I),I=700,755)/
     &-0.347562,-0.321224,+0.075277,+0.000000,-0.009560,+0.0,0.0,0.0,
     &+0.963261,-0.215803,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.013401,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.016882,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/

      FKTN=0.0
      DO 100 I=0,6
         DO 100 J=0,7
            FKTN=FKTN + PZD**I * A(I*8+J+CONFIG*100)* AC**J
100   CONTINUE 
      RETURN 
      END  
C ---------------------------------------------------------
C Berechnung von Kq fuer Propeller/Duesen-Kombinationen ohne Korrektur
C fuer Zaehigkeit
C Folgende Propeller/Duesen-Kombinationen koennen ueber den Parameter
C CONFIG ausgewaehlt werden:
C 0 -->  Ka 3-65  in Duese 19a
C 1 -->  Ka 4-55  in Duese 19a
C 2 -->  Ka 4-70  in Duese 19a
C 3 -->  Ka 5-75  in Duese 19a
C 4 -->  Ka 4-70  in Duese 37
C 5 -->  Ka 4-70  in Duese 24
C 6 -->  Ka 4-70  in Duese 22
C 7 -->  Kd 5-100 in Duese 33
C Literatur: Oosterveld und van Oortmessen, Thruster systems for Improving
C the Manueverability and Position-Keeping Capability Of Floating Objects,
C Offshore Technology Conference 1972, Paper Nr. 1625

      REAL FUNCTION FKQN(PZD,D,VA,N,CONFIG)
      REAL PZD,D,VA,N,AC,A
      INTEGER I,J,CONFIG
      DIMENSION A(0:800)

C fuer die Berechnung der Fortschrittsziffer muessen N,VA und D
C groesser 0.0 bzw. einem sehr kleinem Wert sein
      N = MAX(N,1.E-5) 
      D = MAX(D,1.E-5) 
      VA= MAX(VA,1.E-5) 
      AC=VA/N/D
C      PRINT*,'FKQN-Meldung:' 
C      PRINT*,' PZD= ',PZD,' CONFIG= ',CONFIG
C      PRINT*,'D= ',D,' VA= ',VA,' N= ',N

C Ka 3-65 in Duese 19a
      DATA (A(I),I=0,55)/
     &+0.006260,+0.000000,-0.017942,+0.000000,-0.008089,+0.0,0.0,0.0,
     &+0.000000,+0.000000,-0.016644,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.040041,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.003460,-0.000674,+0.001721,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-55 in Duese 19a
      DATA (A(I),I=100,155)/
     &-0.034700,+0.018568,+0.000000,+0.000000,-0.195582,+0.317452,
     &   -0.093739,+0.022850,
     &+0.158951,-0.048433,+0.000000,+0.024157,+0.000000,-0.123376,
     &   +0.000000,+0.0,
     &-0.212253,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.156133,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.030740,+0.073587,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.031826,-0.014568,-0.109363,+0.000000,+0.043862,+0.0,0.0,0.0,
     &+0.007947,+0.000000,+0.038275,+0.000000,-0.021971,+0.000000,
     &+0.000700,0.0/
C Ka 4-70 in Duese 19a
      DATA (A(I),I=200,255)/
     &+0.006735,+0.000000,-0.016306,+0.000000,-0.007244,+0.0,0.0,0.0,
     &+0.000000,+0.000000,-0.024012,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.005193,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.046605,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.007366,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.001730,-0.000337,+0.000861,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 5-75 in Duese 19a
      DATA (A(I),I=300,355)/
     &+0.007210,+0.000000,-0.014670,+0.000000,-0.006398,+0.0,0.0,0.0,
     &+0.000000,+0.000000,-0.031380,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.010386,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.053169,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.014731,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-70 in Duese 37
      DATA (A(I),I=400,455)/
     &+0.016729,+0.000000,+0.000000,+0.000000,0.000000,+0.000000,
     &                                        +0.030559,-0.012160,
     &-0.048424,-0.011118,-0.056199,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.084376,+0.000000,+0.045637,-0.042003,+0.000000,+0.0,0.0,0.0,
     &-0.008652,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.001176,+0.002441,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-70 in Duese 24
      DATA (A(I),I=500,555)/
     &+0.023557,+0.000000,-0.016989,+0.000000,+0.000000,+0.000000,
     &                                        +0.082386,-0.049039,
     &-0.072021,+0.000000,+0.000000,-0.037596,-0.034871,+0.0,0.0,0.0,
     &+0.103364,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.013447,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,
     &                                        +0.013180,+0.000000,
     &+0.000000,+0.000000,-0.012173,+0.046464,-0.035041,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0/
C Ka 4-70 in Duese 22
      DATA (A(I),I=600,655)/
     &+0.032079,+0.000000,-0.020219,-0.021294,+0.000000,+0.0,0.0,
     &                                                  +0.001334,
     &-0.102805,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.140281,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &-0.026416,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,-0.008516,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.005229,+0.000000,+0.000000,-0.000195,+0.0,0.0,0.0/
C Kd 5-100 in Duese 33
      DATA (A(I),I=700,755)/
     &-0.007789,-0.022424,+0.000000,-0.009087,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,-0.010492,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.082463,+0.000000,+0.026193,-0.009585,+0.000000,+0.001029,
     &                                                       0.0,0.0,
     &+0.000000,-0.007692,+0.000000,+0.000000,+0.000000,+0.0,
     &                                                 -0.000094,0.0,
     &-0.003196,+0.000000,+0.000000,+0.000000,-0.000117,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000000,+0.000000,+0.0,0.0,0.0,
     &+0.000000,+0.000000,+0.000000,+0.000152,+0.000000,+0.0,0.0,0.0/

       FKQN=0
       DO 100 I=0,6
          DO 100 J=0,7
             FKQN=FKQN + PZD**I * A(I*8+J+CONFIG*100)*AC**J
100    CONTINUE 
       RETURN 
       END
C ---------------------------------------------------------
