C.....Program to generate grid on cylinder submerged under free surface
C     preprocessor for SHAL2D
C.....main variables
C     A      area of panel
C     AWET   wetted area of cylinder (approximately circumference of cylinder)
C     D      submergence of centre of cylinder below still water level
C     DELPHI delta phi = angle of each segment within circle
C     DX     delta x between two end points of a segment
C     DZ     delta z between two end points of a segment
C     FAK    desingularisation factor
C     KN     pointer to global numbers of end points of each segment
C     LP     maximum number of points
C     N      inward unit normal
C     NB     number of corner points
C     NPB    number of collocation point = number of sources 
C     R      radius of cylinder
C     XD     coordinates of segments describing surface of cylinder
C     X      collocation points (segment centres)
C     XQ     source points, submerged inside cylinder
      IMPLICIT NONE
      INTEGER I, LP, NB, NPB
      REAL AWET, D, DELPHI, DX, DZ, FAK, PI, R
      PARAMETER (LP=100)
      INTEGER KN(2,LP)
      REAL A(LP), N(2,LP), X(2,LP), XD(2,LP), XQ(2,LP)
      PI = 3.141592654
      FAK = 0.6666
100   CONTINUE
      WRITE(*,*) '>> Input: Radius, Number of Panels, Depth'
      READ(*,*,ERR=100) R, NPB, D
C.....plausibility control
      IF ((R.LE.0).OR.(NPB.LT.4).OR.(NPB.GT.LP)) GOTO 100
C.....generate cylinder first around (0,0); first generate grid corner points
      NB = NPB
      DELPHI = 2*PI/NPB
      XD(1,1) = 0.
      XD(2,1) = R
      KN(1,1) = NPB
      KN(2,1) = 1
      DO I = 2, NPB
         XD(1,I) = R * SIN((I-1)*DELPHI)
         XD(2,I) = R * COS((I-1)*DELPHI)
         KN(1,I) = KN(2,I-1)
         KN(2,I) = KN(1,I)+1
      END DO
C.....generate collocation points, unit normals, source points, wetted area
      AWET = 0.
      DO I=1,NPB
         DX = XD(1,KN(2,I))-XD(1,KN(1,I))
         DZ = XD(2,KN(2,I))-XD(2,KN(1,I))
         A(I) = SQRT(DX**2+DZ**2)
         AWET = AWET+A(I)
         X(1,I) = (XD(1,KN(1,I))+XD(1,KN(2,I)))/2
         X(2,I) = (XD(2,KN(1,I))+XD(2,KN(2,I)))/2
         N(1,I) =  DZ/A(I)
         N(2,I) =  DX/A(I)
         XQ(1,I) = X(1,I) + N(1,I)*A(I)*FAK
         XQ(2,I) = X(2,I) - N(2,I)*A(I)*FAK
      END DO
      WRITE(*,*) 'Wetted surface', AWET, ' (=',2*PI*R,' analytically)'
C.....transformation for submergence
      DO I=1,NPB
         X(2,I)  = - X(2,I) + D
         XD(2,I) = -XD(2,I) + D
         XQ(2,I) = -XQ(2,I) + D
      END DO
C.....output on file
      OPEN(10,FILE='CYL.DAT',STATUS='UNKNOWN')
         WRITE(10,*) NPB,NPB
         WRITE(10,*) AWET
         DO I=1,NPB
            WRITE(10,*) XD(1,I), XD(2,I)
         END DO
         DO I=1,NPB
            WRITE(10,*) KN(1,I), KN(2,I)
            WRITE(10,*) X(1,I),X(2,I),XQ(1,I),XQ(2,I)
            WRITE(10,*) A(I), N(1,I), N(2,I)
         END DO
      CLOSE(10)
      END
