    ---------------------------------------------------------------------
C Test program shallow water added mass
      real zk(120),yk(120),ome(50)
      complex addedm(2,2)                           !zwl= z-coord. waterline
      read(5,*)zwl,zb,nk,(yk(k),k=1,nk),(zk(k),k=1,nk) !zb=1e6 for deep water
c reads depth WL under origin,number of contour points,y values,z values 
      read(5,*)nome,(ome(iome),iome=1,nome)
c reads number of circular frequencies, then circular frequencies
      do i=1,nome
         call hmassef(zwl,zb,yk,zk,nk,ome(i),1,addedm)
         write(6,*)'heave:',ome(i),addedm(1,1)
         call hmassef(zwl,zb,yk,zk,nk,ome(i),2,addedm)
         write(6,*)'sway-roll:',addedm
      end do
      end

ccc    ---------------------------------------------------------------------
cccC Test program for exciting forces in waves
ccc      real zk(120),yk(120),wink(10)
ccc      complex diff1(1,10),diff2(2,10),frkr1(1,10),frkr2(2,10)
ccc      read(5,*)zwl,zb,om,nw,(wink(iw),iw=1,nw)    !zb=1e6 for deep water
ccc      read(5,*)nk,(yk(k),k=1,nk),(zk(k),k=1,nk)     !zwl= z-coord. WL
ccc      do io=1,1
ccc       call werreg(zwl,zb,yk,zk,nk,om,nw,wink,1,diff1,frkr1)
ccc       write(6,*)'Vertikal diff:',(diff1(1,iw),iw=1,nw)
ccc       write(6,*)'Vertikal frkr:',(frkr1(1,iw),iw=1,nw)
ccc       call werreg(zwl,zb,yk,zk,nk,om,nw,wink,2,diff2,frkr2)
ccc       write(6,*)'Hor. und Moment diff:',((diff2(i2,iw),iw=1,nw),i2=1,2)
ccc       write(6,*)'Hor. und Moment frkr:',((frkr2(i2,iw),iw=1,nw),i2=1,2)
ccc      enddo
ccc      end

      function pq(dy,dz)
C pq source potential. dy,dz field point coordinate - source point coordinate
      pq=0.5*log(dy**2+dz**2)
      end

      function pqn(dy1,dz1,dy2,dz2)
C pqn integral normal velocity (to right) between points P1,P2
      pqn=atan2(dy2*dz1-dz2*dy1,dy1*dy2+dz1*dz2)
      end

      function pqs(zb,yk,zk,yi,zi,mp)            !with mirror image to y=0
      pqs=pq(yk-yi,zk-zi)+mp*pq(yk+yi,zk-zi)
      if(zb<1000.)pqs=pqs+pq(yk-yi,zk-(2*zb-zi))
     &                +mp*pq(yk+yi,zk-(2*zb-zi))
      end

      function pqsn(zb,yk1,zk1,yk2,zk2,yi,zi,mp) !with mirror image to y=0
      pqsn=pqn(yk1-yi,zk1-zi,       yk2-yi,zk2-zi)
     & +mp*pqn(yk1+yi,zk1-zi,       yk2+yi,zk2-zi)
      if(zb<1000.)pqsn=pqsn
     &    +pqn(yk1-yi,zk1-(2*zb-zi),yk2-yi,zk2-(2*zb-zi))
     & +mp*pqn(yk1+yi,zk1-(2*zb-zi),yk2+yi,zk2-(2*zb-zi))
      end

      subroutine hmassef(zwl,zb,yk,zk,nk,ome,ns,addedm)
C complex added mass for deep or shallow water (= added mass -i/omega_e*damping); 
C bottom at zb, water surface at zwl
C for deep water: zb=1e6. water density fixed to 1.
C symmetrical cross section
C 
C ns=2 for antimetric motions and forces, =1 for symmetric
C yk,zk must be arrays with 120 elements; elements 1 to nk<=50
C contain points for r.h.s. half of cross section, from bottom to top.
C 
C output: addedm (complex array 1*1 for ns=1, 2*2 for ns=2)
      parameter(pi=3.14159,g=9.81,nfd=55)
      complex a(0:120,0:120+2),ciome,detl,f(2,2),addedm(ns,ns)
      real yk(120),zk(120),yi(0:120),zi(0:120)
      if(nk.gt.50)stop'*** too many points on cross section. max=50.'
      if(ns.ne.1.and.ns.ne.2)stop'*** wrong parameter ns (1 or 2)'
      if(nk+nfd>120)stop'*** too many panels'
      addedm(1:ns,1:ns)=0.
      do nf=25,28,3            !compute with two nf-values, add
C +y to starboard; +z downward. height last point = WL height
      mp=3-2*ns
      ciome=(0.,1.)*ome
      t=zb-zwl !t=water depth
      rk=max(ome**2/g,ome/sqrt(g*t))
      if(rk*t.gt.5)goto 3
      do 4 i=1,10
         rk=rk-(rk*tanh(rk*t)-ome**2/g)/(tanh(rk*t)+rk/cosh(rk*t)**2)/2
 4    continue
 3    hend=2*pi/(12*rk)
      h=sqrt((yk(nk)-yk(nk-1))**2+(zk(nk)-zk(nk-1))**2)
C compute surface collocation points yk,zk and all source points yi,zi
      do 10 k=1,nk-1
         yi(k)=(yk(k)+yk(k+1))/2+(zk(k+1)-zk(k))/20
         zi(k)=(zk(k)+zk(k+1))/2-(yk(k+1)-yk(k))/20
 10   continue
      do 15 k=nk,nk+nfd-1
         h=min(h*1.5,hend)
         yk(k+1)=yk(k)+h
         zk(k+1)=zwl
         yi(k)=yk(k)+0.5*h
         zi(k)=zwl-1.0*h
 15   continue
      yi(0)=0
      zi(0)=zwl-yk(nk+nfd)/2
C hull boundary condition
      do 21 k=1,nk-1
         if(mp.eq.1)then
            a(k,nk+nfd)=-ciome*(yk(k+1)-yk(k))
         else
            a(k,nk+nfd)=+ciome*(zk(k+1)-zk(k))
            a(k,nk+nfd+1)=-ciome*
     &                    (yk(k+1)**2-yk(k)**2+zk(k+1)**2-zk(k)**2)/2
      endif
      do 20 i=ns-1,nk+nfd-1
         a(k,i)=pqsn(zb,yk(k),zk(k),yk(k+1),zk(k+1),yi(i),zi(i),mp)
 20   continue
 21   continue
C surface boundary condition near hull
      do 26 k=nk,nk+nf
      a(k,nk+nfd)=0
      a(k,nk+nfd+1)=0
      dy=yk(k+1)-yk(k)
      do 25 i=ns-1,nk+nfd-1  
C 2-point integration (.316,.684) makes integral source potnetial correct at k=i
      a(k,i)=pqsn(zb,yk(k),zwl,yk(k+1),zwl,yi(i),zi(i),mp)
     &  +ome**2/g*dy*(pqs(zb,yk(k)+0.316*dy,zwl,yi(i),zi(i),mp)
     &               +pqs(zb,yk(k)+0.684*dy,zwl,yi(i),zi(i),mp))/2
 25   continue
 26   continue
C surface boundary condition far away from hull
      do 31 k=nk+nf+1,nk+nfd-1
      a(k,nk+nfd)=0
      a(k,nk+nfd+1)=0
      dy=yk(k+1)-yk(k)
      fak=(real(k-nk-nf)/(nfd-nf-1))**2
      do 30 i=ns-1,nk+nfd-1
      a(k,i)=rk*dy*(pqs(zb,yk(k)+0.316*dy,zwl,yi(i),zi(i),mp)
     & +pqs(zb,yk(k)+0.684*dy,zwl,yi(i),zi(i),mp))/2 -(0.,1.)*
     & (pqs(zb,yk(k+1)-fak*dy,zwl,yi(i),zi(i),mp)
     & -pqs(zb,yk(k  )-fak*dy,zwl,yi(i),zi(i),mp))
 30   continue
 31   continue
C sum of all source strength = 0 at ns=1
      if(ns.eq.1)then
         do 40 i=ns-1,nk+nfd-1
            a(0,i)=1.
 40      continue
         a(0,nk+nfd)=0.
      endif
      call simqcd(a(ns-1,ns-1),nk+nfd+1-ns,ns,120+1,is,1.e-5,detl)
      if(is.ne.0)stop'*** Singular System. 2 coinciding points?'
      write(8,*)real(a(1:nk+nfd-1,nk+nfd))
      write(8,*)aimag(a(1:nk+nfd-1,nk+nfd))
C force integration. n=1 for heave, n=2 for sway and roll
C For ns=2: sway and roll motion (l=1,2), force and moment (m=1,2)
      do 81 m=1,ns
      do 80 l=1,ns
      f(m,l)=0
      do 71 k=1,nk-1
      dy=yk(k+1)-yk(k)
      dz=zk(k+1)-zk(k)
      if(mp.eq.1)then
       fakt=-dy
      else
       if(m.eq.1)fakt=dz
       if(m.eq.2)fakt=-0.5*(yk(k+1)**2-yk(k)**2+zk(k+1)**2-zk(k)**2)
      endif
      do 70 i=ns-1,nk+nfd-1
      f(m,l)=f(m,l)+a(i,nk+nfd-1+l)*fakt*
     &  (pqs(zb,yk(k)+0.316*dy,zk(k)+0.316*dz,yi(i),zi(i),mp)
     &  +pqs(zb,yk(k)+0.684*dy,zk(k)+0.684*dz,yi(i),zi(i),mp))/2
 70   continue
 71   continue
      addedm(m,l)=addedm(m,l)-ciome*f(m,l)/ome**2
 80   continue
 81   continue
      enddo
      end


      subroutine werreg(zwl,zb,yk,zk,nk,om,nw,wink,ns,diff,frkr)
C compute complex amplitude of diffraction force diff and Froude-Krilov force
C frkr in wave of amplitude 1, circular frequency om from direction wink
C (array with nw<=10 values; in degree; 0 longitudinal, 90 from right). 
C symmetric cross sections at free surface on deep or shallow water; density=1.
C ns=2 sway force and roll moment, =1 for heave force
C yk,zk must be arrays with 120 elements. The first nk<=50 elements
C contain coordinates for right half of cross section, from bottom to top.
C diff,frkr must be complex arrays with ns*nw elements.
      parameter(pi=3.14159,g=9.81,nfd=55)
      complex a(0:120,0:130),ciom,detl,f(2,nw),diff(ns,nw),frkr(ns,nw),
     & zw1,zw2,zw1a,zw2a
      real yk(120),zk(120),yi(0:120),zi(0:120),wink(10)
      if(nk.gt.50)stop'*** too many points on section. max. 50.'
      if(ns.ne.1.and.ns.ne.2)stop'*** wrong parameter ns (1 or 2)'
      if(nw.gt.10)stop'*** too many incident angles. max. 10'
      if(nk+nfd>120)stop'too panels'
      diff(1:ns,1:nw)=0.
      do nf=25,28,3 !compute twice, average with Diff. lambda/4
C +y to starboard; +z downwards. height last point = WL height
      mp=3-2*ns
      ciom=(0.,1.)*om
      t=zb-zwl !t=water depth
      rk=max(om**2/g,om/sqrt(g*t))
      if(rk*t<6)then
       do 4 i=1,10
        rk=rk-(rk*tanh(rk*t)-om**2/g)/(tanh(rk*t)+rk/cosh(rk*t)**2)/2
 4     continue
       f1=g*exp( rk*t)/2/cosh(rk*t)
       f2=g*exp(-rk*t)/2/cosh(rk*t)
      else
       f1=g; f2=0.
      endif
      h=sqrt((yk(nk)-yk(nk-1))**2+(zk(nk)-zk(nk-1))**2)
      hend=2*pi/(12*rk)
      print *,'k',rk,h,hend,f1,f2
C compute surface coll. points yk,zk and all source points yi,zi
      do 10 k=1,nk-1
      yi(k)=(yk(k)+yk(k+1))/2+(zk(k+1)-zk(k))/20
      zi(k)=(zk(k)+zk(k+1))/2-(yk(k+1)-yk(k))/20
 10   continue
      do 15 k=nk,nk+nfd-1
      h=min(h*1.5,hend)
      yk(k+1)=yk(k)+h
      zk(k+1)=zwl
      yi(k)=yk(k)+0.5*h
      zi(k)=zwl-1.0*h
 15   continue
      yi(0)=0
      zi(0)=zwl-yk(nk+nfd)/2
      do k=0,nk+nfd-1
       do iw=1,nw
        a(k,nk+nfd-1+iw)=0.   !inhomogeneous terms
       enddo
      enddo
C hull boundary condition (integral over one line section) and Froude-Krilov force
      do is=1,ns; do iw=1,nw
       frkr(is,iw)=0
      enddo; enddo
      do 22 k=1,nk-1
      dy=yk(k+1)-yk(k)
      dz=zk(k+1)-zk(k)
      do 19 iw=1,nw
      sinw=sin(wink(iw)/57.296)
      zw1=cmplx(-rk*dz,rk*sinw*dy)
      if(abs(zw1).lt. .001)zw1=0.001
      zw1a=-conjg(zw1)
      zw2 =exp(rk*cmplx(-zk(k)+zwl,yk(k)*sinw))*(exp(zw1 )-1.)/zw1
      zw2a=exp(rk*cmplx( zk(k)-zwl,yk(k)*sinw))*(exp(zw1a)-1.)/zw1a
      if(mp.eq. 1)then
       a(k,nk+nfd-1+iw)=f1*rk/om*real(cmplx(dy, sinw*dz)*zw2)*(0.,1.)
       frkr(1,iw)=frkr(1,iw)+f1*2*real(zw2 )*dy
       if(f2/=0)then
        a(k,nk+nfd-1+iw)=a(k,nk+nfd-1+iw)
     &   -f2*rk/om*real(cmplx(dy,-sinw*dz)*zw2a)*(0.,1.)
        frkr(1,iw)=frkr(1,iw)+f2*2*real(zw2a)*dy
       endif
      else
       a(k,nk+nfd-1+iw)=f1*rk/om*real(cmplx(-sinw*dz,dy)*zw2)
       frkr(1,iw)=frkr(1,iw)-f1*(0.,2.)*aimag(zw2)*dz
       frkr(2,iw)=frkr(2,iw)+f1*(0.,1.)*aimag(zw2)*
     &  (yk(k+1)**2-yk(k)**2+zk(k+1)**2-zk(k)**2)
       if(f2/=0)then
       a(k,nk+nfd-1+iw)=a(k,nk+nfd-1+iw)
     &  -f2*rk/om*real(cmplx( sinw*dz,dy)*zw2a)
       frkr(1,iw)=frkr(1,iw)-f2*(0.,2.)*aimag(zw2a)*dz
       frkr(2,iw)=frkr(2,iw)+f2*(0.,1.)*aimag(zw2a)*
     &  (yk(k+1)**2-yk(k)**2+zk(k+1)**2-zk(k)**2)
       endif
      endif
 19   continue
      do 20 i=ns-1,nk+nfd-1
      a(k,i)=pqsn(zb,yk(k),zk(k),yk(k+1),zk(k+1),yi(i),zi(i),mp)
 20   continue
 22   continue
C surface boundary condition near hull
      do 26 k=nk,nk+nf
      dy=yk(k+1)-yk(k)
      do 25 i=ns-1,nk+nfd-1
C 2-point integration (.316,.684) makes integral source pot. correct at k=i
      a(k,i)=pqsn(zb,yk(k),zwl,yk(k+1),zwl,yi(i),zi(i),mp)
     & +om**2/g*dy*(pqs(zb,yk(k)+0.316*dy,zwl,yi(i),zi(i),mp)
     &          +pqs(zb,yk(k)+0.684*dy,zwl,yi(i),zi(i),mp))/2
 25   continue
 26   continue
C surface boundary condition far from body
      do 31 k=nk+nf+1,nk+nfd-1
      dy=yk(k+1)-yk(k)
      fak=(real(k-nk-nf)/(nfd-nf-1))**2
      do 30 i=ns-1,nk+nfd-1
      a(k,i)=rk*dy*(
     &  pqs(zb,yk(k)+0.316*dy,zwl,yi(i),zi(i),mp)
     & +pqs(zb,yk(k)+0.684*dy,zwl,yi(i),zi(i),mp))/2
     & -(0.,1.)*(pqs(zb,yk(k+1)-fak*dy,zwl,yi(i),zi(i),mp)
     &          -pqs(zb,yk(k)  -fak*dy,zwl,yi(i),zi(i),mp))
 30   continue
 31   continue
C sum source strengths = 0 at ns=1
      if(ns.eq.1)then
      do 40 i=ns-1,nk+nfd-1
      a(0,i)=1.
 40   continue
      endif
      call simqcd(a(ns-1,ns-1),nk+nfd+1-ns,nw,120+1,is,1.e-5,detl)
      if(is.ne.0)stop'*** Singular system. coinciding contour points?'
C ns=1 for heave force, ns=2 for sway force and roll moment
      do 81 m=1,ns
      do 80 l=1,nw
      f(m,l)=0
      do 71 k=1,nk-1
      dy=yk(k+1)-yk(k)
      dz=zk(k+1)-zk(k)
      if(mp.eq.1)then
       fakt=-dy
      else
       if(m.eq.1)fakt=dz
       if(m.eq.2)fakt=-0.5*(yk(k+1)**2-yk(k)**2+zk(k+1)**2-zk(k)**2)
      endif
      do 70 i=ns-1,nk+nfd-1
      f(m,l)=f(m,l)+a(i,nk+nfd-1+l)*fakt*
     &  (pqs(zb,yk(k)+0.316*dy,zk(k)+0.316*dz,yi(i),zi(i),mp)
     &  +pqs(zb,yk(k)+0.684*dy,zk(k)+0.684*dz,yi(i),zi(i),mp))/2
 70   continue
 71   continue
      diff(m,l)=diff(m,l)-ciom*f(m,l)
 80   continue
 81   continue
      enddo
      end


      SUBROUTINE SIMQCD(A,N,NR,IM,I,S,DETL)
C Gauss algorithm to solve complex system of linear equations
C with possibly several r.h.s. 
C A,DETL are complex!
C A=coefficient matrix + negative right-hand sides; after execution r.h.s. are replaced 
C   by solutions, matrix is destroyed
C   1. Index of A = line (1 to N), 2. Index = column (1 to N+NR)
C N=number of equations
C NR=number of right-hand sides
C IM=dimensioning for 1. index in dimensioning statement for A of calling routine
C I=0 normally; otherwise error occurred: pivot value less than S.
C recommended for S: 1.E-5.
C DETL is the natural log(determinant of coefficient matrix)
      IMPLICIT COMPLEX (A-H,O-Z)
      REAL S
      DIMENSION A(*)
      NNR=(N+NR)*IM
      MMR=N*IM
      K1=-IM
      DETL=0.
      DO 52 I=1,N
      K1=K1+IM+1
      BIGA=A(K1)
      IMAX=K1
      DO 30 J=K1+1,K1+N-I
      IF(ABS(BIGA).LT.ABS(A(J)))THEN
      BIGA=A(J)
      IMAX=J
      ENDIF
30    CONTINUE
      IF(ABS(BIGA).LT.S)RETURN
      DETL=DETL+LOG(BIGA)
      A(IMAX)=A(K1)
      DO 51 K=K1+IM,NNR,IM
      IMAX=IMAX+IM
      SAVE=-A(IMAX)/BIGA
      A(IMAX)=A(K)
      A(K)=SAVE
      K2=K1
      DO 50 J=K+1,K+N-I
      K2=K2+1
      A(J)=A(J)+A(K2)*SAVE
 50   continue
 51   continue
 52   continue
      DO 82 I=MMR,NNR-1,IM
      K1=MMR+1
      DO 81 J=N,2,-1
      K1=K1-IM
      K2=I
      SAVE=A(I+J)
      DO 80 K=K1,K1+J-2
      K2=K2+1
      A(K2)=A(K2)+A(K)*SAVE
 80   continue
 81   continue
 82   continue
      I=0
      END

