      SUBROUTINE JENSEN(NPB,KN,UBOOT,AS,BS,NSYM,
     &                 A,N,R,S,VNK,VSK,VTK,X,XD)
C.....jensen panel for closed body 
C     writes coefficients for normal and tangential velocity on unformatted file channel 20
C     works for monohull, either submarine or surface-piercing ship
C     NPB      number of panels on body
C     KN       array with number of corners of panel (KN(0)) and global indices of corner points (KN(1)...)
C     UBOOT    true: submarine = fully submerged (closed) body       
C     AS, BS   coefficients needed to compute mirror image at 'deckline', compute in SPIDEF, used in MIRROR
C     NSYM     coefficient needed to compute mirror image at 'deckline', compute in SPIDEF, used in MIRROR
C     N        unit normal (inward)
C     R        radius of tangential sphere, center of sphere must be inside body
C     S        unit tangential vector
C     VNK      auxiliary array for normal coefficients
C     VSK      auxiliary array for tangential coefficients in direction of S
C     VTK      as VSK, but tangential vector T perpendicular to N and S
C     X        xyz coordinates of panel centers
C     XD       xyz coordinates of all corner points of elements, KN is pointer to these corner points
C     A        area of panel
C     M        element (source) strength, often denoted by sigma in literature
      IMPLICIT NONE
      INTEGER I, K, KN(0:5,*), NPB, SP, STEP
      LOGICAL UBOOT
      REAL A(*), AS, BS, DUM, F, FF, N(3,*),
     &     NS(3,0:3), NSYM, O(3), OX(3), OXB, PK(3), QX,QY,QZ, 
     &     R(*), S(3,*), T(3), VNK(*), VSK(*), VTK(*), X(3,*), 
     &     XD(3,*), XS(3,0:3)  
      DO K = 1, NPB
C........second tangential vector
         T(1) = S(3,K)*N(2,K)-S(2,K)*N(3,K)
         T(2) = S(1,K)*N(3,K)-S(3,K)*N(1,K)
         T(3) = S(2,K)*N(1,K)-S(1,K)*N(2,K)
C........sphere center
         O(1) = X(1,K) + R(K)*N(1,K)
         O(2) = X(2,K) + R(K)*N(2,K)
         O(3) = X(3,K) + R(K)*N(3,K)
C........initialize coefficients for velocity coefficients
         DO I = 1, NPB
            VNK(I) = 0.0
            VSK(I) = 0.0
            VTK(I) = 0.0
         END DO
         VNK(K) = -0.5
C........hull on hull coefficients
         DO I = 1, NPB
C...........source and mirror images
            CALL QULKY(1,.TRUE.,KN(0,I),X(1,I),X(1,K),XD,A(I),
     &                    AS,BS,1.,NSYM,QX,QY,QZ,
     &                    DUM,DUM,DUM,DUM,DUM,DUM,DUM,DUM,DUM,DUM)
            VNK(I) = VNK(I) + N(1,K)*QX+N(2,K)*QY+N(3,K)*QZ
            VTK(I) = VTK(I) +   T(1)*QX+  T(2)*QY+  T(3)*QZ
            VSK(I) = VSK(I) + S(1,K)*QX+S(2,K)*QY+S(3,K)*QZ
C...........subtract integral over closed body of tangential components
            CALL MIRROR(AS,BS,0.0 ,N(1,I),NS)
            CALL MIRROR(AS,BS,NSYM,X(1,I),XS)
C...........for submarine only mirror images 0  and 2 are needed
            IF (UBOOT) THEN
               STEP = 2
            ELSE
               STEP = 1
            END IF
            DO SP = 0, 3
C..............source and collo point may not coincide
               IF (.NOT.(SP.EQ.0 .AND. I.EQ.K)) THEN
C.................projection of mirror element on sphere
                  OX(1) = XS(1,SP)-O(1)
                  OX(2) = XS(2,SP)-O(2)
                  OX(3) = XS(3,SP)-O(3)
                  OXB = SQRT(OX(1)**2+OX(2)**2+OX(3)**2)
                  F = R(K)/OXB
                  PK(1) = O(1)+F*OX(1)
                  PK(2) = O(2)+F*OX(2)
                  PK(3) = O(3)+F*OX(3)
                  FF =-(NS(1,SP)*OX(1)+NS(2,SP)*OX(2)+NS(3,SP)*OX(3))
     &                    /OXB * F**2
C                 Green function on sphere
                  CALL QUEL3D(1,FF*A(I),X(1,K),X(2,K),X(3,K),
     &                        PK(1),PK(2),PK(3),QX,QY,QZ,
     &                        DUM,DUM,DUM,DUM,DUM,
     &                        DUM,DUM,DUM,DUM,DUM)
                  VTK(K) = VTK(K) - (T(1)  *QX+T(2)  *QY+T(3)  *QZ)
                  VSK(K) = VSK(K) - (S(1,K)*QX+S(2,K)*QY+S(3,K)*QZ)
               END IF
            END DO
         END DO
C........store tangential velocity coefficients
         WRITE(20) (S(I,K),I=1,3),T,
     &             (VSK(I),I=1,NPB),(VTK(I),I=1,NPB),(VNK(I),I=1,NPB)
      END DO
      END


      SUBROUTINE QULKY(LEVEL,CLOSED,KN,XQUELL,XK,XD,A,AS,BS,M,NSYM,
     &           PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
C.....influence of one 'element' (point cluster including all mirror images)
C     LEVEL    level of maximum differentiation (1: velocities, 2: 2nd derivatives, 3: up to xxz etc.
C     CLOSED   logical, true = use mirror image at 'deckline' to close body
C     KN       array with number of corners of panel (KN(0)) and global indices of corner points (KN(1)...)
C     XQUELL   xyz coordinates of element center
C     XK       xyz coordinates of collocation/field point
C     XD       xyz coordinates of all corner points of elements, KN is pointer to these corner points
C     A        area of panel
C     AS, BS   coefficients needed to compute mirror image at 'deckline', compute in SPIDEF, used in MIRROR
C     M        element (source) strength, often denoted by sigma in literature
C     NSYM     coefficients needed to compute mirror image at 'deckline', compute in SPIDEF, used in MIRROR
C     PX,...   derivatives of potential, PX = phi_x = partial derivative with respect to x  
      INTEGER IY, KN(0:5), LEVEL
      LOGICAL CLOSED
      REAL A, AS, BS, M, NSYM, XD(3,*), XK(3), XQ(3), XQUELL(3), 
     &     PX,PY,PZ, PXX,PXY,PXZ,PYY,PYZ, PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &     QX,QY,QZ, QXX,QXY,QXZ,QYY,QYZ, QXXZ,QXYZ,QXZZ,QYYZ,QYZZ
C.....initialize values to zero
      CALL SET0(LEVEL,
     &          PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
C.....mirror image loop for port/starb'd symmetry
      DO IY = 1,-1,-2
         XQ(1) = XQUELL(1)
         XQ(2) = XQUELL(2)
         XQ(3) = XQUELL(3)
         CALL PANEL(LEVEL,IY,KN,A,M,XK,XD,XQ,
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
         CALL SUMM(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
         IF (CLOSED) THEN 
             CALL PANELT(LEVEL,IY,KN,A,AS,BS,M,NSYM,XK,XD,XQ
     &           ,QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
             CALL SUMM(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
         END IF
      END DO
      END

      SUBROUTINE PANEL(LEVEL,IY,KN,A,M,XK,XD,XQ,
     &         PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
C.....one element (mirror image IY)
C     LEVEL    level of maximum differentiation (1: velocities, 2: 2nd derivatives, 3: up to xxz etc.
C     IY       1 or -1 for original and mirror image in Y
C     KN       array with number of corners of panel (KN(0)) and global indices of corner points (KN(1)...)
C     A        area of panel
C     M        element (source) strength, often denoted by sigma in literature
C     XK       xyz coordinates of collocation/field point
C     XD       xyz coordinates of all corner points of elements, KN is pointer to these corner points
C     XQ       xyz coordinates of element center 
C     PX,...   derivatives of potential, PX = phi_x = partial derivative with respect to x  
      INTEGER IECK, IY, KN(0:4), LEVEL, NECK
      REAL A,  M, R2, R2MIN, X,Y,Z, XQM,YQM,ZQM, XD(3,*), XK(3),
     &     XQ(3), XS(3),
     &     PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &     QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ
C.....rename collocation point
      X = XK(1)
      Y = XK(2)
      Z = XK(3)
C.....mirror image of original element center 
      XQM = XQ(1)
      YQM = XQ(2)*IY
      ZQM = XQ(3)
C.....if distance large enough --> simple approximation 
      R2MIN = A*25
      R2 = (X-XQM)**2+(Y-YQM)**2+(Z-ZQM)**2
      IF (R2.LT.R2MIN*1.E-5) THEN
C........avoid problems if element and point are VERY close
	 CALL SET0(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
	 RETURN
      END IF
      IF (R2.GT.R2MIN) THEN
C........simple approximation by point source
	 CALL QUEL3D(LEVEL,M*A,X,Y,Z,XQM,YQM,ZQM,
     &         PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
      ELSE
C........approximation by multiple point sources
	 CALL SET0(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
	 NECK = KN(0)
	 DO IECK = 1, NECK
C...........consider mirror images of corners 
	    XS(1) = (XQM+XD(1,KN(IECK)))/2
	    XS(2) = (YQM+IY*XD(2,KN(IECK)))/2
	    XS(3) = (ZQM+XD(3,KN(IECK)))/2
	    CALL QUEL3D(LEVEL,M*A/NECK,X,Y,Z,XS(1),XS(2),XS(3),
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
	    CALL SUMM(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
         END DO
      END IF
      END
      
      SUBROUTINE PANELT(LEVEL,IY,KN,A,AS,BS,M,NSYM,XK,XD,XQ,
     &         PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
C.....same as PANEL, only mirror image at "deck"   
C     LEVEL    level of maximum differentiation (1: velocities, 2: 2nd derivatives, 3: up to xxz etc.
C     IY       1 or -1 for original and mirror image in Y
C     KN       array with number of corners of panel (KN(0)) and global indices of corner points (KN(1)...)
C     A        area of panel
C     M        element (source) strength, often denoted by sigma in literature
C     XK       xyz coordinates of collocation/field point
C     XD       xyz coordinates of all corner points of elements, KN is pointer to these corner points
C     XQ       xyz coordinates of element center 
C     PX,...   derivatives of potential, PX = phi_x = partial derivative with respect to x  
      INTEGER IECK, IY, KN(0:4), LEVEL, NECK
      REAL A, AS, BS, M, NSYM, R2, R2MIN, X,Y,Z, XQM,YQM,ZQM, 
     & XK(3), XD(3,*), XQ(3), XS(3), XSDECK(3,0:3),
     &      PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &      QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ
C.....rename collocation point
      X = XK(1)
      Y = XK(2)
      Z = XK(3)
C.....mirror image of panel center on "deck"
      CALL MIRROR(AS,BS,NSYM,XQ,XSDECK)
C.....second mirror image for bottom and port/starb'd symmetry
      XQM = XSDECK(1,1)
      YQM = XSDECK(2,1)*IY
      ZQM = XSDECK(3,1)
C.....if distance large enough --> simple approximation 
      R2MIN = 25*A
      R2 = (X-XQM)**2+(Y-YQM)**2+(Z-ZQM)**2
      IF (R2.LT.R2MIN*1.E-5) THEN
	 CALL SET0(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
	 RETURN
      END IF
      IF (R2.GT.R2MIN) THEN
C........simple approximation by point source
	 CALL QUEL3D(LEVEL,M*A,X,Y,Z,XQM,YQM,ZQM,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
      ELSE
C........approximation by multiple point sources
	 CALL SET0(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
	 NECK = KN(0)
	 DO IECK = 1, NECK
C...........consider mirror images of corners
	    CALL MIRROR(AS,BS,NSYM,XD(1,KN(IECK)),XSDECK)
	    XS(1) = (XQM+XSDECK(1,1))/2
	    XS(2) = (YQM+IY*XSDECK(2,1))/2
	    XS(3) = (ZQM+XSDECK(3,1))/2
	    CALL QUEL3D(LEVEL,M*A/NECK,X,Y,Z,XS(1),XS(2),XS(3),
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
	    CALL SUMM(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
         END DO
      END IF
      END
      
      SUBROUTINE SET0(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
C.....sets velocities and higher derivatives up to LEVEL to zero
      INTEGER LEVEL
      REAL PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ
      IF (LEVEL.EQ.0) RETURN
      PX = 0.0
      PY = 0.0
      PZ = 0.0
      IF (LEVEL.EQ.1) RETURN
      PXX = 0.0
      PXY = 0.0
      PXZ = 0.0
      PYY = 0.0
      PYZ = 0.0
      IF (LEVEL.EQ.2) RETURN
      PXXZ = 0.0
      PXYZ = 0.0
      PXZZ = 0.0
      PYYZ = 0.0
      PYZZ = 0.0
      END

      SUBROUTINE SUMM(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &            QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ)
C.....Sums up velocities and higher derivatives (up to LEVEL)
      INTEGER LEVEL
      REAL PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &     QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ
      IF (LEVEL.EQ.0) RETURN
      PX = PX + QX
      PY = PY + QY
      PZ = PZ + QZ
      IF (LEVEL.EQ.1) RETURN
      PXX = PXX + QXX
      PXY = PXY + QXY
      PXZ = PXZ + QXZ
      PYY = PYY + QYY
      PYZ = PYZ + QYZ
      IF (LEVEL.EQ.2) RETURN
      PXXZ = PXXZ + QXXZ
      PXYZ = PXYZ + QXYZ
      PXZZ = PXZZ + QXZZ
      PYYZ = PYYZ + QYYZ
      PYZZ = PYZZ + QYZZ
      END

      SUBROUTINE QUEL3D(LEVEL,M,X,Y,Z,XQ,YQ,ZQ,
     &           POTX,POTY,POTZ,POTXX,POTXY,POTXZ,POTYY,POTYZ,
     &           POTXXZ,POTXYZ,POTXZZ,POTYYZ,POTYZZ)
C.....single Rankine point source of strength M*A
C     LEVEL    level of maximum differentiation (1: velocities, 2: 2nd derivatives, 3: up to xxz etc.
C     M        element (source) strength, often denoted by sigma in literature
C     X,Y,Z    coordinates of field (collocation) point
C     XQ, ...  coordinates of source point
C     POTX,... partial derivatives of potential with respect to x etc
      INTEGER LEVEL 
      REAL DX,DY,DZ, FAKTOR, M, POT,POTX,POTY,POTZ,POTXX,
     &     POTXY,POTXZ,POTYY,POTYZ,POTXXZ,POTXYZ,POTXZZ,POTYYZ,POTYZZ,
     &     R, R2, STORE, STORE2, STORE3, X,Y,Z, XQ,YQ,ZQ
      FAKTOR=-M/12.566371
      DX=X-XQ
      DY=Y-YQ
      DZ=Z-ZQ
      R2=DX**2+DY**2+DZ**2
      R=SQRT(R2)
      POT = FAKTOR/R
      IF (LEVEL.EQ.0) RETURN
      STORE = -POT/R2
      POTX = STORE*DX
      POTY = STORE*DY
      POTZ = STORE*DZ
      IF (LEVEL.EQ.1) RETURN
      STORE2 = -3./R2
      POTXX = STORE2*POTX*DX+STORE
      POTXY = STORE2*POTX*DY
      POTXZ = STORE2*POTX*DZ
      POTYY = STORE2*POTY*DY+STORE
      POTYZ = STORE2*POTY*DZ
      IF (LEVEL.EQ.2) RETURN
      STORE3 = -5*DZ/R2
      POTXXZ = (-0.4*STORE + POTXX)*STORE3
      POTXYZ =               POTXY*STORE3
      POTXZZ =               POTXZ*STORE3 + STORE2*POTX
      POTYYZ = (-0.4*STORE + POTYY)*STORE3
      POTYZZ =               POTYZ*STORE3 + STORE2*POTY
      END

      SUBROUTINE MIRROR(A,B,NSYM,X,XS)
C.....     3 | 1
C         -------    mirror images of x
C          2 | 0
C
C     mirror images at 'deckline' defined by: x' = a * x + b * (z-nsym)
C                                             z' = b * x - a * (z-nsym) + nsym 
      REAL A, B, X(3), XS(3,0:3), NSYM
      XS(1,0) = X(1)
      XS(2,0) = X(2)
      XS(3,0) = X(3)
      XS(1,1) = A*X(1)+B*(X(3)-NSYM)
      XS(2,1) = X(2)
      XS(3,1) = B*X(1)-A*(X(3)-NSYM)+NSYM
      XS(1,2) = X(1)
      XS(2,2) =-X(2)
      XS(3,2) = X(3)
      XS(1,3) = XS(1,1)
      XS(2,3) =-X(2)
      XS(3,3) = XS(3,1)
      END

      SUBROUTINE SPIDEF(MSYM,A,B)
C.....define mirror image auxiliary variables z = MSYM x + NSYM
C     a = (1-MSYM**2)/(1+MSYM**2)
C     b =     2*MSYM /(1+MSYM**2)
c     xs = a x + b (z-NSYM)
c     zs = b x - a (z-NSYM) + NSYM
      REAL A, B, MSYM
      IF (MSYM.EQ.0.0) THEN
	 A = 1.0
	 B = 0.0
      ELSE
	 A = (1-MSYM**2)/(1+MSYM**2)   
	 B =      2*MSYM/(1+MSYM**2)  
      END IF
      END
