C Author: Heinrich Soeding
C Manoeuvring simulation based on coefficients like those in Table L.1
C Here for turning circle. For other manoeuvres change equation for delta 
      DIMENSION VAR(24)
C VAR(1:6) designate nondimens. position (x0,y0), course angle, u-U0, v, r
      COMMON IND(101),IEXP(5,101),COEFF(101),DELTAM,DELTAP,DELTA,
     & RMASSP,XGP,RIZZP,XUD,XUDU2,YVD,YVDV2,YRD,YRDR2,
     & RNVD,RNVDV2,RNRD,RNRDR2,N
      EXTERNAL DERIV
      DATA VAR/24*0./
C Reads: Simulation time (s), Lpp (m), Approach speed (m/s), rudder angle (rad),
C        rudder turning rate (rad/s), m' (nondimensional mass), xG', Izz'
      READ(5,*)TMAX,RLPP,U0,DELTAM,DELTAP,RMASSP,XGP,RIZZP
      DELTAP=DELTAP*RLPP/U0

C Reads: Nondimensional acceleration coefficients. E. g.
C        XUD means X'.  and RNRDR2 means N'. 2 .
C                    u                     rr        (this line = Index to above)
      READ(5,*)XUD,XUDU2,YVD,YVDV2,YRD,YRDR2,RNVD,RNVDV2,RNRD,RNRDR2

      DO 10 N=1,101
C Reads one nondimensional velocity coefficient per line. Each line contains:
C (1,2 or 3) for (X', Y' or N'); powers of u, v, r, delta in index of coeff.;
C and value of coefficient. Use negative powers to indicate absolute values.
C E.g. to indicate that N'     = -0.0072, input: 3 0 -1 1 0 -0.0072
C                         r|v|                       (this line = Index to above)
C Last line of input: 0 0 0 0 0 0
      READ(5,*)IND(N),(IEXP(J,N),J=1,4),COEFF(N)
          write(6,*)ind(n),(IEXP(J,N),J=1,4),COEFF(N)
      IEXP(5,N)=2-ABS(IEXP(1,N))-ABS(IEXP(2,N))-ABS(IEXP(3,N))     
      IF(IND(N).EQ.0) GOTO 20
10    CONTINUE
      STOP ' > 100 coefficients not allowed'
20    N=N-1
      WRITE(6,*)'   Time(s)    x0(m)     y0(m) psi(degr)    u(m/s)    v(       
     &m/s) r(degr/s) delta(degr)'
      T=0
25    DO 30 K=1,4
30    CALL RUNGE(VAR,VAR,DERIV,T,0.125,6)
C Rechnen mit dimensionslosen Groessen; Ausgabe dimensionsbehaftet
      WRITE(6,'(8F10.3)')T*RLPP/U0,VAR(1)*RLPP,VAR(2)*RLPP,VAR(3)*57.296 
     & ,VAR(4)*U0+U0,VAR(5)*U0,VAR(6)*57.296*U0/RLPP,DELTA*57.296
      IF(T*RLPP/U0.LT.TMAX)GOTO 25
      END

      SUBROUTINE DERIV(VAR,T,VARDOT)
      DIMENSION VAR(6),VARDOT(6),XYN(3)
      COMMON IND(101),IEXP(5,101),COEFF(101),DELTAM,DELTAP,DELTA,
     & RMASSP,XGP,RIZZP,XUD,XUDU2,YVD,YVDV2,YRD,YRDR2,
     & RNVD,RNVDV2,RNRD,RNRDR2,N
C Determination of time derivatives of position and course angle 
      VARDOT(1)=(VAR(4)+1)*COS(VAR(3))-VAR(5)*SIN(VAR(3))
      VARDOT(2)=(VAR(4)+1)*SIN(VAR(3))+VAR(5)*COS(VAR(3))
      VARDOT(3)=VAR(6)
C Centrifugal terms
      DO 35 I=1,3
      XYN(1)=RMASSP*(VAR(5)+XGP*VAR(6))*VAR(6)
      XYN(2)=-RMASSP*(VAR(4)+1)*VAR(6)
35    XYN(3)=-RMASSP*(VAR(4)+1)*XGP*VAR(6)
C Hydrodynamic terms in (X, Y, N)/...
      DELTA=MAX(1.E-10,MIN(ABS(DELTAP*T),ABS(DELTAM)))
      DELTA=SIGN(DELTA,DELTAM)
      DO 60 I=1,N
      TERM=COEFF(I)*DELTA**ABS(IEXP(4,I))*(VAR(4)+1)**IEXP(5,I)
      IF(IEXP(4,I).LT.0)TERM=TERM*SIGN(1.,DELTA)
      DO 50 J=1,3
      IF(IEXP(J,I))40,50,45
40    TERM=TERM*SIGN(1.,VAR(3+J))
45    TERM=TERM*VAR(3+J)**ABS(IEXP(J,I))
50    CONTINUE
      XYN(IND(I))=XYN(IND(I))+TERM
60    CONTINUE
C Determination of time derivatives of u, v, r
      VARDOT(4)=XYN(1)/(RMASSP-XUD-XUDU2*(VAR(4)/(VAR(4)+1))**2)
      A11=RMASSP-YVD-YVDV2*(VAR(5)/(VAR(4)+1))**2
      A12=RMASSP*XGP-YRD-YRDR2*(VAR(6)/(VAR(4)+1))**2
      A21=RMASSP*XGP-RNVD-RNVDV2*(VAR(5)/(VAR(4)+1))**2
      A22=RIZZP-RNRD-RNRDR2*(VAR(6)/(VAR(4)+1))**2
      DET=A11*A22-A12*A21
      VARDOT(5)=(XYN(2)*A22-XYN(3)*A12)/DET
      VARDOT(6)=(A11*XYN(3)-A21*XYN(2))/DET
      END

      SUBROUTINE RUNGE(Y1,Y2,YP,T,DT,N)
C Runge-Kutta integration. Y1 input values at time T, Y2 output at time T+DT.
C N no. of functions to be integrated. T is increased by DT. Note Dimensions
C of Y1! YP subroutine determining the time derivatives of Y1; arguments are
C array of input values, time, array of time derivatives.
      DIMENSION Y1(N,4),Y2(N)
      CALL YP(Y1(1,1),T,Y1(1,2))
      DO 10 I=1,N
   10 Y1(I,3)=Y1(I,1)+Y1(I,2)*DT/2.
      T=T+DT/2.
      CALL YP(Y1(1,3),T,Y1(1,4))
      DO 20 I=1,N
      Y1(I,2)=Y1(I,2)+2.*Y1(I,4)
   20 Y1(I,3)=Y1(I,1)+Y1(I,4)*DT/2.
      CALL YP(Y1(1,3),T,Y1(1,4))
      DO 30 I=1,N
      Y1(I,2)=Y1(I,2)+2.*Y1(I,4)
   30 Y1(I,3)=Y1(I,1)+Y1(I,4)*DT
      T=T+DT/2.
      CALL YP(Y1(1,3),T,Y1(1,4))
      DO 40 I=1,N
      Y1(I,2)=(Y1(I,2)+Y1(I,4))/6.
   40 Y2(I)=Y1(I,1)+Y1(I,2)*DT
      RETURN
      END

