C Author: Heinrich Soeding
      PROGRAM MICHEL
Computes wave resistance according to Michell for given shape offsets.
C All data and results in m, t, s and combinations of these units.
      COMMON /MICH/X(21),NZD(21),DZ(21),Y(11),NSP,I,G,PI,U0,K,A1,RNUE
     *,FA1(21,5),FA2(21,5),FA3(21,5),FA4(21,5),FA5(21,5)
      COMPLEX I
      EXTERNAL F
      DATA G,PI,I/9.81,3.14159,(0.,1.)/
C NSP = number of offset sections (odd, 3<=nsp<=21)
      READ(5,*) NSP
      DO 10 ISP=1,NSP
C NP  = number of offsets on a section (odd, 3<=np<=11)
C X   = long. coord. of section. x pairwize equidistant like for Simpson's rule
C DZ  = vertical distance of offsets (constant on a section)
C Y   = halfbreadths. First at waterline, last at keel 
         READ(5,*) NP,X(ISP),DZ(ISP),(Y(IP),IP=1,NP)
         NZD(ISP)=NP/2
         IZ = 0
         DO 9 IP=3,NP,2
            IZ = IZ+1
            F1 = Y(IP-2)
            F2 = Y(IP-1)
            F3 = Y(IP)
            FA1(ISP,IZ) = 0.5*F1-2.*F2+1.5*F3
            FA2(ISP,IZ) = 1.5*F1-2.*F2+0.5*F3
            FA3(ISP,IZ) =     F1-2.*F2+    F3
            FA4(ISP,IZ) = F1
            FA5(ISP,IZ) = F3
9        CONTINUE
10    CONTINUE
      WRITE(6,*) '   U0 (m/s)  Rw (kN)  for density 1 t/m3'
C U0 = ship velocity (m/s)
16    READ(5,*) U0
      RNUE = G/U0**2
      A1 =A(1.)
      SUM = A1 * ALOG(2.+SQRT(3.))
C K counts int. intervals of lengths 1,2,4... S = integral for k, SVOR for k-1
      K=0
      SVOR=SUM
C AREA = integration routine. See STR 37 (1990), 139-140
20    S=SIMPS(F,2.**K,2.**(K+1),2.**(K-2),0.02*ABS(SUM/SVOR))
      SUM = SUM+S
      IF (K.GT.1.AND.ABS(S).LT.0.05*SUM) GOTO 30
      SVOR=S
      K=K+1
      GOTO 20
30    RT = 4./PI*(RNUE*U0)**2*SUM
      WRITE(6,35) U0,RT
35    FORMAT(2F10.3)
C Next speed
      GOTO 16
      END

      FUNCTION F(RLAM)
Computes integrand of Michell integral depending on lambda
      COMMON /MICH/X(21),NZD(21),DZ(21),Y(11),NSP,I,G,PI,U0,K,A1,RNUE
     *,FA1(21,5),FA2(21,5),FA3(21,5),FA4(21,5),FA5(21,5)
      COMPLEX I      
C Due to separation of singularity, F=0 for lambda=1
      IF(RLAM.GT.1.0001)GOTO 10
      F=0.
      RETURN
10    RNENN=SQRT(RLAM**2-1.)
      F=A(RLAM)*RLAM**2/RNENN
C Different integrand in first integral
      IF(K.EQ.0) F=F-A1/RNENN
      END

      FUNCTION A(RLAM)
Computes square of modulus of A(lambda)
      COMMON /MICH/X(21),NZD(21),DZ(21),Y(11),NSP,I,G,PI,U0,K,A1,RNUE
     *,FA1(21,5),FA2(21,5),FA3(21,5),FA4(21,5),FA5(21,5)
      COMPLEX I,SUM,CA
      SUM=0
C ZINT(ISP,RLAM) computes z integral for section ISP depending on lambda
      F1=ZINT(1,RLAM)
C x integral by special integration formula for each double spacing
      DO 10 ISP=2,NSP,2
         F2=ZINT(ISP,RLAM)
         F3=ZINT(ISP+1,RLAM)
         D2F=F3-2*F2+F1
         RK=RNUE*RLAM
         HK=0.5*(X(ISP+1)-X(ISP-1))*RK
         SUM=SUM+CEXP(I*RK*X(ISP-1))/RK*(CEXP(2.*I*HK)*((0.5*F1-2.*F2
     *   +1.5*F3)/HK-I*F3+I*D2F/HK**2)+(1.5*F1-2.*F2+0.5*F3)
     *   /HK+I*F1-I*D2F/HK**2)
         F1=F3
10    CONTINUE
      CA=-I*RK*SUM
      A=CA*CONJG(CA)
      END

      FUNCTION ZINT(ISP,RLAM)
Computes z integral for section ISP depending on lambda
      COMMON /MICH/X(21),NZD(21),DZ(21),Y(11),NSP,I,G,PI,U0,K,A1,RNUE
     *,FA1(21,5),FA2(21,5),FA3(21,5),FA4(21,5),FA5(21,5)
      COMPLEX I
      ZINT=0
      NZ=NZD(ISP)
C FAKT = ratio of exponential function for 2 double spacings
      FAKT=0.
      HK=-RNUE*RLAM**2*DZ(ISP)
      IF(HK.GT.-100.)FAKT=EXP(HK*2.)
      EHOCH=DZ(ISP)/HK
      DO 10 IZ=1,NZ
C For all double spacings of a section
         ZINT=ZINT+EHOCH*(FAKT*((FA3(ISP,IZ)/HK-FA1(ISP,IZ))/HK+
     *   FA5(ISP,IZ))-((FA3(ISP,IZ)/HK+FA2(ISP,IZ))/HK+FA4(ISP,IZ)))
         EHOCH=EHOCH*FAKT
10    CONTINUE
      END

