      PROGRAM SHAL2D
C.....2-D program for wave resistance for submerged body
C     z upward positive
      IMPLICIT NONE
      INTEGER IT, KS, LP, NIT, NP, NPB, NPO
      REAL BOTTOM, C, DX, R2MIN
      PARAMETER (LP=200)
      INTEGER KN(2,LP)
      REAL A(LP), AP(2,LP), B(LP), M(LP), N(2,LP), V(2,LP), X(2,LP), 
     &     XD(2,LP), XQ(2,LP)
C.....trick compiler into accepting larger arrays
      COMMON C(LP,LP+1)
C.....display name and purpose of program
      CALL MELDE
C.....read all input
      CALL INPUT(LP,NIT,NPB,NPO,NP,KN,AWET,BOTTOM,DX,R2MIN,U,
     &           A,AP,B,M,N,V,X,XD,XQ)
C.....iterative solution of problem
      DO IT = 0, NIT
C........make linear system of equations from boundary conditions
         CALL LGSMAK(LP,NP,NPB,NPO,KN,U,A,AP,B,N,V,X,XD,XQ,C)
C........solve system of equations to determine element strengths
         CALL SIMQ2(C,NP,1,LP,KS,1.E-5)
         IF (KS.EQ.4) STOP'>> System of equations singular, fatal error'
         DO I = 1, NP
            M(I) = C(I,NP+1)
         END DO
C........determine velocities with now known element strengths
         CALL VELALL(NP,NPB,NPO,KN,U,A,AP,B,M,X,XD,XQ,V)
      END DO
C.....integrate pressure to forces
      CALL KRAFT(NPB,AWET,U,A,N,V,X)
C.....write output
      CALL OUTPUT(NPB,NPO,U,M,V,X,XQ)
      END

      SUBROUTINE INPUT(LP,NIT,NPB,NPO,NP,KN,AWET,BOTTOM,DX,R2MIN,U,
     &                 A,AP,B,M,N,V,X,XD,XQ)
C.....read all input and initialise all arrays
      IMPLICIT NONE
      CHARACTER FILE*80, TEXT*80
      INTEGER LP, NIT, NP, NPB, NPO
      REAL AWET, BOTTOM, DX, G, R2MIN, U
      INTEGER KN(2,*)
      REAL A(*), AP(2,*), B(*), M(*), N(2,*), V(2,*), X(2,*), XD(2,*),
     &     XQ(2,*)
      PARAMETER (G = 9.81)
      NPO = 0
C.....read control file <CONTRL.CTL>
      CALL INPST(TEXT,NIT,BOTTOM,U,FILE)
C.....read grid for body 
      CALL INBODY(FILE,LP,NB,NPB,KN,AWET,A,N,X,XD,XQ)
      WRITE(*,*) '>> ', NPB, ' panels on body'
C.....read grid for free surface
      CALL INWAT(LP,NB,NPB,NO,NPO,KN,DX,A,X,XD,XQ)
      WRITE(*,*) '>> ', NPO, ' panels on water surface'
C.....minimum r^2 to use better approximation for element 
      R2MIN = 9*DX**2
C.....total number of elements (unknowns)
      NP = NPB+NPO
      WRITE(*,*) '>> ', NP, ' panels in total'
C.....depth Froude number
      IF (BOTTOM.LT.9999) THEN
         WRITE(*,*) '>> Depth Froude number: ', U/SQRT(G*BOTTOM)
      END IF
C.....initialise all arrays (set potential to uniform flow)
      CALL INITVA(NP,U,AP,B,M,V)
      WRITE(*,*) '>> finished initial preparations'
      END

      SUBROUTINE INPST(TEXT,NIT,BOTTOM,U,FILE)
C.....read control file <CONTRL.CTL>
      IMPLICIT NONE
      CHARACTER*80 TEXT, FILE
      INTEGER NIT
      REAL BOTTOM, U
      OPEN(10,FILE='CONTRL.CTL',STATUS='OLD',ERR=100)
         READ(10,'(A)',ERR=200) TEXT
         WRITE(*,'(1X,A)') TEXT
         READ(10,'(A)',ERR=200) FILE
         READ(10,*,ERR=200) U
         READ(10,*,ERR=200) NIT
         READ(10,*,ERR=200) BOTTOM
      CLOSE(10)
      WRITE(*,*) '>> control file <CONTRL.CTL> successfully read'
      RETURN
100   CONTINUE
      WRITE(*,*) '>> control file <CONTRL.CTL> not found'
      STOP
200   CONTINUE
      WRITE(*,*) '>> error reading control file <CONTRL.CTL>'
      STOP
      END

      SUBROUTINE INBODY(FILE,LP,NB,NPB,KN,AWET,A,N,X,XD,XQ)
C.....read input file for body
      IMPLICIT NONE
      CHARACTER FILE*80
      INTEGER I, ID, IECK, ILEN, LP, NB, NPB
      REAL AWET
      INTEGER KN(2,*)
      REAL A(*), N(2,*), X(2,*), XD(2,*), XQ(2,*)
      OPEN(10,FILE=FILE,STATUS='OLD',ERR=100)
         READ(10,*,ERR=200) NB, NPB
         IF (NB.GT.LP) THEN
            WRITE(*,*) '>> Too many points on body'
            GOTO 200
         END IF
         IF (NPB.GT.LP) THEN
            WRITE(*,*) '>> Too many panels on body'
            GOTO 200
         END IF
         READ(10,*,ERR=200) AWET
         DO I = 1, NB
            READ(10,*,ERR=200) (XD(ID,I),ID=1,2)
         END DO
         DO I = 1, NPB
            READ(10,*,ERR=200) (KN(IECK,I),IECK=1,2)
            READ(10,*,ERR=200) (X(ID,I),ID=1,2),(XQ(ID,I),ID=1,2)
            READ(10,*,ERR=200) A(I),(N(ID,I),ID=1,2)
         END DO
      CLOSE(10)
      RETURN
100   CONTINUE
      WRITE(*,*) '>> File', FILE(1:ILEN(FILE,80)), ' not found'
      STOP
200   CONTINUE
      WRITE(*,*) '>> Error reading file ', FILE(1:ILEN(FILE,80))
      STOP
      END

      SUBROUTINE INWAT(LP,NB,NPB,NO,NPO,KN,DX,A,X,XD,XQ)
C.....read input file for water surface
      IMPLICIT NONE
      INTEGER I, ID, IECK, LP, NB, NO, NPB, NPO
      REAL DX, HQ
      INTEGER KN(2,*)
      REAL A(*), X(2,*), XD(2,*), XQ(2,*)
      OPEN(10,FILE='SHAL2D.ZUS',ERR=100)
         READ(10,*,ERR=200) NO, NPO
         IF (NB+NO.GT.LP) THEN
            WRITE(*,*) '>> Too many points'
            STOP
         END IF
         IF (NPB+NPO.GT.LP) THEN
            WRITE(*,*) '>> Too many panels'
            STOP
         END IF
         READ(10,*,ERR=200) HQ, DX
         DO I = NB+1, NB+NO
            READ(10,*,ERR=200) (XD(ID,I),ID=1,2)
         END DO
         DO I = NPB+1, NPB+NPO
            READ(10,*,ERR=200) (KN(IECK,I),IECK=1,2)
            DO IECK = 1, 2
               KN(IECK,I) = KN(IECK,I) + NB
            END DO
            READ(10,*,ERR=200)(X(ID,I),ID=1,2), A(I), (XQ(ID,I),ID=1,2)
C...........make sure that sources are above free surface (z upward positive)
            XQ(2,I) = ABS(XQ(2,I))
         END DO
      CLOSE(10)
      RETURN
100   CONTINUE
      WRITE(*,*) '>> File SHAL2D.ZUS not found'
      STOP
200   CONTINUE
      WRITE(*,*) '>> Error reading file SHAL2D.ZUS '
      STOP
      END

      INTEGER FUNCTION ILEN(R,ISIZE)
C.....finds last non-blank character of string R of size ISIZE
      IMPLICIT NONE
      INTEGER I, ISIZE
      CHARACTER R*(*)
      DO I = ISIZE, 1, -1
         IF (R(I:I).NE.' ') GOTO 100
      END DO
100   CONTINUE
      ILEN = I
      END

      SUBROUTINE INITVA(NP,U,AP,B,M,V)
C.....Initialise all arrays
      IMPLICIT NONE
      INTEGER I, NP
      REAL U
      REAL AP(2,*), B(*), M(*), V(2,*)
      DO I = 1, NP
         AP(1,I) = 0.
         AP(2,I) = 0.
         B(I)    = 0.
         M(I)    = 0.
         V(1,I)  = -U
         V(2,I)  = 0.
      END DO
      END

      SUBROUTINE LGSMAK(LP,NP,NPB,NPO,KN,BOTTOM,DX,R2MIN,U,
     &                  A,AP,B,N,V,X,XD,XQ,C)
C.....set up matrix of coefficient to determine unknown source strengths
      IMPLICIT NONE
      INTEGER I, LP, NP, NPB, NPO
      REAL BOTTOM, DX, R2MIN
      INTEGER KN(2,*)
      REAL A(*), AP(2,*), B(*), C(LP,*), N(2,*), X(2,*),XQ(2,*)
C.....equations for body
      CALL LGSBOD(LP,NP,NPB,NPO,KN,U,A,N,X,XD,XQ,C)
C.....equations for free surface
      CALL LGSFS(LP,NP,NPB,NPO,KN,U,A,AP,B,V,X,XD,XQ,C)
      END

      SUBROUTINE LGSBOD(LP,NP,NPB,NPO,KN,BOTTOM,DX,R2MIN,U,
     &                  A,N,X,XD,XQ,C)
C     equations for body (Neumann condition)
      IMPLICIT NONE
      INTEGER I, K, LP, NP, NPB, NPO
      REAL BOTTOM, DX, QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ, R2MIN, U
      INTEGER KN(2,*)
      REAL A(*), C(LP,*), N(2,*), X(2,*), XD(2,*), XQ(2,*)
      DO K = 1, NPB
         C(K,NP+1) = -N(1,K)*U
         DO I = 1, NPB+NPO
            IF (I.LE.NPB) THEN
               CALL QULBD(1,BOTTOM,A(I),XQ(1,I),X(1,K),
     &                    QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            ELSE
               CALL QULFS(1,BOTTOM,A(I),DX,R2MIN,XQ(1,I),X(1,K),
     &                    QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            END IF
            C(K,I)=N(1,K)*QX+N(2,K)*QZ
         END DO
      END DO
      END

      SUBROUTINE LGSFS(LP,NP,NPB,NPO,KN,BOTTOM,DX,R2MIN,U,
     &                 A,AP,B,V,X,XD,XQ,C)
C.....equations for free surface
      IMPLICIT NONE
      INTEGER I, K, LEVEL, LP, NP, NPB, NPO 
      REAL AX,AZ, BR, BOTTOM, DX, G, PX, PZ, R2MIN, DX, U
      PARAMETER (G=-9.81)
      INTEGER KN(2,*)
      REAL A(*), AP(2,*), B(*), C(LP,*), V(2,*),
     &     X(2,*), XD(2,*), XQ(2,*)
      LEVEL = 2
      DO K = NPB+1, NPB+NPO
         AX = AP(1,K)
         AZ = AP(2,K)
         PX = V(1,K)
         PZ = V(2,K)
         BR = B(K)
         DO I = 1, NP
            IF (I.LE.NPB) THEN
               CALL QULBD(LEVEL,BOTTOM,A(I),XQ(1,I),X(1,K),
     &                    QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            ELSE
               CALL QULFS(LEVEL,BOTTOM,A(I),DX,R2MIN,XQ(1,I),X(1,K),
     &                    QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            END IF
            C(K,I) = (2*(QX*AX+QZ*AZ+PX*PZ*QXZ)+PX**2*QXX+PZ**2*QZZ
     &               -G*QZ+BR*(PX*QX+PZ*QZ))/G
         END DO
C........constant part
         C(K,NP+1) =-(2*(PX*AX+PZ*AZ)-BR*((PX**2+PZ**2+U**2)/2+G*X(2,K))
     &               +U*(2*AX+BR*PX))/G
      END DO
      END

      SUBROUTINE KRAFT(NPB,AWET,U,A,N,V,X)
C.....direct pressure integration gives forces
      IMPLICIT NONE
      INTEGER K, NPB
      REAL AWET, CW, FORCE, P, U
      REAL A(*), N(2,*), V(2,*), X(2,*)
C     rho is not considered as it cancels out for the drag coefficient CW
      FORCE = 0.
      DO K = 1, NPB
         P = (U**2-(V(1,K)**2+V(2,K)**2))/2.
         FORCE = FORCE + P*N(1,K)*A(K)
      END DO
C.....wave resistance coefficient
      CW = -FORCE/(0.5*U**2*AWET)
      WRITE(*,*) '>> CW = ', CW
      END

      SUBROUTINE MELDE
      WRITE(*,1)'******************************************************'
      WRITE(*,1)'*                                                    *'
      WRITE(*,1)'*                      SHAL2D                        *'
      WRITE(*,1)'*                                                    *'
      WRITE(*,1)'*      Steady 2-D Free-surface Potential Flow        *'
      WRITE(*,1)'*                                                    *'
      WRITE(*,1)'******************************************************'
      WRITE(*,*)
1     FORMAT(12X,A)
      END

      SUBROUTINE OUTPUT(NPB,NPO,U,M,V,X,XQ)
C.....output on file 
      IMPLICIT NONE
      INTEGER I, ID, NPB, NPO
      REAL CP, U
      REAL M(*), V(2,*), X(2,*), XQ(2,*)
      OPEN(12,FILE='OUTPUT.DAT')
         WRITE(12,*) NPB, NPO
         DO I = 1, NPB
            CP = 1.0-(V(1,I)**2+V(2,I)**2)/U**2
            WRITE(12,1000) (X(ID,I),ID=1,2), M(I),
    &                      (V(ID,I),ID=1,2), CP
         END DO
         DO I = NPB+1, NPB+NPO
            WRITE(12,1000) (X(ID,I),ID=1,2), M(I), 
     &                     (V(ID,I),ID=1,2), (XQ(ID,I),ID=1,2)
         END DO
      CLOSE(12)
      WRITE(*,*) '>> File OUTPUT.DAT created'
1000  FORMAT(7(1X,G13.7))
      END

      SUBROUTINE QULFS(LEVEL,BOTTOM,M,DX,R2MIN,XQ,XK,
     &                 PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ)
C.....influence of free-surface element
      IMPLICIT NONE
      INTEGER I, IBOD, LEVEL
      LOGICAL SHALLW
      REAL BOTTOM, DX, M, PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ, 
     &     QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ, R2, R2MIN, ZQ
      REAL XQ(2), XK(2)
      SHALLW = BOTTOM.LT.9999
C.....set influence function initially to 0
      CALL SET0(LEVEL,P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ)
C.....Loop to account for ocean bottom by using mirror image at bottom
      DO IBOD = 0, 2, 2
         ZQ = XQ(2) - IBOD*(XQ(2)-BOTTOM)
         R2 = (XK(1)-XQ(1))**2 + (XK(2)-ZQ)**2
         IF (R2.GT.R2MIN) THEN
            CALL RANK2D(LEVEL,M,XK(1),XK(2),XQ(1),ZQ,
     &                  QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            CALL SUMM(LEVEL,
     &                P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ,
     &                Q,QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
         ELSE
            DO I = -1, 1, 2
               CALL RANK2D(LEVEL,M/2,
     &                     XK(1),XK(2),XQ(1)-I*DX/4,ZQ,
     &                     QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
               CALL SUMM(LEVEL,
     &                   P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ,
     &                   Q,QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            END DO
         END IF
         IF (.NOT.SHALLW) RETURN
      END DO
      END

      SUBROUTINE QULBD(LEVEL,BOTTOM,M,XQ,XK,
     &                 PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ)
C.....influence of body element
      IMPLICIT NONE
      INTEGER IBOD, LEVEL
      REAL BOTTOM, M, P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ,
     &                Q,QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ
      REAL XQ(2), XK(2)
      LOGICAL SHALLW
      SHALLW = BOTTOM.LT.9999
C.....initialise influence functions to 0
      CALL SET0(LEVEL,P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ)
C.....mirror images at ocean bottom
      DO IBOD = 0, 2, 2
         ZQ = XQ(2) - IBOD*(XQ(2)-BOTTOM)
         CALL RANK2D(LEVEL,M,XK(1),XK(2),XQ(1),ZQ,
     &               QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
         CALL SUMM(LEVEL,P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ,
     &                   Q,QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
         IF (.NOT.SHALLW) RETURN
      END DO
      END

      SUBROUTINE SET0(LEVEL,P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ)
C.....sets potential and derivatives to 0
      IMPLICIT NONE
      INTEGER LEVEL
      REAL P, PX, PZ, PXX, PXZ, PZZ, PXXZ, PXZZ, PZZZ
      P = 0.
      IF (LEVEL.EQ.0) RETURN
      PX = 0.
      PZ = 0.
      IF (LEVEL.EQ.1) RETURN
      PXX = 0.
      PXZ = 0.
      PZZ = 0.
      IF (LEVEL.EQ.2) RETURN
      PXXZ = 0.
      PXZZ = 0.
      PZZZ = 0.
      END

      SUBROUTINE SUMM(LEVEL,P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ,
     &                      Q,QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
C.....sums up potential and derivatives
      IMPLICIT NONE
      INTEGER LEVEL
      REAL P, PX, PZ, PXX, PXZ, PZZ, PXXZ, PXZZ, PZZZ,
     &     Q, QX, QZ, QXX, QXZ, QZZ, QXXZ, QXZZ, QZZZ
      P = P + Q
      IF (LEVEL.EQ.0) RETURN
      PX = PX + QX
      PZ = PZ + QZ
      IF (LEVEL.EQ.1) RETURN
      PXX = PXX + QXX
      PXZ = PXZ + QXZ
      PZZ = PZZ + QZZ
      IF (LEVEL.EQ.2) RETURN
      PXXZ = PXXZ + QXXZ
      PXZZ = PXZZ + QXZZ
      PZZZ = PZZZ + QZZZ
      END

      SUBROUTINE VELALL(NP,NPB,NPO,KN,DX,R2MIN,U,
     &                  A,AP,B,M,X,XD,XQ,V)
C.....calculation of all velocities with now known source strengths
      IMPLICIT NONE
      INTEGER NP, NPB, NPO
      REAL BOTTOM, DX, R2MIN, U
      INTEGER KN(2,*)
      REAL A(*), AP(2,*), B(*), M(*), V(2,*), X(2,*), XD(2,*), XQ(2,*)
C.....velocity at free water surface
      CALL VELFS(NP,NPB,NPO,KN,BOTTOM,DX,R2MIN,U,A,M,X,XD,XQ,
     &                 AP,B,V)
C.....velocity on body
      CALL       VELBOD(NP,NPB,NPO,KN,DX,R2MIN,U,A,M,X,XD,XQ,V)
      END

      SUBROUTINE VELBOD(NP,NPB,NPO,KN,DX,R2MIN,U,A,M,X,XD,XQ,V)
C     velocity on body
      IMPLICIT NONE
      INTEGER I, K, NP, NPB, NPO
      REAL DX, R2MIN, U,
     &     QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ
      INTEGER KN(2,*)
      REAL A(*), M(*), V(2,*), X(2,*), XD(2,*), XQ(2,*)
      DO K = 1, NPB
         V(1,K) = -U
         V(2,K) = 0.
         DO I = 1, NP
            IF (I.LE.NPB) THEN
               CALL QULBD(1,A(I)*M(I),XQ(1,I),X(1,K),
     &                    QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            ELSE
               CALL QULFS(1,BOTTOM,A(I)*M(I),DX,R2MIN,XQ(1,I),X(1,K),
     &                    QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            END IF
            V(1,K) = V(1,K) + QX
            V(2,K) = V(2,K) + QZ
         END DO
      END DO
      END

      SUBROUTINE VELFS(NP,NPB,NPO,KN,BOTTOM,DX,R2MIN,U,A,M,X,XD,XQ,
     &                 AP,B,V)
C.....velocity and higher derivatives at free surface
      IMPLICIT NONE
      INTEGER I, IAZ, IERR, K, LEVEL, NP, NPB, NPO
      REAL AZMAX, BOTTOM, DX, ERR, ERRMAX, G, 
     &     P,PX,PZ, PXX,PXZ,PZZ, PXXZ,PXZZ,PZZZ, 
     &     Q,QX,QZ, QXX,QXZ,QZZ, QXXZ,QXZZ,QZZZ, R2MIN, U, ZMIN, ZMAX
      INTEGER KN(2,*)
      REAL A(*), AP(2,*), B(*), M(*), V(2,*), X(2,*), XD(2,*), XQ(2,*)
      PARAMETER (G=-9.81)
      LEVEL = 3
      ERRMAX=0
      AZMAX=-1.E10
      ZMAX = -1.E10
      ZMIN =  1.E10
      DO K = NPB+1, NPB+NPO
         CALL SET0(LEVEL,P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ)
         PX = -U
         PZ = 0.
         DO I = 1, NP
            IF (I.LE.NPB) THEN
               CALL QULBD(LEVEL,BOTTOM,A(I)*M(I),XQ(1,I),X(1,K),
     &                    QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            ELSE
               CALL QULFS(LEVEL,BOTTOM,A(I)*M(I),DX,R2MIN,XQ(1,I),
     &                    X(1,K),QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
            END IF
            CALL SUMM(LEVEL,P,PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ,
     &                      Q,QX,QZ,QXX,QXZ,QZZ,QXXZ,QXZZ,QZZZ)
         END DO
C........new velocity, acceleration, etc
         V(1,K) = PX
         V(2,K) = PZ
         AP(1,K) = PX*PXX + PZ*PXZ
         AP(2,K) = PX*PXZ + PZ*PZZ
         B(K)=(2*(AP(1,K)*PXZ+AP(2,K)*PZZ+PX**2*PXXZ/2
     &       +PZ*PX*PXZZ+PZ**2*PZZZ/2)-G*PZZ)/(G-AP(2,K))
C........nonlinear Bernoulli
         X(2,K) = (PX**2+PZ**2-U**2)/(2*G)
         ZMAX = MAX(ZMAX,X(2,K))
         ZMIN = MIN(ZMIN,X(2,K))
C........determine maximum acceleration and its location
         IF (AP(2,K).GT.AZMAX) THEN
            AZMAX  = AP(2,K)
            IAZ    = K
         END IF
C........nondimensional error
         ERR=ABS(PX*AP(1,K)+PZ*AP(2,K)-G)/(G*U)
         IF (ERR .GT. ERRMAX) THEN
            ERRMAX = ERR
            IERR = K
         END IF
      END DO
      WRITE(*,*) '>> Max. error:        ',ERRMAX,' at ', X(1,IERR)
      WRITE(*,*) '>> Max. acceleration: ',AZMAX ,' at ', X(1,IAZ)
      WRITE(*,*) '>> Z between:', ZMAX, ZMIN
      END

      SUBROUTINE RANK2D(LEVEL,M,X,Z,XQ,ZQ,
     &                  PX,PZ,PXX,PXZ,PZZ,PXXZ,PXZZ,PZZZ)
C.....2-D Rankine source of strength M
      IMPLICIT NONE
      INTEGER LEVEL
      REAL AUX, DX, DZ, FACTOR, M, PI, PX,PZ, PXX,PXZ,PZZ,
     &     PXXZ,PXZZ,PZZZ, R2, X, XQ, Z, ZQ
      PI = 3.141592654
      FACTOR = M/(4*PI)
      DX = X-XQ
      DZ = Z-ZQ
      R2 = DX**2+DZ**2
      IF (R2.EQ.0.) THEN
         WRITE(*,*) '>> Fatal error, singular distance in RANK2D'
         STOP
      END IF
      AUX = 2./R2
      PX = FACTOR*AUX*DX
      PZ = FACTOR*AUX*DZ
      IF (LEVEL.EQ.1) RETURN
      PXX = AUX*(FACTOR-DX*PX)
      PXZ = -AUX*DX*PZ
      PZZ = -PXX
      IF (LEVEL.EQ.2) RETURN
      PXXZ = -AUX*(PXX*DZ+PXZ*DX)
      PXZZ = -AUX*(PXZ*DZ+PZZ*DX)
      PZZZ = -PXXZ
      END
