C.....grid generation on free surface for SHAL2D
C     preprocessor for SHAL2D
C.....main variables
C     A      segment area
C     DX     grid spacing
C     HQ     height of desingularised sources above still free surface
C     KN     pointer to global numbers of end points of each segment
C     LAMBDA wave length
C     LP     maximum number of points
C     NO     number of corner points for segments on free surface
C     NPO    number of collocation point = number of sources 
C     NPWL   number of panels per wave length
C     U      speed of body [m/s]
C     RATIO  desingularisation ratio = desing. height/grid spacing
C     WL     grid length/wave length
C     X      coordinates of collocation points (segment centres)
C     X1     first x (most forward)
C     XD     coordinates of segments describing surface of cylinder
C     XQ     source point
      IMPLICIT NONE
      INTEGER I, LP, NO, NPO, NPWL
      PARAMETER (LP=1000)
      REAL DX, HQ, LAMBDA, RATIO, U, WL, X1
      REAL A(LP), X(2,LP), XD(2,LP), XQ(2,LP)
C.....read speed U from control file 
      OPEN(10,FILE='CONTRL.CTL',STATUS='OLD',ERR=200)
         READ(10,*)
         READ(10,*)
         READ(10,*) U
      CLOSE(10)
C.....remaining interactive input
100   CONTINUE
      WRITE(*,*) '>> Input:'
      WRITE(*,*) '>> first x (most forward)'
      WRITE(*,*) '>> number of panels per wave length'
      WRITE(*,*) '>> grid length / wave length'
      READ(*,*,ERR=100) X1, NPWL, WL
      RATIO = 1.0
      LAMBDA = U**2*0.64
      DX = LAMBDA/NPWL
      NPO = NINT(NPWL*WL)
      IF (NPO.LT.0.OR.NPO.GT.LP) GOTO 100
      NO = NPO+1
C.....generate segments on free surface
      DO I = 1, NO
         XD(1,I) = X1+DX/2-(I-1)*DX
      END DO
C.....generate collocation points, sources (shifted by DX), areas
      HQ = RATIO*DX
      DO I = 1, NPO
         X(1,I) = (XD(1,I)+XD(1,I+1))/2
         X(2,I) = 0.0
         XQ(1,I) = X(1,I)-DX
         XQ(2,I) = HQ
         A(I) = DX
      END DO
C.....output on file
      OPEN(10,FILE='SHAL2D.ZUS',STATUS='UNKNOWN')
         WRITE(10,*) NO, NPO
         WRITE(10,1000) HQ, DX
         DO I = 1, NO
            WRITE(10,1000) XD(1,I), XD(2,I)
         END DO
         DO I = 1, NPO
            WRITE(10,*) I, I+1
            WRITE(10,1000) X(1,I), X(2,I), A(I), XQ(1,I), XQ(2,I) 
         END DO
      CLOSE(10)
      WRITE(*,*) '>> File SHAL2D.ZUS created'
1000  FORMAT(5(1X,G10.4))
      STOP
C.....Error in opening control file
200   CONTINUE
      WRITE(*,*) '>> Control file not found'
      END
