C Thiart elements, osciallting dublet
      SUBROUTINE DUBLET(LEVEL,KN,X,XD,RF,Q,QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ)
C***********************************************************************
C***  LEVEL: indicator for only first derivatives (=1) or for both     *
C***         first and second derivatives (=2)                         *
C***  XD(1,KN(1)): X-coordinate at leading edge of mega-panel    *
C***  XD(1,KN(2)): X-coordinate at trailing edge of mega-panel   *
C***  XD(3,KN(1)): Z-coordinate at bottom edge of mega-panel     *
C***  XD(3,KN(2)): Z-coordinate at top edge of mega-panel        *
C***  X(1): X-coordinate of field point                            *
C***  X(2): Y-coordinate of field point                            *
C***  X(3): Z-coordinate of field point                            *
C***  RF: encounter frequency/ship speed                            *
C***  Q: velocity potential at field point                         *
C***  QX: X-derivative of velocity potential at field point         *
C***  QY: Y-derivative of velocity potential at field point         *
C***  QZ: Z-derivative of velocity potential at field point         *
C***  QXX: XX-derivative of velocity potential at field point        *
C***  QYY: YY-derivative of velocity potential at field point        *
C***  QXY: XY-derivative of velocity potential at field point        *
C***  QYZ: YZ-derivative of velocity potential at field point        *
C***  QXZ: XZ-derivative of velocity potential at field point        *
C***********************************************************************
      
      dimension KN(0:4)
      dimension XD(3,*),X(3)
      complex Q,QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ
      complex*16 cons0,cons1,cons2,cons3,cons4
      complex*16 ci400,ci4x0,ci4y0,ci4z0,ci4xx,ci4yy,ci4xy,ci4yz,ci4xz
      real*8 delxf,delxa,delyc,delzu,delzo
      real*8 xaysq,xfysq,zuysq,zoysq
      real*8 radao,radfo,radau,radfu
      real*8 rfox3,raox3,rfux3,raux3
      real*8 rfoz3,raoz3,rfuz3,rauz3
      real*8 wakel,xwake,delxw,cocos,cosin
      real*8 radio,radiu,radic,ratio
      real*8 ri100,ri1x0,ri1y0,ri1z0,ri1xx,ri1yy,ri1xy,ri1yz,ri1xz
      real*8 ri200,ri2x0,ri2y0,ri2z0,ri2xx,ri2yy,ri2xy,ri2yz,ri2xz
      real*8 ri300,ri3x0,ri3y0,ri3z0,ri3xx,ri3yy,ri3xy,ri3yz,ri3xz
      real*8 xleft,xcent,xriht,delxl,delxc,delxr,xylsq,xycsq,xyrsq
      real*8 radol,radoc,rador,radul,raduc,radur
      real*8 fleft,fcent,friht,delfl,delfr,delf2
      real*8 FI400,FI4x0,FI4y0,FI4z0,FI4xx,FI4yy,FI4xy,FI4yz,FI4xz
C***  make things smaller to preserve conditioning of system
      SMALL = 1.E-6
      
C***  Often-used distances and squares of distances

      delxf=dble(X(1)-XD(1,KN(1)))
      delxa=dble(X(1)-XD(1,KN(2)))
      delyc=dble(X(2))
      delzu=dble(X(3)-XD(3,KN(1)))
      delzo=dble(X(3)-XD(3,KN(2)))
      xaysq=delxa**2+delyc**2
      xfysq=delxf**2+delyc**2
      zuysq=delzu**2+delyc**2
      zoysq=delzo**2+delyc**2
      radao=dsqrt(xaysq+delzo**2)
      radfo=dsqrt(xfysq+delzo**2)
      radau=dsqrt(xaysq+delzu**2)
      radfu=dsqrt(xfysq+delzu**2)
      rfox3=3.0d0*radfo**2-delxf**2
      raox3=3.0d0*radao**2-delxa**2
      rfux3=3.0d0*radfu**2-delxf**2
      raux3=3.0d0*radau**2-delxa**2
      rfoz3=3.0d0*radfo**2-delzo**2
      raoz3=3.0d0*radao**2-delzo**2
      rfuz3=3.0d0*radfu**2-delzu**2
      rauz3=3.0d0*radau**2-delzu**2

C***  Multiplicative and other constants

      wakel=0.2d0*dble(XD(1,KN(1))-XD(1,KN(2)))
      xwake=dble(X(1))-0.5d0*wakel
      if (xwake.gt.dble(XD(1,KN(2)))) then
        nsubp=0
        delxw=0.0d0
      else
        wakel=dmin1(dble(XD(1,KN(2))),dble(X(1))+0.5d0*wakel)
     *       -xwake
        radio=sqrt(delyc**2+delzo**2)
        radiu=sqrt(delyc**2+delzu**2)
        radic=sqrt(delyc**2+0.25d0*(delzo+delzu)**2)
        ratio=wakel/dmin1(radio,radiu,radic)
        nsubp=max(1,int(0.5d0+8.0d0*ratio))
        delxw=wakel/float(nsubp)
      end if
      cons1=cmplx(delyc,0.0d0)
      cons2=cmplx(-delxf,0.0d0)
      cons3=cmplx((delxa-delxf),-dble(1.0/RF))
      cons4=cmplx(-dsin(dble(XD(1,KN(2))*RF))/dble(RF)
     *     ,-dcos(dble(XD(1,KN(2))*RF))/dble(RF))
      cocos=dcos(dble(RF)*delxw)
      cosin=dsin(dble(RF)*delxw)

C***  Integral "I1" and its derivatives

      ri100=dlog(((delzo+radao)*(delzu+radfu))
     *     /((delzo+radfo)*(delzu+radau)))
      ri1x0=delxa*(1.0d0/(radao*(delzo+radao))
     *     -1.0d0/(radau*(delzu+radau)))
     *     -delxf*(1.0d0/(radfo*(delzo+radfo))
     *     -1.0d0/(radfu*(delzu+radfu)))
      ri1y0=delyc*(1.0d0/(radao*(delzo+radao))
     *     -1.0d0/(radau*(delzu+radau))
     *     -1.0d0/(radfo*(delzo+radfo))
     *     +1.0d0/(radfu*(delzu+radfu)))
      ri1z0=1.0d0/radao+1.0d0/radfu-1.0d0/radfo-1.0d0/radau
      if (LEVEL.ne.1) then
        ri1xx=(radao**2*(delzo+radao)-delxa**2*(delzo+2.0d0*radao))
     *       /(radao**3*(delzo+radao)**2)
     *       +(radfu**2*(delzu+radfu)-delxf**2*(delzu+2.0d0*radfu))
     *       /(radfu**3*(delzu+radfu)**2)
     *       -(radfo**2*(delzo+radfo)-delxf**2*(delzo+2.0d0*radfo))
     *       /(radfo**3*(delzo+radfo)**2)
     *       -(radau**2*(delzu+radau)-delxa**2*(delzu+2.0d0*radau))
     *       /(radau**3*(delzu+radau)**2)
        ri1yy=(radao**2*(delzo+radao)-delyc**2*(delzo+2.0d0*radao))
     *       /(radao**3*(delzo+radao)**2)
     *       +(radfu**2*(delzu+radfu)-delyc**2*(delzu+2.0d0*radfu))
     *       /(radfu**3*(delzu+radfu)**2)
     *       -(radfo**2*(delzo+radfo)-delyc**2*(delzo+2.0d0*radfo))
     *       /(radfo**3*(delzo+radfo)**2)
     *       -(radau**2*(delzu+radau)-delyc**2*(delzu+2.0d0*radau))
     *       /(radau**3*(delzu+radau)**2)
        ri1xy=delyc*(delxa*((delzu+2.0d0*radau)
     *       /(radau**3*(delzu+radau)**2)
     *       -(delzo+2.0*radao)/(radao**3*(delzo+radao)**2))
     *       -delxf*((delzu+2.0d0*radfu)/(radfu**3*(delzu+radfu)**2)
     *       -(delzo+2.0d0*radfo)/(radfo**3*(delzo+radfo)**2)))
        ri1yz=-delyc*(1.0d0/radao**3+1.0d0/radfu**3-1.0d0/radfo**3
     *       -1.0d0/radau**3)
        ri1xz=delxa*(1.0d0/radau**3-1.0d0/radao**3)
     *       -delxf*(1.0d0/radfo**3-1.0d0/radfu**3)
      end if

C***  Integral "I2"

      ri300=datan((delzo*delxa)/(delyc*radao))
     *     -datan((delzu*delxa)/(delyc*radau))
      ri200=datan((delzo*delxf)/(delyc*radfo))
     *     -datan((delzu*delxf)/(delyc*radfu))
     *     -ri300
      ri3x0=delyc*(delzo/radao-delzu/radau)/xaysq
      ri2x0=delyc*(delzo/radfo-delzu/radfu)/xfysq
     *     -ri3x0
      ri3y0=delxa*((delzu/radau)*(1.0d0/xaysq+1.0d0/zuysq)
     *     -(delzo/radao)*(1.0d0/xaysq+1.0d0/zoysq))
      ri2y0=delxf*((delzu/radfu)*(1.0d0/xfysq+1.0d0/zuysq)
     *     -(delzo/radfo)*(1.0d0/xfysq+1.0d0/zoysq))
     *     -ri3y0
      ri3z0=delyc*delxa*(1.0d0/(radao*zoysq)-1.0d0/(radau*zuysq))
      ri2z0=delyc*delxf*(1.0d0/(radfo*zoysq)-1.0d0/(radfu*zuysq))
     *     -ri3z0
      if (LEVEL.ne.1) then
        ri3xx=delxa*delyc*(delzu*rauz3/radau**3-delzo*raoz3/radao**3)
     *       /xaysq**2
        ri2xx=delxf*delyc*(delzu*rfuz3/radfu**3-delzo*rfoz3/radfo**3)
     *       /xfysq**2
     *       -ri3xx
        ri3yy=delxa*delyc*(delzo*(raoz3/xaysq**2+raox3/zoysq**2)
     *       /radao**3
     *       -delzu*(rauz3/xaysq**2+raux3/zuysq**2)/radau**3)
        ri2yy=delxf*delyc*(delzo*(rfoz3/xfysq**2+rfox3/zoysq**2)
     *       /radfo**3
     *       -delzu*(rfuz3/xfysq**2+rfux3/zuysq**2)/radfu**3)
     *       -ri3yy
        ri3xy=(delzo*(xaysq/radao-delyc**2*raoz3/radao**3)
     *       -delzu*(xaysq/radau-delyc**2*rauz3/radau**3))/xaysq**2
        ri2xy=(delzo*(xfysq/radfo-delyc**2*rfoz3/radfo**3)
     *       -delzu*(xfysq/radfu-delyc**2*rfuz3/radfu**3))/xfysq**2
     *       -ri3xy
        ri3yz=delxa*((zoysq/radao-delyc**2*raox3/radao**3)/zoysq**2
     *       -(zuysq/radau-delyc**2*raux3/radau**3)/zuysq**2)
        ri2yz=delxf*((zoysq/radfo-delyc**2*rfox3/radfo**3)/zoysq**2
     *       -(zuysq/radfu-delyc**2*rfux3/radfu**3)/zuysq**2)
     *       -ri3yz
        ri3xz=delyc/radao**3-delyc/radau**3
        ri2xz=delyc/radfo**3-delyc/radfu**3
     *       -ri3xz
      end if

C***  Integral "I3"

      ri300=ri300-datan(delzo/delyc)+datan(delzu/delyc)
      ri3y0=ri3y0+delzo/zoysq-delzu/zuysq
      ri3z0=ri3z0-delyc*(1.0d0/zoysq-1.0d0/zuysq)
      if (LEVEL.ne.1) then
        ri3yy=ri3yy-2.0d0*delyc*(delzo/zoysq**2-delzu/zuysq**2)
        ri3yz=ri3yz-(delzo**2-delyc**2)/zoysq**2
     *       +(delzu**2-delyc**2)/zuysq**2
      end if

C***  Integral "I4"

      ci400=cmplx(0.0d0,0.0d0)
      ci4x0=cmplx(0.0d0,0.0d0)
      ci4y0=cmplx(0.0d0,0.0d0)
      ci4z0=cmplx(0.0d0,0.0d0)
      ci4xx=cmplx(0.0d0,0.0d0)
      ci4yy=cmplx(0.0d0,0.0d0)
      ci4xy=cmplx(0.0d0,0.0d0)
      ci4yz=cmplx(0.0d0,0.0d0)
      ci4xz=cmplx(0.0d0,0.0d0)
      do isubp=1,nsubp
        xleft=xwake+float(isubp-1)*delxw
        xcent=xleft+0.5d0*delxw
        xriht=xleft+delxw
        delxl=dble(X(1))-xleft
        delxc=dble(X(1))-xcent
        delxr=dble(X(1))-xriht
        xylsq=delxl**2+delyc**2
        xycsq=delxc**2+delyc**2
        xyrsq=delxr**2+delyc**2
        radol=dsqrt(xylsq+delzo**2)
        radoc=dsqrt(xycsq+delzo**2)
        rador=dsqrt(xyrsq+delzo**2)
        radul=dsqrt(xylsq+delzu**2)
        raduc=dsqrt(xycsq+delzu**2)
        radur=dsqrt(xyrsq+delzu**2)
        cons0=cmplx(dcos(dble(RF*xleft))/dble(RF)
     *       ,dsin(dble(RF*xleft))/dble(RF))
        fleft=FI400(delzo,delzu,radol,radul,xylsq)
        fcent=FI400(delzo,delzu,radoc,raduc,xycsq)
        friht=FI400(delzo,delzu,rador,radur,xyrsq)
        delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
        delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
        delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
        ci400=ci400+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *       ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
        fleft=FI4X0(delxl,delzo,delzu,radol,radul,xylsq)
        fcent=FI4X0(delxc,delzo,delzu,radoc,raduc,xycsq)
        friht=FI4X0(delxr,delzo,delzu,rador,radur,xyrsq)
        delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
        delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
        delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
        ci4x0=ci4x0+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *       ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
        fleft=FI4Y0(delyc,delzo,delzu,radol,radul,xylsq)
        fcent=FI4Y0(delyc,delzo,delzu,radoc,raduc,xycsq)
        friht=FI4Y0(delyc,delzo,delzu,rador,radur,xyrsq)
        delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
        delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
        delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
        ci4y0=ci4y0+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *       ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
        fleft=FI4Z0(radol,radul)
        fcent=FI4Z0(radoc,raduc)
        friht=FI4Z0(rador,radur)
        delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
        delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
        delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
        ci4z0=ci4z0+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *       ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
        if (LEVEL.ne.1) then
          fleft=FI4XX(delxl,delzo,delzu,radol,radul,xylsq)
          fcent=FI4XX(delxc,delzo,delzu,radoc,raduc,xycsq)
          friht=FI4XX(delxr,delzo,delzu,rador,radur,xyrsq)
          delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
          delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
          delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
          ci4xx=ci4xx+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *         ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
          fleft=FI4YY(delyc,delzo,delzu,radol,radul,xylsq)
          fcent=FI4YY(delyc,delzo,delzu,radoc,raduc,xycsq)
          friht=FI4YY(delyc,delzo,delzu,rador,radur,xyrsq)
          delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
          delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
          delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
          ci4yy=ci4yy+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *         ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
          fleft=FI4XY(delxl,delyc,delzo,delzu,radol,radul,xylsq)
          fcent=FI4XY(delxc,delyc,delzo,delzu,radoc,raduc,xycsq)
          friht=FI4XY(delxr,delyc,delzo,delzu,rador,radur,xyrsq)
          delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
          delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
          delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
          ci4xy=ci4xy+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *         ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
          fleft=FI4YZ(delyc,radol,radul)
          fcent=FI4YZ(delyc,radoc,raduc)
          friht=FI4YZ(delyc,rador,radur)
          delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
          delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
          delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
          ci4yz=ci4yz+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *         ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
          fleft=FI4XZ(delxl,radol,radul)
          fcent=FI4XZ(delxc,radoc,raduc)
          friht=FI4XZ(delxr,rador,radur)
          delfl=(3.0d0*fleft-4.0d0*fcent+friht)/dble(RF*delxw)
          delfr=(fleft-4.0d0*fcent+3.0d0*friht)/dble(RF*delxw)
          delf2=(fleft-2.0d0*fcent+friht)/dble(0.5*RF*delxw)**2
          ci4xz=ci4xz+cmplx((delfr*cocos-(delf2-friht)*cosin+delfl)
     *         ,(delfr*cosin+(delf2-friht)*cocos-delf2+fleft))*cons0
        end if
      end do

C***  Potential and derivatives

      Q=cons1*cmplx(ri100,0.0d0)+cons2*cmplx(ri200,0.0d0)
     *     +cons3*cmplx(ri300,0.0d0)+cons4*cons1*ci400
      QX=cons1*cmplx(ri1x0,0.0d0)-cmplx(ri200,0.0d0)
     *     +cons2*cmplx(ri2x0,0.0d0)
     *     +cons3*cmplx(ri3x0,0.0d0)+cons4*cons1*ci4x0
      QY=cmplx(ri100,0.0d0)+cons1*cmplx(ri1y0,0.0d0)
     *     +cons2*cmplx(ri2y0,0.0d0)
     *     +cons3*cmplx(ri3y0,0.0d0)+cons4*(ci400+cons1*ci4y0)
      QZ=cons1*cmplx(ri1z0,0.0d0)+cons2*cmplx(ri2z0,0.0d0)
     *     +cons3*cmplx(ri3z0,0.0d0)+cons4*cons1*ci4z0
      if (LEVEL.ne.1) then
        QXX=cons1*cmplx(ri1xx,0.0d0)-cmplx(2.0d0*ri2x0,0.0d0)
     *       +cons2*cmplx(ri2xx,0.0d0)
     *       +cons3*cmplx(ri3xx,0.0d0)+cons4*cons1*ci4xx
        QYY=cmplx(2.0d0*ri1y0,0.0d0)+cons1*cmplx(ri1yy,0.0d0)
     *       +cons2*cmplx(ri2yy,0.0d0)+cons3*cmplx(ri3yy,0.0d0)
     *       +cons4*(cmplx(2.0d0,0.0d0)*ci4y0+cons1*ci4yy)
        QXY=cmplx(ri1x0,0.0d0)+cons1*cmplx(ri1xy,0.0d0)
     *       -cmplx(ri2y0,0.0d0)+cons2*cmplx(ri2xy,0.0d0)
     *       +cons3*cmplx(ri3xy,0.0d0)
     *       +cons4*(ci4x0+cons1*ci4xy)
        QYZ=cmplx(ri1z0,0.0d0)+cons1*cmplx(ri1yz,0.0d0)
     *       +cons2*cmplx(ri2yz,0.0d0)
     *       +cons3*cmplx(ri3yz,0.0d0)+cons4*(ci4z0+cons1*ci4yz)
        QXZ=cons1*cmplx(ri1xz,0.0d0)-cmplx(ri2z0,0.0d0)
     *       +cons2*cmplx(ri2xz,0.0d0)
     *       +cons3*cmplx(ri3xz,0.0d0)+cons4*cons1*ci4xz
      end if
      Q = Q * SMALL
      QX = QX * SMALL
      QY = QY * SMALL
      QZ = QZ * SMALL
      QXX = QXX * SMALL
      QXY = QXY * SMALL
      QXZ = QXZ * SMALL
      QYY = QYY * SMALL 
      QYZ = QYZ * SMALL
      end


      Function FI400(delzo,delzu,radio,radiu,xysqr)
      real*8 FI400
      real*8 delzo,delzu,radio,radiu,xysqr
      FI400=(delzo/radio-delzu/radiu)/xysqr
      end


      Function FI4X0(delxx,delzo,delzu,radio,radiu,xysqr)
      real*8 FI4X0
      real*8 delxx,delzo,delzu,radio,radiu,xysqr
      FI4X0=-delxx*(delzo*(3.0d0*radio**2-delzo**2)/radio**3
     *     -delzu*(3.0d0*radiu**2-delzu**2)/radiu**3)/xysqr**2
      end


      Function FI4Y0(delyc,delzo,delzu,radio,radiu,xysqr)
      real*8 FI4Y0
      real*8 delyc,delzo,delzu,radio,radiu,xysqr
      FI4Y0=-delyc*(delzo*(3.0d0*radio**2-delzo**2)/radio**3
     *     -delzu*(3.0d0*radiu**2-delzu**2)/radiu**3)/xysqr**2
      end


      Function FI4Z0(radio,radiu)
      real*8 FI4Z0
      real*8 radio,radiu
      FI4Z0=1.0d0/radio**3-1.0d0/radiu**3
      end

      
      Function FI4XX(delxx,delzo,delzu,radio,radiu,xysqr)
      real*8 FI4XX
      real*8 delxx,delzo,delzu,radio,radiu,xysqr
      FI4XX=-delzo*((radio**2-3.0d0*delxx**2)/(radio**5*xysqr)
     *     +(2.0d0*radio**2-4.0d0*delxx**2)/(radio**3*xysqr**2)
     *     -8.0d0*delxx**2/(radio*xysqr**3))
     *     +delzu*((radiu**2-3.0d0*delxx**2)/(radiu**5*xysqr)
     *     +(2.0d0*radiu**2-4.0d0*delxx**2)/(radiu**3*xysqr**2)
     *     -8.0d0*delxx**2/(radiu*xysqr**3))
      end

      
      Function FI4YY(delyc,delzo,delzu,radio,radiu,xysqr)
      real*8 FI4YY
      real*8 delyc,delzo,delzu,radio,radiu,xysqr
      FI4YY=-delzo*((radio**2-3.0d0*delyc**2)/(radio**5*xysqr)
     *     +(2.0d0*radio**2-4.0d0*delyc**2)/(radio**3*xysqr**2)
     *     -8.0d0*delyc**2/(radio*xysqr**3))
     *     +delzu*((radiu**2-3.0d0*delyc**2)/(radiu**5*xysqr)
     *     +(2.0d0*radiu**2-4.0d0*delyc**2)/(radiu**3*xysqr**2)
     *     -8.0d0*delyc**2/(radiu*xysqr**3))
      end


      Function FI4XY(delxx,delyc,delzo,delzu,radio,radiu,xysqr)
      real*8 FI4XY
      real*8 delxx,delyc,delzo,delzu,radio,radiu,xysqr
      FI4XY=delxx*delyc*(delzo*(3.0d0/(radio**5*xysqr)
     *     +(12.0d0*radio**2-4.0d0*delzo**2)/(radio**3*xysqr**3))
     *     -delzu*(3.0d0/(radiu**5*xysqr)
     *     +(12.0d0*radiu**2-4.0d0*delzu**2)/(radiu**3*xysqr**3)))
      end


      Function FI4YZ(delyc,radio,radiu)
      real*8 FI4YZ
      real*8 delyc,radio,radiu
      FI4YZ=-delyc*(3.0d0/radio**5-3.0d0/radiu**5)
      end

      
      Function FI4XZ(delxx,radio,radiu)
      real*8 FI4XZ
      real*8 delxx,radio,radiu
      FI4XZ=-delxx*(3.0d0/radio**5-3.0d0/radiu**5)
      end
