***************** BEGIN EXAMPLE **********************

C     sample calls of h-o routines
C     z positive upwards
C     Compute geometry coefficients for all body panels
C     NPB    number of elements on body 
C     NP     total number of elements/collocation points
C     NPO    number of points outside body, where simplified computation is used
C     NTRANS number of flat panels which are not higher-order (with knuckle to other panels)
C            the first NTRANS panels in NPB are of this type
C     KN     array of  pointers with global point numbers of corner points of elements
C            KN(0) gives number of corners, either 3 or 4
C     NACHB  array of pointer to global numbers of neighboring panels, 
C            NEIGHB(0) gives true number of neighbors
C     N      unit normal (is new computed and output of GEOMHO)
C     X      xyz coordinates of collocation points (is new computed and output of GEOMHO for points on body)
C     XD     xyz coordinates of corner points
C     XQ     xyz coordinates of centers (coincides with X unless desingularised panels are used)  
C
C     GEOMHO may be modified depending on how you arrange your various panel types
C     if you just use exclusively h-o panels, set NTRANS and NPO to zero, set NP to NPB
C     nothing terrible should happen if you compute N differently and dummy the N computation in GEOMHO out

      CALL GEOMHO(NPB,NP,NPO,NTRANS,KN,NACHB,N,X,XD,XQ,GEOP)

C     once the geometry coefficients have been computed, you use the h-o panels as  follows


C     velocities and accelerations at one point
      INTEGER I, IFO, IK, IS, J, LEVEL, NP, NPB, NPO, NS, NTRANS
      REAL BOTTOM,
     &     PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &     QX,QY,QZ,QXX,QXY,QXZ,QYY,QYZ,QXXZ,QXYZ,QXZZ,QYYZ,QYZZ,
     &     VXX,VYX,VZX,VXY,VYY,VZY,VXXDX,VXXDY,VXXDZ,
     &     VYXDY,VYXDZ,VXYDX,VXYDY,VXYDZ,VYYDY,VYYDZ,
     &     VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,VXYDXZ,
     &     VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ, U
      INTEGER NACHB(0:4,*)
      REAL CX(0:4), CY(0:4), GEOP(44,*), M(*), X(3), XQ(3,*), VZ(4,2:3),
     &     XS(3,4)
      DATA VZ / 1.0,  -1.0,   1.0, -1.0,  1.0,   1.0,  -1.0, -1.0  /
      NS = 8           ! number of mirror images, here closed body and bottom mirror image
C.... Initialize velocities and higher derivatives
      CALL SET0(LEVEL,PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,
     &          PXXZ,PXYZ,PXZZ,PYYZ,PYZZ)
C.... Loop over all panels
      DO 10 I = 1, NP
C....... On free surface use point source clusters, set IFO=2
         IF (I.GT.NPB .AND. I.LE.NPB+NPO) THEN
            IFO=2                    ! simple panel
         ELSE
            IFO=0                    ! h-o panel
         END IF
C....... Compute coeff.s to interpolate linearly source strengths
         CALL LINSOR(XQ,GEOP(12,I),NACHB,I,CX,CY)
         DO 20 IS = 1, NS
C.......... Compute higher-order panel influence for this mirror image
            CALL PANHO(XS(1,IS),XS(2,IS),XS(3,IS),XQ(1,I),
     &                 GEOP(1,I),LEVEL,QX,QY,QZ,VXX,VYX,VZX,VXY,
     &                 VYY,VZY,QXX,QXY,QXZ,QYY,QYZ,
     &                 VXXDX,VXXDY,VXXDZ,VYXDY,VYXDZ,
     &                 VXYDX,VXYDY,VXYDZ,VYYDY,VYYDZ,
     &                 QXXZ,QXYZ,QXZZ,QYYZ,QYZZ,
     &                 VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     &                 VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,IFO)
            QY = QY*VZ(IS,2)
            QZ = QZ*VZ(IS,3)
            IF (LEVEL.GT.1) THEN
               QXY = QXY*VZ(IS,2)
               QXZ = QXZ*VZ(IS,3)
               QYZ = QYZ*VZ(IS,2)*VZ(IS,3)
               IF (LEVEL.EQ.3) THEN
                  QXXZ = QXXZ*VZ(IS,3)
                  QXYZ = QXYZ*VZ(IS,2)*VZ(IS,3)
                  QYYZ = QYYZ*VZ(IS,3)
                  QYZZ = QYZZ*VZ(IS,2)
               END IF
            END IF
            CALL SUMM(LEVEL,
     &         PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &         QX*M(I),QY*M(I),QZ*M(I),
     &         QXX*M(I),QXY*M(I),QXZ*M(I),QYY*M(I),QYZ*M(I),
     &         QXXZ*M(I),QXYZ*M(I),QXZZ*M(I),QYYZ*M(I),QYZZ*M(I))
C.......... transom stern panels only linear
            IF (I.LE.NTRANS) GOTO 20
C.......... free surface point source clusters also only linear
            IF (I.GT.NPB .AND. I.LE.NPB+NPO) GOTO 20
C.......... Add influence from present panel source strength due to
C           linear source variation
            QX =  CX(0)*VXX + CY(0)*VXY
            QY = (CX(0)*VYX + CY(0)*VYY)*VZ(IS,2)
            QZ = (CX(0)*VZX + CY(0)*VZY)*VZ(IS,3)
            IF (LEVEL.GT.1) THEN
               QXX =  CX(0)*VXXDX+CY(0)*VXYDX
               QXY = (CX(0)*VXXDY+CY(0)*VXYDY)*VZ(IS,2)
               QXZ = (CX(0)*VXXDZ+CY(0)*VXYDZ)*VZ(IS,3)
               QYY =  CX(0)*VYXDY+CY(0)*VYYDY
               QYZ = (CX(0)*VYXDZ+CY(0)*VYYDZ)*VZ(IS,2)*VZ(IS,3)
               IF (LEVEL.EQ.3) THEN
                  QXXZ = (CX(0)*VXXDXZ+CY(0)*VXYDXZ)*VZ(IS,3)
                  QXYZ = (CX(0)*VXXDYZ+CY(0)*VXYDYZ)*VZ(IS,2)*VZ(IS,3)
                  QXZZ = (CX(0)*VXXDZZ+CY(0)*VXYDZZ)
                  QYYZ = (CX(0)*VYXDYZ+CY(0)*VYYDYZ)*VZ(IS,3)
                  QYZZ = (CX(0)*VYXDZZ+CY(0)*VYYDZZ)*VZ(IS,2)
               END IF
            END IF
            CALL SUM(LEVEL,
     &         PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,PYZZ,
     &         QX*M(I),QY*M(I),QZ*M(I),
     &         QXX*M(I),QXY*M(I),QXZ*M(I),QYY*M(I),QYZ*M(I),
     &         QXXZ*M(I),QXYZ*M(I),QXZZ*M(I),QYYZ*M(I),QYZZ*M(I))
C.......... Add the influence from the neighboring panels due to
C           the linear variation in the source strength
            DO 30 J = 1, NACHB(0,I)
               IK = NACHB(J,I)
               QX =  CX(J)*VXX + CY(J)*VXY
               QY = (CX(J)*VYX + CY(J)*VYY)*VZ(IS,2)
               QZ = (CX(J)*VZX + CY(J)*VZY)*VZ(IS,3)
               IF (LEVEL.GT.1) THEN
                  QXX =  CX(J)*VXXDX+CY(J)*VXYDX
                  QXY = (CX(J)*VXXDY+CY(J)*VXYDY)*VZ(IS,2)
                  QXZ = (CX(J)*VXXDZ+CY(J)*VXYDZ)*VZ(IS,3)
                  QYY =  CX(J)*VYXDY+CY(J)*VYYDY
                  QYZ = (CX(J)*VYXDZ+CY(J)*VYYDZ)*VZ(IS,2)*VZ(IS,3)
                  IF (LEVEL.EQ.3) THEN
                     QXXZ = (CX(J)*VXXDXZ+CY(J)*VXYDXZ)*VZ(IS,3)
                     QXYZ=(CX(J)*VXXDYZ+CY(J)*VXYDYZ)*VZ(IS,2)*VZ(IS,3)
                     QXZZ = (CX(J)*VXXDZZ+CY(J)*VXYDZZ)
                     QYYZ = (CX(J)*VYXDYZ+CY(J)*VYYDYZ)*VZ(IS,3)
                     QYZZ = (CX(J)*VYXDZZ+CY(J)*VYYDZZ)*VZ(IS,2)
                  END IF
               END IF
               CALL SUM(LEVEL,
     &            PX,PY,PZ,PXX,PXY,PXZ,PYY,PYZ,PXXZ,PXYZ,PXZZ,PYYZ,
     &            PYZZ,QX*M(IK),QY*M(IK),QZ*M(IK),QXX*M(IK),
     &            QXY*M(IK),QXZ*M(IK),QYY*M(IK),QYZ*M(IK),QXXZ*M(IK),
     &            QXYZ*M(IK),QXZZ*M(IK),QYYZ*M(IK),QYZZ*M(IK))
 30         CONTINUE
 20      CONTINUE
 10   CONTINUE
      END

************* END EXAMPLE **************************











C***********************************************************************
C
C     SUBROUTINE GEOHO
C
C     Subroutine written for XSHIP, to compute all of the higher -
C     order panel geometry coefficients to be used in a h.o. panel routines
C
C-----------------------------------------------------------------------
      SUBROUTINE GEOMHO(NPB,NP,NPO,NTRANS,KN,NACHB,N,X,XD,XQ,GEOP)

      INTEGER NPB,NV,NBR,NP,NPO,NTRANS,I,J,K,L
      INTEGER KN(0:4,*), NACHB(0:4,*)
      REAL N(3,*), X(3,*), XD(3,*), XQ(3,*)
      LOGICAL FLATP
C..... Add declarations for higher order panel coefficients
      REAL GEOP(44,*),SI(15),P,Q,RR,XV(4,3),XVP(9,3),DIR(3,3),
     &     XI(4),ETA(4),D12,D23,D34,D41,DM32,DM41,B32,B41,W,TLEN
      INTEGER KNACH(9)

C..... First compute the body panels
      DO 100 I=1,NP
         FLATP = .FALSE.
         IF (I.LE.NTRANS .OR. I.GT.NPB) FLATP=.TRUE.
         DO 20 J=1,KN(0,I)
            DO 10 K=1,3
               XV(J,K) = XD(K,KN(J,I))
 10         CONTINUE
 20      CONTINUE
         IF (KN(0,I).LT.4) THEN
            XV(4,1) = XV(3,1)
            XV(4,2) = XV(3,2)
            XV(4,3) = XV(3,3)
         END IF
C........ determine the number of not on the border neighboring
C         panel corner points.
         NV = 0
         DO 60 J=1,NACHB(0,I)
            NBR = NACHB(J,I)
            DO 50 K=1,KN(0,NBR)
               KNEW = KN(K,NBR)
               DO 30 L = 1,NV
                  IF (KNEW.EQ.KNACH(L)) GOTO 40
 30            CONTINUE
               DO 35 L=1,KN(0,I)
                  IF (KNEW.EQ.KN(L,I)) GOTO 40
 35            CONTINUE
               NV = NV+1
               KNACH(NV) = KNEW
 40            CONTINUE
 50         CONTINUE
 60      CONTINUE
         DO 80 J=1,NV
            DO 70 K=1,3
               XVP(J,K) = XD(K,KNACH(J))
 70         CONTINUE
 80      CONTINUE

C........ X und N neu berechnen
         CALL GEOPAN(XV,XI,ETA,N(1,I),XQ(1,I),DIR,IK,Q,P,RR,NV,XVP,
     &               FLATP,D12,D23,D34,D41,DM32,DM41,B32,B41,W,TLEN,SI)

         GEOP(1,I)  =  XI(1)
         GEOP(2,I)  =  XI(2)
         GEOP(3,I)  =  XI(3)
         GEOP(4,I)  =  XI(4)
         GEOP(5,I)  =  ETA(1)
         GEOP(6,I)  =  ETA(2)
         GEOP(7,I)  =  ETA(3)
         GEOP(8,I)  =  ETA(4)
         GEOP(9,I)  =  TLEN
         GEOP(10,I) =  DM32
         GEOP(11,I) =  DM41
         GEOP(12,I) =  DIR(1,1)
         GEOP(13,I) =  DIR(2,1)
         GEOP(14,I) =  DIR(3,1)
         GEOP(15,I) =  DIR(1,2)
         GEOP(16,I) =  DIR(2,2)
         GEOP(17,I) =  DIR(3,2)
         GEOP(18,I) =  DIR(1,3)
         GEOP(19,I) =  DIR(2,3)
         GEOP(20,I) =  DIR(3,3)
         GEOP(21,I) =  B32
         GEOP(22,I) =  B41
         GEOP(23,I) =  D12
         GEOP(24,I) =  D23
         GEOP(25,I) =  D34
         GEOP(26,I) =  D41
         GEOP(27,I) =  P
         GEOP(28,I) =  Q
         GEOP(29,I) =  RR
         DO 90 J=1,15
            GEOP(J+29,I) = SI(J)
 90      CONTINUE
         IF (I.LE.NPB .OR. I.GT.(NPB+NPO) ) THEN
            X(1,I) = XQ(1,I)
            X(2,I) = XQ(2,I)
            X(3,I) = XQ(3,I)
         END IF
         IF (I.LE.NTRANS) THEN
            N(1,I) = -1.0
            N(2,I) = 0.0
            N(3,I) = 0.0
         END IF
 100  CONTINUE
      END
C)))))))))))))))))))) End of Subroutine GEOHO (((((((((((((((((((((((


C***********************************************************************
C
C      Subroutine GEOPAN, computes the normal vector of the panel
C      from the four corner points, projects the four corner points
C      onto a flat surface with this normal vector, computes the
C      control point for the panel, and rotates the projected flat
C      panel into the x-y plane.  The coefficients are then determined
C      for a parabolic panel geometry in the form:
C        ZETA = ZO + A*XI + B*ETA + P*XI**2 + 2*Q*XI*ETA + R*ETA**2
C
C      The other parameters and coefficients relating to the panel
C      geometry as required for the Hess Higher order method described
C      in MDC Report No. J8519, June 1979, are also computed.
C
C
C      The panel is assumed to be a quadrilateral with two parallel
C      ( or nearly parallel ) sides.
C      (The line between input points 1 & 2 should be parallel (or at
C      least close to parallel) to the line between input points 3 & 4.)
C      Panel indecies should be numbered clockwise, when viewed from
C      the fluid domain.  The normal vector points outward, into the
C      fluid domain.
C
C      Viewing from fluid domain:
C                                                          (xi4,eta4)
C     (x3,y3,z3) *-----* (x4,y4,z4)         (xi3,eta3)*------*
C               /      |        _______\        _____/___    |
C              /       |               /      xi    /    |   |
C  (x2,y2,z2) *--------* (x1,y1,z1)                * --------*
C                                           (xi2,eta2)   | (xi1,eta1)
C         Normal vector is out from screen.             eta
C
C      Note: If the panel is triangular, it must be the sides between
C            verticies 1&2 or between sides 3&4 which are set to zero.
C
C      Programmed by Michael Hughes
C      June 1994
C
C      Modified in October 1994 in order to use an iterative procedure
C       for determining the panel geometry coefficients
C
C
C      Input:
C         XV - coordinates of panel corner points in global system
C         NV - Number of extra points used to determine geometry
C         XVP - coordinates of the extra points
C
C      Output -
C         XI,ETA - coordinates of the flat panel in local system
C         XC - panel control point in global system
C         XN - panel normal vector in global system
C         DIRP - transformation matrix from global to local system
C         IK - error flag (=1 if curved panel crosses symmetry plane)
C         Z0,A,B,Q,P,R - Coefficients for panel geometry
C            zeta = Z0 +A(xi) +B(eta) +P(xi^2) +2Q(xi)(eta) +R(eta^2)
C         D12,D23,D34,D41 - length of each panel side
C***********************************************************************

      SUBROUTINE GEOPAN(XV,XI,ETA,XN,XC,DIRP,IK,Q,P,R,NV,XVP,FLATP,
     &                  D12,D23,D34,D41,DM32,DM41,B32,B41,W,TLEN,SI)

      LOGICAL FLATP
      DIMENSION XV(4,3),ZV(4,3),XI(4),ETA(4),XN(3),UP(3),VP(3),
     *          D1(3),D2(3),XC(3),DIRP(3,3),PF(3),PS(3),
     *          XVP(9,3),XIV(4),ETAV(4),ZETAV(4),ZVP(4,3),XF(3),XS(3),
     *          ALPH(3),BETA(3),GAMM(3),DIROLD(3,3),SI(15),AMAT(5,9)
      DIMENSION ZVOLD(4,3),xm(3),xnm(3),xim(4),etam(4),dirm(3,3)
      DIMENSION MM(15),NN(15)
      DATA HALF,ZERO,TWO/0.5E00,0.0E00,2.0E00/
      DATA ONE,TOLT /1.0E00,1.0E-5/
      DATA MM/0,0,1,0,1,2,0,1,2,3,0,1,2,3,4/
      DATA NN/0,1,0,2,1,0,3,2,1,0,4,3,2,1,0/

      IK=0

C..... Compute the flat panel geometry associated to the four input
C      vertex points, also compute vector UP, which is the direction
C      of the horizontal axis for the local panel coordinate system
      DF = ZERO
      DS = ZERO
      DU = ZERO
      DO 10 I=1,3
         PF(I) = XV(2,I)-XV(1,I)
         PS(I) = XV(3,I)-XV(4,I)
         UP(I) = PF(I) + PS(I)
         DF = DF + PF(I)*PF(I)
         DS = DS + PS(I)*PS(I)
         DU = DU + UP(I)*UP(I)
         XF(I) = HALF*(XV(1,I)+XV(2,I))
         XS(I) = HALF*(XV(3,I)+XV(4,I))
         XC(I) = HALF*(XF(I)+XS(I))
         xm(i) = xc(i)
 10   CONTINUE
      DF = SQRT(DF)
      DS = SQRT(DS)
      DU = SQRT(DU)
      DO 20 I=1,3
         UP(I) = UP(I)/DU
         ZV(1,I) = XF(I) - HALF*DF*UP(I)
         ZV(2,I) = XF(I) + HALF*DF*UP(I)
         ZV(3,I) = XS(I) + HALF*DS*UP(I)
         ZV(4,I) = XS(I) - HALF*DS*UP(I)
         D1(I) = ZV(3,I)-ZV(1,I)
         D2(I) = ZV(4,I)-ZV(2,I)
         DO 18 J=1,4
            ZVOLD(J,I) = ZV(J,I)
 18      CONTINUE
 20   CONTINUE
C..... Compute cross-product of diagonal vectors to find normal vector
      XN(1) = D2(2)*D1(3) - D1(2)*D2(3)
      XN(2) = D1(1)*D2(3) - D2(1)*D1(3)
      XN(3) = D2(1)*D1(2) - D1(1)*D2(2)
      XNNORM = SQRT( XN(1)**2 + XN(2)**2 + XN(3)**2 )
      DO 30 I=1,3
         XN(I) = XN(I)/XNNORM
 30   CONTINUE

C..... Compute cross-product of normal vector and local panel horiz.
C      axis vector to find local panel vertical axis vector.
      VP(1) = XN(2)*UP(3) - UP(2)*XN(3)
      VP(2) = UP(1)*XN(3) - XN(1)*UP(3)
      VP(3) = XN(1)*UP(2) - UP(1)*XN(2)
      VPNORM= SQRT( VP(1)**2 + VP(2)**2 + VP(3)**2 )
      DO 40 J=1,3
         VP(J) = VP(J)/VPNORM
 40   CONTINUE

C----------------------------------------------------------------------
C     Compute the direction cosines for the rotation
C----------------------------------------------------------------------
      DO 50 J=1,3
         DIRP(1,J) = UP(J)
         DIRP(2,J) = VP(J)
         DIRP(3,J) = XN(J)
 50   CONTINUE

      ITMX = 20
      POLD=999.9
      QOLD=999.9
      ROLD=999.9
      AOLD=999.9
      BOLD=999.9
      ZOLD=999.9
      DO 470 ITER=1,ITMX
         DO 55 I=1,4
            DO 53 J=1,3
               ZV(I,J) = ZVOLD(I,J)
 53         CONTINUE
 55      CONTINUE
c          write(*,*) 'Iteration Number', iter
C..... Translate the projected corner points so that the origin is
C      at the control points
      DO 70 I=1,4
         DO 60 J=1,3
            ZVP(I,J) = ZV(I,J) - XC(J)
 60      CONTINUE
 70   CONTINUE

C..... Rotate the panel so that it lies in the x-y plane
C      (i.e. the new normal vector should be (0,0,1) and the origin
C            should be at the panel control point )
      DO 100 I=1,4
         XI(I) = DIRP(1,1)*ZVP(I,1) + DIRP(1,2)*ZVP(I,2) +
     *                 DIRP(1,3)*ZVP(I,3)
         ETA(I) = DIRP(2,1)*ZVP(I,1) + DIRP(2,2)*ZVP(I,2) +
     *                 DIRP(2,3)*ZVP(I,3)
 100  CONTINUE
C..... Translate the verticies so the centroid is at the origin
C      Delete next 12 lines to place origin at average of four corners
      CALL CENTROID(XI,ETA,XIC,ETAC)
      DO 105 I=1,4
         XI(I) = XI(I)-XIC
         ETA(I) = ETA(I)-ETAC
 105  CONTINUE
C..... Transform new control point back into global coordinate system
      XCC = DIRP(1,1)*XIC + DIRP(2,1)*ETAC
      YCC = DIRP(1,2)*XIC + DIRP(2,2)*ETAC
      ZCC = DIRP(1,3)*XIC + DIRP(2,3)*ETAC
      XC(1) = XC(1)+XCC
      XC(2) = XC(2)+YCC
      XC(3) = XC(3)+ZCC

C..... Store results for flat panel incase curved panel is unacceptable
      if (iter.eq.1) then
         do 106 i=1,4
            xim(i) = xi(i)
            etam(i) = eta(i)
 106     continue
         do 108 i=1,3
            xm(i) = xc(i)
            xnm(i) = xn(i)
            do 107 j=1,3
               dirm(i,j) = dirp(i,j)
 107        continue
 108     continue
      end if

C..... Translate the projected corner points so that the origin is
C      at the control points
      DO 115 I=1,4
         DO 110 J=1,3
            ZV(I,J) = ZV(I,J) - XC(J)
 110     CONTINUE
 115  CONTINUE

C..... Compute the characteristic length of the panel
      XI31 = XI(3)-XI(1)
      ETA24 = ETA(2)-ETA(4)
      DIAG1=SQRT( XI31*XI31 + (ETA(3)-ETA(1))**2 )
      DIAG2=SQRT( (XI(4)-XI(2))**2 + ETA24*ETA24 )
      TLEN=MAX(DIAG1,DIAG2)

C..... Compute other constants required by Hess method near field
C      velocity formulas
      W = ETA(1)-ETA(3)
      XI32 = XI(2)-XI(3)
      XI41 = XI(1)-XI(4)
      DM32 = XI32/W
      DM41 = XI41/W

C..... If panel is known to be flat, skip p,q,r computations
      IF (FLATP) THEN
         IK = 1
         p = zero
         q = zero
         r = zero
         GOTO 524
      END IF

C...... Compute XIV,ETAV, and ZETAV, the coordinates of the original
C       input panel vertex points in the local panel coordinate system
      DO 130 I=1,4
         DO 120 J=1,3
            ZVP(I,J) = XV(I,J) - XC(J)
 120     CONTINUE
 130  CONTINUE
      DO 140 I=1,4
         XIV(I) = DIRP(1,1)*ZVP(I,1) + DIRP(1,2)*ZVP(I,2) +
     *            DIRP(1,3)*ZVP(I,3)
         ETAV(I) = DIRP(2,1)*ZVP(I,1) + DIRP(2,2)*ZVP(I,2) +
     *             DIRP(2,3)*ZVP(I,3)
         ZETAV(I) = DIRP(3,1)*ZVP(I,1) + DIRP(3,2)*ZVP(I,2) +
     *              DIRP(3,3)*ZVP(I,3)
 140  CONTINUE

C..... Compute terms related to the surface curvature
      IF ( (DF.LT.TOLT) .OR. (DS.LT.TOLT) ) THEN
C........ Triangluar Panel
C         print*,'triangular panel'
         DO 150 J=1,3
           JJ=J
           IF (DF.LT.TOLT) JJ=J+1
           AMAT(J,1) = ONE
           AMAT(J,2) = XIV(JJ)
           AMAT(J,3) = ETAV(JJ)
           AMAT(J,4) = XIV(JJ)*XIV(JJ)
           AMAT(J,5) = ETAV(JJ)*ETAV(JJ)
           AMAT(J,6) = TWO*XIV(JJ)*ETAV(JJ)
           AMAT(J,7) = -ZETAV(JJ)
 150     CONTINUE
         S=0.00000000001
         N=3
         NR=4
         IM=5
         CALL SIMQS(AMAT,N,NR,IM,S)
         ZP = AMAT(1,4)
         ZR = AMAT(1,5)
         ZQ = AMAT(1,6)
         ZOO = AMAT(1,7)
         AP = AMAT(2,4)
         AR = AMAT(2,5)
         AQ = AMAT(2,6)
         AO = AMAT(2,7)
         BP = AMAT(3,4)
         BR = AMAT(3,5)
         BQ = AMAT(3,6)
         BO = AMAT(3,7)

C----------------------------------------------------------------------
C        Use least squares method to determine parameters P, Q and R;
C         Find P and R such that the error**2 for the zeta values
C         at the input vertex points of the neighboring panels which
C         are not also verticies of the panel in question is minimized.
C        Depending on whether the panel is on an edge of the body and
C        whether the panel is triangular or quadrilateral, the number
C        of surrounding vertex points used will vary.
C
C        NV is the number of surrounding vertex points to be used, and
C        the X,Y,Z coordintes in the global coordinate system of these
C        points is XVP(I,J)
C----------------------------------------------------------------------
         CPP = ZERO
         CQQ = ZERO
         CRR = ZERO
         CPQ = ZERO
         CPR = ZERO
         CQR = ZERO
         XLP = ZERO
         XLQ = ZERO
         XLR = ZERO
         DO 170 M=1,NV
C........... Transform the vertex point to the local coordinate system
            XP = XVP(M,1) - XC(1)
            YP = XVP(M,2) - XC(2)
            ZZP = XVP(M,3) - XC(3)
            XIP =   DIRP(1,1)*XP + DIRP(1,2)*YP + DIRP(1,3)*ZZP
            ETAP =  DIRP(2,1)*XP + DIRP(2,2)*YP + DIRP(2,3)*ZZP
            ZETAP = DIRP(3,1)*XP + DIRP(3,2)*YP + DIRP(3,3)*ZZP
C........... Determine sums for least square solution for P and R.
            DXEM = ZOO + AO*XIP + BO*ETAP
            PXEM = ZP + AP*XIP + BP*ETAP + XIP*XIP
            QXEM = ZQ + AQ*XIP + BQ*ETAP + TWO*XIP*ETAP
            RXEM = ZR + AR*XIP + BR*ETAP + ETAP*ETAP
            CPP = CPP + PXEM**2
            CQQ = CQQ + QXEM**2
            CRR = CRR + RXEM**2
            CPQ = CPQ + PXEM*QXEM
            CPR = CPR + PXEM*RXEM
            CQR = CQR + QXEM*RXEM
            XLP = XLP + (ZETAP-DXEM)*PXEM
            XLQ = XLQ + (ZETAP-DXEM)*QXEM
            XLR = XLR + (ZETAP-DXEM)*RXEM
C........... First point is panel control point (add its effect twice)
c            IF (M.EQ.1) THEN
c               CPP = CPP + PXEM**2
c               CQQ = CQQ + QXEM**2
c               CRR = CRR + RXEM**2
c               CPQ = CPQ + PXEM*QXEM
c               CPR = CPR + PXEM*RXEM
c               CQR = CQR + QXEM*RXEM
c               XLP = XLP + (ZETAP-DXEM)*PXEM
c               XLQ = XLQ + (ZETAP-DXEM)*QXEM
c               XLR = XLR + (ZETAP-DXEM)*RXEM
c            END IF
 170     CONTINUE
         AMAT(1,1) = CPP
         AMAT(1,2) = CPQ
         AMAT(1,3) = CPR
         AMAT(1,4) = -XLP
         AMAT(2,1) = CPQ
         AMAT(2,2) = CQQ
         AMAT(2,3) = CQR
         AMAT(2,4) = -XLQ
         AMAT(3,1) = CPR
         AMAT(3,2) = CQR
         AMAT(3,3) = CRR
         AMAT(3,4) = -XLR
C........ Solve for P,Q and R
         S=0.00000000001
         N=3
         NR=1
         IM=5
         CALL SIMQS(AMAT,N,NR,IM,S)
         P = AMAT(1,4)
         Q = AMAT(2,4)
         R = AMAT(3,4)
C........ Compute the remaining coefficients
         A = AO + AQ*Q + AP*P + AR*R
         B = BO + BQ*Q + BP*P + BR*R
         ZO = ZOO + ZQ*Q + ZP*P + ZR*R

      ELSE
C........ Quadrilateral Panel
C         print*,'quadrilateral panel'

         DO 180 J=1,4
            AMAT(J,1) = ONE
            AMAT(J,2) = XIV(J)
            AMAT(J,3) = ETAV(J)
            AMAT(J,4) = TWO*XIV(J)*ETAV(J)
            AMAT(J,5) = -ZETAV(J)
            AMAT(J,6) = XIV(J)*XIV(J)
            AMAT(J,7) = ETAV(J)*ETAV(J)
 180     CONTINUE
         S=0.00000000001
         N=4
         NR=3
         IM=5
         CALL SIMQS(AMAT,N,NR,IM,S)
         ZOO=AMAT(1,5)
         ZP=AMAT(1,6)
         ZR=AMAT(1,7)
         AO=AMAT(2,5)
         AP=AMAT(2,6)
         AR=AMAT(2,7)
         BO=AMAT(3,5)
         BP=AMAT(3,6)
         BR=AMAT(3,7)
         QO=AMAT(4,5)
         QP=AMAT(4,6)
         QR=AMAT(4,7)

C----------------------------------------------------------------------
C        Use least squares method to determine parameters P and R;
C         Find P and R such that the error**2 for the zeta values
C         at the input vertex points of the neighboring panels which
C         are not also verticies of the panel in question is minimized.
C        Depending on whether the panel is on an edge of the body and
C        whether the panel is triangular or quadrilateral, the number
C        of surrounding vertex points used will vary.
C
C        NV is the number of surrounding vertex points to be used, and
C        the X,Y,Z coordintes in the global coordinate system of these
C        points is XVP(I,J)
C----------------------------------------------------------------------
         DXE2 = TWO*QO
         EXE2 = TWO*QP
         FXE2 = TWO*QR
         CRR = ZERO
         CPP = ZERO
         CRP = ZERO
         XL1 = ZERO
         XL2 = ZERO
         DO 200 M=1,NV
C........... Transform the vertex point to the local coordinate system
            XP = XVP(M,1) - XC(1)
            YP = XVP(M,2) - XC(2)
            ZZP = XVP(M,3) - XC(3)
            XIP =   DIRP(1,1)*XP + DIRP(1,2)*YP + DIRP(1,3)*ZZP
            ETAP =  DIRP(2,1)*XP + DIRP(2,2)*YP + DIRP(2,3)*ZZP
            ZETAP = DIRP(3,1)*XP + DIRP(3,2)*YP + DIRP(3,3)*ZZP
C........... Determine sums for least square solution for P and R.
            DXEM = ZOO + AO*XIP + BO*ETAP + DXE2*XIP*ETAP
            EXEM = ZP + AP*XIP + BP*ETAP + XIP*XIP + EXE2*XIP*ETAP
            FXEM = ZR + AR*XIP + BR*ETAP + ETAP*ETAP + FXE2*XIP*ETAP
            CRR = CRR + FXEM**2
            CPP = CPP + EXEM**2
            CRP = CRP + FXEM*EXEM
            XL1 = XL1 + (ZETAP-DXEM)*FXEM
            XL2 = XL2 + (ZETAP-DXEM)*EXEM
C........... First point is panel control point (add its effect twice)
c            IF (M.EQ.1) THEN
c               CRR = CRR + FXEM**2
c               CPP = CPP + EXEM**2
c               CRP = CRP + FXEM*EXEM
c               XL1 = XL1 + (ZETAP-DXEM)*FXEM
c               XL2 = XL2 + (ZETAP-DXEM)*EXEM
c            END IF
 200     CONTINUE
C........ Solve for P and R
         CALL SOLVETWO(R,P,CRR,CRP,XL1,CRP,CPP,XL2)
C........ Compute the remaining coefficients
         A = AO + AP*P + AR*R
         B = BO + BP*P + BR*R
         Q = QO + QP*P + QR*R
         ZO = ZOO + ZP*P + ZR*R
      END IF

C..... Now that the geometry of the panel has been defined, adjust
C      the position of the control point, so that the control point
C      lies on the curved surface at the point closest to the origin.
C      Then redefine the flat panel surface, so that it is tangent to
C      the curved surface at this point.  This is done in an iterative
C      manner, using a Newton-Raphson method.
      ITMAX=100
      XIXC = ZERO
      ETAXC = ZERO
      DO 300 K=1,ITMAX
         ZETAXC = ZO + A*XIXC + B*ETAXC + P*XIXC**2 +
     *            TWO*Q*XIXC*ETAXC + R*ETAXC**2
         ZZX = A+TWO*(P*XIXC + Q*ETAXC)
         ZZN = B+TWO*(Q*XIXC + R*ETAXC)
         GXE = XIXC + ZETAXC*ZZX
         HXE = ETAXC + ZETAXC*ZZN
         GX = ONE + ZZX**2 + TWO*P*ZETAXC
         GN = ZZX*ZZN + TWO*Q*ZETAXC
         HX = GN
         HN = ONE + ZZN**2 + TWO*R*ZETAXC
         RHSG = GX*XIXC + GN*ETAXC - GXE
         RHSH = HX*XIXC + HN*ETAXC - HXE
         CALL SOLVETWO(XINEW,ETANEW,GX,GN,RHSG,HX,HN,RHSH)
         ERRX = ABS(XINEW-XIXC)
         ERRN = ABS(ETANEW-ETAXC)
         ERR = MAX(ERRX,ERRN)
         XIXC = XINEW
         ETAXC = ETANEW
         IF (ERR.LT.TOLT) GOTO 305
 300  CONTINUE
      WRITE(*,*) 'Newton Rapson Iteration did not converge'
 305  CONTINUE
      ZETAXC = ZO + A*XIXC + B*ETAXC + P*XIXC**2 +
     *         TWO*Q*XIXC*ETAXC + R*ETAXC**2

      ZZX = A+TWO*(P*XIXC + Q*ETAXC)
      ZZN = B+TWO*(Q*XIXC + R*ETAXC)
      GAMMA = SQRT( ONE+ ZZX*ZZX + ZZN*ZZN )
      GAMM(1) = -ZZX/GAMMA
      GAMM(2) = -ZZN/GAMMA
      GAMM(3) = SQRT(ONE - GAMM(1)*GAMM(1) - GAMM(2)*GAMM(2))
      G12 = SQRT( GAMM(1)*GAMM(1) + GAMM(2)*GAMM(2) )
      IF ( G12 .LT. 0.001 ) THEN
         G1 = HALF*SQRT(TWO)
         G2 = G1
      ELSE
         G1 = GAMM(1)/G12
         G2 = GAMM(2)/G12
      END IF
      ALPH(1) = G2*G2 + GAMM(3)*G1*G1
      ALPH(2) = (GAMM(3)-ONE)*G1*G2
      ALPH(3) = -G1*SQRT(ONE-GAMM(3)*GAMM(3))
      CALL CROSSP(GAMM,ALPH,BETA)

C..... Compute the new control point
      XC(1) = XC(1) + DIRP(1,1)*XIXC+DIRP(2,1)*ETAXC+DIRP(3,1)*ZETAXC
      XC(2) = XC(2) + DIRP(1,2)*XIXC+DIRP(2,2)*ETAXC+DIRP(3,2)*ZETAXC
      XC(3) = XC(3) + DIRP(1,3)*XIXC+DIRP(2,3)*ETAXC+DIRP(3,3)*ZETAXC
C..... Compute the new transfromation matrix
C..... Store old transformation matrix
      DO 400 I=1,3
         DO 390 J=1,3
            DIROLD(I,J) = DIRP(I,J)
 390     CONTINUE
 400  CONTINUE
      DO 450 I=1,3
         DO 440 J=1,3
            DIRP(I,J) = ALPH(I)*DIROLD(1,J) + BETA(I)*DIROLD(2,J) +
     *                  GAMM(I)*DIROLD(3,J)
C....... Let's try some damping
            DIRP(I,J) = DIROLD(I,J) + 0.0001*(DIRP(I,J)-DIROLD(I,J))
 440     CONTINUE
 450  CONTINUE
      DO 460 I=1,3
         XN(I) = DIRP(3,I)
 460  CONTINUE
      GERROR = ABS(P-POLD)+ABS(R-ROLD)+ABS(Q-QOLD)+ABS(A-AOLD)+
     *         ABS(B-BOLD)+ABS(ZO-ZOLD)
      POLD=P
      QOLD=Q
      ROLD=R
      AOLD=A
      BOLD=B
      ZOLD=ZO
      IF ( GERROR.LT.0.0001) GOTO 461
 470  CONTINUE



C-----------------------------------------------------------------------
C    Check to see if panel is curved too much,  crosses symmetry plane,
C    is not converged, or is otherwise bad.  In these cases, force the
C    panel to be a flat panel
C-----------------------------------------------------------------------

      IF (XC(2).LT.ZERO) WRITE(*,*) 'Y Control point less than zero'
      WRITE(*,*) 'PANEL GEOMETRY NOT CONVERGED IN',ITMX,'ITERATIONS'
      WRITE(*,*) 'Using flat panel'
      IK=3
      FLATP=.TRUE.
      GOTO 462
 461  CONTINUE
C..... if panel crosses symmetry plane, use flat panel
      IF (XC(2).LT.ZERO) THEN
         WRITE(*,*)
     &   'Y Control point less than zero, changing to flat panel'
         FLATP=.TRUE.
         IK=1
      END IF
      IF (ITER.GT.6) THEN
        WRITE(*,*) 'Panel Geometry converged in ',iter,' iterations'
        WRITE(*,'(a6,6f14.6)') 'panel',(xc(jj),jj=1,3),(xn(jj),jj=1,3)
        WRITE(*,'(a6,6f14.6)') 'flat',(xm(jj),jj=1,3),(xnm(jj),jj=1,3)
        WRITE(*,'(a10,3f17.8)') 'pqr',p,q,r
      END IF
 462  CONTINUE
C..... if any of the panel curvature terms are so large that
C      panel curves out more than 1/8th of its half length or width
C      set the panel to be a flat panel.
      ETMAX = MAX(ABS(ETA(1)),ABS(ETA(2)))
      ETMAX = MAX(ETMAX,ABS(ETA(3)))
      ETMAX = MAX(ETMAX,ABS(ETA(4)))
      XIMAX = MAX(ABS(XI(1)),ABS(XI(2)))
      XIMAX = MAX(XIMAX,ABS(XI(3)))
      XIMAX = MAX(XIMAX,ABS(XI(4)))
      TESTP = ONE/(8.0*XIMAX*XIMAX)
      TESTR = ONE/(8.0*ETMAX*ETMAX)
      TESTQ = ONE/(8.0*ETMAX*XIMAX)
      IF ( (ABS(P) .GT. TESTP)  .OR. (ABS(Q) .GT. TESTQ) .OR.
     &     (ABS(R) .GT. TESTR) ) THEN
         WRITE(*,*)
     &   'Panel is curved too much; Setting equal to flat panel'
         WRITE(*,'(A12,5F12.5)') 'pqr,xi,eta',p,q,r,ximax,etmax
         FLATP = .TRUE.
         IK=2
      END IF

      IF (FLATP) THEN
         WRITE(*,'(a10,3f17.8)') 'old',(xc(jj),jj=1,3)
         WRITE(*,'(a10,3f17.8)') 'new',(xm(jj),jj=1,3)
         WRITE(*,'(a10,3f17.8)') 'pqr old',p,q,r
         DO 476 I=1,4
            XI(I) = XIM(I)
            ETA(I) = ETAM(I)
 476     CONTINUE
         do 478 I=1,3
            XC(I) = XM(I)
            XN(I) = XNM(I)
            DO 477 J=1,3
               DIRP(I,J) = DIRM(I,J)
 477        CONTINUE
 478     CONTINUE
         IK = 1
         P = ZERO
         Q = ZERO
         R = ZERO
         XI31 = XI(3)-XI(1)
         ETA24 = ETA(2)-ETA(4)
         DIAG1=SQRT( XI31*XI31 + (ETA(3)-ETA(1))**2 )
         DIAG2=SQRT( (XI(4)-XI(2))**2 + ETA24*ETA24 )
         TLEN=MAX(DIAG1,DIAG2)
         W = ETA(1)-ETA(3)
         XI32 = XI(2)-XI(3)
         XI41 = XI(1)-XI(4)
         DM32 = XI32/W
         DM41 = XI41/W
      END IF
 524  CONTINUE
      B32 = (XI(3)*ETA(2) - XI(2)*ETA(3))/W
      B41 = (XI(4)*ETA(1) - XI(1)*ETA(4))/W
C..... Compute the characteristic length of the panel
      XI31V = XI(3)-XI(1)
      ETA24V = ETA(2)-ETA(4)
      DIAG1=SQRT( XI31V*XI31V + (ETA(3)-ETA(1))**2 )
      DIAG2=SQRT( (XI(4)-XI(2))**2 + ETA24V*ETA24V )
      TLEN=MAX(DIAG1,DIAG2)
      D12 = DF
      D34 = DS
      DM32S = DM32*DM32
      DM32C = DM32S*DM32
      DM32Q = DM32C*DM32
      DM32F = DM32Q*DM32
      DM41S = DM41*DM41
      DM41C = DM41S*DM41
      DM41Q = DM41C*DM41
      DM41F = DM41Q*DM41
      D23 = W*SQRT( ONE + DM32S )
      D41 = W*SQRT( ONE + DM41S )

C..... Compute the panel moments
      X1 = XI(1)/TLEN
      X2 = XI(2)/TLEN
      X3 = XI(3)/TLEN
      X4 = XI(4)/TLEN
      Y1 = ETA(1)/TLEN
      Y2 = ETA(2)/TLEN
      Y3 = ETA(3)/TLEN
      Y4 = ETA(4)/TLEN
      DO 500 I=1,15
         M=MM(I)
         N=NN(I)
         IF ( ABS(DM32).GT.ONE) THEN
            N1 = (M+1)*(N+1)
            N2 = (N+1)*(N+2)
            N3 = N2*(N+3)
            N4 = N3*(N+4)
            N5 = N4*(N+5)
            N6 = N5*(N+6)
            M4 = M*(M-1)
            M5 = M4*(M-2)
            M6 = M5*(M-3)
            S1 = (X2**(N+1))*(Y2**(M+1)) - (X3**(N+1))*(Y3**(M+1))
            S2 = (X2**(N+2))*(Y2**M) - (X3**(N+2))*(Y3**M)
            S3 = (X2**(N+3))*(Y2**(M-1)) - (X3**(N+3))*(Y3**(M-1))
            S4 = (X2**(N+4))*(Y2**(M-2)) - (X3**(N+4))*(Y3**(M-2))
            S5 = (X2**(N+5))*(Y2**(M-3)) - (X3**(N+5))*(Y3**(M-3))
            S6 = (X2**(N+6))*(Y2**(M-4)) - (X3**(N+6))*(Y3**(M-4))
            S1 = S1/FLOAT(N1)
            S2 = S2/( DM32*FLOAT(N2) )
            S3 = FLOAT(M)*S3/( DM32S*FLOAT(N3) )
            S4 = FLOAT(M4)*S4/( DM32C*FLOAT(N4) )
            S5 = FLOAT(M5)*S5/( DM32Q*FLOAT(N5) )
            S6 = FLOAT(M6)*S6/( DM32F*FLOAT(N6) )
            S32 = S1-S2+S3-S4+S5-S6
         ELSE
            M1 = (M+1)*(M+2)
            M2 = M1*(M+3)
            M3 = M2*(M+4)
            M4 = M3*(M+5)
            M5 = M4*(M+6)
            N3 = N*(N-1)
            N4 = N3*(N-2)
            N5 = N4*(N-3)
            S1 = (X2**N)*(Y2**(M+2)) - (X3**N)*(Y3**(M+2))
            S2 = (X2**(N-1))*(Y2**(M+3)) - (X3**(N-1))*(Y3**(M+3))
            S3 = (X2**(N-2))*(Y2**(M+4)) - (X3**(N-2))*(Y3**(M+4))
            S4 = (X2**(N-3))*(Y2**(M+5)) - (X3**(N-3))*(Y3**(M+5))
            S5 = (X2**(N-4))*(Y2**(M+6)) - (X3**(N-4))*(Y3**(M+6))
            S1 = DM32*S1/FLOAT(M1)
            S2 = DM32S*FLOAT(N)*S2/FLOAT(M2)
            S3 = DM32C*FLOAT(N3)*S3/FLOAT(M3)
            S4 = DM32Q*FLOAT(N4)*S4/FLOAT(M4)
            S5 = DM32F*FLOAT(N5)*S5/FLOAT(M5)
            S32 = S1-S2+S3-S4+S5
         END IF
         IF ( ABS(DM41).GT.ONE) THEN
            N1 = (M+1)*(N+1)
            N2 = (N+1)*(N+2)
            N3 = N2*(N+3)
            N4 = N3*(N+4)
            N5 = N4*(N+5)
            N6 = N5*(N+6)
            M4 = M*(M-1)
            M5 = M4*(M-2)
            M6 = M5*(M-3)
            S1 = (X1**(N+1))*(Y1**(M+1)) - (X4**(N+1))*(Y4**(M+1))
            S2 = (X1**(N+2))*(Y1**M) - (X4**(N+2))*(Y4**M)
            S3 = (X1**(N+3))*(Y1**(M-1)) - (X4**(N+3))*(Y4**(M-1))
            S4 = (X1**(N+4))*(Y1**(M-2)) - (X4**(N+4))*(Y4**(M-2))
            S5 = (X1**(N+5))*(Y1**(M-3)) - (X4**(N+5))*(Y4**(M-3))
            S6 = (X1**(N+6))*(Y1**(M-4)) - (X4**(N+6))*(Y4**(M-4))
            S1 = S1/FLOAT(N1)
            S2 = S2/( DM41*FLOAT(N2) )
            S3 = FLOAT(M)*S3/( DM41S*FLOAT(N3) )
            S4 = FLOAT(M4)*S4/( DM41C*FLOAT(N4) )
            S5 = FLOAT(M5)*S5/( DM41Q*FLOAT(N5) )
            S6 = FLOAT(M6)*S6/( DM41F*FLOAT(N6) )
            S41 = S1-S2+S3-S4+S5-S6
         ELSE
            M1 = (M+1)*(M+2)
            M2 = M1*(M+3)
            M3 = M2*(M+4)
            M4 = M3*(M+5)
            M5 = M4*(M+6)
            N3 = N*(N-1)
            N4 = N3*(N-2)
            N5 = N4*(N-3)
            S1 = (X1**N)*(Y1**(M+2)) - (X4**N)*(Y4**(M+2))
            S2 = (X1**(N-1))*(Y1**(M+3)) - (X4**(N-1))*(Y4**(M+3))
            S3 = (X1**(N-2))*(Y1**(M+4)) - (X4**(N-2))*(Y4**(M+4))
            S4 = (X1**(N-3))*(Y1**(M+5)) - (X4**(N-3))*(Y4**(M+5))
            S5 = (X1**(N-4))*(Y1**(M+6)) - (X4**(N-4))*(Y4**(M+6))
            S1 = DM41*S1/FLOAT(M1)
            S2 = DM41S*FLOAT(N)*S2/FLOAT(M2)
            S3 = DM41C*FLOAT(N3)*S3/FLOAT(M3)
            S4 = DM41Q*FLOAT(N4)*S4/FLOAT(M4)
            S5 = DM41F*FLOAT(N5)*S5/FLOAT(M5)
            S41 = S1-S2+S3-S4+S5
         END IF
         SS = Y1**(M+1)*( X2**(N+1)-X1**(N+1) ) +
     *        Y3**(M+1)*( X4**(N+1)-X3**(N+1) )
         SI(I) = S41-S32 + SS/FLOAT( (M+1)*(N+1) )
 500  CONTINUE
      END

C)))))))))))))))))))) End of Subroutine GEOPAN ((((((((((((((((((((((((


C**********************************************************************
      SUBROUTINE CROSSP(A,B,C)
C----- Compute the cross product of two vectors
C          C = A x B
      DIMENSION A(3),B(3),C(3)

      C(1) = A(2)*B(3) - B(2)*A(3)
      C(2) = B(1)*A(3) - A(1)*B(3)
      C(3) = A(1)*B(2) - B(1)*A(2)
      RETURN
      END
C))))))))))))))))))))))))) End of Subroutine CROSSP (((((((((((((((((((



C**********************************************************************
      SUBROUTINE SOLVETWO(X,Y,A,B,C,D,E,F)
C----- Solve a 2x2 set of linear equations in the form:
C          A*X + B*Y = C
C          D*X + E*Y = F
      DATA ZERO,TOLL,TWO/0.00E00,1.0E-11,2.0E00/

      det = a*e - b*d
      if (abs(det).lt.toll) then
         if ( abs(c).lt.toll .and. abs(f).lt.toll ) then
            x=zero
            y=zero
         else if ( abs(a).lt.toll .and. abs(d).lt.toll ) then
            x=zero
            if ( abs(b+e).gt.toll ) then
               y = (c+f)/(b+e)
            else if (abs(b).gt.toll) then
               y = (two*c+f)/(two*b+e)
            else
               y=zero
            end if
         else if ( abs(b).lt.toll .and. abs(e).lt.toll ) then
            y=zero
            if ( abs(a+d).gt.toll ) then
               x = (c+f)/(a+d)
            else if (abs(a).gt.toll) then
               y = (two*c+f)/(two*a+d)
            else
               x=zero
            end if
         else if ( abs(a+b).gt.0.00000001 ) then
            x = c/(a+b)
            y = x
         else if (abs(d+e).gt.0.00000001) then
            x = f/(d+e)
            y=x
         end if
      else
         x = (c*e - f*b)/det
         y = (a*f - d*c)/det
      end if
      END
C)))))))))))))))))))))))) End of Subroutine SOLVETWO ((((((((((((((((((

C***********************************************************************
C
C      SUBROUTINE CENTROID(XI,ETA,XIC,ETAC)
C
C      Computes the center of area of a flat quadrilateral element
C      given the four corners of the element, in the plane of the
C      element.
C      If the element is triangular, the zero length element must
C      be either between corner points 1&2, or between corner
C      points 3&4.
C      It is assumed that side 1-2 is PARALLEL to side 3-4.
C
C      Program by: Michael Hughes, June 1994
C
C***********************************************************************

      SUBROUTINE CENTROID(XI,ETA,XIC,ETAC)

      DIMENSION XI(4),ETA(4)
      DATA HALF,THIRD /0.50E00,0.3333333333333333/

C..... Compute distances between corners
      X23 = XI(3) - XI(2)
      Y23 = ETA(3) - ETA(2)
      X41 = XI(4) - XI(1)
      Y41 = ETA(4) - ETA(1)
      YAVG = HALF*(Y41 + Y23)
      Y23S = ETA(3)**2 - ETA(2)**2
      Y41S = ETA(4)**2 - ETA(1)**2
      YAVGS = HALF*(Y41S+Y23S)
      Y23C = ETA(3)**3 - ETA(2)**3
      Y41C = ETA(4)**3 - ETA(1)**3
      YAVGC = HALF*(Y41C+Y23C)

C..... Slope (X/Y) between 1 and 4 and between 2 and 3
      DM1 = X41/Y41
      DM2 = X23/Y23
      B1 = XI(1) - DM1*ETA(1)
      B2 = XI(2) - DM2*ETA(2)
      BD = B1-B2
      DMD = DM1-DM2
      BX = B1+B2
      DMX = DM1+DM2
      A = BD*YAVG + HALF*DMD*YAVGS
      YMOM = HALF*BD*YAVGS + THIRD*DMD*YAVGC
      BDX = BD*BX
      BMDX = BX*DMD + BD*DMX
      DMDX = DMD*DMX
      XMOM = HALF*(BDX*YAVG + HALF*BMDX*YAVGS + THIRD*DMDX*YAVGC)
      XIC = XMOM/A
      ETAC = YMOM/A
      END


      SUBROUTINE SIMQS(A,N,NR,IM,S)

C Gauss algorithm for solving a non-singular, real linear system of equations
C A X + B = 0, where A, B and X are (n by n), (n by r) and (n by r) matrices,
C respectively. (n by r) means (n rows, r columns).

C Parameter A (input; changed): Two-index array containing the elements of the
C       (n by n+r) matrix (A, B) which is defined with index ranges of
C       (1:im>=n, 1:arbitrary_value>=n+nr + 1) in the program calling SIMQS.
C                                        ----
C       After regular program execution, the part of A containing initially the
C       matrix B will contain the solution X.
C Parameter N (input): Number of unknowns n
C Parameter NR (input): r (number of absolue-value vectors; 1 in many cases)
C Parameter IM (input): im (maximum defined value of first index of A)
C Parameter S (input): Lower limit of absolute value of pivot elements. If
C       a division by an element the absolute value of which is smaller than S
C       would be required, the system is regarded as singular, and elimination
C       is stopped with an error message indicating the row indices of those
C       rows which are (approximately) linearly dependent on the other ones.
C       It is recommended to generate the equation system such that maximum
C       absolute values of the elements of A in each row and column do not
C       differ by orders of magnitude. Then, for S a value of 10**-5 times that
C       maximum absolute value may be recommended.

      DIMENSION A(*)
      NNR=(N+NR)*IM
      MMR=N*IM
      K1=-IM
      DO 10 I=1,N
10    A(NNR+I)=I+0.5
      DO 50 I=1,N
      K1=K1+IM+1
      BIGA=A(K1)
      IMAX=K1
      DO 30 J=K1+1,K1+N-I
      IF(ABS(BIGA).LT.ABS(A(J)))THEN
      BIGA=A(J)
      IMAX=J
      ENDIF
30    CONTINUE
      IF(ABS(BIGA).LT.S)THEN
        WRITE(6,*)'Singular system of equations'
        WRITE(6,*)'The following equations are combinations of others:'
c        WRITE(6,*) (IFIX(A(NNR+I1)),I1=I,N)
        STOP
      ENDIF
      A(IMAX)=A(K1)
      A(NNR+IMAX-IM*(I-1)) = A(NNR+I)
      DO 50 K=K1+IM,NNR,IM
      IMAX=IMAX+IM
      SAVE=-A(IMAX)/BIGA
      A(IMAX)=A(K)
      A(K)=SAVE
      K2=K1
      DO 50 J=K+1,K+N-I
      K2=K2+1
50    A(J)=A(J)+A(K2)*SAVE
      DO 80 I=MMR,NNR-1,IM
      K1=MMR+1
      DO 80 J=N,2,-1
      K1=K1-IM
      K2=I
      SAVE=A(I+J)
      DO 80 K=K1,K1+J-2
      K2=K2+1
80    A(K2)=A(K2)+A(K)*SAVE
      I=0
      END

C***********************************************************************
C
C     SUBROUTINE PANHO()
C
C     Computes the induced velocity from a curved panel with a
C     linear source distribution.
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Programmed by: Michael Hughes, July 1994
C
c
c   ifo=0 normal higher order panel panel
c   ifo=1 use intermediate or far field only
c   ifo=2 use only far field approx
c
C***********************************************************************

      SUBROUTINE PANHO(XFP,YFP,ZFP,XC,GEOP,LEVEL,
     *                 VX,VY,VZ,VXX,VYX,VZX,VXY,VYY,VZY,
     *                 VXDX,VXDY,VXDZ,VYDY,VYDZ,
     *                 VXXDX,VXXDY,VXXDZ,VYXDY,VYXDZ,
     *                 VXYDX,VXYDY,VXYDZ,VYYDY,VYYDZ,
     *                 VXDXZ,VXDYZ,VXDZZ,VYDYZ,VYDZZ,
     *                 VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     *                 VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,ifo)

      DIMENSION XI(4),ETA(4),SI(15),XC(3),DIRP(3,3),GEOP(44)

      DATA HALF,THREE,FIFT,ONE,TOL,TOLT,ZERO /0.50E00,3.0E00,
     *                   15.0E00,1.0E00,0.000001,0.001,0.0E00/
      DATA TWOPI,TWO,FIVE/ 6.283185308E+00,2.0E00,5.0E00 /

      XI(1) = GEOP(1)
      XI(2) = GEOP(2)
      XI(3) = GEOP(3)
      XI(4) = GEOP(4)
      ETA(1) = GEOP(5)
      ETA(2) = GEOP(6)
      ETA(3) = GEOP(7)
      ETA(4) = GEOP(8)
      TLEN = GEOP(9)
      DM32 = GEOP(10)
      DM41 = GEOP(11)
      DIRP(1,1) = GEOP(12)
      DIRP(2,1) = GEOP(13)
      DIRP(3,1) = GEOP(14)
      DIRP(1,2) = GEOP(15)
      DIRP(2,2) = GEOP(16)
      DIRP(3,2) = GEOP(17)
      DIRP(1,3) = GEOP(18)
      DIRP(2,3) = GEOP(19)
      DIRP(3,3) = GEOP(20)
      B32 = GEOP(21)
      B41 = GEOP(22)
      D12 = GEOP(23)
      D23 = GEOP(24)
      D34 = GEOP(25)
      D41 = GEOP(26)
      P = GEOP(27)
      Q = GEOP(28)
      RR = GEOP(29)
      DO 10 I=1,15
         SI(I) = GEOP(I+29)
 10   CONTINUE

C..... Check for flat panel
      IF ( (ABS(P)+ABS(Q)+ABS(RR)).LT.(0.0001/TLEN) ) THEN
         CALL PANFLT(XFP,YFP,ZFP,XC,LEVEL,XI,ETA,TLEN,DM32,DM41,
     &               DIRP,B32,B41,D12,D23,D34,D41,SI,RAT,
     &               VX,VY,VZ,VXX,VYX,VZX,VXY,VYY,VZY,
     *               VXDX,VXDY,VXDZ,VYDY,VYDZ,
     *               VXXDX,VXXDY,VXXDZ,VYXDY,VYXDZ,
     *               VXYDX,VXYDY,VXYDZ,VYYDY,VYYDZ,
     *               VXDXZ,VXDYZ,VXDZZ,VYDYZ,VYDZZ,
     *               VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     *               VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,ifo)

         VZDZ = -VXDX - VYDY
         VYDX = VXDY
         VZDX = VXDZ
         VZDY = VYDZ
         VZXDZ = -VXXDX - VYXDY
         VYXDX = VXXDY
         VZXDX = VXXDZ
         VZXDY = VYXDZ
         VZYDZ = -VXYDX - VYYDY
         VYYDX = VXYDY
         VZYDX = VXYDZ
         VZYDY = VYYDZ
         GOTO 200
      END IF

C..... Compute radius to panel control point and determine whether to
C      use near, intermediate or far field
      RS = (XFP-XC(1))**2 + (YFP-XC(2))**2 + (ZFP-XC(3))**2
      R = SQRT(RS)
      RAT = R/TLEN
      IF (RAT.LT.5.00 .and. ifo.lt.2) THEN
C........ Convert to panel coordinates
         XP = XFP-XC(1)
         YP = YFP-XC(2)
         ZP = ZFP-XC(3)
         X = DIRP(1,1)*XP + DIRP(1,2)*YP + DIRP(1,3)*ZP
         Y = DIRP(2,1)*XP + DIRP(2,2)*YP + DIRP(2,3)*ZP
         Z = DIRP(3,1)*XP + DIRP(3,2)*YP + DIRP(3,3)*ZP
         XS = X*X
         YS = Y*Y
         ZS = Z*Z
      END IF

      IF (LEVEL.GT.1) GOTO 100

      IF (RAT.LT.2.5 .and. ifo.eq.0)  THEN
c      print*,'near field'
C----------------------------------------------------------------------
C        Near Field (Exact closed form integral solutions)
C----------------------------------------------------------------------
C........ First compute quantities related to the flat panel
C........ Compute radius to each corner point
         X1 = X-XI(1)
         X2 = X-XI(2)
         X3 = X-XI(3)
         X4 = X-XI(4)
         Y1 = Y-ETA(1)
         Y2 = Y-ETA(2)
         Y3 = Y-ETA(3)
         Y4 = Y-ETA(4)
         X1S = X1*X1
         X2S = X2*X2
         X3S = X3*X3
         X4S = X4*X4
         R1S = X1S + Y1**2 + ZS
         R2S = X2S + Y2**2 + ZS
         R3S = X3S + Y3**2 + ZS
         R4S = X4S + Y4**2 + ZS
         R1 = SQRT( R1S )
         R2 = SQRT( R2S )
         R3 = SQRT( R3S )
         R4 = SQRT( R4S )

C........ Compute other distances required for near field equations
         ALPH1 = X1/R1
         BETA1 = Y1/R1
         GAMM1 = Z/R1
         ALPH2 = X2/R2
         BETA2 = Y2/R2
         GAMM2 = Z/R2
         ALPH3 = X3/R3
         BETA3 = Y3/R3
         GAMM3 = Z/R3
         ALPH4 = X4/R4
         BETA4 = Y4/R4
         GAMM4 = Z/R4
         P322 = DM32*(ZS+Y2**2) - X2*Y2
         P323 = DM32*(ZS+Y3**2) - X3*Y3
         P411 = DM41*(ZS+Y1**2) - X1*Y1
         P414 = DM41*(ZS+Y4**2) - X4*Y4

C........ Compute Vx, Vy, and Vz
         XLOG23 = ALOG( (R2+R3-D23)/(R2+R3+D23) )
         XLOG41 = ALOG( (R4+R1-D41)/(R4+R1+D41) )
C........ Check for triangular panels
         IF (D12.LT.TOL) THEN
            XLOG12=ZERO
         ELSE
            XLOG12 = ALOG( (R1+R2-D12)/(R1+R2+D12) )
         END IF
         IF (D34.LT.TOL) THEN
            XLOG34=ZERO
         ELSE
            XLOG34 = ALOG( (R3+R4-D34)/(R3+R4+D34) )
         END IF
         ZR1=Z*R1
         ZR2=Z*R2
         ZR3=Z*R3
         ZR4=Z*R4
         BOT32S = ONE/( ONE + DM32**2 )
         BOT41S = ONE/( ONE + DM41**2 )
         BOT32 = SQRT( BOT32S )
         BOT41 = SQRT( BOT41S )
         BOT32C = BOT32S*BOT32
         BOT41C = BOT41S*BOT41
         VXO = BOT41*XLOG41 - BOT32*XLOG23
         VYO = XLOG34 - XLOG12 + DM32*BOT32*XLOG23 - DM41*BOT41*XLOG41
         H32 = DM32*Y3 - X3
         H41 = DM41*Y1 - X1
         IF ( ABS(Z/TLEN).LT. TOLT ) THEN
            IF ( (RAT.LT.0.5) .AND. (H32*H41.LT.0.0)
     *            .AND. (Y1*Y3.LT.0) ) THEN
               VZO = TWOPI
            ELSE
               VZO = ZERO
            END IF
         ELSE
            DT322 = ATAN(P322/ZR2)
            DT323 = ATAN(P323/ZR3)
            DT411 = ATAN(P411/ZR1)
            DT414 = ATAN(P414/ZR4)
            VZO = DT323 - DT322 + DT411 - DT414
         END IF

         D322 = ZS*R2S + P322**2
         D323 = ZS*R3S + P323**2
         D411 = ZS*R1S + P411**2
         D414 = ZS*R4S + P414**2
         DT322X = -Z*(R2S*BETA2 + P322*ALPH2)/D322
         DT322Y = Z*( (TWO*DM32*BETA2-ALPH2)*R2S - P322*BETA2 )/D322
         DT322Z = (TWO*DM32*ZS*R2 - P322*(R2 + Z*GAMM2))/D322
         DT323X = -Z*(R3S*BETA3 + P323*ALPH3)/D323
         DT323Y = Z*( (TWO*DM32*BETA3-ALPH3)*R3S - P323*BETA3 )/D323
         DT323Z = (TWO*DM32*ZS*R3 - P323*(R3 + Z*GAMM3))/D323
         DT411X = -Z*(R1S*BETA1 + P411*ALPH1)/D411
         DT411Y = Z*( (TWO*DM41*BETA1-ALPH1)*R1S - P411*BETA1 )/D411
         DT411Z = (TWO*DM41*ZS*R1 - P411*(R1 + Z*GAMM1))/D411
         DT414X = -Z*(R4S*BETA4 + P414*ALPH4)/D414
         DT414Y = Z*( (TWO*DM41*BETA4-ALPH4)*R4S - P414*BETA4 )/D414
         DT414Z = (TWO*DM41*ZS*R4 - P414*(R4 + Z*GAMM4))/D414
         DDD12 = TWO*D12/((R1+R2)**2-D12**2)
         DDD23 = TWO*D23/((R2+R3)**2-D23**2)
         DDD34 = TWO*D34/((R3+R4)**2-D34**2)
         DDD41 = TWO*D41/((R4+R1)**2-D41**2)
         DL12DX = DDD12*(ALPH1+ALPH2)
         DL23DX = DDD23*(ALPH2+ALPH3)
         DL34DX = DDD34*(ALPH3+ALPH4)
         DL41DX = DDD41*(ALPH4+ALPH1)
         DL12DY = DDD12*(BETA1+BETA2)
         DL23DY = DDD23*(BETA2+BETA3)
         DL34DY = DDD34*(BETA3+BETA4)
         DL41DY = DDD41*(BETA4+BETA1)
         DL12DZ = DDD12*(GAMM1+GAMM2)
         DL23DZ = DDD23*(GAMM2+GAMM3)
         DL34DZ = DDD34*(GAMM3+GAMM4)
         DL41DZ = DDD41*(GAMM4+GAMM1)

         DVXDX = BOT41*DL41DX - BOT32*DL23DX
         DVXDY = BOT41*DL41DY - BOT32*DL23DY
         DVXDZ = BOT41*DL41DZ - BOT32*DL23DZ
         DVYDX = DL34DX-DL12DX + DM32*BOT32*DL23DX - DM41*BOT41*DL41DX
         DVYDY = DL34DY-DL12DY + DM32*BOT32*DL23DY - DM41*BOT41*DL41DY
         DVYDZ = DL34DZ-DL12DZ + DM32*BOT32*DL23DZ - DM41*BOT41*DL41DZ

         PHI00X = DT323X - DT322X + DT411X - DT414X
         PHI00Y = DT323Y - DT322Y + DT411Y - DT414Y
         IF ( ABS(Y1/TLEN).LT.TOLT ) THEN
            PHI00Z = DM41/ABS(X1) - DM32/ABS(X2) + DT323Z - DT414Z
         ELSE IF ( ABS(H32/TLEN).LT.TOLT ) THEN
            PHI00Z = DM32*BOT32*(ONE/ABS(Y3)-ONE/ABS(Y1)) + DT411Z -
     *               DT414Z
         ELSE IF ( ABS(Y3/TLEN).LT.TOLT ) THEN
            PHI00Z = DM32/ABS(X3) - DM41/ABS(X4) - DT322Z + DT411Z
         ELSE IF ( ABS(H41/TLEN).LT.TOLT ) THEN
            PHI00Z = DM41*BOT41*(ONE/ABS(Y1)-ONE/ABS(Y3)) + DT323Z -
     *               DT322Z
         ELSE
            PHI00Z = DT323Z - DT322Z + DT411Z - DT414Z
         END IF
         PHI01X = Y*PHI00X - Z*DVYDX
         PHI01Y = Y*PHI00Y - Z*DVYDY + VZO
         PHI01Z = Y*PHI00Z - Z*DVYDZ - VYO
         PHI10X = X*PHI00X - Z*DVXDX + VZO
         PHI10Y = X*PHI00Y - Z*DVXDY
         PHI10Z = X*PHI00Z - Z*DVXDZ - VXO

         XMB32 = X-DM32*Y-B32
         XMB41 = X-DM41*Y-B41
         XJ11 = (R3-R2)*BOT32S + (R1-R4)*BOT41S +
     *          DM32*BOT32C*XMB32*XLOG23 -
     *          DM41*BOT41C*XMB41*XLOG41

         DJ11X = (ALPH3-ALPH2)*BOT32S + DM32*BOT32C*XLOG23 +
     *           DM32*BOT32C*XMB32*DL23DX +
     *           (ALPH1-ALPH4)*BOT41S - DM41*BOT41C*XLOG41 -
     *           DM41*BOT41C*XMB41*DL41DX
         DJ11Y = (BETA3-BETA2)*BOT32S + DM32*DM32*BOT32C*XLOG23 +
     *           DM32*BOT32C*XMB32*DL23DY +
     *           (BETA1-BETA4)*BOT41S + DM41*DM41*BOT41C*XLOG41 -
     *           DM41*BOT41C*XMB41*DL41DY
         DJ11Z = (GAMM3-GAMM2)*BOT32S +
     *           DM32*BOT32C*XMB32*DL23DZ +
     *           (GAMM1-GAMM4)*BOT41S -
     *           DM41*BOT41C*XMB41*DL41DZ

         PHI11X = Z*DJ11X + X*PHI01X + Y*PHI10X - X*Y*PHI00X - Z*VYO
         PHI11Y = Z*DJ11Y + X*PHI01Y + Y*PHI10Y - X*Y*PHI00Y - Z*VXO
         PHI11Z = Z*DJ11Z + X*PHI01Z + Y*PHI10Z - X*Y*PHI00Z + XJ11

         HO = DM32*(R2-R3)*BOT32S + DM41*(R4-R1)*BOT41S +
     *        BOT32C*XMB32*XLOG23 - BOT41C*XMB41*XLOG41
         DHODX = DM32*BOT32S*(ALPH2-ALPH3) + BOT32C*XLOG23 +
     *           XMB32*BOT32C*DL23DX + DM41*BOT41S*(ALPH4-ALPH1) -
     *           BOT41C*XLOG41 - XMB41*BOT41C*DL41DX
         DHODY = DM32*BOT32S*(BETA2-BETA3) - DM32*BOT32C*XLOG23 +
     *           XMB32*BOT32C*DL23DY + DM41*BOT41S*(BETA4-BETA1) +
     *           DM41*BOT41C*XLOG41 - XMB41*BOT41C*DL41DY
         DHODZ = DM32*BOT32S*(GAMM2-GAMM3) + XMB32*BOT32C*DL23DZ +
     *           DM41*BOT41S*(GAMM4-GAMM1) - XMB41*BOT41C*DL41DZ

         PHI02X = Z*DHODX + TWO*Y*PHI01X - (YS+ZS)*PHI00X
         PHI02Y = Z*DHODY + TWO*Y*PHI01Y - (YS+ZS)*PHI00Y - TWO*Z*VYO
         PHI02Z = Z*DHODZ + TWO*Y*PHI01Z - (YS+ZS)*PHI00Z +
     *            HO - TWO*Z*VZO

         PHIS = Y1*XLOG12 + (X2-DM32*Y2)*BOT32*XLOG23 -
     *          Y3*XLOG34 - (X4-DM41*Y4)*BOT41*XLOG41 - Z*VZO

         XJ20 = PHIS-HO
         XJ02 = HO - Z*VZO
         DJ20DX = -VXO - DHODX
         DJ20DY = -VYO - DHODY
         DJ20DZ = -VZO - DHODZ

         VXR = -PHI02X
         VYR = -PHI02Y
         VZR = -PHI02Z
         VXQ = -PHI11X
         VYQ = -PHI11Y
         VZQ = -PHI11Z
         VXP = -( Z*DJ20DX + TWO*X*PHI10X - XS*PHI00X - TWO*Z*VXO )
         VYP = -( Z*DJ20DY + TWO*X*PHI10Y - XS*PHI00Y )
         VZP = -( Z*DJ20DZ + TWO*X*PHI10Z - XS*PHI00Z + XJ20 )

         VXX = X*VXO - XJ20
         VYX = X*VYO - XJ11
         VZX = X*VZO - Z*VXO
         VXY = Y*VXO - XJ11
         VYY = Y*VYO - XJ02
         VZY = Y*VZO - Z*VYO

      ELSE IF (RAT.LT.5.0 .and. ifo.lt.2 ) THEN
C-------------------------------------------------------------------
C        Multipole expansion for intermediate field
C-------------------------------------------------------------------
         ALPH = X/R
         BETA = Y/R
         GAMM = Z/R
         ALPHS = ALPH*ALPH
         BETAS = BETA*BETA
         GAMMS = GAMM*GAMM
         UX = -ALPH
         UY = -BETA
         UZ = -GAMM
         UXX = THREE*ALPHS - ONE
         UXY = THREE*ALPH*BETA
         UYY = THREE*BETAS - ONE
         UXZ = THREE*ALPH*GAMM
         UYZ = THREE*BETA*GAMM
         UZZ = THREE*GAMMS - ONE
         UXXX = THREE*ALPH*(THREE-FIVE*ALPHS)
         UXXY = THREE*BETA*( ONE-FIVE*ALPHS )
         UXXZ = THREE*GAMM*( ONE-FIVE*ALPHS )
         UXYY = THREE*ALPH*( ONE-FIVE*BETAS )
         UXYZ = -FIFT*ALPH*BETA*GAMM
         UXZZ = THREE*ALPH*( ONE-FIVE*GAMMS )
         UYYY = THREE*BETA*(THREE-FIVE*BETAS)
         UYYZ = THREE*GAMM*( ONE-FIVE*BETAS )
         UYZZ = THREE*BETA*( ONE-FIVE*GAMMS )
         UXXXZ = FIFT*ALPH*GAMM*(7.0*ALPHS - THREE)
         UXXYZ = FIFT*BETA*GAMM*(7.0*ALPHS-ONE)
         UXXZZ = THREE - FIFT*(ALPHS+GAMMS) + 105.0*ALPHS*GAMMS
         UXYYZ = FIFT*ALPH*GAMM*(7.0*BETAS - ONE)
         UXYZZ = FIFT*ALPH*BETA*(7.0*GAMMS - ONE)
         UYYYZ = FIFT*BETA*GAMM*(7.0*BETAS - THREE)
         UYYZZ = THREE - FIFT*(BETAS+GAMMS) + 105.0*BETAS*GAMMS

         TR = TLEN/R
         TRS = TR*TR
         R3 = RS*R
         RCUB = ONE/R3
         TS = TLEN*TLEN
         TC = TLEN*TS
         TQ = TS*TS
         TQR = TQ*RCUB
         TCR = TC/RS

         VXO = TRS*( -SI(1)*UX + TR*(SI(2)*UXX+SI(3)*UXY)
     *       -  HALF*TRS*(SI(4)*UXXX + TWO*SI(5)*UXXY + SI(6)*UXYY) )
         VYO = TRS*( -SI(1)*UY + TR*(SI(2)*UXY+SI(3)*UYY)
     *       -  HALF*TRS*(SI(4)*UXXY + TWO*SI(5)*UXYY + SI(6)*UYYY) )
         VZO = TRS*( -SI(1)*UZ + TR*(SI(2)*UXZ+SI(3)*UYZ)
     *       -  HALF*TRS*(SI(4)*UXXZ + TWO*SI(5)*UXYZ + SI(6)*UYYZ) )

         VXR = TQR*( SI(6)*UXZ - TR*( SI(9)*UXXZ + SI(10)*UXYZ ) +
     *         HALF*TRS*(SI(13)*UXXXZ+TWO*SI(14)*UXXYZ+SI(15)*UXYYZ) )
         VYR = TQR*( SI(6)*UYZ - TR*( SI(9)*UXYZ + SI(10)*UYYZ ) +
     *         HALF*TRS*(SI(13)*UXXYZ+TWO*SI(14)*UXYYZ+SI(15)*UYYYZ) )
         VZR = TQR*( SI(6)*UZZ - TR*( SI(9)*UXZZ + SI(10)*UYZZ ) +
     *         HALF*TRS*(SI(13)*UXXZZ+TWO*SI(14)*UXYZZ+SI(15)*UYYZZ) )

         VXQ = TQR*( SI(5)*UXZ - TR*( SI(8)*UXXZ + SI(9)*UXYZ ) +
     *         HALF*TRS*(SI(12)*UXXXZ+TWO*SI(13)*UXXYZ+SI(14)*UXYYZ) )
         VYQ = TQR*( SI(5)*UYZ - TR*( SI(8)*UXYZ + SI(9)*UYYZ ) +
     *         HALF*TRS*(SI(12)*UXXYZ+TWO*SI(13)*UXYYZ+SI(14)*UYYYZ) )
         VZQ = TQR*( SI(5)*UZZ - TR*( SI(8)*UXZZ + SI(9)*UYZZ ) +
     *         HALF*TRS*(SI(12)*UXXZZ+TWO*SI(13)*UXYZZ+SI(14)*UYYZZ) )

         VXP = TQR*( SI(4)*UXZ - TR*( SI(7)*UXXZ + SI(8)*UXYZ ) +
     *         HALF*TRS*(SI(11)*UXXXZ+TWO*SI(12)*UXXYZ+SI(13)*UXYYZ) )
         VYP = TQR*( SI(4)*UYZ - TR*( SI(7)*UXYZ + SI(8)*UYYZ ) +
     *         HALF*TRS*(SI(11)*UXXYZ+TWO*SI(12)*UXYYZ+SI(13)*UYYYZ) )
         VZP = TQR*( SI(4)*UZZ - TR*( SI(7)*UXZZ + SI(8)*UYZZ ) +
     *         HALF*TRS*(SI(11)*UXXZZ+TWO*SI(12)*UXYZZ+SI(13)*UYYZZ) )

         VXX = -TCR*( SI(2)*UX - TR*(SI(4)*UXX + SI(5)*UXY) +
     *         HALF*TRS*(SI(7)*UXXX + TWO*SI(8)*UXXY + SI(9)*UXYY) )
         VYX = -TCR*( SI(2)*UY - TR*(SI(4)*UXY + SI(5)*UYY) +
     *         HALF*TRS*(SI(7)*UXXY + TWO*SI(8)*UXYY + SI(9)*UYYY) )
         VZX = -TCR*( SI(2)*UZ - TR*(SI(4)*UXZ + SI(5)*UYZ) +
     *         HALF*TRS*(SI(7)*UXXZ + TWO*SI(8)*UXYZ + SI(9)*UYYZ) )

         VXY = -TCR*( SI(3)*UX - TR*(SI(5)*UXX + SI(6)*UXY) +
     *         HALF*TRS*(SI(8)*UXXX + TWO*SI(9)*UXXY + SI(10)*UXYY) )
         VYY = -TCR*( SI(3)*UY - TR*(SI(5)*UXY + SI(6)*UYY) +
     *         HALF*TRS*(SI(8)*UXXY + TWO*SI(9)*UXYY + SI(10)*UYYY) )
         VZY = -TCR*( SI(3)*UZ - TR*(SI(5)*UXZ + SI(6)*UYZ) +
     *         HALF*TRS*(SI(8)*UXXZ + TWO*SI(9)*UXYZ + SI(10)*UYYZ) )


      ELSE

C----------------------------------------------------------------------
C        Far Field Expansion (point source)
C----------------------------------------------------------------------
         ROX = XFP - XC(1)
         ROY = YFP - XC(2)
         ROZ = ZFP - XC(3)
         RDOTI = THREE*(ROX*DIRP(1,1)+ROY*DIRP(1,2)+ROZ*DIRP(1,3))/RS
         RDOTJ = THREE*(ROX*DIRP(2,1)+ROY*DIRP(2,2)+ROZ*DIRP(2,3))/RS
         RDOTN = THREE*(ROX*DIRP(3,1)+ROY*DIRP(3,2)+ROZ*DIRP(3,3))/RS
         DX = DIRP(3,1) - ROX*RDOTN
         DY = DIRP(3,2) - ROY*RDOTN
         DZ = DIRP(3,3) - ROZ*RDOTN
         DIX = DIRP(1,1) - ROX*RDOTI
         DIY = DIRP(1,2) - ROY*RDOTI
         DIZ = DIRP(1,3) - ROZ*RDOTI
         DJX = DIRP(2,1) - ROX*RDOTJ
         DJY = DIRP(2,2) - ROY*RDOTJ
         DJZ = DIRP(2,3) - ROZ*RDOTJ

         R3 = RS*R
         RCUB = ONE/R3
         TS = TLEN*TLEN
         TC = TLEN*TS
         TQ = TS*TS
         TQR = TQ*RCUB

         VOO = TS*SI(1)*RCUB
         VXO = VOO*ROX
         VYO = VOO*ROY
         VZO = VOO*ROZ
         VPP = -TQR*SI(4)
         VXP = VPP*DX
         VYP = VPP*DY
         VZP = VPP*DZ
         VQQ = -TQR*SI(5)
         VXQ = VQQ*DX
         VYQ = VQQ*DY
         VZQ = VQQ*DZ
         VRR = -TQR*SI(6)
         VXR = VRR*DX
         VYR = VRR*DY
         VZR = VRR*DZ
         VX1 = TC*SI(2)*RCUB
         VX2 = TQR*SI(4)
         VX3 = TQR*SI(5)
         VY1 = TC*SI(3)*RCUB
         VY2 = VX3
         VY3 = TQR*SI(6)
         VXX = VX1*ROX - VX2*DIX - VX3*DJX
         VYX = VX1*ROY - VX2*DIY - VX3*DJY
         VZX = VX1*ROZ - VX2*DIZ - VX3*DJZ
         VXY = VY1*ROX - VY2*DIX - VY3*DJX
         VYY = VY1*ROY - VY2*DIY - VY3*DJY
         VZY = VY1*ROZ - VY2*DIZ - VY3*DJZ

      END IF

      VX = VXO + P*VXP + TWO*Q*VXQ + RR*VXR
      VY = VYO + P*VYP + TWO*Q*VYQ + RR*VYR
      VZ = VZO + P*VZP + TWO*Q*VZQ + RR*VZR

C..... Transfer the Velocities from the panel coordinate system to the
C      global coordinate system if not in far field
      IF (RAT.LT.5.0 .and. ifo.lt.2) THEN
         VXT = VX*DIRP(1,1) + VY*DIRP(2,1) + VZ*DIRP(3,1)
         VYT = VX*DIRP(1,2) + VY*DIRP(2,2) + VZ*DIRP(3,2)
         VZT = VX*DIRP(1,3) + VY*DIRP(2,3) + VZ*DIRP(3,3)
         VX=VXT
         VY=VYT
         VZ=VZT
         VXT = VXX*DIRP(1,1) + VYX*DIRP(2,1) + VZX*DIRP(3,1)
         VYT = VXX*DIRP(1,2) + VYX*DIRP(2,2) + VZX*DIRP(3,2)
         VZT = VXX*DIRP(1,3) + VYX*DIRP(2,3) + VZX*DIRP(3,3)
         VXX=VXT
         VYX=VYT
         VZX=VZT
         VXT = VXY*DIRP(1,1) + VYY*DIRP(2,1) + VZY*DIRP(3,1)
         VYT = VXY*DIRP(1,2) + VYY*DIRP(2,2) + VZY*DIRP(3,2)
         VZT = VXY*DIRP(1,3) + VYY*DIRP(2,3) + VZY*DIRP(3,3)
         VXY=VXT
         VYY=VYT
         VZY=VZT
      END IF


      GOTO 999
 100  CONTINUE
      IF (RAT.LT.0.1 .and. ifo.lt.2) THEN
       if (ifo.eq.1) print*,'big trouble in fo panel'
         XFPP = XFP + DIRP(3,1)*0.00001*TLEN
         YFPP = YFP + DIRP(3,2)*0.00001*TLEN
         ZFPP = ZFP + DIRP(3,3)*0.00001*TLEN
         RS = (XFPP-XC(1))**2 + (YFPP-XC(2))**2 + (ZFPP-XC(3))**2
         R = SQRT(RS)
         RAT = R/TLEN
C........ Convert to panel coordinates
         XP = XFPP-XC(1)
         YP = YFPP-XC(2)
         ZP = ZFPP-XC(3)
         X = DIRP(1,1)*XP + DIRP(1,2)*YP + DIRP(1,3)*ZP
         Y = DIRP(2,1)*XP + DIRP(2,2)*YP + DIRP(2,3)*ZP
         Z = DIRP(3,1)*XP + DIRP(3,2)*YP + DIRP(3,3)*ZP
      END IF
      IF (RAT.LT.2.5 .and. ifo.eq.0) THEN
C----------------------------------------------------------------------
C        Near Field (Exact closed form integral solutions)
C         Maple generated fortran code
C----------------------------------------------------------------------
         CALL NEARFLDA(X,Y,Z,XI,ETA,RAT,TLEN,DM32,DM41,
     *                 B32,B41,D12,D23,D34,D41,P,Q,RR,VX,VY,VZ,
     *                 VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                 VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                 VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                 VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                 VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      ELSE IF (RAT.LT.5.0 .and. ifo.lt.2) THEN
C-------------------------------------------------------------------
C        Multipole expansion for intermediate field
C-------------------------------------------------------------------
c       print*,'intermediate field'
        CALL INTERFLD(X,Y,Z,TLEN,P,Q,RR,SI,VX,VY,VZ,
     *                    VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                    VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                    VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                    VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                    VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      ELSE
c        print*,'far field',xfp,yfp,zfp,(xc(jjj),jjj=1,3)
C----------------------------------------------------------------------
C        Far Field Expansion (point source)
C----------------------------------------------------------------------
         CALL FARFLD(XFP,YFP,ZFP,TLEN,P,Q,RR,SI,XC,DIRP,VX,VY,VZ,
     *                    VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                    VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                    VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                    VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                    VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      END IF

      IF (LEVEL.EQ.3) CALL FFDZ(XC,SI,TLEN,XFP,YFP,ZFP,P,Q,RR,
     *                          VXDXZ,VXDYZ,VXDZZ,VYDYZ,VYDZZ,
     *                          VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     *                          VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,DIRP)

 200  CONTINUE
C..... Transfer the Velocities from the panel coordinate system to the
C      global coordinate system if not in far field

      IF (RAT.LT.5.0 .and. ifo.lt.2) THEN
         VXT = VX*DIRP(1,1) + VY*DIRP(2,1) + VZ*DIRP(3,1)
         VYT = VX*DIRP(1,2) + VY*DIRP(2,2) + VZ*DIRP(3,2)
         VZT = VX*DIRP(1,3) + VY*DIRP(2,3) + VZ*DIRP(3,3)
         VX=VXT
         VY=VYT
         VZ=VZT
         VXT = VXX*DIRP(1,1) + VYX*DIRP(2,1) + VZX*DIRP(3,1)
         VYT = VXX*DIRP(1,2) + VYX*DIRP(2,2) + VZX*DIRP(3,2)
         VZT = VXX*DIRP(1,3) + VYX*DIRP(2,3) + VZX*DIRP(3,3)
         VXX=VXT
         VYX=VYT
         VZX=VZT
         VXT = VXY*DIRP(1,1) + VYY*DIRP(2,1) + VZY*DIRP(3,1)
         VYT = VXY*DIRP(1,2) + VYY*DIRP(2,2) + VZY*DIRP(3,2)
         VZT = VXY*DIRP(1,3) + VYY*DIRP(2,3) + VZY*DIRP(3,3)
         VXY=VXT
         VYY=VYT
         VZY=VZT

C........ Rotate the derivative of the velocity field from the panel
C         coordinate system, into the global coordinate system.
C                                              T
C               [DV'DX'] = [DIRP]*[DVDX]*[DIRP]

C........ First multiply [DIRP]*[DVDX]
         VXDXP = VXDX*DIRP(1,1) + VYDX*DIRP(2,1) + VZDX*DIRP(3,1)
         VXDYP = VXDY*DIRP(1,1) + VYDY*DIRP(2,1) + VZDY*DIRP(3,1)
         VXDZP = VXDZ*DIRP(1,1) + VYDZ*DIRP(2,1) + VZDZ*DIRP(3,1)
         VYDXP = VXDX*DIRP(1,2) + VYDX*DIRP(2,2) + VZDX*DIRP(3,2)
         VYDYP = VXDY*DIRP(1,2) + VYDY*DIRP(2,2) + VZDY*DIRP(3,2)
         VYDZP = VXDZ*DIRP(1,2) + VYDZ*DIRP(2,2) + VZDZ*DIRP(3,2)
         VZDXP = VXDX*DIRP(1,3) + VYDX*DIRP(2,3) + VZDX*DIRP(3,3)
         VZDYP = VXDY*DIRP(1,3) + VYDY*DIRP(2,3) + VZDY*DIRP(3,3)
         VZDZP = VXDZ*DIRP(1,3) + VYDZ*DIRP(2,3) + VZDZ*DIRP(3,3)

C                                    T
C........ Then multiply [DVDX]*[DIRP]
         VXDX = VXDXP*DIRP(1,1) + VXDYP*DIRP(2,1) + VXDZP*DIRP(3,1)
         VXDY = VXDXP*DIRP(1,2) + VXDYP*DIRP(2,2) + VXDZP*DIRP(3,2)
         VXDZ = VXDXP*DIRP(1,3) + VXDYP*DIRP(2,3) + VXDZP*DIRP(3,3)
         VYDX = VYDXP*DIRP(1,1) + VYDYP*DIRP(2,1) + VYDZP*DIRP(3,1)
         VYDY = VYDXP*DIRP(1,2) + VYDYP*DIRP(2,2) + VYDZP*DIRP(3,2)
         VYDZ = VYDXP*DIRP(1,3) + VYDYP*DIRP(2,3) + VYDZP*DIRP(3,3)
         VZDX = VZDXP*DIRP(1,1) + VZDYP*DIRP(2,1) + VZDZP*DIRP(3,1)
         VZDY = VZDXP*DIRP(1,2) + VZDYP*DIRP(2,2) + VZDZP*DIRP(3,2)
         VZDZ = VZDXP*DIRP(1,3) + VZDYP*DIRP(2,3) + VZDZP*DIRP(3,3)

C........ Rotate the derivative of the velocity field from the panel
C         coordinate system, into the global coordinate system.
C                                              T
C               [DVX'DX'] = [DIRP]*[DVXDX]*[DIRP]

C........ First multiply [DIRP]*[DVDX]
         VXDXP = VXXDX*DIRP(1,1) + VYXDX*DIRP(2,1) + VZXDX*DIRP(3,1)
         VXDYP = VXXDY*DIRP(1,1) + VYXDY*DIRP(2,1) + VZXDY*DIRP(3,1)
         VXDZP = VXXDZ*DIRP(1,1) + VYXDZ*DIRP(2,1) + VZXDZ*DIRP(3,1)
         VYDXP = VXXDX*DIRP(1,2) + VYXDX*DIRP(2,2) + VZXDX*DIRP(3,2)
         VYDYP = VXXDY*DIRP(1,2) + VYXDY*DIRP(2,2) + VZXDY*DIRP(3,2)
         VYDZP = VXXDZ*DIRP(1,2) + VYXDZ*DIRP(2,2) + VZXDZ*DIRP(3,2)
         VZDXP = VXXDX*DIRP(1,3) + VYXDX*DIRP(2,3) + VZXDX*DIRP(3,3)
         VZDYP = VXXDY*DIRP(1,3) + VYXDY*DIRP(2,3) + VZXDY*DIRP(3,3)
         VZDZP = VXXDZ*DIRP(1,3) + VYXDZ*DIRP(2,3) + VZXDZ*DIRP(3,3)

C                                    T
C........ Then multiply [DVDX]*[DIRP]
         VXXDX = VXDXP*DIRP(1,1) + VXDYP*DIRP(2,1) + VXDZP*DIRP(3,1)
         VXXDY = VXDXP*DIRP(1,2) + VXDYP*DIRP(2,2) + VXDZP*DIRP(3,2)
         VXXDZ = VXDXP*DIRP(1,3) + VXDYP*DIRP(2,3) + VXDZP*DIRP(3,3)
         VYXDX = VYDXP*DIRP(1,1) + VYDYP*DIRP(2,1) + VYDZP*DIRP(3,1)
         VYXDY = VYDXP*DIRP(1,2) + VYDYP*DIRP(2,2) + VYDZP*DIRP(3,2)
         VYXDZ = VYDXP*DIRP(1,3) + VYDYP*DIRP(2,3) + VYDZP*DIRP(3,3)
         VZXDX = VZDXP*DIRP(1,1) + VZDYP*DIRP(2,1) + VZDZP*DIRP(3,1)
         VZXDY = VZDXP*DIRP(1,2) + VZDYP*DIRP(2,2) + VZDZP*DIRP(3,2)
         VZXDZ = VZDXP*DIRP(1,3) + VZDYP*DIRP(2,3) + VZDZP*DIRP(3,3)

C........ Rotate the derivative of the velocity field from the panel
C         coordinate system, into the global coordinate system.
C                                              T
C               [DVY'DX'] = [DIRP]*[DVYDX]*[DIRP]

C........ First multiply [DIRP]*[DVDX]
         VXDXP = VXYDX*DIRP(1,1) + VYYDX*DIRP(2,1) + VZYDX*DIRP(3,1)
         VXDYP = VXYDY*DIRP(1,1) + VYYDY*DIRP(2,1) + VZYDY*DIRP(3,1)
         VXDZP = VXYDZ*DIRP(1,1) + VYYDZ*DIRP(2,1) + VZYDZ*DIRP(3,1)
         VYDXP = VXYDX*DIRP(1,2) + VYYDX*DIRP(2,2) + VZYDX*DIRP(3,2)
         VYDYP = VXYDY*DIRP(1,2) + VYYDY*DIRP(2,2) + VZYDY*DIRP(3,2)
         VYDZP = VXYDZ*DIRP(1,2) + VYYDZ*DIRP(2,2) + VZYDZ*DIRP(3,2)
         VZDXP = VXYDX*DIRP(1,3) + VYYDX*DIRP(2,3) + VZYDX*DIRP(3,3)
         VZDYP = VXYDY*DIRP(1,3) + VYYDY*DIRP(2,3) + VZYDY*DIRP(3,3)
         VZDZP = VXYDZ*DIRP(1,3) + VYYDZ*DIRP(2,3) + VZYDZ*DIRP(3,3)

C                                    T
C........ Then multiply [DVDX]*[DIRP]
         VXYDX = VXDXP*DIRP(1,1) + VXDYP*DIRP(2,1) + VXDZP*DIRP(3,1)
         VXYDY = VXDXP*DIRP(1,2) + VXDYP*DIRP(2,2) + VXDZP*DIRP(3,2)
         VXYDZ = VXDXP*DIRP(1,3) + VXDYP*DIRP(2,3) + VXDZP*DIRP(3,3)
         VYYDX = VYDXP*DIRP(1,1) + VYDYP*DIRP(2,1) + VYDZP*DIRP(3,1)
         VYYDY = VYDXP*DIRP(1,2) + VYDYP*DIRP(2,2) + VYDZP*DIRP(3,2)
         VYYDZ = VYDXP*DIRP(1,3) + VYDYP*DIRP(2,3) + VYDZP*DIRP(3,3)
         VZYDX = VZDXP*DIRP(1,1) + VZDYP*DIRP(2,1) + VZDZP*DIRP(3,1)
         VZYDY = VZDXP*DIRP(1,2) + VZDYP*DIRP(2,2) + VZDZP*DIRP(3,2)
         VZYDZ = VZDXP*DIRP(1,3) + VZDYP*DIRP(2,3) + VZDZP*DIRP(3,3)

      END IF

 999  CONTINUE
      END

C***********************************************************************
C
C     SUBROUTINE NEARFLDA()
C
C     Computes the induced velocity and the derivatives of the velocity
C     from a curved panel with a linear source distribution, for the
C     case where the panel has four sides
C
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Programmed by: Michael Hughes, July 1994
C
C***********************************************************************

      SUBROUTINE NEARFLDA(X,Y,Z,XI,ETA,RAT,TLEN,DM32,DM41,
     *                    B32,B41,D12,D23,D34,D41,P,Q,RR,VX,VY,VZ,
     *                    VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                    VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                    VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                    VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                    VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )


      DIMENSION XI(4),ETA(4)
      DATA TWOPI,TOL,TOLT,ZERO/6.283185308E+00,0.000001,0.001,0.0E00/

      if (abs(z/tlen) .lt. tol) z = sign(min(tol,(tol*tlen)),z)
      t1 = DM41**2
      t3 = 1/(0.1E1+t1)
      t4 = sqrt(t3)
      t5 = X**2
      t7 = XI(4)**2
      t8 = Y**2
      t10 = ETA(4)**2
      t11 = Z**2
      t12 = t5-2*X*XI(4)+t7+t8-2*Y*ETA(4)+t10+t11
      t13 = sqrt(t12)
      t15 = XI(1)**2
      t17 = ETA(1)**2
      t18 = t5-2*X*XI(1)+t15+t8-2*Y*ETA(1)+t17+t11
      t19 = sqrt(t18)
      t20 = t13+t19-D41
      t21 = t13+t19+D41
      t22 = 1/t21
      t24 = alog(t20*t22)
      t25 = t4*t24
      t26 = DM32**2
      t28 = 1/(0.1E1+t26)
      t29 = sqrt(t28)
      t31 = XI(2)**2
      t33 = ETA(2)**2
      t34 = t5-2*X*XI(2)+t31+t8-2*Y*ETA(2)+t33+t11
      t35 = sqrt(t34)
      t37 = XI(3)**2
      t39 = ETA(3)**2
      t40 = t5-2*X*XI(3)+t37+t8-2*Y*ETA(3)+t39+t11
      t41 = sqrt(t40)
      t42 = t35+t41-D23
      t43 = t35+t41+D23
      t44 = 1/t43
      t46 = alog(t42*t44)
      t47 = t29*t46
      t48 = DM32*t28
      t49 = X-XI(2)
      t50 = 1/t35
      t51 = t49*t50
      t52 = X-XI(3)
      t53 = 1/t41
      t54 = t52*t53
      t56 = t48*(t51-t54)
      t57 = t28*t29
      t58 = t57*t46
      t60 = X-DM32*Y-B32
      t61 = t60*t28
      t62 = t61*t29
      t63 = t35+t41
      t64 = t63**2
      t65 = D23**2
      t66 = t64-t65
      t67 = 1/t66
      t68 = D23*t67
      t69 = t51+t54
      t70 = t68*t69
      t71 = t62*t70
      t72 = DM41*t3
      t73 = X-XI(4)
      t74 = 1/t13
      t75 = t73*t74
      t76 = X-XI(1)
      t77 = 1/t19
      t78 = t76*t77
      t80 = t72*(t75-t78)
      t81 = t3*t4
      t82 = t81*t24
      t84 = X-DM41*Y-B41
      t85 = t84*t3
      t86 = t85*t4
      t87 = t13+t19
      t88 = t87**2
      t89 = D41**2
      t90 = t88-t89
      t91 = 1/t90
      t92 = D41*t91
      t93 = t75+t78
      t94 = t92*t93
      t95 = t86*t94
      t98 = t52**2
      t99 = Y-ETA(3)
      t100 = t99**2
      t101 = t98+t100+t11
      t102 = t101*t99
      t107 = DM32*(t11+t100)-t52*t99
      t108 = t107*t52
      t110 = t102*t53+t108*t53
      t111 = Z*t110
      t113 = t107**2
      t114 = t11*t101+t113
      t115 = 1/t114
      t116 = t111*t115
      t117 = t49**2
      t118 = Y-ETA(2)
      t119 = t118**2
      t120 = t117+t119+t11
      t121 = t120*t118
      t126 = DM32*(t11+t119)-t49*t118
      t127 = t126*t49
      t129 = t121*t50+t127*t50
      t130 = Z*t129
      t132 = t126**2
      t133 = t11*t120+t132
      t134 = 1/t133
      t135 = t130*t134
      t136 = t76**2
      t137 = Y-ETA(1)
      t138 = t137**2
      t139 = t136+t138+t11
      t140 = t139*t137
      t145 = DM41*(t11+t138)-t76*t137
      t146 = t145*t76
      t148 = t140*t77+t146*t77
      t149 = Z*t148
      t151 = t145**2
      t152 = t11*t139+t151
      t153 = 1/t152
      t154 = t149*t153
      t155 = t73**2
      t156 = Y-ETA(4)
      t157 = t156**2
      t158 = t155+t157+t11
      t159 = t158*t156
      t164 = DM41*(t11+t157)-t73*t156
      t165 = t164*t73
      t167 = t159*t74+t165*t74
      t168 = Z*t167
      t170 = t164**2
      t171 = t11*t158+t170
      t172 = 1/t171
      t173 = t168*t172
      t174 = -t116+t135-t154+t173
      t176 = t4*D41
      t177 = t91*t93
      t178 = t176*t177
      t179 = t29*D23
      t180 = t67*t69
      t181 = t179*t180
      t183 = Z*(0.2e1*t178-0.2e1*t181)
      t184 = 1/Z
      t185 = t107*t184
      t188 = t126*t184
      t191 = t145*t184
      t194 = t164*t184

      H32 = DM32*t99 - t52
      H41 = DM41*t137 - t76
      IF ( ABS(Z/TLEN).LT. TOLT ) THEN
         IF ( (RAT.LT.0.5) .AND. (H32*H41.LT.0.0)
     *         .AND. (t137*t99.LT.0) ) THEN
            VZO = TWOPI
         ELSE
            VZO = ZERO
         END IF
      ELSE
         t187 = atan(t185*t53)
         t190 = atan(t188*t50)
         t193 = atan(t191*t77)
         t196 = atan(t194*t74)
         VZO = t187-t190+t193-t196
      END IF

      t197 = X*t174-t183+ vzo
      t200 = 0.1e1*t25-t47
      t201 = Z*t200
      t205 = (t54-t51)*t28
      t206 = t48*t47
      t207 = t48*t29
      t208 = t60*D23
      t210 = t207*t208*t180
      t212 = (t78-t75)*t3
      t213 = t72*t25
      t214 = t72*t4
      t215 = t84*D41
      t217 = t214*t215*t177
      t221 = t41+t13
      t222 = t221**2
      t223 = D34**2
      t224 = t222-t223
      t226 = D34/t224
      t227 = t54+t75
      t228 = t226*t227
      t229 = t19+t35
      t230 = t229**2
      t231 = D12**2
      t232 = t230-t231
      t234 = D12/t232
      t235 = t78+t51
      t236 = t234*t235
      t237 = DM32*t29
      t238 = t237*t70
      t239 = DM41*t4
      t240 = t239*t94
      t242 = Z*(0.2e1*t228-0.2e1*t236+0.2e1*t238-0.2e1*t240)
      t243 = Y*t174-t242
      t246 = X*Y
      t248 = t41+t13-D34
      t249 = t41+t13+D34
      t250 = 1/t249
      IF (D34.LT.TOL) THEN
         t252=ZERO
      ELSE
         t252 = alog(t248*t250)
      END IF
      t253 = t19+t35-D12
      t254 = t19+t35+D12
      t255 = 1/t254
      IF (D12.LT.TOL) THEN
         t257=ZERO
      ELSE
         t257 = alog(t253*t255)
      END IF
      t258 = t237*t46
      t259 = t239*t24
      t260 = t252-t257+0.1e1*t258-t259
      t261 = Z*t260
      t267 = t8+t11
      t272 = t118*t50
      t273 = t99*t53
      t275 = t48*(t272-t273)
      t276 = t272+t273
      t277 = t68*t276
      t278 = t62*t277
      t279 = t156*t74
      t280 = t137*t77
      t282 = t72*(t279-t280)
      t283 = t279+t280
      t284 = t92*t283
      t285 = t86*t284
      t288 = DM32*t99
      t290 = 0.2e1*t288*t53-t54
      t292 = t107*t99
      t294 = t290*t101-t292*t53
      t295 = Z*t294
      t296 = t295*t115
      t297 = DM32*t118
      t299 = 0.2e1*t297*t50-t51
      t301 = t126*t118
      t303 = t299*t120-t301*t50
      t304 = Z*t303
      t305 = t304*t134
      t306 = DM41*t137
      t308 = 0.2e1*t306*t77-t78
      t310 = t145*t137
      t312 = t308*t139-t310*t77
      t313 = Z*t312
      t314 = t313*t153
      t315 = DM41*t156
      t317 = 0.2e1*t315*t74-t75
      t319 = t164*t156
      t321 = t317*t158-t319*t74
      t322 = Z*t321
      t323 = t322*t172
      t324 = t296-t305+t314-t323
      t326 = t91*t283
      t327 = t176*t326
      t328 = t67*t276
      t329 = t179*t328
      t331 = Z*(0.2e1*t327-0.2e1*t329)
      t332 = X*t324-t331
      t338 = (t273-t272)*t28
      t339 = t26*t28
      t340 = t339*t47
      t342 = t207*t208*t328
      t344 = (t280-t279)*t3
      t345 = t1*t3
      t346 = t345*t25
      t348 = t214*t215*t326
      t352 = t273+t279
      t353 = t226*t352
      t354 = t280+t272
      t355 = t234*t354
      t356 = t237*t277
      t357 = t239*t284
      t359 = Z*(0.2e1*t353-0.2e1*t355+0.2e1*t356-0.2e1*t357)
      t360 = Y*t324-t359+vzo
      t373 = Z*t50
      t374 = Z*t53
      t376 = t48*(t373-t374)
      t377 = t373+t374
      t378 = t68*t377
      t379 = t62*t378
      t380 = Z*t74
      t381 = Z*t77
      t383 = t72*(t380-t381)
      t384 = t380+t381
      t385 = t92*t384
      t386 = t86*t385
      t389 = DM32*t11
      t402 = DM41*t11
      t415 = (0.2e1*t389*t41-t107*(t41+t11*t53))*t115-(0.2e1*t389*t35 -
     #        t126*(t35+t11*t50))*t134+(0.2e1*t402*t19-t145*(t19+t11*
     #        t77))*t153-(0.2e1*t402*t13-t164*(t13+t11*t74))*t172
      t417 = t91*t384
      t419 = t67*t377
      t423 = X*t415-Z*(0.2e1*t176*t417-0.2e1*t179*t419)-t25+0.1e1*t47
      t426 = t137*t257
      t428 = (X-XI(2)-t297)*t29
      t429 = t428*t46
      t430 = t99*t252
      t432 = (X-XI(4)-t315)*t4
      t433 = t432*t24
      t434 = vzo
      t435 = Z*t434
      t438 = DM32*(t35-t41)*t28
      t441 = DM41*(t13-t19)*t3
      t443 = t57*t60*t46
      t445 = t81*t84*t24
      t446 = -Z*(-vzo-t376-0.2e1*t379-t383+0.2e1*t386)-0.2e1*X*t423 +
     #       t5*t415-t426-t429+t430+0.1e1*t433+t435+0.1e1*t438 +
     #       0.1e1*t441+0.1e1*t443-t445
      t449 = (t374-t373)*t28
      t453 = (t381-t380)*t3
      t459 = t374+t380
      t461 = t381+t373
      t467 = Y*t415-Z*(0.2e1*t226*t459-0.2e1*t234*t461+0.2e1*t237*t378
     #     -0.2e1*t239*t385)-t252+t257-t258+0.1e1*t259
      t472 = (t41-t35)*t28
      t474 = (t19-t13)*t3
      t475 = t29*t60
      t477 = t48*t475*t46
      t478 = t4*t84
      t480 = t72*t478*t24
      t490 = 2*X-2*XI(4)
      t491 = t74*t490
      t492 = 2*X-2*XI(1)
      t493 = t77*t492
      t494 = t491/2+t493/2
      t496 = t21**2
      t498 = t20/t496
      t500 = t494*t22-t498*t494
      t502 = 1/t20
      t503 = t502*t21
      t504 = t4*t500*t503
      t505 = 2*X-2*XI(2)
      t506 = t50*t505
      t507 = 2*X-2*XI(3)
      t508 = t53*t507
      t509 = t506/2+t508/2
      t511 = t43**2
      t513 = t42/t511
      t515 = t509*t44-t513*t509
      t517 = 1/t42
      t518 = t517*t43
      t519 = t29*t515*t518
      t520 = t35**2
      t522 = 1/t520/t35
      t523 = t49*t522
      t524 = t523*t505
      t525 = t41**2
      t527 = 1/t525/t41
      t528 = t52*t527
      t529 = t528*t507
      t531 = t48*(t50-t524/2-t53+t529/2)
      t533 = t515*t517*t43
      t534 = t57*t533
      t535 = t57*t70
      t536 = t61*t179
      t537 = t66**2
      t538 = 1/t537
      t539 = t538*t69
      t541 = t539*t63*t509
      t542 = t536*t541
      t543 = t50-t524/2+t53-t529/2
      t544 = t68*t543
      t545 = t62*t544
      t546 = t13**2
      t548 = 1/t546/t13
      t549 = t73*t548
      t550 = t549*t490
      t551 = t19**2
      t553 = 1/t551/t19
      t554 = t76*t553
      t555 = t554*t492
      t557 = t72*(t74-t550/2-t77+t555/2)
      t559 = t500*t502*t21
      t560 = t81*t559
      t561 = t81*t94
      t562 = t85*t176
      t563 = t90**2
      t564 = 1/t563
      t565 = t564*t93
      t567 = t565*t87*t494
      t568 = t562*t567
      t569 = t74-t550/2+t77-t555/2
      t570 = t92*t569
      t571 = t86*t570
      t572 = -t504+0.1e1*t519-t531-t534-0.2e1*t535+0.4e1*t542-0.2e1*t545
     #     -t557+0.1e1*t560+0.2e1*t561-0.4e1*t568+0.2e1*t571
      t576 = t527*t507
      t578 = -Y+ETA(3)
      t581 = t107*t53
      t586 = t114**2
      t587 = 1/t586
      t595 = t522*t505
      t597 = -Y+ETA(2)
      t600 = t126*t50
      t605 = t133**2
      t606 = 1/t605
      t614 = t553*t492
      t616 = -Y+ETA(1)
      t619 = t145*t77
      t624 = t152**2
      t625 = 1/t624
      t633 = t548*t490
      t635 = -Y+ETA(4)
      t638 = t164*t74
      t643 = t171**2
      t644 = 1/t643
      t650 = -Z*(t507*t99*t53-t102*t576/2+t578*t52*t53+t581-t108*t576/2)
     #     *t115+t111*t587*(t11*t507+2*t107*t578)+Z*(t505*t118*t50-t121
     #     *t595/2+t597*t49*t50+t600-t127*t595/2)*t134-t130*t606*(t11
     #     *t505+2*t126*t597)-Z*(t492*t137*t77-t140*t614/2+t616*t76*t77
     #     +t619-t146*t614/2)*t153+t149*t625*(t11*t492+2*t145*t616)
     #     +Z*(t490*t156*t74-t159*t633/2+t635*t73*t74+t638-t165*t633/2)
     #     *t172-t168*t644*(t11*t490+2*t164*t635)
      t652 = t176*t564
      t653 = t93*t87
      t654 = t653*t494
      t656 = t91*t569
      t658 = t179*t538
      t659 = t69*t63
      t660 = t659*t509
      t662 = t67*t543
      t670 = 1/t11
      t675 = 1/(1+t113*t670/t40)
      t676 = (t578*t184*t53-t185*t576/2)*t675
      t685 = 1/(1+t132*t670/t34)
      t686 = (t597*t184*t50-t188*t595/2)*t685
      t695 = 1/(1+t151*t670/t18)
      t696 = (t616*t184*t77-t191*t614/2)*t695
      t705 = 1/(1+t170*t670/t12)
      t706 = (t635*t184*t74-t194*t633/2)*t705
      t707 = -t116+t135-t154+t173+X*t650-Z*(-0.4e1*t652*t654+0.2e1*t176*
     #        t656+0.4e1*t658*t660-0.2e1*t179*t662)+t676-t686+t696-t706
      t710 = 0.1e1*t504-t519
      t717 = t207*t70
      t718 = t48*t475
      t719 = D23*t538
      t727 = t214*t94
      t728 = t72*t478
      t729 = D41*t564
      t737 = t224**2
      t739 = D34/t737
      t740 = t227*t221
      t741 = t508/2+t491/2
      t746 = t232**2
      t748 = D12/t746
      t749 = t235*t229
      t750 = t493/2+t506/2
      t755 = t237*D23
      t758 = t239*D41
      t763 = Y*t650-Z*(-0.4e1*t739*t740*t741+0.2e1*t226*(t53-t529/2+t74-
     #t550/2)+0.4e1*t748*t749*t750-0.2e1*t234*(t77-t555/2+t50-t524/2)-
     #0.4e1*t755*t541+0.2e1*t237*t544+0.4e1*t758*t567-0.2e1*t239*t570)
      t768 = t249**2
      t770 = t248/t768
      t772 = t741*t250-t770*t741
      t773 = 1/t248
      t777 = t254**2
      t779 = t253/t777
      t781 = t750*t255-t779*t750
      t782 = 1/t253
      t798 = 2*Y-2*ETA(4)
      t799 = t74*t798
      t800 = 2*Y-2*ETA(1)
      t801 = t77*t800
      t802 = t799/2+t801/2
      t805 = t802*t22-t498*t802
      t807 = t4*t805*t503
      t808 = 2*Y-2*ETA(2)
      t809 = t50*t808
      t810 = 2*Y-2*ETA(3)
      t811 = t53*t810
      t812 = t809/2+t811/2
      t815 = t812*t44-t513*t812
      t817 = t29*t815*t518
      t818 = t523*t808
      t819 = t528*t810
      t821 = t48*(-t818/2+t819/2)
      t823 = t815*t517*t43
      t824 = t57*t823
      t825 = t63*t812
      t826 = t539*t825
      t827 = t536*t826
      t828 = -t818/2-t819/2
      t829 = t68*t828
      t830 = t62*t829
      t831 = t549*t798
      t832 = t554*t800
      t834 = t72*(-t831/2+t832/2)
      t836 = t805*t502*t21
      t837 = t81*t836
      t838 = t87*t802
      t839 = t565*t838
      t840 = t562*t839
      t841 = -t831/2-t832/2
      t842 = t92*t841
      t843 = t86*t842
      t844 = -t807+0.1e1*t817-t821-t824+0.2e1*t717+0.4e1*t827-0.2e1*t830
     #-t834+0.1e1*t837-0.2e1*t727-0.4e1*t840+0.2e1*t843
      t849 = t527*t810
      t852 = DM32*t810-X+XI(3)
      t862 = t587*(t11*t810+2*t107*t852)
      t867 = t522*t808
      t870 = DM32*t808-X+XI(2)
      t880 = t606*(t11*t808+2*t126*t870)
      t885 = t553*t800
      t888 = DM41*t800-X+XI(1)
      t898 = t625*(t11*t800+2*t145*t888)
      t903 = t548*t798
      t906 = DM41*t798-X+XI(4)
      t916 = t644*(t11*t798+2*t164*t906)
      t918 = -Z*(t810*t99*t53+t101*t53-t102*t849/2+t852*t52*t53-t108
     #     *t849/2)*t115+t111*t862+Z*(t808*t118*t50+t120*t50-t121
     #     *t867/2+t870*t49*t50-t127*t867/2)*t134-t130*t880-Z*(t800
     #     *t137*t77+t139*t77-t140*t885/2+t888*t76*t77-t146*t885/2)
     #     *t153+t149*t898+Z*(t798*t156*t74+t158*t74-t159*t903/2+t906
     #     *t73*t74-t165*t903/2)*t172-t168*t916
      t920 = t653*t802
      t922 = t91*t841
      t924 = t659*t812
      t926 = t67*t828
      t934 = (t852*t184*t53-t185*t849/2)*t675
      t939 = (t870*t184*t50-t188*t867/2)*t685
      t944 = (t888*t184*t77-t191*t885/2)*t695
      t949 = (t906*t184*t74-t194*t903/2)*t705
      t950 = X*t918-Z*(-0.4e1*t652*t920+0.2e1*t176*t922 +
     #       0.4e1*t658*t924-0.2e1*t179*t926)+t934-t939+t944-t949
      t953 = 0.1e1*t807-t817
      t954 = Z*t953
      t959 = t207*t823
      t960 = t339*t29
      t968 = t214*t836
      t969 = t345*t4
      t978 = t811/2+t799/2
      t983 = t801/2+t809/2
      t994 = -t116+t135-t154+t173+Y*t918-Z*(-0.4e1*t739*t740*t978 +
     #       0.2e1*t226*(-t819/2-t831/2)+0.4e1*t748*t749*t983 -
     #       0.2e1*t234*(-t832/2-t818/2)-0.4e1*t755*t826 +
     #       0.2e1*t237*t829+0.4e1*t758*t839-0.2e1*t239*t842)
      t1000 = t978*t250-t770*t978
      t1002 = t1000*t773*t249
      t1005 = t983*t255-t779*t983
      t1007 = t1005*t782*t254
      t1008 = t237*t823
      t1009 = t239*t836
      t1010 = t1002-t1007+0.1e1*t1008-t1009
      t1011 = Z*t1010
      t1023 = t384*t22-t498*t384
      t1025 = t4*t1023*t503
      t1028 = t377*t44-t513*t377
      t1030 = t29*t1028*t518
      t1031 = t523*Z
      t1032 = t528*Z
      t1034 = t48*(-t1031+t1032)
      t1036 = t1028*t517*t43
      t1037 = t57*t1036
      t1038 = t63*t377
      t1039 = t539*t1038
      t1040 = t536*t1039
      t1041 = -t1031-t1032
      t1042 = t68*t1041
      t1043 = t62*t1042
      t1044 = t549*Z
      t1045 = t554*Z
      t1047 = t72*(-t1044+t1045)
      t1049 = t1023*t502*t21
      t1050 = t81*t1049
      t1051 = t87*t384
      t1052 = t565*t1051
      t1053 = t562*t1052
      t1054 = -t1044-t1045
      t1055 = t92*t1054
      t1056 = t86*t1055
      t1062 = t527*Z
      t1064 = DM32*Z
      t1071 = t11*Z
      t1075 = t587*(2*Z*t101+2*t1071+4*t107*DM32*Z)
      t1080 = t522*Z
      t1091 = t606*(2*Z*t120+2*t1071+4*t126*DM32*Z)
      t1096 = t553*Z
      t1098 = DM41*Z
      t1108 = t625*(2*Z*t139+2*t1071+4*t145*DM41*Z)
      t1113 = t548*Z
      t1124 = t644*(2*Z*t158+2*t1071+4*t164*DM41*Z)
      t1126 = -t110*t115-Z*(2*Z*t99*t53-t102*t1062+2*t1064*t54 -
     #        t108*t1062)*t115+t111*t1075+t129*t134+Z*(2*Z*t118*t50 -
     #        t121*t1080+2*t1064*t51-t127*t1080)*t134-t130*t1091 -
     #        t148*t153-Z*(2*Z*t137*t77-t140*t1096+2*t1098*t78 -
     #        t146*t1096)*t153+t149*t1108+t167*t172+Z*(2*Z*t156*t74 -
     #        t159*t1113+2*t1098*t75-t165*t1113)*t172-t168*t1124
      t1128 = t653*t384
      t1130 = t91*t1054
      t1132 = t659*t377
      t1134 = t67*t1041
      t1138 = DM32*t53
      t1143 = (2*t1138-t107*t670*t53-t107*t527)*t675
      t1144 = DM32*t50
      t1149 = (2*t1144-t126*t670*t50-t126*t522)*t685
      t1150 = DM41*t77
      t1155 = (2*t1150-t145*t670*t77-t145*t553)*t695
      t1156 = DM41*t74
      t1161 = (2*t1156-t164*t670*t74-t164*t548)*t705
      t1162 = X*t1126-0.2e1*t178+0.2e1*t181-Z*(-0.4e1*t652*t1128 +
     #        0.2e1*t176*t1130+0.4e1*t658*t1132-0.2e1*t179*t1134) +
     #        t1143-t1149+t1155-t1161
      t1165 = 0.1e1*t1025-t1030
      t1166 = Z*t1165
      t1167 = 0.3e1*t25-0.3e1*t47+0.1e1*t56+0.1e1*t58+0.2e1*t71 +
     #      0.1e1*t80-t82-0.2e1*t95-Z*(-t1025+0.1e1*t1030-t1034 -
     #      t1037+0.4e1*t1040-0.2e1*t1043-t1047+0.1e1*t1050 -
     #      0.4e1*t1053+0.2e1*t1056)-0.2e1*X*t1162+t5*t1126+0.2e1*t1166
      t1171 = t207*t1036
      t1178 = t214*t1049
      t1200 = Y*t1126-0.2e1*t228+0.2e1*t236-0.2e1*t238+0.2e1*t240 -
     #        Z*(-0.4e1*t739*t740*t459+0.2e1*t226*(-t1032-t1044) +
     #        0.4e1*t748*t749*t461-0.2e1*t234*(-t1045-t1031) -
     #        0.4e1*t755*t1039+0.2e1*t237*t1042+0.4e1*t758*t1052 -
     #        0.2e1*t239*t1055)
      t1206 = t459*t250-t770*t459
      t1208 = t1206*t773*t249
      t1211 = t461*t255-t779*t461
      t1213 = t1211*t782*t254
      t1214 = t237*t1036
      t1215 = t239*t1049
      t1216 = t1208-t1213+0.1e1*t1214-t1215
      t1217 = Z*t1216
      t1218 = -t205-t206-0.2e1*t210-t212+0.1e1*t213+0.2e1*t217 -
     #        Z*(0.1e1*(-t1032+t1031)*t28+0.1e1*t1171-0.4e1*t718*t719
     #        *t1132+0.2e1*t207*t208*t1134+0.1e1*(-t1045+t1044)*t3 -
     #        t1178+0.4e1*t728*t729*t1128-0.2e1*t214*t215*t1130) -
     #        X*t1200-Y*t1162+t246*t1126+t252-t257+0.1e1*t258-t259+t1217
      t1228 = t118*t522
      t1229 = t1228*t808
      t1230 = t99*t527
      t1231 = t1230*t810
      t1233 = t48*(t50-t1229/2-t53+t1231/2)
      t1234 = t207*t277
      t1235 = t538*t276
      t1236 = t1235*t825
      t1237 = t536*t1236
      t1238 = t50-t1229/2+t53-t1231/2
      t1239 = t68*t1238
      t1240 = t62*t1239
      t1241 = t156*t548
      t1242 = t1241*t798
      t1243 = t137*t553
      t1244 = t1243*t800
      t1246 = t72*(t74-t1242/2-t77+t1244/2)
      t1247 = t214*t284
      t1248 = t564*t283
      t1249 = t1248*t838
      t1250 = t562*t1249
      t1251 = t74-t1242/2+t77-t1244/2
      t1252 = t92*t1251
      t1253 = t86*t1252
      t1254 = -t1002+t1007-t1008+0.1e1*t1009-t1233+0.1e1*t959 +
     #        0.2e1*t1234+0.4e1*t1237-0.2e1*t1240-t1246-t968 -
     #        0.2e1*t1247-0.4e1*t1250+0.2e1*t1253
      t1300 = Z*((0.2e1*t1138-t288*t849+t819/2)*t101+t290*t810 -
     #        t852*t99*t53-t581+t292*t849/2)*t115-t295*t862 -
     #        Z*((0.2e1*t1144-t297*t867+t818/2)*t120+t299*t808 -
     #        t870*t118*t50-t600+t301*t867/2)*t134+t304*t880 +
     #        Z*((0.2e1*t1150-t306*t885+t832/2)*t139+t308*t800 -
     #        t888*t137*t77-t619+t310*t885/2)*t153-t313*t898 -
     #        Z*((0.2e1*t1156-t315*t903+t831/2)*t158+t317*t798 -
     #        t906*t156*t74-t638+t319*t903/2)*t172+t322*t916
      t1302 = t283*t87
      t1303 = t1302*t802
      t1305 = t91*t1251
      t1307 = t276*t63
      t1308 = t1307*t812
      t1310 = t67*t1238
      t1314 = X*t1300-Z*(-0.4e1*t652*t1303+0.2e1*t176*t1305 +
     #        0.4e1*t658*t1308-0.2e1*t179*t1310)
      t1338 = t352*t221
      t1343 = t354*t229
      t1354 = t296-t305+t314-t323+Y*t1300-Z*(-0.4e1*t739*t1338*t978 +
     #        0.2e1*t226*(t53-t1231/2+t74-t1242/2)+0.4e1*t748*t1343*
     #        t983-0.2e1*t234*(t77-t1244/2+t50-t1229/2)-0.4e1*t755*
     #        t1236+0.2e1*t237*t1239+0.4e1*t758*t1249-0.2e1*t239*t1252)
     #        +t934-t939+t944-t949
      t1367 = t1228*Z
      t1368 = t1230*Z
      t1370 = t48*(-t1367+t1368)
      t1371 = t1235*t1038
      t1372 = t536*t1371
      t1373 = -t1367-t1368
      t1374 = t68*t1373
      t1375 = t62*t1374
      t1376 = t1241*Z
      t1377 = t1243*Z
      t1379 = t72*(-t1376+t1377)
      t1380 = t1248*t1051
      t1381 = t562*t1380
      t1382 = -t1376-t1377
      t1383 = t92*t1382
      t1384 = t86*t1383
      t1385 = -t1208+t1213-t1214+0.1e1*t1215-t1370+0.1e1*t1171+0.4e1*
     #         t1372-0.2e1*t1375-t1379-t1178-0.4e1*t1381+0.2e1*t1384
      t1431 = t294*t115+Z*((-0.2e1*t288*t1062+t1032)*t101+2*t290*Z -
     #        2*t1064*t273+t292*t1062)*t115-t295*t1075-t303*t134 -
     #        Z*((-0.2e1*t297*t1080+t1031)*t120+2*t299*Z-2*t1064*t272 +
     #        t301*t1080)*t134+t304*t1091+t312*t153+Z*((-0.2e1*t306*
     #        t1096+t1045)*t139+2*t308*Z-2*t1098*t280+t310*t1096)*t153-
     #        t313*t1108-t321*t172-Z*((-0.2e1*t315*t1113+t1044)*t158 +
     #        2*t317*Z-2*t1098*t279+t319*t1113)*t172+t322*t1124
      t1433 = t1302*t384
      t1435 = t91*t1382
      t1437 = t1307*t377
      t1439 = t67*t1373
      t1443 = X*t1431-0.2e1*t327+0.2e1*t329-Z*(-0.4e1*t652*t1433 +
     #        0.2e1*t176*t1435+0.4e1*t658*t1437-0.2e1*t179*t1439)
      t1446 = t252-t257+0.1e1*t258-t259+0.1e1*t275-t206+0.2e1*t278+
     #  0.1e1*t282+0.1e1*t213-0.2e1*t285-Z*t1385-0.2e1*X*t1443+t5*t1431
      t1479 = Y*t1431-0.2e1*t353+0.2e1*t355-0.2e1*t356+0.2e1*t357 -
     #        Z*(-0.4e1*t739*t1338*t459+0.2e1*t226*(-t1368-t1376) +
     #        0.4e1*t748*t1343*t461-0.2e1*t234*(-t1377-t1367)-0.4e1*
     #        t755*t1371+0.2e1*t237*t1374+0.4e1*t758*t1380-0.2e1*t239*
     #        t1383)+t1143-t1149+t1155-t1161
      t1483 = -t338-t340-0.2e1*t342-t344-t346+0.2e1*t348-Z*(0.1e1*
     #       (-t1368+t1367)*t28+0.1e1*t960*t1036-0.4e1*t718*t719*t1437+
     #       0.2e1*t207*t208*t1439+0.1e1*(-t1377+t1376)*t3+0.1e1*t969*
     #       t1049+0.4e1*t728*t729*t1433-0.2e1*t214*t215*t1435) -
     #       X*t1479-Y*t1443+t246*t1431+0.1e1*t25-t47+t1166
      t1490 = -t275+0.1e1*t206-0.2e1*t278-t282-t213+0.2e1*t285 -
     #        Z*(0.1e1*t1370-t1171-0.4e1*t1372+0.2e1*t1375+0.1e1*t1379+
     #        0.1e1*t1178+0.4e1*t1381-0.2e1*t1384)-0.2e1*Y*t1479+2*Z*
     #        t324+t267*t1431+0.2e1*t252-0.2e1*t257+0.2e1*t258 -
     #        0.2e1*t259+0.2e1*t1217
      t1501 = t782*t254
      t1505 = t773*t249
      t1518 = 0.2e1*t25-0.2e1*t47+X*t710-t137*t781*t1501-t428*t533 +
     #       t99*t772*t1505+0.1e1*t432*t559+Z*(t676-t686+t696-t706)+
     #       0.1e1*DM32*(t506/2-t508/2)*t28+0.1e1*DM41*(t491/2-t493/2)*
     #       t3+0.1e1*t58+0.1e1*t62*t533-t82-t86*t559
      t1527 = Z*(t934-t939+t944-t949)
      t1530 = DM32*(t809/2-t811/2)*t28
      t1533 = DM41*(t799/2-t801/2)*t3
      t1534 = t62*t823
      t1535 = t86*t836
      t1536 = X*t953-t257-t137*t1005*t1501+0.1e1*t258-t428*t823+t252+
     #       t99*t1000*t1505-t259+0.1e1*t432*t836+t1527+0.1e1*t1530+
     #       0.1e1*t1533-t206+0.1e1*t1534+0.1e1*t213-t1535
      t1545 = Z*(t1143-t1149+t1155-t1161)
      t1546 = t62*t1036
      t1547 = t86*t1049
      t1548 = X*t1165-t137*t1211*t1501-t428*t1036+t99*t1206*t1505 +
     #0.1e1*t432*t1049+vzo+t1545+0.1e1*t376+0.1e1*t383+0.1e1*t1546-t1547
      t1551 = (t811/2-t809/2)*t28
      t1553 = (t801/2-t799/2)*t3
      t1556 = t207*t60*t815*t518
      t1559 = t214*t84*t805*t503
      t1564 = t207*t60*t1028*t518
      t1567 = t214*t84*t1023*t503
      t1592 = t252-t257+0.1e1*t258-t259+Y*t1010-t1530-t1533+0.1e1*t206-
     #        t1534-t213+0.1e1*t1535+t1527
      VX = 0.1e1*t25-t47+P*(-Z*(-t25+0.1e1*t47-t56-t58-0.2e1*t71-t80 +
     #   0.1e1*t82+0.2e1*t95)-0.2e1*X*t197+t5*t174+0.2e1*t201)+0.2e1*Q*
     #   (-Z*(0.1e1*t205+0.1e1*t206+0.2e1*t210+0.1e1*t212-t213-0.2e1*
     #   t217)-X*t243-Y*t197+t246*t174+t261)+RR*(-Z*(0.1e1*t56+0.1e1*
     #   t58+0.2e1*t71+0.1e1*t80-t82-0.2e1*t95)-0.2e1*Y*t243+t267*t174)
      VY = t252-t257+0.1e1*t258-t259+P*(-Z*(-t252+t257-t258+0.1e1*t259-
     #   t275+0.1e1*t206-0.2e1*t278-t282-t213+0.2e1*t285)-0.2e1*X*t332+
     #   t5*t324)+0.2e1*Q*(-Z*(0.1e1*t338+0.1e1*t340+0.2e1*t342+0.1e1*
     #   t344+0.1e1*t346-0.2e1*t348)-X*t360-Y*t332+t246*t324+t201)+RR*
     #   (-Z*(0.1e1*t275-t206+0.2e1*t278+0.1e1*t282+0.1e1*t213-0.2e1*
     #   t285)-0.2e1*Y*t360+t267*t324+0.2e1*t261)
      VZ = vzo + P*t446+0.2e1*Q*(-Z*(0.1e1*t449+0.2e1*t207*t208*t419+
     #   0.1e1*t453-0.2e1*t214*t215*t417)-X*t467-Y*t423+t246*t415-t472-
     #   t474-t477+0.1e1*t480)+RR*(-Z*(0.1e1*t376+0.2e1*t379+0.1e1*t383
     #   -0.2e1*t386)-0.2e1*Y*t467+t267*t415-t438-t441-t443+0.1e1
     #   *t445+0.2e1*t435)
      s1 = 0.1e1*t504-t519
      s2 = s1+P*(-Z*t572+0.2e1*t183-0.2e1*vzo
     #   - 0.2e1*X*t707+t5*t650+0.2e1*Z*t710)
      VXDX = s2+0.2e1*Q*(-Z*(0.1e1*(t53-t529/2-t50+t524/2)*t28+0.1e1*
     #  t207*t533+0.2e1*t717-0.4e1*t718*t719*t660+0.2e1*t207*t208*t662+
     #  0.1e1*(t77-t555/2-t74+t550/2)*t3-t214*t559-0.2e1*t727+0.4e1*
     #  t728*t729*t654-0.2e1*t214*t215*t656)+t242-X*t763-Y*t707+t246*
     #  t650+Z*(t772*t773*t249-t781*t782*t254+0.1e1*t237*t533-t239*
     #  t559))+RR*(-Z*(0.1e1*t531+0.1e1*t534+0.2e1*t535-0.4e1*t542+
     #  0.2e1*t545+0.1e1*t557-t560-0.2e1*t561+0.4e1*t568-0.2e1*t571)-
     #  0.2e1*Y*t763+t267*t650)
      VXDY =0.1e1*t807-t817+P*(-Z*t844-0.2e1*X*t950+t5*t918+0.2e1*t954)
     #  +0.2e1*Q*(-Z*(0.1e1*(-t819/2+t818/2)*t28+0.1e1*t959-0.2e1*t960
     #  *t70-0.4e1*t718*t719*t924+0.2e1*t207*t208*t926+0.1e1*(-t832/2+
     #  t831/2)*t3-t968+0.2e1*t969*t94+0.4e1*t728*t729*t920-0.2e1*t214*
     #  t215*t922)-X*t994+t183-vzo-Y*t950+t246*t918+t1011)+RR*(-Z*(
     #  0.1e1*t821+0.1e1*t824-0.2e1*t717-0.4e1*t827+0.2e1*t830+0.1e1*
     #  t834-t837+0.2e1*t727+0.4e1*t840-0.2e1*t843)+0.2e1*t242-0.2e1*Y*
     #  t994+t267*t918)
      VXDZ = 0.1e1*t1025-t1030+P*t1167+0.2e1*Q*t1218+RR*(-t56-t58-0.2e1
     #  *t71-t80+0.1e1*t82+0.2e1*t95-Z*(0.1e1*t1034+0.1e1*t1037-0.4e1*
     #  t1040+0.2e1*t1043+0.1e1*t1047-t1050+0.4e1*t1053-0.2e1*t1056)-
     #  0.2e1*Y*t1200+2*Z*t174+t267*t1126)
      VYDY = t1002-t1007+0.1e1*t1008-t1009+P*(-Z*t1254-0.2e1*X*t1314+t5
     #  *t1300)+0.2e1*Q*(-Z*(0.1e1*(t53-t1231/2-t50+t1229/2)*t28+0.1e1*
     #  t960*t823-0.2e1*t960*t277-0.4e1*t718*t719*t1308+0.2e1*t207*t208*
     #  t1310+0.1e1*(t77-t1244/2-t74+t1242/2)*t3+0.1e1*t969*t836+0.2e1*
     #  t969*t284+0.4e1*t728*t729*t1303-0.2e1*t214*t215*t1305)-X*t1354+
     #  t331-Y*t1314+t246*t1300+t954)+RR*(-Z*(0.1e1*t1233-t959-0.2e1*
     #  t1234-0.4e1*t1237+0.2e1*t1240+0.1e1*t1246+0.1e1*t968+0.2e1*
     #  t1247+0.4e1*t1250-0.2e1*t1253)+0.2e1*t359-0.2e1*vzo
     #  -0.2e1*Y*t1354+t267*t1300+0.2e1*t1011)
      VYDZ=t1208-t1213+0.1e1*t1214-t1215+P*t1446+0.2e1*Q*t1483+RR*t1490
      VXX = X*t200-t426-t429+t430+0.1e1*t433+t435+0.1e1*t438 +
     #      0.1e1*t441+0.1e1*t443-t445
      VYX = X*t260-t472-t474-t477+0.1e1*t480
      VZX = X*t434-t201
      VXXDX = t1518
      VXXDY = t1536
      VXXDZ = t1548
      VYXDY = X*t1010-t1551-t1553+0.1e1*t340-t1556-t346+0.1e1*t1559
      VYXDZ = X*t1216-t449-t453-t1564+0.1e1*t1567
      VXY = Y*t200-t472-t474-t477+0.1e1*t480
      VYY = Y*t260-t438-t441-t443+0.1e1*t445+t435
      VZY = Y*t434-t261
      VXYDX = Y*t710-(t508/2-t506/2)*t28-(t493/2-t491/2)*t3-t206 -
     #        t207*t60*t515*t518+0.1e1*t213+0.1e1*t214*t84*t500*t503
      VXYDY = 0.1e1*t25-t47+Y*t953-t1551-t1553+0.1e1*t340-t1556-t346
     #      + 0.1e1*t1559
      VXYDZ = Y*t1165-t449-t453-t1564+0.1e1*t1567
      VYYDY = t1592
      VYYDZ = Y*t1216-t376-t383-t1546+0.1e1*t1547+vzo+t1545

      VZDZ = -VXDX - VYDY
      VYDX = VXDY
      VZDX = VXDZ
      VZDY = VYDZ
      VZXDZ = -VXXDX - VYXDY
      VYXDX = VXXDY
      VZXDX = VXXDZ
      VZXDY = VYXDZ
      VZYDZ = -VXYDX - VYYDY
      VYYDX = VXYDY
      VZYDX = VXYDZ
      VZYDY = VYYDZ

      RETURN
      END

C***********************************************************************
C
C     SUBROUTINE INTERFLD()
C
C     Computes the induced velocity and the derivatives of the velocity
C     from a curved panel with a linear source distribution, for the
C     case where the panel has four sides
C
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Programmed by: Michael Hughes, July 1994
C
C***********************************************************************

      SUBROUTINE INTERFLD(X,Y,Z,TLEN,P,Q,RR,SI,VX,VY,VZ,
     *                    VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                    VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                    VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                    VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                    VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )


      DIMENSION SI(15)

      t1 = TLEN**2
      t2 = X**2
      t3 = Y**2
      t4 = Z**2
      t5 = t2+t3+t4
      t6 = 1/t5
      t7 = t1*t6
      t8 = SI(1)*X
      t9 = sqrt(t5)
      t10 = 1/t9
      t12 = TLEN*t10
      t13 = t2*t6
      t14 = 0.3E1*t13-0.1E1
      t16 = SI(3)*X
      t17 = Y*t6
      t19 = SI(2)*t14+0.3E1*t16*t17
      t21 = SI(4)*X
      t22 = 0.3E1-0.5E1*t13
      t23 = t10*t22
      t25 = SI(5)*Y
      t26 = 0.1E1-0.5E1*t13
      t27 = t10*t26
      t29 = SI(6)*X
      t30 = t3*t6
      t31 = 0.1E1-0.5E1*t30
      t32 = t10*t31
      t34 = 0.3E1*t21*t23+0.6E1*t25*t27+0.3E1*t29*t32
      t36 = t8*t10+t12*t19-0.5E0*t7*t34
      t38 = t1**2
      t39 = P*t38
      t40 = t9**2
      t42 = 1/t40/t9
      t43 = t6*Z
      t44 = t21*t43
      t45 = SI(7)*Z
      t46 = t45*t27
      t47 = SI(8)*X
      t48 = t42*Y
      t49 = t48*Z
      t50 = t47*t49
      t51 = 0.3E1*t46-0.15E2*t50
      t53 = SI(11)*X
      t54 = 0.7E1*t13-0.3E1
      t55 = t43*t54
      t57 = SI(12)*Y
      t58 = 0.7E1*t13-0.1E1
      t59 = t43*t58
      t60 = t57*t59
      t61 = SI(13)*X
      t62 = 0.7E1*t30-0.1E1
      t63 = t43*t62
      t64 = t61*t63
      t65 = 0.15E2*t53*t55+0.3E2*t60+0.15E2*t64
      t67 = 0.3E1*t44-t12*t51+0.5E0*t7*t65
      t70 = Q*t38
      t71 = SI(5)*X
      t72 = t71*t43
      t73 = SI(8)*Z
      t74 = t73*t27
      t75 = SI(9)*X
      t76 = t75*t49
      t77 = 0.3E1*t74-0.15E2*t76
      t79 = SI(12)*X
      t81 = SI(13)*Y
      t82 = t81*t59
      t83 = SI(14)*X
      t84 = t83*t63
      t85 = 0.15E2*t79*t55+0.3E2*t82+0.15E2*t84
      t87 = 0.3E1*t72-t12*t77+0.5E0*t7*t85
      t90 = RR*t38
      t92 = SI(9)*Z
      t94 = SI(10)*X
      t96 = 0.3E1*t92*t27-0.15E2*t94*t49
      t99 = SI(14)*Y
      t101 = SI(15)*X
      t103 = 0.15E2*t61*t55+0.3E2*t99*t59+0.15E2*t101*t63
      t105 = 0.3E1*t29*t43-t12*t96+0.5E0*t7*t103
      t109 = t1*TLEN
      t110 = t109*t6
      t111 = SI(2)*X
      t114 = t71*t17
      t115 = SI(4)*t14+0.3E1*t114
      t117 = SI(7)*X
      t119 = SI(8)*Y
      t120 = t119*t27
      t121 = t75*t32
      t122 = 0.3E1*t117*t23+0.6E1*t120+0.3E1*t121
      t124 = -t111*t10-t12*t115+0.5E0*t7*t122
      t129 = SI(5)*t14+0.3E1*t29*t17
      t132 = SI(9)*Y
      t135 = 0.3E1*t47*t23+0.6E1*t132*t27+0.3E1*t94*t32
      t137 = -t16*t10-t12*t129+0.5E0*t7*t135
      t139 = SI(1)*Y
      t142 = 0.3E1*t30-0.1E1
      t144 = 0.3E1*t111*t17+SI(3)*t142
      t146 = SI(4)*Y
      t149 = SI(6)*Y
      t150 = 0.3E1-0.5E1*t30
      t151 = t10*t150
      t153 = 0.3E1*t146*t27+0.6E1*t71*t32+0.3E1*t149*t151
      t155 = t139*t10+t12*t144-0.5E0*t7*t153
      t160 = -0.15E2*t117*t49+0.3E1*t73*t32
      t162 = SI(11)*Y
      t165 = 0.7E1*t30-0.3E1
      t166 = t43*t165
      t168 = 0.15E2*t162*t59+0.3E2*t79*t63+0.15E2*t81*t166
      t170 = 0.3E1*t146*t43-t12*t160+0.5E0*t7*t168
      t173 = t25*t43
      t174 = t92*t32
      t175 = -0.15E2*t50+0.3E1*t174
      t178 = 0.15E2*t60+0.3E2*t64+0.15E2*t99*t166
      t180 = 0.3E1*t173-t12*t175+0.5E0*t7*t178
      t183 = t149*t43
      t184 = SI(10)*Z
      t185 = t184*t32
      t186 = -0.15E2*t76+0.3E1*t185
      t188 = SI(15)*Y
      t190 = 0.15E2*t82+0.3E2*t84+0.15E2*t188*t166
      t192 = 0.3E1*t183-t12*t186+0.5E0*t7*t190
      t196 = SI(2)*Y
      t200 = 0.3E1*t21*t17+SI(5)*t142
      t206 = 0.3E1*SI(7)*Y*t27+0.6E1*t47*t32+0.3E1*t132*t151
      t208 = -t196*t10-t12*t200+0.5E0*t7*t206
      t210 = SI(3)*Y
      t213 = 0.3E1*t114+SI(6)*t142
      t215 = SI(10)*Y
      t217 = 0.3E1*t120+0.6E1*t121+0.3E1*t215*t151
      t219 = -t210*t10-t12*t213+0.5E0*t7*t217
      t236 = t4*t6
      t237 = 0.3E1*t236-0.1E1
      t240 = t10*(0.1E1-0.5E1*t236)
      t245 = t5**2
      t246 = 1/t245
      t249 = 0.3E1-0.15E2*t13-0.15E2*t236+0.105e3*t2*t246*t4
      t252 = t17*(0.7E1*t236-0.1E1)
      t254 = t3*t246
      t256 = 0.3E1-0.15E2*t30-0.15E2*t236+0.105e3*t254*t4
      t306 = t1*t246
      t309 = SI(1)*t10
      t312 = TLEN*t42
      t315 = X*t6
      t316 = t2*X
      t317 = t316*t246
      t318 = 0.6E1*t315-0.6E1*t317
      t322 = SI(3)*t2
      t323 = t246*Y
      t329 = SI(4)*t10
      t331 = SI(4)*t2
      t332 = t42*t22
      t335 = t10*(-0.1E2*t315+0.1E2*t317)
      t337 = t42*t26
      t338 = t337*X
      t341 = SI(6)*t10
      t343 = SI(6)*t2
      t344 = t42*t31
      t346 = t40**2
      t348 = 1/t346/t9
      t349 = t348*t3
      t355 = t348*t67
      t359 = SI(4)*t6*Z
      t360 = t246*Z
      t361 = t331*t360
      t367 = Y*Z
      t369 = SI(8)*t2
      t370 = t348*Y
      t371 = t370*Z
      t372 = t369*t371
      t377 = SI(11)*t6
      t378 = Z*t54
      t381 = t360*t54
      t384 = t43*(0.14E2*t315-0.14E2*t317)
      t387 = Z*t58
      t388 = t387*X
      t391 = SI(13)*t6
      t392 = Z*t62
      t394 = SI(13)*t2
      t395 = t360*t62
      t398 = 1/t245/t5
      t399 = t398*Z
      t400 = t399*t3
      t401 = t394*t400
      t407 = t348*t87
      t411 = SI(5)*t6*Z
      t412 = SI(5)*t2
      t413 = t412*t360
      t420 = SI(9)*t2
      t421 = t420*t371
      t426 = SI(12)*t6
      t434 = SI(14)*t6
      t436 = SI(14)*t2
      t438 = t436*t400
      t444 = t348*t105
      t448 = SI(6)*t6*Z
      t456 = SI(10)*t2
      t468 = SI(15)*t6
      t470 = SI(15)*t2
      t479 = t109*t246
      t482 = SI(2)*t10
      t483 = SI(2)*t2
      t488 = t25*t6
      t489 = t412*t323
      t494 = SI(7)*t10
      t501 = SI(9)*t10
      t502 = t501*t31
      t504 = t420*t349
      t512 = SI(3)*t10
      t517 = t149*t6
      t523 = SI(8)*t10
      t529 = SI(10)*t10
      t530 = t529*t31
      t550 = t332*Y
      t552 = SI(4)*t316
      t556 = SI(5)*t3
      t558 = t348*t2
      t560 = t344*Y
      t562 = t3*Y
      t563 = t562*t246
      t565 = t10*(-0.1E2*t17+0.1E2*t563)
      t573 = t360*Y
      t574 = t21*t573
      t577 = t337*Y
      t578 = t45*t577
      t580 = t45*t558*Y
      t581 = t349*Z
      t582 = t47*t581
      t583 = t42*Z
      t584 = t47*t583
      t590 = t378*Y
      t592 = SI(11)*t316
      t593 = t399*Y
      t595 = t426*t387
      t596 = SI(12)*t3
      t597 = t360*t58
      t598 = t596*t597
      t599 = t399*t2
      t600 = t596*t599
      t601 = t61*t246
      t602 = t392*Y
      t603 = t601*t602
      t605 = t43*(0.14E2*t17-0.14E2*t563)
      t606 = t61*t605
      t614 = t71*t573
      t617 = t73*t577
      t618 = t75*t581
      t619 = t75*t583
      t624 = t79*t246
      t626 = SI(12)*t316
      t628 = t391*t387
      t629 = SI(13)*t3
      t630 = t629*t597
      t632 = t83*t246*t602
      t633 = t83*t605
      t641 = t29*t573
      t644 = t92*t577
      t645 = t94*t581
      t652 = SI(13)*t316
      t655 = SI(14)*t3
      t672 = t71*t254
      t673 = t71*t6
      t679 = SI(7)*t316
      t681 = t523*t26
      t682 = SI(8)*t3
      t683 = t682*t337
      t684 = t682*t558
      t685 = t75*t560
      t686 = t75*t565
      t698 = t29*t6
      t704 = SI(8)*t316
      t706 = t501*t26
      t707 = SI(9)*t3
      t727 = t332*Z
      t729 = t348*Z
      t731 = t337*Z
      t733 = t558*Z
      t735 = t344*Z
      t744 = t246*t4
      t746 = t21*t6
      t749 = t494*t26
      t750 = SI(7)*t4
      t753 = t370*t4
      t754 = t47*t753
      t755 = t47*t48
      t760 = t744*t54
      t762 = t6*t54
      t764 = t398*t4
      t766 = t744*t58
      t767 = t57*t766
      t768 = t6*t58
      t769 = t57*t768
      t770 = t764*t2
      t771 = t57*t770
      t772 = t744*t62
      t773 = t61*t772
      t774 = t6*t62
      t775 = t61*t774
      t776 = t764*t3
      t777 = t61*t776
      t788 = SI(8)*t4
      t791 = t75*t753
      t792 = t75*t48
      t800 = t81*t766
      t801 = t81*t768
      t802 = t81*t770
      t803 = t83*t772
      t804 = t83*t774
      t805 = t83*t776
      t816 = SI(9)*t4
      t851 = t75*t735
      t882 = 0.6E1*t17-0.6E1*t563
      t889 = SI(4)*t3
      t895 = SI(6)*t3
      t896 = t42*t150
      t903 = t348*t170
      t918 = SI(11)*t3
      t923 = Z*t165
      t925 = t360*t165
      t933 = t348*t180
      t936 = t556*t360
      t953 = t348*t192
      t956 = t895*t360
      t987 = SI(7)*t3
      t1002 = SI(3)*t3
      t1035 = t896*Z
      t1064 = t744*t165
      t1066 = t6*t165
      t1100 = SI(10)*t4
      VX = t7*t36+0.1E1*t39*t42*t67+0.2E1*t70*t42*t87+0.1E1*t90*t42*t105
      VXX = -t110*t124
      VXY = -t110*t137
      VY = t7*t155+0.1E1*t39*t42*t170+0.2E1*t70*t42*t180 +
     #     0.1E1*t90*t42*t192
      VYX = -t110*t208
      VYY = -t110*t219
      VZ = t7*(SI(1)*Z*t10+t12*(0.3E1*t111*t43+0.3E1*t210*t43)-0.5E0*
     #    t7*(0.3E1*SI(4)*Z*t27-0.3E2*t71*t49+0.3E1*SI(6)*Z*t32))+0.1E1*
     #    t39*t42*(SI(4)*t237-t12*(0.3E1*t117*t240+0.3E1*t119*t240)+
     #    0.5E0*t7*(SI(11)*t249+0.3E2*t79*t252+SI(13)*t256))+0.2E1*t70*
     #    t42*(SI(5)*t237-t12*(0.3E1*t47*t240+0.3E1*t132*t240)+0.5E0*t7*
     #    (SI(12)*t249+0.3e2*t61*t252+SI(14)*t256))+0.1E1*t90*t42*(SI(6)
     #    *t237-t12*(0.3E1*t75*t240+0.3E1*t215*t240)+0.5E0*t7*(SI(13)*
     #    t249+0.3E2*t83*t252+SI(15)*t256))
      VZX = -t110*(-SI(2)*Z*t10-t12*(0.3E1*t44+0.3E1*t173) +
     #        0.5E0*t7*(0.3E1*t46-0.3E2*t50+0.3E1*t174))
      VZY = -t110*(-SI(3)*Z*t10-t12*(0.3E1*t72+0.3E1*t183) +
     #        0.5E0*t7*(0.3E1*t74-0.3E2*t76+0.3E1*t185))
      s1 = -2*t306*t36*X+t7*(t309-SI(1)*t2*t42-t312*t19*X+t12*(SI(2)*
     #  t318+0.3E1*SI(3)*t6*Y-0.6E1*t322*t323)+0.1E1*t306*t34*X-0.5E0*
     #  t7*(0.3e1*t329*t22-0.3E1*t331*t332+0.3E1*t21*t335-0.6E1*t25*
     #  t338+0.6E1*t25*t335+0.3E1*t341*t31-0.3E1*t343*t344+0.3E2*t343*
     #  t349))-0.3E1*t39*t355*X+0.1E1*t39*t42*(0.3E1*t359-0.6E1*t361+
     #  t312*t51*X-t12*(-0.3E1*t45*t338+0.3E1*t45*t335-0.15E2*SI(8)*t42
     #  *t367+0.45E2*t372)-t306*t65*X+0.5E0*t7*(0.15E2*t377*t378-0.3E2*
     #  SI(11)*t2*t381+0.15E2*t53*t384-0.6E2*t57*t246*t388+0.3E2*t57*
     #  t384+0.15E2*t391*t392-0.3E2*t394*t395-0.21e3*t401))
      VXDX = s1-0.6E1*t70*t407*X+0.2E1*t70*t42*(0.3E1*t411-0.6E1*t413 +
     #   t312*t77*X-t12*(-0.3E1*t73*t338+0.3E1*t73*t335-0.15E2*SI(9)*t42
     #   *t367+0.45E2*t421)-t306*t85*X+0.5E0*t7*(0.15E2*t426*t378-0.3E2*
     #   SI(12)*t2*t381+0.15E2*t79*t384-0.6E2*t81*t246*t388+0.3E2*t81*
     #   t384+0.15E2*t434*t392-0.3E2*t436*t395-0.21e3*t438))-0.3E1*t90*
     #   t444*X+0.1E1*t90*t42*(0.3E1*t448-0.6E1*t343*t360+t312*t96*X-
     #   t12*(-0.3E1*t92*t338+0.3E1*t92*t335-0.15E2*SI(10)*t42*t367+
     #   0.45E2*t456*t371)-t306*t103*X+0.5E0*t7*(0.15E2*t391*t378-0.3E2*
     #   t394*t381+0.15E2*t61*t384-0.6E2*t99*t246*t388+0.3E2*t99*t384+
     #   0.15E2*t468*t392-0.3E2*t470*t395-0.21e3*t470*t400))
      VXXDX = 2*t479*t124*X-t110*(-t482+t483*t42+t312*t115*X-t12*(SI(4)
     #   *t318+0.3E1*t488-0.6E1*t489)-t306*t122*X+0.5E0*t7*(0.3E1*t494*
     #   t22-0.3E1*SI(7)*t2*t332+0.3E1*t117*t335-0.6E1*t119*t338+0.6E1*
     #   t119*t335+0.3E1*t502-0.3E1*t420*t344+0.3E2*t504))
      VXYDX = 2*t479*t137*X-t110*(-t512+t322*t42+t312*t129*X-t12*(SI(5)*
     #   t318+0.3E1*t517-0.6E1*t343*t323)-t306*t135*X+0.5E0*t7*(0.3E1*
     #   t523*t22-0.3E1*t369*t332+0.3E1*t47*t335-0.6E1*t132*t338+0.6E1*
     #   t132*t335+0.3E1*t530-0.3E1*t456*t344+0.3E2*t456*t349))
      s1 = -2*t306*t36*Y+t7*(-t8*t48-t312*t19*Y+t12*(-0.6E1*t483*t323-
     #  0.6e1*t16*t254+0.3E1*t16*t6)+0.1E1*t306*t34*Y-0.5E0*t7*(-0.3E1*
     #  t21*t550+0.3E2*t552*t370+0.6E1*SI(5)*t10*t26-0.6E1*t556*t337+
     #  0.6E2*t556*t558-0.3E1*t29*t560+0.3E1*t29*t565))-0.3E1*t39*t355*
     #  Y+0.1E1*t39*t42*(-0.6E1*t574+t312*t51*Y-t12*(-0.3E1*t578+0.3E2*
     #  t580+0.45E2*t582-0.15E2*t584)-t306*t65*Y+0.5E0*t7*(-0.3E2*t53*
     #  t246*t590-0.21e3*t592*t593+0.3E2*t595-0.6E2*t598-0.42e3*t600-
     #  0.3E2*t603+0.15E2*t606))
      VXDY = s1-0.6E1*t70*t407*Y+0.2E1*t70*t42*(-0.6E1*t614+t312*t77*Y-
     #  t12*(-0.3E1*t617+0.3E2*t372+0.45E2*t618-0.15E2*t619)-t306*t85*
     #  Y+0.5E0*t7*(-0.3E2*t624*t590-0.21e3*t626*t593+0.3E2*t628-0.6E2*
     #  t630-0.42e3*t401-0.3E2*t632+0.15E2*t633))-0.3E1*t90*t444*Y+
     #  0.1E1*t90*t42*(-0.6E1*t641+t312*t96*Y-t12*(-0.3E1*t644+0.3E2*
     #  t421+0.45E2*t645-0.15E2*t94*t583)-t306*t103*Y+0.5E0*t7*(-0.3E2*
     #  t601*t590-0.21e3*t652*t593+0.3E2*t434*t387-0.6E2*t655*t597-
     #  0.42e3*t438-0.3E2*t101*t246*t602+0.15E2*t101*t605))
      VXXDY = 2*t479*t124*Y-t110*(t111*t48+t312*t115*Y-t12*(-0.6E1*t331*
     #  t323-0.6E1*t672+0.3E1*t673)-t306*t122*Y+0.5E0*t7*(-0.3E1*t117*
     #  t550+0.3E2*t679*t370+0.6E1*t681-0.6E1*t683+0.6E2*t684-0.3E1*
     #  t685+0.3e1*t686))
      VXYDY = 2*t479*t137*Y-t110*(t16*t48+t312*t129*Y-t12*(-0.6E1*t489-
     #  0.6E1*t29*t254+0.3E1*t698)-t306*t135*Y+0.5E0*t7*(-0.3E1*t47*
     #  t550+0.3E2*t704*t370+0.6E1*t706-0.6E1*t707*t337+0.6E2*t504-
     #  0.3E1*t94*t560+0.3E1*t94*t565))
      s1 = -2*t306*t36*Z+t7*(-t8*t583-t312*t19*Z+t12*(-0.6E1*t483*t360-
     #  0.6e1*t16*t573)+0.1E1*t306*t34*Z-0.5E0*t7*(-0.3E1*t21*t727+
     #  0.3E2*t552*t729-0.6E1*t25*t731+0.6E2*t25*t733-0.3E1*t29*t735+
     #  0.3E2*t29*t581))-0.3E1*t39*t355*Z+0.1E1*t39*t42*(-0.6E1*t21*
     #  t744+0.3E1*t746+t312*t51*Z-t12*(0.3E1*t749-0.3E1*t750*t337+
     #  0.3E2*t750*t558+0.45E2*t754-0.15E2*t755)-t306*t65*Z+0.5E0*t7*
     #  (-0.3E2*t53*t760+0.15E2*t53*t762-0.21e3*t592*t764-0.6E2*t767+
     #  0.3E2*t769-0.42e3*t771-0.3E2*t773+0.15E2*t775-0.21e3*t777))
      VXDZ = s1-0.6E1*t70*t407*Z+0.2E1*t70*t42*(-0.6E1*t71*t744+0.3e1*
     #  t673+t312*t77*Z-t12*(0.3E1*t681-0.3E1*t788*t337+0.3E2*t788*t558+
     #  0.45E2*t791-0.15E2*t792)-t306*t85*Z+0.5E0*t7*(-0.3E2*t79*t760+
     #  0.15E2*t79*t762-0.21e3*t626*t764-0.6E2*t800+0.3E2*t801-0.42e3*
     #  t802-0.3E2*t803+0.15E2*t804-0.21E3*t805))-0.3E1*t90*t444*Z+
     #  0.1E1*t90*t42*(-0.6E1*t29*t744+0.3E1*t698+t312*t96*Z-t12*(0.3E1*
     #  t706-0.3E1*t816*t337+0.3E2*t816*t558+0.45E2*t94*t753-0.15E2*t94*
     #  t48)-t306*t103*Z+0.5E0*t7*(-0.3E2*t61*t760+0.15E2*t61*t762-
     #  0.21E3*t652*t764-0.6E2*t99*t766+0.3E2*t99*t768-0.42E3*t99*t770-
     #  0.3E2*t101*t772+0.15E2*t101*t774-0.21E3*t101*t776))
      VXXDZ = 2*t479*t124*Z-t110*(t111*t583+t312*t115*Z-t12*(-0.6E1*
     #  t361-0.6E1*t614)-t306*t122*Z+0.5E0*t7*(-0.3E1*t117*t727+0.3E2*
     #  t679*t729-0.6E1*t617+0.6E2*t372-0.3E1*t851+0.3E2*t618))
      VXYDZ = 2*t479*t137*Z-t110*(t16*t583+t312*t129*Z-t12*(-0.6E1*
     #   t413-0.6E1*t641)-t306*t135*Z+0.5E0*t7*(-0.3E1*t47*t727+0.3E2*
     #   t704*t729-0.6E1*t644+0.6E2*t421-0.3E1*t94*t735+0.3E2*t645))
      s1 = -2*t306*t155*Y+t7*(t309-SI(1)*t3*t42-t312*t144*Y+t12*(-0.6E1*
     #  t111*t254+0.3E1*t111*t6+SI(3)*t882)+0.1E1*t306*t153*Y-0.5E0*t7*
     #  (0.3e1*t329*t26-0.3E1*t889*t337+0.3E2*t889*t558-0.6E1*t71*t560+
     #  0.6E1*t71*t565+0.3E1*t341*t150-0.3E1*t895*t896+0.3E1*t149*t565))
     #  -0.3E1*t39*t903*Y+0.1E1*t39*t42*(0.3E1*t359-0.6E1*t889*t360+
     #  t312*t160*Y-t12*(0.45E2*t117*t581-0.15E2*t117*t583-0.3E1*t73*
     #  t560+0.3E1*t73*t565)-t306*t168*Y+0.5E0*t7*(0.15E2*t377*t387-
     #  0.3E2*t918*t597-0.21E3*t918*t599-0.6E2*t624*t602+0.3E2*t79*t605+
     #  0.15E2*t391*t923-0.3E2*t629*t925+0.15E2*t81*t605))
      VYDY = s1-0.6E1*t70*t933*Y+0.2E1*t70*t42*(0.3E1*t411-0.6E1*t936+
     #  t312*t175*Y-t12*(0.45E2*t582-0.15E2*t584-0.3E1*t92*t560+0.3E1*
     #  t92*t565)-t306*t178*Y+0.5E0*t7*(0.15E2*t595-0.3E2*t598-0.21E3*
     #  t600-0.6E2*t603+0.3E2*t606+0.15E2*t434*t923-0.3E2*t655*t925+
     #  0.15E2*t99*t605))-0.3E1*t90*t953*Y+0.1E1*t90*t42*(0.3E1*t448-
     #  0.6E1*t956+t312*t186*Y-t12*(0.45E2*t618-0.15E2*t619-0.3E1*t184*
     #  t560+0.3E1*t184*t565)-t306*t190*Y+0.5E0*t7*(0.15E2*t628-0.3E2*
     #  t630-0.21E3*t401-0.6E2*t632+0.3E2*t633+0.15E2*t468*t923-0.3E2*
     #  SI(15)*t3*t925+0.15E2*t188*t605))
      VYXDY = 2*t479*t208*Y-t110*(-t482+SI(2)*t3*t42+t312*t200*Y-t12*
     #  (-0.6E1*t21*t254+0.3E1*t746+SI(5)*t882)-t306*t206*Y+0.5E0*t7*
     #  (0.3e1*t749-0.3E1*t987*t337+0.3E2*t987*t558-0.6E1*t47*t560+
     #  0.6E1*t47*t565+0.3E1*t501*t150-0.3E1*t707*t896+0.3E1*t132*t565))
      VYYDY = 2*t479*t219*Y-t110*(-t512+t1002*t42+t312*t213*Y-t12*
     #  (-0.6E1*t672+0.3E1*t673+SI(6)*t882)-t306*t217*Y+0.5E0*t7*(0.3E1*
     #  t681-0.3E1*t683+0.3E2*t684-0.6E1*t685+0.6E1*t686+0.3E1*t529*
     #  t150-0.3E1*SI(10)*t3*t896+0.3E1*t215*t565))
      s1 = -2*t306*t155*Z+t7*(-t139*t583-t312*t144*Z+t12*(-0.6E1*t111*
     #  t573-0.6E1*t1002*t360)+0.1E1*t306*t153*Z-0.5E0*t7*(-0.3E1*t146*
     #  t731+0.3E2*t146*t733-0.6E1*t71*t735+0.6E2*t71*t581-0.3E1*t149*
     #  t1035+0.3E2*SI(6)*t562*t729))-0.3E1*t39*t903*Z+0.1E1*t39*t42*
     #  (-0.6E1*t146*t744+0.3E1*t146*t6+t312*t160*Z-t12*(0.45E2*t117*
     #  t753-0.15E2*t117*t48+0.3E1*t523*t31-0.3E1*t788*t344+0.3E2*t788*
     #  t349)-t306*t168*Z+0.5e0*t7*(-0.3E2*t162*t766+0.15E2*t162*t768-
     #  0.21E3*t162*t770-0.6E2*t79*t772+0.3E2*t79*t774-0.42E3*t79*t776-
     #  0.3E2*t81*t1064+0.15E2*t81*t1066-0.21E3*SI(13)*t562*t764))
      VYDZ = s1-0.6E1*t70*t933*Z+0.2E1*t70*t42*(-0.6E1*t25*t744+0.3e1*
     #  t488+t312*t175*Z-t12*(0.45E2*t754-0.15E2*t755+0.3E1*t502-0.3E1*
     #  t816*t344+0.3E2*t816*t349)-t306*t178*Z+0.5E0*t7*(-0.3E2*t767+
     #  0.15e2*t769-0.21E3*t771-0.6E2*t773+0.3E2*t775-0.42E3*t777-0.3E2*
     #  t99*t1064+0.15E2*t99*t1066-0.21E3*SI(14)*t562*t764))-0.3E1*t90*
     #  t953*Z+0.1e1*t90*t42*(-0.6E1*t149*t744+0.3E1*t517+t312*t186*Z-
     #  t12*(0.45E2*t791-0.15E2*t792+0.3E1*t530-0.3E1*t1100*t344+0.3E2*
     #  t1100*t349)-t306*t190*Z+0.5E0*t7*(-0.3E2*t800+0.15E2*t801-
     #  0.21E3*t802-0.6E2*t803+0.3E2*t804-0.42E3*t805-0.3E2*t188*t1064+
     #  0.15E2*t188*t1066-0.21E3*SI(15)*t562*t764))
      VYXDZ = 2*t479*t208*Z-t110*(t196*t583+t312*t200*Z-t12*(-0.6E1*t574
     #  - 0.6E1*t936)-t306*t206*Z+0.5E0*t7*(-0.3E1*t578+0.3E2*t580-0.6e1
     #  * t47*t735+0.6E2*t582-0.3E1*t132*t1035+0.3E2*SI(9)*t562*t729))
      VYYDZ = 2*t479*t219*Z-t110*(t210*t583+t312*t213*Z-t12*(-0.6E1*
     #  t614-0.6E1*t956)-t306*t217*Z+0.5E0*t7*(-0.3E1*t617+0.3E2*t372-
     #  0.6e1*t851+0.6E2*t618-0.3E1*t215*t1035+0.3E2*SI(10)*t562*t729))

      VZDZ = -VXDX - VYDY
      VYDX = VXDY
      VZDX = VXDZ
      VZDY = VYDZ
      VZXDZ = -VXXDX - VYXDY
      VYXDX = VXXDY
      VZXDX = VXXDZ
      VZXDY = VYXDZ
      VZYDZ = -VXYDX - VYYDY
      VYYDX = VXYDY
      VZYDX = VXYDZ
      VZYDY = VYYDZ

      RETURN
      END

C***********************************************************************
C
C     SUBROUTINE FARFLD()
C
C     Computes the induced velocity and the derivatives of the velocity
C     from a curved panel with a linear source distribution, for the
C     case where the panel has four sides
C
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Programmed by: Michael Hughes, July 1994
C
C***********************************************************************

      SUBROUTINE FARFLD(XFP,YFP,ZFP,TLEN,P,Q,RR,SI,XC,DIRP,VX,VY,VZ,
     *                  VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                  VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                  VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                  VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                  VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      DIMENSION SI(15),XC(3),DIRP(3,3)

      t1 = TLEN**2
      t2 = t1*SI(1)
      t3 = XFP-XC(1)
      t4 = t3**2
      t5 = YFP-XC(2)
      t6 = t5**2
      t7 = ZFP-XC(3)
      t8 = t7**2
      t9 = t4+t6+t8
      t10 = 1/t9
      t11 = XFP**2
      t13 = XC(1)**2
      t14 = YFP**2
      t16 = XC(2)**2
      t17 = ZFP**2
      t19 = XC(3)**2
      t21 = sqrt(t11-2*XFP*XC(1)+t13+t14-2*YFP*XC(2)+t16+t17 -
     #      2*ZFP*XC(3)+t19)
      t22 = 1/t21
      t23 = t10*t22
      t24 = t23*t3
      t26 = t1**2
      t27 = P*t26
      t28 = t27*t10
      t29 = t22*SI(4)
      t30 = t3*DIRP(3,1)
      t31 = t5*DIRP(3,2)
      t33 = t30+t31+t7*DIRP(3,3)
      t34 = t3*t33
      t36 = DIRP(3,1)-0.3E1*t34*t10
      t39 = Q*t26
      t40 = t39*t10
      t41 = t22*SI(5)
      t44 = RR*t26
      t45 = t44*t10
      t46 = t22*SI(6)
      t50 = t1*TLEN
      t51 = t50*SI(2)
      t53 = t26*t10
      t54 = t3*DIRP(1,1)
      t55 = t5*DIRP(1,2)
      t57 = t54+t55+t7*DIRP(1,3)
      t58 = t3*t57
      t60 = DIRP(1,1)-0.3E1*t58*t10
      t63 = t3*DIRP(2,1)
      t64 = t5*DIRP(2,2)
      t66 = t63+t64+t7*DIRP(2,3)
      t67 = t3*t66
      t69 = DIRP(2,1)-0.3E1*t67*t10
      t73 = t50*SI(3)
      t80 = t23*t5
      t82 = t5*t33
      t84 = DIRP(3,2)-0.3E1*t82*t10
      t93 = t5*t57
      t95 = DIRP(1,2)-0.3E1*t93*t10
      t98 = t5*t66
      t100 = DIRP(2,2)-0.3E1*t98*t10
      t110 = t23*t7
      t114 = DIRP(3,3)-0.3E1*t7*t33*t10
      t125 = DIRP(1,3)-0.3E1*t7*t57*t10
      t130 = DIRP(2,3)-0.3E1*t7*t66*t10
      t140 = t9**2
      t141 = 1/t140
      t142 = t2*t141
      t143 = t22*t3
      t144 = 2*XFP-2*XC(1)
      t145 = t143*t144
      t147 = t2*t10
      t148 = t21**2
      t150 = 1/t148/t21
      t151 = t150*t3
      t152 = t151*t144
      t154 = t2*t23
      t155 = t27*t141
      t156 = t36*t144
      t159 = t150*SI(4)
      t162 = t33*t10
      t164 = t141*t144
      t166 = -0.3E1*t162-0.3E1*t30*t10+0.3E1*t34*t164
      t169 = t39*t141
      t172 = t150*SI(5)
      t177 = t44*t141
      t180 = t150*SI(6)
      t185 = -t142*t145-0.5E0*t147*t152+0.1E1*t154+0.1E1*t155*t29*t156+
     #   0.5E0*t28*t159*t156-t28*t29*t166+0.2E1*t169*t41*t156+0.1E1*
     #   t40*t172*t156-0.2E1*t40*t41*t166+0.1E1*t177*t46*t156+0.5E0*
     #   t45*t180*t156-t45*t46*t166
      t186 = t51*t141
      t188 = t51*t10
      t190 = t51*t23
      t192 = t26*t141*t22
      t193 = SI(4)*t60
      t194 = t193*t144
      t196 = t53*t150
      t198 = t57*t10
      t201 = -0.3E1*t198-0.3E1*t54*t10+0.3E1*t58*t164
      t204 = SI(5)*t69
      t205 = t204*t144
      t208 = t66*t10
      t211 = -0.3E1*t208-0.3E1*t63*t10+0.3E1*t67*t164
      t215 = t73*t141
      t217 = t73*t10
      t219 = t73*t23
      t220 = SI(5)*t60
      t221 = t220*t144
      t226 = SI(6)*t69
      t227 = t226*t144
      t233 = 2*YFP-2*XC(2)
      t234 = t143*t233
      t236 = t151*t233
      t238 = t36*t233
      t245 = t141*t233
      t247 = -0.3E1*t3*DIRP(3,2)*t10+0.3E1*t34*t245
      t262 = -t142*t234-0.5E0*t147*t236+0.1E1*t155*t29*t238+0.5E0*t28*
     #  t159*t238-t28*t29*t247+0.2E1*t169*t41*t238+0.1E1*t40*t172*t238-
     #  0.2E1*t40*t41*t247+0.1E1*t177*t46*t238+0.5E0*t45*t180*t238-
     #  t45*t46*t247
      t265 = t193*t233
      t271 = -0.3E1*t3*DIRP(1,2)*t10+0.3E1*t58*t245
      t274 = t204*t233
      t280 = -0.3E1*t3*DIRP(2,2)*t10+0.3E1*t67*t245
      t286 = t220*t233
      t291 = t226*t233
      t297 = 2*ZFP-2*XC(3)
      t298 = t143*t297
      t300 = t151*t297
      t302 = t36*t297
      t309 = t141*t297
      t311 = -0.3E1*t3*DIRP(3,3)*t10+0.3E1*t34*t309
      t326 = -t142*t298-0.5E0*t147*t300+0.1E1*t155*t29*t302+0.5E0*t28*
     #   t159*t302-t28*t29*t311+0.2E1*t169*t41*t302+0.1E1*t40*t172*t302-
     #   0.2E1*t40*t41*t311+0.1E1*t177*t46*t302+0.5E0*t45*t180*t302-
     #   t45*t46*t311
      t329 = t193*t297
      t335 = -0.3E1*t3*DIRP(1,3)*t10+0.3E1*t58*t309
      t338 = t204*t297
      t344 = -0.3E1*t3*DIRP(2,3)*t10+0.3E1*t67*t309
      t350 = t220*t297
      t355 = t226*t297
      t361 = t22*t5
      t362 = t361*t233
      t364 = t150*t5
      t365 = t364*t233
      t367 = t84*t233
      t374 = -0.3E1*t162-0.3E1*t31*t10+0.3E1*t82*t245
      t389 = -t142*t362-0.5E0*t147*t365+0.1E1*t154+0.1E1*t155*t29*t367+
     #  0.5E0*t28*t159*t367-t28*t29*t374+0.2E1*t169*t41*t367+0.1E1*t40*
     #  t172*t367-0.2E1*t40*t41*t374+0.1E1*t177*t46*t367+0.5E0*t45*
     #  t180*t367-t45*t46*t374
      t392 = SI(4)*t95
      t393 = t392*t233
      t398 = -0.3E1*t198-0.3E1*t55*t10+0.3E1*t93*t245
      t401 = SI(5)*t100
      t402 = t401*t233
      t407 = -0.3E1*t208-0.3E1*t64*t10+0.3E1*t98*t245
      t413 = SI(5)*t95
      t414 = t413*t233
      t419 = SI(6)*t100
      t420 = t419*t233
      t426 = t361*t297
      t428 = t364*t297
      t430 = t84*t297
      t438 = -0.3E1*t5*DIRP(3,3)*t10+0.3E1*t82*t309
      t453 = -t142*t426-0.5E0*t147*t428+0.1E1*t155*t29*t430+0.5E0*t28*
     #   t159*t430-t28*t29*t438+0.2E1*t169*t41*t430+0.1E1*t40*t172*
     #   t430-0.2E1*t40*t41*t438+0.1E1*t177*t46*t430+0.5E0*t45*t180*
     #   t430-t45*t46*t438
      t456 = t392*t297
      t462 = -0.3E1*t5*DIRP(1,3)*t10+0.3E1*t93*t309
      t465 = t401*t297
      t471 = -0.3E1*t5*DIRP(2,3)*t10+0.3E1*t98*t309
      t477 = t413*t297
      t482 = t419*t297
      VX = 0.1E1*t2*t24-t28*t29*t36-0.2E1*t40*t41*t36-t45*t46*t36
      VXX = 0.1E1*t51*t24-t53*t29*t60-t53*t41*t69
      VXY = 0.1E1*t73*t24-t53*t41*t60-t53*t46*t69
      VY = 0.1E1*t2*t80-t28*t29*t84-0.2E1*t40*t41*t84-t45*t46*t84
      VYX = 0.1E1*t51*t80-t53*t29*t95-t53*t41*t100
      VYY = 0.1E1*t73*t80-t53*t41*t95-t53*t46*t100
      VZ = 0.1E1*t2*t110-t28*t29*t114-0.2E1*t40*t41*t114-t45*t46*t114
      VZX = 0.1E1*t51*t110-t53*t29*t125-t53*t41*t130
      VZY = 0.1E1*t73*t110-t53*t41*t125-t53*t46*t130
      VXDX = t185
      VXXDX = -t186*t145-0.5E0*t188*t152+0.1E1*t190+0.1E1*t192*t194+
     #   0.5E0*t196*t194-t53*t29*t201+0.1E1*t192*t205+0.5E0*t196*t205-
     #   t53*t41*t211
      VXYDX = -t215*t145-0.5E0*t217*t152+0.1E1*t219+0.1E1*t192*t221+
     #   0.5E0*t196*t221-t53*t41*t201+0.1E1*t192*t227+0.5E0*t196*t227-
     #   t53*t46*t211
      VXDY = t262
      VXXDY = -t186*t234-0.5E0*t188*t236+0.1E1*t192*t265+0.5E0*t196*
     #   t265-t53*t29*t271+0.1E1*t192*t274+0.5E0*t196*t274-t53*t41*t280
      VXYDY = -t215*t234-0.5E0*t217*t236+0.1E1*t192*t286+0.5E0*t196*
     #   t286-t53*t41*t271+0.1E1*t192*t291+0.5E0*t196*t291-t53*t46*t280
      VXDZ = t326
      VXXDZ = -t186*t298-0.5E0*t188*t300+0.1E1*t192*t329+0.5E0*t196*
     #   t329-t53*t29*t335+0.1E1*t192*t338+0.5E0*t196*t338-t53*t41*t344
      VXYDZ = -t215*t298-0.5E0*t217*t300+0.1E1*t192*t350+0.5E0*t196*
     #   t350-t53*t41*t335+0.1E1*t192*t355+0.5E0*t196*t355-t53*t46*t344
      VYDY = t389
      VYXDY = -t186*t362-0.5E0*t188*t365+0.1E1*t190+0.1E1*t192*t393+
     #   0.5E0*t196*t393-t53*t29*t398+0.1E1*t192*t402+0.5E0*t196*t402-
     #   t53*t41*t407
      VYYDY = -t215*t362-0.5E0*t217*t365+0.1E1*t219+0.1E1*t192*t414+
     #   0.5E0*t196*t414-t53*t41*t398+0.1E1*t192*t420+0.5E0*t196*t420-
     #   t53*t46*t407
      VYDZ = t453
      VYXDZ = -t186*t426-0.5E0*t188*t428+0.1E1*t192*t456+0.5E0*t196*
     #   t456-t53*t29*t462+0.1E1*t192*t465+0.5E0*t196*t465-t53*t41*t471
      VYYDZ = -t215*t426-0.5E0*t217*t428+0.1E1*t192*t477+0.5E0*t196*
     #   t477-t53*t41*t462+0.1E1*t192*t482+0.5E0*t196*t482-t53*t46*t471

      VZDZ = -VXDX - VYDY
      VYDX = VXDY
      VZDX = VXDZ
      VZDY = VYDZ
      VZXDZ = -VXXDX - VYXDY
      VYXDX = VXXDY
      VZXDX = VXXDZ
      VZXDY = VYXDZ
      VZYDZ = -VXYDX - VYYDY
      VYYDX = VXYDY
      VZYDX = VXYDZ
      VZYDY = VYYDZ

      RETURN
      END



      SUBROUTINE FFDZ(XC,SI,TLEN,XFP,YFP,ZFP,P,Q,RR,VXDXZ,VXDYZ,VXDZZ,
     *                VYDYZ,VYDZZ,VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     *                VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,DIRP)

      DIMENSION XC(3),SI(15),DIRP(3,3)

      t1 = TLEN**2
      t2 = t1*SI(1)
      t3 = XFP-XC(1)
      t4 = t3**2
      t5 = YFP-XC(2)
      t6 = t5**2
      t7 = ZFP-XC(3)
      t8 = t7**2
      t9 = t4+t6+t8
      t10 = 1/t9
      t11 = t2*t10
      t12 = XFP**2
      t14 = XC(1)**2
      t15 = YFP**2
      t17 = XC(2)**2
      t18 = ZFP**2
      t20 = XC(3)**2
      t22 = sqrt(t12-2*XFP*XC(1)+t14+t15-2*YFP*XC(2)+t17+t18-
     #           2*ZFP*XC(3)+t20)
      t23 = t22**2
      t24 = t23**2
      t26 = 1/t24/t22
      t27 = t26*t3
      t28 = 2*XFP-2*XC(1)
      t29 = 2*ZFP-2*XC(3)
      t30 = t28*t29
      t31 = t27*t30
      t33 = t9**2
      t35 = 1/t33/t9
      t36 = t2*t35
      t37 = 1/t22
      t38 = t37*t3
      t39 = t38*t30
      t41 = 1/t33
      t42 = t2*t41
      t44 = 1/t23/t22
      t45 = t44*t3
      t46 = t45*t30
      t48 = t41*t37
      t49 = t48*t29
      t50 = t2*t49
      t51 = t10*t44
      t52 = t51*t29
      t53 = t2*t52
      t54 = 2.00*Q
      t55 = t1**2
      t56 = t54*t55
      t57 = t10*t37
      t58 = DIRP(3,3)*t10
      t59 = t3*DIRP(3,1)
      t60 = t5*DIRP(3,2)
      t62 = t59+t60+t7*DIRP(3,3)
      t64 = t62*t41*t29
      t65 = t41*t29
      t67 = t3*DIRP(3,3)
      t68 = t41*t28
      t70 = t3*t62
      t72 = t35*t28*t29
      t74 = -0.3E1*t58+0.3E1*t64+0.3E1*t59*t65+0.3E1*t67*t68 -
     #      0.6E1*t70*t72
      t78 = RR*t55
      t79 = t78*t41
      t80 = t37*SI(6)
      t83 = -0.3E1*t67*t10+0.3E1*t70*t65
      t84 = t83*t28
      t87 = t41*t44
      t88 = t78*t87
      t90 = DIRP(3,1)-0.3E1*t70*t10
      t91 = SI(6)*t90
      t92 = t91*t30
      t94 = t55*t10
      t95 = t54*t94
      t96 = t44*SI(5)
      t97 = t62*t10
      t101 = (-0.3E1*t97-0.3E1*t59*t10+0.3E1*t70*t68)*t29
      t104 = t55*t41
      t105 = t54*t104
      t106 = t37*SI(5)
      t109 = t35*t37
      t110 = t78*t109
      t112 = P*t55
      t113 = t112*t109
      t114 = SI(4)*t90
      t115 = t114*t30
      t117 = t112*t87
      t119 = t112*t41
      t120 = t37*SI(4)
      t123 = 0.75E0*t11*t31+0.2E1*t36*t39+0.1E1*t42*t46-t50-0.5E0*t53-
     #   t56*t57*SI(5)*t74+0.1E1*t79*t80*t84-t88*t92+0.5E0*t95*t96*
     #   t101+0.1E1*t105*t106*t101-0.2E1*t110*t92-0.2E1*t113*t115-t117*
     #   t115+0.1E1*t119*t120*t84
      t124 = t10*t26
      t125 = t112*t124
      t127 = t112*t10
      t128 = t44*SI(4)
      t135 = t78*t10
      t136 = t44*SI(6)
      t141 = t55*t35
      t142 = t54*t141
      t144 = t90*t28*t29
      t151 = t78*t124
      t157 = t26*SI(5)
      t164 = -0.75E0*t125*t115+0.5E0*t127*t128*t84+0.1E1*t119*t120*t101+
     #   0.5E0*t127*t128*t101+0.5E0*t135*t136*t101-t127*t120*t74-0.2E1*
     #   t142*t106*t144-t105*t96*t144+0.1E1*t105*t106*t84-0.75E0*t151*
     #   t92+0.5E0*t135*t136*t84+0.1E1*t79*t80*t101-0.75E0*t95*t157*
     #   t144+0.5E0*t95*t96*t84-t135*t80*t74
      t166 = t3*DIRP(3,2)
      t168 = 2*YFP-2*XC(2)
      t169 = t41*t168
      t172 = t35*t168*t29
      t174 = 0.3E1*t166*t65+0.3E1*t67*t169-0.6E1*t70*t172
      t181 = (-0.3E1*t166*t10+0.3E1*t70*t169)*t29
      t188 = t83*t168
      t192 = t90*t168*t29
      t201 = t168*t29
      t202 = t45*t201
      t204 = t27*t201
      t206 = t114*t201
      t209 = -t56*t57*SI(5)*t174+0.1E1*t119*t120*t181+0.5E0*t127*t128*
     #   t181-t127*t120*t174+0.1E1*t105*t106*t188-0.75E0*t95*t157*t192+
     #   0.5E0*t95*t96*t188+0.1E1*t105*t106*t181+0.5E0*t95*t96*t181+
     #   0.1E1*t42*t202+0.75E0*t11*t204-0.2E1*t113*t206-t117*t206
      t215 = t91*t201
      t229 = t38*t201
      t235 = 0.1E1*t119*t120*t188-0.75E0*t125*t206+0.5E0*t127*t128*t188-
     #0.2E1*t110*t215-t88*t215+0.1E1*t79*t80*t188-0.75E0*t151*t215+0.5E0
     #*t135*t136*t188+0.1E1*t79*t80*t181+0.5E0*t135*t136*t181-t135*t80*
     #t174+0.2E1*t36*t229-0.2E1*t142*t106*t192-t105*t96*t192
      t237 = t29**2
      t238 = t38*t237
      t240 = t45*t237
      t242 = t48*t3
      t244 = t27*t237
      t246 = t51*t3
      t248 = t112*t35
      t249 = t90*t237
      t254 = t83*t29
      t260 = t35*t237
      t263 = 0.6E1*t67*t65-0.6E1*t70*t260+0.6E1*t70*t41
      t270 = SI(5)*t90
      t275 = 0.2E1*t36*t238+0.1E1*t42*t240-0.2E1*t2*t242+0.75E0*t11*t244
     #-t2*t246-0.2E1*t248*t120*t249-t119*t128*t249+0.2E1*t119*t120*t254+
     #0.1E1*t127*t128*t90-t127*t120*t263-t105*t96*t249+0.2E1*t105*t106*
     #t254+0.2E1*t56*t48*t270-0.75E0*t95*t157*t249
      t285 = t26*SI(4)
      t290 = t78*t35
      t299 = t26*SI(6)
      t310 = 0.1E1*t95*t96*t254+0.1E1*t56*t51*t270-t56*t57*SI(5)*t263+
     #   0.2E1*t119*t120*t90-0.75E0*t127*t285*t249+0.1E1*t127*t128*t254-
     #   0.2E1*t290*t80*t249-t79*t136*t249+0.2E1*t79*t80*t254+0.2E1*t79*
     #   t80*t90-0.75E0*t135*t299*t249+0.1E1*t135*t136*t254+0.1E1*t135*
     #   t136*t90-t135*t80*t263-0.2E1*t142*t106*t249
      t313 = t5*t62
      t316 = (-0.3E1*t97-0.3E1*t60*t10+0.3E1*t313*t169)*t29
      t320 = DIRP(3,2)-0.3E1*t313*t10
      t322 = t320*t168*t29
      t327 = t5*DIRP(3,3)
      t330 = -0.3E1*t327*t10+0.3E1*t313*t65
      t331 = t330*t168
      t345 = -0.3E1*t58+0.3E1*t64+0.3E1*t60*t65+0.3E1*t327*t169 -
     #       0.6E1*t313*t172
      t350 = SI(4)*t320*t201
      t356 = 0.1E1*t119*t120*t316-t50-0.5E0*t53-0.2E1*t142*t106*t322-
     #   t105*t96*t322+0.1E1*t105*t106*t331-0.75E0*t95*t157*t322+0.5E0*
     #   t95*t96*t331+0.1E1*t105*t106*t316+0.5E0*t95*t96*t316-t56*t57*
     #   SI(5)*t345-0.75E0*t125*t350+0.5E0*t127*t128*t331 +
     #   0.5E0*t127*t128*t316
      t360 = SI(6)*t320*t201
      t374 = t37*t5
      t375 = t374*t201
      t377 = t44*t5
      t378 = t377*t201
      t380 = t26*t5
      t381 = t380*t201
      t387 = -t127*t120*t345-0.2E1*t110*t360-t88*t360+0.1E1*t79*t80*
     #   t331-0.75E0*t151*t360+0.5E0*t135*t136*t331+0.1E1*t79*t80*t316+
     #   0.5E0*t135*t136*t316-t135*t80*t345+0.2E1*t36*t375+0.1E1*t42*
     #   t378+0.75E0*t11*t381-0.2E1*t113*t350-t117*t350+0.1E1*t119*
     #   t120*t331
      t391 = t374*t237
      t393 = t377*t237
      t395 = t320*t237
      t398 = t330*t29
      t406 = 0.6E1*t327*t65-0.6E1*t313*t260+0.6E1*t313*t41
      t409 = SI(5)*t320
      t416 = t48*t5
      t418 = t380*t237
      t420 = t51*t5
      t424 = 0.2E1*t119*t120*t320+0.2E1*t36*t391+0.1E1*t42*t393-0.75E0*
     #   t127*t285*t395+0.1E1*t127*t128*t398+0.1E1*t127*t128*t320-t127*
     #   t120*t406+0.2E1*t56*t48*t409-0.75E0*t95*t157*t395+0.1E1*t95*
     #   t96*t398-0.2E1*t2*t416+0.75E0*t11*t418-t2*t420-0.2E1*t248*t120*
     #   t395
      t456 = -t119*t128*t395+0.2E1*t119*t120*t398+0.1E1*t56*t51*t409-
     #   t56*t57*SI(5)*t406-0.2E1*t290*t80*t395-t79*t136*t395+0.2E1*t79*
     #   t80*t398+0.2E1*t79*t80*t320-0.75E0*t135*t299*t395+0.1E1*t135*
     #   t136*t398+0.1E1*t135*t136*t320-t135*t80*t406-0.2E1*t142*t106*
     #   t395-t105*t96*t395+0.2E1*t105*t106*t398
      t458 = t1*TLEN
      t459 = t458*SI(2)
      t460 = t459*t35
      t462 = t459*t41
      t464 = t459*t10
      t466 = t459*t49
      t467 = t459*t52
      t468 = t141*t37
      t469 = t3*DIRP(1,1)
      t470 = t5*DIRP(1,2)
      t472 = t469+t470+t7*DIRP(1,3)
      t473 = t3*t472
      t475 = DIRP(1,1)-0.3E1*t473*t10
      t476 = SI(4)*t475
      t477 = t476*t30
      t479 = t104*t44
      t481 = t104*t37
      t482 = t3*DIRP(1,3)
      t485 = -0.3E1*t482*t10+0.3E1*t473*t65
      t486 = SI(4)*t485
      t487 = t486*t28
      t489 = t94*t26
      t491 = t94*t44
      t494 = t472*t10
      t497 = -0.3E1*t494-0.3E1*t469*t10+0.3E1*t473*t68
      t499 = SI(4)*t497*t29
      t502 = DIRP(1,3)*t10
      t504 = t472*t41*t29
      t508 = -0.3E1*t502+0.3E1*t504+0.3E1*t469*t65+0.3E1*t482*t68 -
     #       0.6E1*t473*t72
      t511 = t3*DIRP(2,1)
      t512 = t5*DIRP(2,2)
      t514 = t511+t512+t7*DIRP(2,3)
      t515 = t3*t514
      t517 = DIRP(2,1)-0.3E1*t515*t10
      t518 = SI(5)*t517
      t519 = t518*t30
      t522 = t3*DIRP(2,3)
      t525 = -0.3E1*t522*t10+0.3E1*t515*t65
      t526 = SI(5)*t525
      t527 = t526*t28
      t531 = t514*t10
      t534 = -0.3E1*t531-0.3E1*t511*t10+0.3E1*t515*t68
      t536 = SI(5)*t534*t29
      t539 = DIRP(2,3)*t10
      t541 = t514*t41*t29
      t545 = -0.3E1*t539+0.3E1*t541+0.3E1*t511*t65+0.3E1*t522*t68 -
     #       0.6E1*t515*t72
      t548 = 0.1E1*t481*t499+0.5E0*t491*t499-t94*t120*t508-0.2E1*t468*
     #   t519-t479*t519+0.1E1*t481*t527-0.75E0*t489*t519+0.5E0*t491*
     #   t527+0.1e1*t481*t536+0.5E0*t491*t536-t94*t106*t545
      t553 = t476*t201
      t556 = t486*t168
      t560 = t3*DIRP(1,2)
      t563 = -0.3E1*t560*t10+0.3E1*t473*t169
      t565 = SI(4)*t563*t29
      t571 = 0.3E1*t560*t65+0.3E1*t482*t169-0.6E1*t473*t172
      t574 = t518*t201
      t577 = t526*t168
      t581 = t3*DIRP(2,2)
      t584 = -0.3E1*t581*t10+0.3E1*t515*t169
      t586 = SI(5)*t584*t29
      t592 = 0.3E1*t581*t65+0.3E1*t522*t169-0.6E1*t515*t172
      t595 = 0.2E1*t460*t229+0.1E1*t462*t202+0.75E0*t464*t204-0.2E1*
     #   t468*t553-t479*t553+0.1E1*t481*t556-0.75E0*t489*t553+0.5E0*
     #   t491*t556+0.1E1*t481*t565+0.5E0*t491*t565-t94*t120*t571-0.2E1*
     #   t468*t574-t479*t574+0.1E1*t481*t577-0.75E0*t489*t574+0.5E0*
     #   t491*t577+0.1E1*t481*t586+0.5E0*t491*t586-t94*t106*t592
      t601 = 0.6E1*t482*t65-0.6E1*t473*t260+0.6E1*t473*t41
      t604 = t518*t237
      t607 = t526*t29
      t615 = t476*t237
      t618 = t486*t29
      t625 = 0.6E1*t522*t65-0.6E1*t515*t260+0.6E1*t515*t41
      t636 = 0.1E1*t491*t618+0.1E1*t94*t96*t517-t94*t106*t625+0.2E1*
     #   t460*t238+0.1E1*t462*t240-0.2E1*t459*t242+0.75E0*t464*t244-
     #   t459*t246-0.2E1*t468*t615-t479*t615+0.2E1*t481*t618
      t641 = t5*t472
      t643 = DIRP(1,2)-0.3E1*t641*t10
      t644 = SI(4)*t643
      t645 = t644*t201
      t648 = t5*DIRP(1,3)
      t651 = -0.3E1*t648*t10+0.3E1*t641*t65
      t652 = SI(4)*t651
      t653 = t652*t168
      t660 = -0.3E1*t494-0.3E1*t470*t10+0.3E1*t641*t169
      t662 = SI(4)*t660*t29
      t668 = -0.3E1*t502+0.3E1*t504+0.3E1*t470*t65+0.3E1*t648*t169 -
     #   0.6E1*t641*t172
      t671 = t5*t514
      t673 = DIRP(2,2)-0.3E1*t671*t10
      t674 = SI(5)*t673
      t675 = t674*t201
      t678 = t5*DIRP(2,3)
      t681 = -0.3E1*t678*t10+0.3E1*t671*t65
      t682 = SI(5)*t681
      t683 = t682*t168
      t689 = -0.3E1*t531-0.3E1*t512*t10+0.3E1*t671*t169
      t691 = SI(5)*t689*t29
      t697 = -0.3E1*t539+0.3E1*t541+0.3E1*t512*t65+0.3E1*t678*t169 -
     #   0.6E1*t671*t172
      t700 = 0.1E1*t481*t662+0.5E0*t491*t662-t94*t120*t668-0.2E1*t468*
     #   t675-t479*t675+0.1E1*t481*t683-0.75E0*t489*t675+0.5E0*t491*
     #   t683+0.1e1*t481*t691+0.5E0*t491*t691-t94*t106*t697
      t705 = t644*t237
      t707 = t652*t29
      t714 = 0.6E1*t648*t65-0.6E1*t641*t260+0.6E1*t641*t41
      t719 = t674*t237
      t721 = t682*t29
      t736 = 0.6E1*t678*t65-0.6E1*t671*t260+0.6E1*t671*t41
      t742 = -0.2E1*t459*t416+0.75E0*t464*t418-t459*t420-0.2E1*t468*
     #   t705-t479*t705+0.2E1*t481*t707+0.1E1*t94*t96*t673-t94*t106*
     #   t736-0.2E1*t468*t719-t479*t719+0.2E1*t481*t721
      t744 = t458*SI(3)
      t745 = t744*t35
      t747 = t744*t41
      t749 = t744*t10
      t751 = t744*t49
      t752 = t744*t52
      t753 = SI(5)*t475
      t754 = t753*t30
      t757 = SI(5)*t485
      t758 = t757*t28
      t764 = SI(5)*t497*t29
      t769 = SI(6)*t517
      t770 = t769*t30
      t773 = SI(6)*t525
      t774 = t773*t28
      t779 = SI(6)*t534*t29
      t784 = 0.1E1*t481*t764+0.5E0*t491*t764-t94*t106*t508-0.2E1*t468*
     #   t770-t479*t770+0.1E1*t481*t774-0.75E0*t489*t770+0.5E0*t491*
     #   t774+0.1e1*t481*t779+0.5E0*t491*t779-t94*t80*t545
      t789 = t753*t201
      t792 = t757*t168
      t797 = SI(5)*t563*t29
      t802 = t769*t201
      t805 = t773*t168
      t810 = SI(6)*t584*t29
      t815 = 0.2E1*t745*t229+0.1E1*t747*t202+0.75E0*t749*t204-0.2E1*
     #   t468*t789-t479*t789+0.1E1*t481*t792-0.75E0*t489*t789+0.5E0*
     #   t491*t792+0.1E1*t481*t797+0.5E0*t491*t797-t94*t106*t571-0.2E1*
     #   t468*t802-t479*t802+0.1E1*t481*t805-0.75E0*t489*t802+0.5E0*
     #   t491*t805+0.1E1*t481*t810+0.5E0*t491*t810-t94*t80*t592
      t821 = t753*t237
      t824 = t757*t29
      t833 = t769*t237
      t836 = t773*t29
      t848 = 0.1E1*t491*t824+0.1E1*t94*t96*t475-0.2E1*t468*t833-t479*
     #   t833+0.2E1*t481*t836+0.2E1*t104*t80*t517-0.75E0*t489*t833+
     #   0.1E1*t491*t836+0.1E1*t94*t136*t517-t94*t80*t625-t94*t106*t601
      t853 = SI(5)*t643
      t854 = t853*t201
      t857 = SI(5)*t651
      t858 = t857*t168
      t864 = SI(5)*t660*t29
      t869 = SI(6)*t673
      t870 = t869*t201
      t873 = SI(6)*t681
      t874 = t873*t168
      t879 = SI(6)*t689*t29
      t884 = 0.1E1*t481*t864+0.5E0*t491*t864-t94*t106*t668-0.2E1*t468*
     #   t870-t479*t870+0.1E1*t481*t874-0.75E0*t489*t870+0.5E0*t491*
     #   t874+0.1e1*t481*t879+0.5E0*t491*t879-t94*t80*t697
      t888 = t853*t237
      t890 = t869*t237
      t892 = t873*t29
      t906 = t857*t29
      t918 = 0.75E0*t749*t418-t744*t420-0.2E1*t468*t888-t479*t888+0.2E1*
     #   t481*t906+0.1E1*t94*t136*t673-t94*t80*t736+0.1E1*t491*t906+
     #   0.1E1*t94*t96*t643-t94*t106*t714-0.2E1*t468*t890
      VXDXZ = t123+t164
      VXDYZ = t209+t235
      VXDZZ = t275+t310
      VYDYZ = t356+t387
      VYDZZ = t424+t456
      VXXDXZ = 0.2E1*t460*t39+0.1E1*t462*t46+0.75E0*t464*t31-t466-
     #   0.5E0*t467-0.2E1*t468*t477-t479*t477+0.1E1*t481*t487-0.75E0*
     #   t489*t477+0.5E0*t491*t487+t548
      VXXDYZ = t595
      VXXDZZ = 0.1E1*t94*t128*t475-t94*t120*t601-0.2E1*t468*t604-t479*
     #   t604+0.2E1*t481*t607+0.2E1*t104*t106*t517-0.75E0*t489*t604+
     #   0.1E1*t491*t607+0.2E1*t104*t120*t475-0.75E0*t489*t615+t636
      VYXDYZ = 0.2E1*t460*t375+0.1E1*t462*t378+0.75E0*t464*t381-t466-
     #   0.5E0*t467-0.2E1*t468*t645-t479*t645+0.1E1*t481*t653-0.75E0*
     #   t489*t645+0.5E0*t491*t653+t700
      VYXDZZ = 0.2E1*t460*t391+0.2E1*t104*t120*t643-0.75E0*t489*t705+
     #   0.1E1*t491*t707+0.1E1*t94*t128*t643-t94*t120*t714+0.2E1*t104*
     #   t106*t673-0.75E0*t489*t719+0.1E1*t491*t721+0.1E1*t462*t393+t742
      VXYDXZ = 0.2E1*t745*t39+0.1E1*t747*t46+0.75E0*t749*t31-t751-
     #   0.5e0*t752-0.2E1*t468*t754-t479*t754+0.1E1*t481*t758-0.75E0*
     #   t489*t754+0.5E0*t491*t758+t784
      VXYDYZ = t815
      VXYDZZ = 0.2E1*t745*t238+0.1E1*t747*t240-0.2E1*t744*t242+0.75E0*
     #   t749*t244-t744*t246-0.2E1*t468*t821-t479*t821+0.2E1*t481*t824+
     #   0.2e1*t104*t106*t475-0.75E0*t489*t821+t848
      VYYDYZ = 0.2E1*t745*t375+0.1E1*t747*t378+0.75E0*t749*t381-t751-
     #   0.5E0*t752-0.2E1*t468*t854-t479*t854+0.1E1*t481*t858-0.75E0*
     #   t489*t854+0.5E0*t491*t858+t884
      VYYDZZ = 0.2E1*t104*t106*t643-0.75E0*t489*t888-t479*t890+0.2E1*
     #   t481*t892+0.2E1*t104*t80*t673-0.75E0*t489*t890+0.1E1*t491*
     #   t892+0.2e1*t745*t391+0.1E1*t747*t393-0.2E1*t744*t416+t918

      RETURN
      END


C***********************************************************************
C
C     SUBROUTINE LINSOR()
C
C     Computes the coefficients needed to determine the slope of the
C     linear variation of the source strength, based on the geometry
C     of the panel and the neighboring panels
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Revised (August 1994) to include symmetry about x-y and/or x-z
C     planes  (removed June 1995)
C
C     Programmed by: Michael Hughes, July 1994
C
C***********************************************************************

      SUBROUTINE LINSOR(XC,DIRP,IPAN,I,CX,CY)

      DIMENSION XI(6),ETA(6),DIRP(3,3),CX(0:4),CY(0:4),XC(3,*)
      DIMENSION IPAN(0:4,*)
      DATA ZERO,ONE/0.00E00,1.0E00/

C..... Rotate the control points for the neighboring panels into
C      the current panel coordinate system
      M = IPAN(0,I)

C..... Add check to turn off linear term if panel has no neighbors
      DO 5 JJ=0,4
         CX(JJ) = ZERO
         CY(JJ) = ZERO
 5    CONTINUE
      IF (M.EQ.0) RETURN

      DO 10 JJ=1,M
         J=IPAN(JJ,I)
         XXP = XC(1,J)-XC(1,I)
         YYP = XC(2,J)-XC(2,I)
         ZZP = XC(3,J)-XC(3,I)
         XI(JJ) = DIRP(1,1)*XXP + DIRP(1,2)*YYP + DIRP(1,3)*ZZP
         ETA(JJ) = DIRP(2,1)*XXP + DIRP(2,2)*YYP + DIRP(2,3)*ZZP
 10   CONTINUE

      X = ZERO
      Y = ZERO
      S = ZERO
      T = ZERO
      U = ZERO
      DO 20 J=1,M
         X = X + XI(J)
         Y = Y + ETA(J)
         S = S + XI(J)*XI(J)
         T = T + XI(J)*ETA(J)
         U = U + ETA(J)*ETA(J)
 20   CONTINUE

      D = S*U - T*T
      DET = ONE/D

      CX(0) = DET*(T*Y - U*X)
      CY(0) = DET*(T*X - S*Y)
      DO 30 K=1,M
         CX(K) = DET*( U*XI(K) - T*ETA(K) )
         CY(K) = DET*( S*ETA(K) - T*XI(K) )
 30   CONTINUE
      END

      SUBROUTINE PANFLT(XFP,YFP,ZFP,XC,LEVEL,XI,ETA,TLEN,DM32,DM41,
     &               DIRP,B32,B41,D12,D23,D34,D41,SI,RAT,
     &               VX,VY,VZ,VXX,VYX,VZX,VXY,VYY,VZY,
     *               VXDX,VXDY,VXDZ,VYDY,VYDZ,
     *               VXXDX,VXXDY,VXXDZ,VYXDY,VYXDZ,
     *               VXYDX,VXYDY,VXYDZ,VYYDY,VYYDZ,
     *               VXDXZ,VXDYZ,VXDZZ,VYDYZ,VYDZZ,
     *               VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     *               VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,ifo)

      DIMENSION XI(4),ETA(4),SI(15),XC(3),DIRP(3,3)

C..... Compute radius to panel control point and determine whether to
C      use near, intermediate or far field
      RS = (XFP-XC(1))**2 + (YFP-XC(2))**2 + (ZFP-XC(3))**2
      R = SQRT(RS)
      RAT = R/TLEN
      IF (RAT.LT.0.1 .and. ifo.eq.0) THEN
         XFPP = XFP + DIRP(3,1)*0.00001*TLEN
         YFPP = YFP + DIRP(3,2)*0.00001*TLEN
         ZFPP = ZFP + DIRP(3,3)*0.00001*TLEN
         RS = (XFPP-XC(1))**2 + (YFPP-XC(2))**2 + (ZFPP-XC(3))**2
         R = SQRT(RS)
         RAT = R/TLEN
      ELSE
         XFPP = XFP
         YFPP = YFP
         ZFPP = ZFP
      END IF
      IF (RAT.LT.5.00 .and. ifo.lt.2) THEN
C........ Convert to panel coordinates
         XP = XFPP-XC(1)
         YP = YFPP-XC(2)
         ZP = ZFPP-XC(3)
         X = DIRP(1,1)*XP + DIRP(1,2)*YP + DIRP(1,3)*ZP
         Y = DIRP(2,1)*XP + DIRP(2,2)*YP + DIRP(2,3)*ZP
         Z = DIRP(3,1)*XP + DIRP(3,2)*YP + DIRP(3,3)*ZP
      END IF

      IF (RAT.LT.2.5 .and. ifo.eq.0) THEN
c        print*,' near field flat'
C----------------------------------------------------------------------
C        Near Field (Exact closed form integral solutions)
C         Maple generated fortran code
C----------------------------------------------------------------------
         CALL FLNEAR(X,Y,Z,XI,ETA,RAT,TLEN,DM32,DM41,
     *                 B32,B41,D12,D23,D34,D41,VX,VY,VZ,
     *                 VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                 VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                 VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                 VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                 VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      ELSE IF (RAT.LT.5.0 .and. ifo.lt.2) THEN
C-------------------------------------------------------------------
C        Multipole expansion for intermediate field
C-------------------------------------------------------------------
c       print*,'intermediate field flat'
         CALL FLINT(X,Y,Z,TLEN,SI,VX,VY,VZ,
     *              VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *              VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *              VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *              VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *              VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      ELSE
c        print*,'far field flat'
C----------------------------------------------------------------------
C        Far Field Expansion (point source)
C----------------------------------------------------------------------
         CALL FLFAR(XFP,YFP,ZFP,TLEN,SI,XC,DIRP,VX,VY,VZ,
     *              VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *              VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *              VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *              VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *              VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      END IF

      IF (LEVEL.EQ.3) CALL FLATDZ(XC,SI,TLEN,XFP,YFP,ZFP,
     *                          VXDXZ,VXDYZ,VXDZZ,VYDYZ,VYDZZ,
     *                          VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     *                          VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,DIRP)

      RETURN
      END

C***********************************************************************
C
C     SUBROUTINE FLNEAR()
C
C     Computes the induced velocity and the derivatives of the velocity
C     from a FLAT panel with a linear source distribution, for the
C     case where the panel has four sides
C
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Programmed by: Michael Hughes, July 1994
C
C***********************************************************************

      SUBROUTINE FLNEAR(X,Y,Z,XI,ETA,RAT,TLEN,DM32,DM41,
     *                  B32,B41,D12,D23,D34,D41,VX,VY,VZ,
     *                  VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                  VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                  VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                  VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                  VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      DIMENSION XI(4),ETA(4)
      DATA TWOPI,TOL,TOLT,ZERO/6.283185308E+00,0.000001,0.001,0.0E00/

      if (abs(z/tlen) .lt. tol) z = sign(min(tol,(tol*tlen)),z)
      t1 = DM41**2
      t3 = 1/(0.1E1+t1)
      t4 = sqrt(t3)
      t5 = X**2
      t7 = XI(4)**2
      t8 = Y**2
      t10 = ETA(4)**2
      t11 = Z**2
      t12 = t5-2*X*XI(4)+t7+t8-2*Y*ETA(4)+t10+t11
      t13 = sqrt(t12)
      t15 = XI(1)**2
      t17 = ETA(1)**2
      t18 = t5-2*X*XI(1)+t15+t8-2*Y*ETA(1)+t17+t11
      t19 = sqrt(t18)
      t20 = t13+t19-D41
      t21 = t13+t19+D41
      t22 = 1/t21
      t24 = alog(t20*t22)
      t25 = t4*t24
      t26 = DM32**2
      t28 = 1/(0.1E1+t26)
      t29 = sqrt(t28)
      t31 = XI(2)**2
      t33 = ETA(2)**2
      t34 = t5-2*X*XI(2)+t31+t8-2*Y*ETA(2)+t33+t11
      t35 = sqrt(t34)
      t37 = XI(3)**2
      t39 = ETA(3)**2
      t40 = t5-2*X*XI(3)+t37+t8-2*Y*ETA(3)+t39+t11
      t41 = sqrt(t40)
      t42 = t35+t41-D23
      t43 = t35+t41+D23
      t44 = 1/t43
      t46 = alog(t42*t44)
      t47 = t29*t46
      t48 = 0.1E1*t25-t47
      t49 = t41+t13-D34
      t50 = t41+t13+D34
      t51 = 1/t50
      IF (D34.LT.TOL) THEN
         t53 = 0.000e00
      ELSE
         t53 = alog(t49*t51)
      END IF
      t54 = t19+t35-D12
      t55 = t19+t35+D12
      t56 = 1/t55
      IF (D12.LT.TOL) THEN
         t58=ZERO
      ELSE
         t58 = alog(t54*t56)
      END IF
      t59 = DM32*t29
      t60 = t59*t46
      t61 = DM41*t4
      t62 = t61*t24
      t63 = t53-t58+0.1E1*t60-t62
      t64 = Y-ETA(3)
      t65 = t64**2
      t70 = DM32*(t11+t65)-(X-XI(3))*t64
      t71 = 1/Z
      t72 = t70*t71
      t73 = 1/t41
      t76 = Y-ETA(2)
      t77 = t76**2
      t82 = DM32*(t11+t77)-(X-XI(2))*t76
      t83 = t82*t71
      t84 = 1/t35
      t87 = Y-ETA(1)
      t88 = t87**2
      t93 = DM41*(t11+t88)-(X-XI(1))*t87
      t94 = t93*t71
      t95 = 1/t19
      t98 = Y-ETA(4)
      t99 = t98**2
      t104 = DM41*(t11+t99)-(X-XI(4))*t98
      t105 = t104*t71
      t106 = 1/t13

      H32 = DM32*t64 - (X-XI(3))
      H41 = DM41*t87 - (X-XI(1))
      IF ( ABS(Z/TLEN).LT. TOLT ) THEN
         IF ( (RAT.LT.0.5) .AND. (H32*H41.LT.0.0)
     *         .AND. (t64*t87.LT.0) ) THEN
            VZO = TWOPI
         ELSE
            VZO = ZERO
         END IF
      ELSE
         t75 = atan(t72*t73)
         t86 = atan(t83*t84)
         t97 = atan(t94*t95)
         t108 = atan(t105*t106)
         vzo = t75-t86+t97-t108
      END IF

      t110 = 2*X-2*XI(4)
      t111 = t106*t110
      t112 = 2*X-2*XI(1)
      t113 = t95*t112
      t114 = t111/2+t113/2
      t116 = t21**2
      t118 = t20/t116
      t120 = t114*t22-t118*t114
      t122 = 1/t20
      t123 = t122*t21
      t125 = 2*X-2*XI(2)
      t126 = t84*t125
      t127 = 2*X-2*XI(3)
      t128 = t73*t127
      t129 = t126/2+t128/2
      t131 = t43**2
      t133 = t42/t131
      t135 = t129*t44-t133*t129
      t137 = 1/t42
      t138 = t137*t43
      t140 = 0.1E1*t4*t120*t123-t29*t135*t138
      t141 = 2*Y-2*ETA(4)
      t142 = t106*t141
      t143 = 2*Y-2*ETA(1)
      t144 = t95*t143
      t145 = t142/2+t144/2
      t148 = t145*t22-t118*t145
      t151 = 2*Y-2*ETA(2)
      t152 = t84*t151
      t153 = 2*Y-2*ETA(3)
      t154 = t73*t153
      t155 = t152/2+t154/2
      t158 = t155*t44-t133*t155
      t161 = 0.1E1*t4*t148*t123-t29*t158*t138
      t162 = Z*t106
      t163 = Z*t95
      t164 = t162+t163
      t167 = t164*t22-t118*t164
      t170 = Z*t84
      t171 = Z*t73
      t172 = t170+t171
      t175 = t172*t44-t133*t172
      t178 = 0.1E1*t4*t167*t123-t29*t175*t138
      t179 = t154/2+t142/2
      t181 = t50**2
      t183 = t49/t181
      t185 = t179*t51-t183*t179
      t186 = 1/t49
      t189 = t144/2+t152/2
      t191 = t55**2
      t193 = t54/t191
      t195 = t189*t56-t193*t189
      t196 = 1/t54
      t200 = t158*t137*t43
      t203 = t148*t122*t21
      t205 = t185*t186*t50-t195*t196*t55+0.1E1*t59*t200-t61*t203
      t206 = t171+t162
      t209 = t206*t51-t183*t206
      t212 = t163+t170
      t215 = t212*t56-t193*t212
      t219 = t175*t137*t43
      t222 = t167*t122*t21
      t224 = t209*t186*t50-t215*t196*t55+0.1E1*t59*t219-t61*t222
      t229 = (X-XI(2)-DM32*t76)*t29
      t234 = (X-XI(4)-DM41*t98)*t4
      t236 = Z*vzo
      t239 = DM32*(t35-t41)*t28
      t242 = DM41*(t13-t19)*t3
      t243 = t28*t29
      t245 = X-DM32*Y-B32
      t247 = t243*t245*t46
      t248 = t3*t4
      t250 = X-DM41*Y-B41
      t252 = t248*t250*t24
      t256 = (t41-t35)*t28
      t258 = (t19-t13)*t3
      t259 = DM32*t28
      t262 = t259*t29*t245*t46
      t263 = DM41*t3
      t266 = t263*t4*t250*t24
      t272 = t113/2+t126/2
      t277 = t196*t55
      t280 = t135*t137*t43
      t282 = t128/2+t111/2
      t287 = t186*t50
      t290 = t120*t122*t21
      t295 = t41**2
      t297 = 1/t295/t41
      t301 = t70**2
      t302 = 1/t11
      t307 = 1/(1+t301*t302/t40)
      t312 = t35**2
      t314 = 1/t312/t35
      t318 = t82**2
      t323 = 1/(1+t318*t302/t34)
      t328 = t19**2
      t330 = 1/t328/t19
      t334 = t93**2
      t339 = 1/(1+t334*t302/t18)
      t344 = t13**2
      t346 = 1/t344/t13
      t350 = t104**2
      t355 = 1/(1+t350*t302/t12)
      t366 = t243*t245
      t369 = t248*t250
      t371 = 0.2E1*t25-0.2E1*t47+X*t140-t87*(t272*t56-t193*t272)*t277-
     #   t229*t280+t64*(t282*t51-t183*t282)*t287+0.1E1*t234*t290+Z*
     #   (((-Y+ETA(3))*t71*t73-t72*t297*t127/2)*t307-((-Y+ETA(2))*t71*
     #   t84-t83*t314*t125/2)*t323+((-Y+ETA(1))*t71*t95-t94*t330*t112/2)
     #   *t339-((-Y+ETA(4))*t71*t106-t105*t346*t110/2)*t355)+0.1E1*DM32*
     #   (t126/2-t128/2)*t28+0.1E1*DM41*(t111/2-t113/2)*t3+0.1E1*t243*
     #   t46+0.1E1*t366*t280-t248*t24-t369*t290
      t412 = Z*(((DM32*t153-X+XI(3))*t71*t73-t72*t297*t153/2)*t307-
     #   ((DM32*t151-X+XI(2))*t71*t84-t83*t314*t151/2)*t323+((DM41*t143-
     #   X+XI(1))*t71*t95-t94*t330*t143/2)*t339-((DM41*t141-X+XI(4))*
     #   t71*t106-t105*t346*t141/2)*t355)
      t415 = DM32*(t152/2-t154/2)*t28
      t418 = DM41*(t142/2-t144/2)*t3
      t419 = t259*t47
      t420 = t366*t200
      t421 = t263*t25
      t422 = t369*t203
      t423 = X*t161-t58-t87*t195*t277+0.1E1*t60-t229*t200+t53+t64*t185*
     #   t287-t62+0.1E1*t234*t203+t412+0.1E1*t415+0.1E1*t418-t419+
     #   0.1E1*t420+0.1E1*t421-t422
      t456 = Z*((2*DM32*t73-t70*t302*t73-t70*t297)*t307-(2*DM32*t84-t82*
     #   t302*t84-t82*t314)*t323+(2*DM41*t95-t93*t302*t95-t93*t330)*
     #   t339-(2*DM41*t106-t104*t302*t106-t104*t346)*t355)
      t458 = t259*(t170-t171)
      t460 = t263*(t162-t163)
      t461 = t366*t219
      t462 = t369*t222
      t463 = X*t178-t87*t215*t277-t229*t219+t64*t209*t287+0.1E1*t234*
     #  t222+vzo+t456+0.1E1*t458+0.1E1*t460+0.1E1*t461-t462
      t466 = (t154/2-t152/2)*t28
      t468 = (t144/2-t142/2)*t3
      t470 = t26*t28*t47
      t471 = t259*t29
      t474 = t471*t245*t158*t138
      t476 = t1*t3*t25
      t477 = t263*t4
      t480 = t477*t250*t148*t123
      t484 = (t171-t170)*t28
      t486 = (t163-t162)*t3
      t489 = t471*t245*t175*t138
      t492 = t477*t250*t167*t123
      t518 = t53-t58+0.1E1*t60-t62+Y*t205-t415-t418+0.1E1*t419-t420-
     #   t421+0.1E1*t422+t412
      VX = t48
      VY = t63
      VZ = vzo
      VXDX = t140
      VXDY = t161
      VXDZ = t178
      VYDY = t205
      VYDZ = t224
      VXX = X*t48-t87*t58-t229*t46+t64*t53+0.1E1*t234*t24+t236 +
     #      0.1E1*t239+0.1E1*t242+0.1E1*t247-t252
      VYX = X*t63-t256-t258-t262+0.1E1*t266
      VZX = X*vzo-Z*t48
      VXXDX = t371
      VXXDY = t423
      VXXDZ = t463
      VYXDY = X*t205-t466-t468+0.1E1*t470-t474-t476+0.1E1*t480
      VYXDZ = X*t224-t484-t486-t489+0.1E1*t492
      VXY = Y*t48-t256-t258-t262+0.1E1*t266
      VYY = Y*t63-t239-t242-t247+0.1E1*t252+t236
      VZY = Y*vzo-Z*t63
      VXYDX = Y*t140-(t128/2-t126/2)*t28-(t113/2-t111/2)*t3-t419-
     #   t471*t245*t135*t138+0.1E1*t421+0.1E1*t477*t250*t120*t123
      VXYDY = 0.1E1*t25-t47+Y*t161-t466-t468+0.1E1*t470-t474-t476+
     #        0.1E1*t480
      VXYDZ = Y*t178-t484-t486-t489+0.1E1*t492
      VYYDY = t518
      VYYDZ = Y*t224-t458-t460-t461+0.1E1*t462+vzo+t456

      VZDZ = -VXDX - VYDY
      VYDX = VXDY
      VZDX = VXDZ
      VZDY = VYDZ
      VZXDZ = -VXXDX - VYXDY
      VYXDX = VXXDY
      VZXDX = VXXDZ
      VZXDY = VYXDZ
      VZYDZ = -VXYDX - VYYDY
      VYYDX = VXYDY
      VZYDX = VXYDZ
      VZYDY = VYYDZ

      RETURN
      END

C***********************************************************************
C
C     SUBROUTINE FLINT()
C
C     Computes the induced velocity and the derivatives of the velocity
C     from a FLAT panel with a linear source distribution, for the
C     case where the panel has four sides
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Programmed by: Michael Hughes, July 1994
C
C***********************************************************************

      SUBROUTINE FLINT(X,Y,Z,TLEN,SI,VX,VY,VZ,
     *                 VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                 VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                 VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                 VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                 VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      DIMENSION SI(15)

      t1 = TLEN**2
      t2 = X**2
      t3 = Y**2
      t4 = Z**2
      t5 = t2+t3+t4
      t6 = 1/t5
      t7 = t1*t6
      t8 = SI(1)*X
      t9 = sqrt(t5)
      t10 = 1/t9
      t12 = TLEN*t10
      t13 = t2*t6
      t14 = 0.3E1*t13-0.1E1
      t16 = SI(3)*X
      t17 = Y*t6
      t19 = SI(2)*t14+0.3E1*t16*t17
      t21 = SI(4)*X
      t22 = 0.3E1-0.5E1*t13
      t23 = t10*t22
      t25 = SI(5)*Y
      t26 = 0.1E1-0.5E1*t13
      t27 = t10*t26
      t29 = SI(6)*X
      t30 = t3*t6
      t31 = 0.1E1-0.5E1*t30
      t32 = t10*t31
      t34 = 0.3E1*t21*t23+0.6E1*t25*t27+0.3E1*t29*t32
      t36 = t8*t10+t12*t19-0.5E0*t7*t34
      t38 = t1*TLEN
      t39 = t38*t6
      t40 = SI(2)*X
      t43 = SI(5)*X
      t44 = t43*t17
      t45 = SI(4)*t14+0.3E1*t44
      t47 = SI(7)*X
      t49 = SI(8)*Y
      t50 = t49*t27
      t51 = SI(9)*X
      t52 = t51*t32
      t53 = 0.3E1*t47*t23+0.6E1*t50+0.3E1*t52
      t55 = -t40*t10-t12*t45+0.5E0*t7*t53
      t60 = SI(5)*t14+0.3E1*t29*t17
      t62 = SI(8)*X
      t64 = SI(9)*Y
      t66 = SI(10)*X
      t68 = 0.3E1*t62*t23+0.6E1*t64*t27+0.3E1*t66*t32
      t70 = -t16*t10-t12*t60+0.5E0*t7*t68
      t72 = SI(1)*Y
      t75 = 0.3E1*t30-0.1E1
      t77 = 0.3E1*t40*t17+SI(3)*t75
      t79 = SI(4)*Y
      t82 = SI(6)*Y
      t83 = 0.3E1-0.5E1*t30
      t84 = t10*t83
      t86 = 0.3E1*t79*t27+0.6E1*t43*t32+0.3E1*t82*t84
      t88 = t72*t10+t12*t77-0.5E0*t7*t86
      t90 = SI(2)*Y
      t94 = 0.3E1*t21*t17+SI(5)*t75
      t96 = SI(7)*Y
      t100 = 0.3E1*t96*t27+0.6E1*t62*t32+0.3E1*t64*t84
      t102 = -t90*t10-t12*t94+0.5E0*t7*t100
      t104 = SI(3)*Y
      t107 = 0.3E1*t44+SI(6)*t75
      t109 = SI(10)*Y
      t111 = 0.3E1*t50+0.6E1*t52+0.3E1*t109*t84
      t113 = -t104*t10-t12*t107+0.5E0*t7*t111
      t117 = t6*Z
      t124 = t9**2
      t126 = 1/t124/t9
      t127 = t126*Y
      t128 = t127*Z
      t166 = t5**2
      t167 = 1/t166
      t168 = t1*t167
      t171 = SI(1)*t10
      t174 = TLEN*t126
      t177 = X*t6
      t178 = t2*X
      t179 = t178*t167
      t180 = 0.6E1*t177-0.6E1*t179
      t184 = SI(3)*t2
      t185 = t167*Y
      t191 = SI(4)*t10
      t193 = SI(4)*t2
      t194 = t126*t22
      t197 = t10*(-0.1E2*t177+0.1E2*t179)
      t199 = t126*t26
      t200 = t199*X
      t203 = SI(6)*t10
      t205 = SI(6)*t2
      t206 = t126*t31
      t208 = t124**2
      t210 = 1/t208/t9
      t211 = t210*t3
      t218 = t38*t167
      t221 = SI(2)*t10
      t222 = SI(2)*t2
      t229 = SI(5)*t2
      t230 = t229*t185
      t235 = SI(7)*t10
      t242 = SI(9)*t10
      t244 = SI(9)*t2
      t246 = t244*t211
      t254 = SI(3)*t10
      t266 = SI(8)*t10
      t273 = SI(10)*t10
      t275 = SI(10)*t2
      t289 = t167*t3
      t296 = t194*Y
      t298 = SI(4)*t178
      t299 = t210*Y
      t303 = SI(5)*t3
      t305 = t210*t2
      t307 = t206*Y
      t309 = t3*Y
      t310 = t309*t167
      t312 = t10*(-0.1E2*t17+0.1E2*t310)
      t325 = t43*t289
      t326 = t43*t6
      t332 = SI(7)*t178
      t334 = t266*t26
      t335 = SI(8)*t3
      t336 = t335*t199
      t337 = t335*t305
      t338 = t51*t307
      t339 = t51*t312
      t357 = SI(8)*t178
      t360 = SI(9)*t3
      t371 = t126*Z
      t375 = t167*Z
      t377 = t185*Z
      t383 = t194*Z
      t385 = t210*Z
      t387 = t199*Z
      t389 = t305*Z
      t391 = t206*Z
      t393 = t211*Z
      t406 = t43*t377
      t413 = t49*t387
      t414 = t49*t389
      t415 = t51*t391
      t416 = t51*t393
      t452 = 0.6E1*t17-0.6E1*t310
      t459 = SI(4)*t3
      t465 = SI(6)*t3
      t466 = t126*t83
      t488 = SI(7)*t3
      t503 = SI(3)*t3
      t536 = t466*Z
      VX = t7*t36
      VXX = -t39*t55
      VXY = -t39*t70
      VY = t7*t88
      VYX = -t39*t102
      VYY = -t39*t113
      VZ = t7*(SI(1)*Z*t10+t12*(0.3E1*t40*t117+0.3E1*t104*t117)-
     #  0.5E0*t7*(0.3E1*SI(4)*Z*t27-0.3E2*t43*t128+0.3E1*SI(6)*Z*t32))
      VZX = -t39*(-SI(2)*Z*t10-t12*(0.3E1*t21*t117+0.3E1*t25*t117)
     #  +0.5E0*t7*(0.3E1*SI(7)*Z*t27-0.3E2*t62*t128+0.3E1*SI(9)*Z*t32))
      VZY = -t39*(-SI(3)*Z*t10-t12*(0.3E1*t43*t117+0.3E1*t82*t117)
     #  +0.5E0*t7*(0.3E1*SI(8)*Z*t27-0.3E2*t51*t128+0.3E1*SI(10)*Z*t32))
      VXDX = -2*t168*t36*X+t7*(t171-SI(1)*t2*t126-t174*t19*X+t12*(SI(2)*
     #  t180+0.3E1*SI(3)*t6*Y-0.6E1*t184*t185)+0.1E1*t168*t34*X-0.5E0*
     #  t7*(0.3E1*t191*t22-0.3E1*t193*t194+0.3E1*t21*t197-0.6E1*t25*
     #  t200+0.6E1*t25*t197+0.3E1*t203*t31-0.3E1*t205*t206+0.3E2*t205*
     #  t211))
      VXXDX = 2*t218*t55*X-t39*(-t221+t222*t126+t174*t45*X-t12*(SI(4)*
     #  t180+0.3E1*SI(5)*t6*Y-0.6E1*t230)-t168*t53*X+0.5E0*t7*(0.3E1*
     #  t235*t22-0.3E1*SI(7)*t2*t194+0.3E1*t47*t197-0.6E1*t49*t200+
     #  0.6E1*t49*t197+0.3E1*t242*t31-0.3E1*t244*t206+0.3E2*t246))
      VXYDX = 2*t218*t70*X-t39*(-t254+t184*t126+t174*t60*X-t12*(SI(5)*
     #  t180+0.3E1*SI(6)*t6*Y-0.6E1*t205*t185)-t168*t68*X+0.5E0*t7*
     #  (0.3E1*t266*t22-0.3E1*SI(8)*t2*t194+0.3E1*t62*t197-0.6E1*t64*
     #  t200+0.6E1*t64*t197+0.3E1*t273*t31-0.3E1*t275*t206+0.3E2*t275*
     #  t211))
      VXDY = -2*t168*t36*Y+t7*(-t8*t127-t174*t19*Y+t12*(-0.6E1*t222*
     #  t185-0.6E1*t16*t289+0.3E1*t16*t6)+0.1E1*t168*t34*Y-0.5E0*t7*
     #  (-0.3E1*t21*t296+0.3E2*t298*t299+0.6E1*SI(5)*t10*t26-0.6E1*
     #  t303*t199+0.6E2*t303*t305-0.3E1*t29*t307+0.3E1*t29*t312))
      VXXDY = 2*t218*t55*Y-t39*(t40*t127+t174*t45*Y-t12*(-0.6E1*t193*
     #   t185-0.6E1*t325+0.3E1*t326)-t168*t53*Y+0.5E0*t7*(-0.3E1*t47*
     #   t296+0.3E2*t332*t299+0.6E1*t334-0.6E1*t336+0.6E2*t337-0.3E1*
     #   t338+0.3E1*t339))
      VXYDY = 2*t218*t70*Y-t39*(t16*t127+t174*t60*Y-t12*(-0.6E1*t230-
     #   0.6E1*t29*t289+0.3E1*t29*t6)-t168*t68*Y+0.5E0*t7*(-0.3E1*t62*
     #   t296+0.3E2*t357*t299+0.6E1*t242*t26-0.6E1*t360*t199+0.6E2*t246-
     #   0.3E1*t66*t307+0.3E1*t66*t312))
      VXDZ = -2*t168*t36*Z+t7*(-t8*t371-t174*t19*Z+t12*(-0.6E1*t222*
     #   t375-0.6E1*t16*t377)+0.1E1*t168*t34*Z-0.5E0*t7*(-0.3E1*t21*
     #   t383+0.3E2*t298*t385-0.6E1*t25*t387+0.6E2*t25*t389-0.3E1*t29*
     #   t391+0.3E2*t29*t393))
      VXXDZ = 2*t218*t55*Z-t39*(t40*t371+t174*t45*Z-t12*(-0.6E1*t193*
     #   t375-0.6E1*t406)-t168*t53*Z+0.5E0*t7*(-0.3E1*t47*t383+0.3E2*
     #   t332*t385-0.6E1*t413+0.6E2*t414-0.3E1*t415+0.3E2*t416))
      VXYDZ = 2*t218*t70*Z-t39*(t16*t371+t174*t60*Z-t12*(-0.6E1*t229*
     #   t375-0.6E1*t29*t377)-t168*t68*Z+0.5E0*t7*(-0.3E1*t62*t383+
     #   0.3E2*t357*t385-0.6E1*t64*t387+0.6E2*t64*t389-0.3E1*t66*t391+
     #   0.3E2*t66*t393))
      VYDY = -2*t168*t88*Y+t7*(t171-SI(1)*t3*t126-t174*t77*Y+t12*
     #   (-0.6E1*t40*t289+0.3E1*t40*t6+SI(3)*t452)+0.1E1*t168*t86*Y-
     #   0.5E0*t7*(0.3E1*t191*t26-0.3E1*t459*t199+0.3E2*t459*t305-
     #   0.6E1*t43*t307+0.6E1*t43*t312+0.3E1*t203*t83-0.3E1*t465*t466+
     #   0.3E1*t82*t312))
      VYXDY = 2*t218*t102*Y-t39*(-t221+SI(2)*t3*t126+t174*t94*Y-t12*
     #   (-0.6E1*t21*t289+0.3E1*t21*t6+SI(5)*t452)-t168*t100*Y+0.5E0*t7*
     #   (0.3E1*t235*t26-0.3E1*t488*t199+0.3E2*t488*t305-0.6E1*t62*t307+
     #   0.6E1*t62*t312+0.3E1*t242*t83-0.3E1*t360*t466+0.3E1*t64*t312))
      VYYDY = 2*t218*t113*Y-t39*(-t254+t503*t126+t174*t107*Y-t12*
     #   (-0.6E1*t325+0.3E1*t326+SI(6)*t452)-t168*t111*Y+0.5E0*t7*
     #   (0.3E1*t334-0.3E1*t336+0.3E2*t337-0.6E1*t338+0.6E1*t339+0.3E1*
     #   t273*t83-0.3E1*SI(10)*t3*t466+0.3E1*t109*t312))
      VYDZ = -2*t168*t88*Z+t7*(-t72*t371-t174*t77*Z+t12*(-0.6E1*t40*
     #   t377-0.6E1*t503*t375)+0.1E1*t168*t86*Z-0.5E0*t7*(-0.3E1*t79*
     #   t387+0.3E2*t79*t389-0.6E1*t43*t391+0.6E2*t43*t393-0.3E1*t82*
     #   t536+0.3E2*SI(6)*t309*t385))
      VYXDZ = 2*t218*t102*Z-t39*(t90*t371+t174*t94*Z-t12*(-0.6E1*t21*
     #   t377-0.6E1*t303*t375)-t168*t100*Z+0.5E0*t7*(-0.3E1*t96*t387+
     #   0.3E2*t96*t389-0.6E1*t62*t391+0.6E2*t62*t393-0.3E1*t64*t536+
     #   0.3E2*SI(9)*t309*t385))
      VYYDZ = 2*t218*t113*Z-t39*(t104*t371+t174*t107*Z-t12*(-0.6E1*t406-
     #   0.6E1*t465*t375)-t168*t111*Z+0.5E0*t7*(-0.3E1*t413+0.3E2*t414-
     #   0.6E1*t415+0.6E2*t416-0.3E1*t109*t536+0.3E2*SI(10)*t309*t385))

      VZDZ = -VXDX - VYDY
      VYDX = VXDY
      VZDX = VXDZ
      VZDY = VYDZ
      VZXDZ = -VXXDX - VYXDY
      VYXDX = VXXDY
      VZXDX = VXXDZ
      VZXDY = VYXDZ
      VZYDZ = -VXYDX - VYYDY
      VYYDX = VXYDY
      VZYDX = VXYDZ
      VZYDY = VYYDZ

      RETURN
      END

C***********************************************************************
C
C     SUBROUTINE FLFAR()
C
C     Computes the induced velocity and the derivatives of the velocity
C     from a FLAT panel with a linear source distribution, for the
C     case where the panel has four sides
C
C
C     The formulas used are those derived from John Hess,
C     Report No. MDC J8519, June 1979.
C
C     Programmed by: Michael Hughes, July 1994
C
C***********************************************************************

      SUBROUTINE FLFAR(XFP,YFP,ZFP,TLEN,SI,XC,DIRP,VX,VY,VZ,
     *                 VXX,VYX,VZX,VXY,VYY,VZY,VXDX,VXDY,VXDZ,
     *                 VYDX,VYDY,VYDZ,VZDX,VZDY,VZDZ,
     *                 VXXDX,VXXDY,VXXDZ,VYXDX,VYXDY,VYXDZ,
     *                 VZXDX,VZXDY,VZXDZ,VXYDX,VXYDY,VXYDZ,
     *                 VYYDX,VYYDY,VYYDZ,VZYDX,VZYDY,VZYDZ  )

      DIMENSION SI(15),XC(3),DIRP(3,3)

      t1 = TLEN**2
      t2 = t1*SI(1)
      t3 = XFP-XC(1)
      t4 = t3**2
      t5 = YFP-XC(2)
      t6 = t5**2
      t7 = ZFP-XC(3)
      t8 = t7**2
      t9 = t4+t6+t8
      t10 = 1/t9
      t11 = XFP**2
      t13 = XC(1)**2
      t14 = YFP**2
      t16 = XC(2)**2
      t17 = ZFP**2
      t19 = XC(3)**2
      t21 = sqrt(t11-2*XFP*XC(1)+t13+t14-2*YFP*XC(2)+t16+t17-
     #      2*ZFP*XC(3)+t19)
      t22 = 1/t21
      t23 = t10*t22
      t24 = t23*t3
      t26 = t1*TLEN
      t27 = t26*SI(2)
      t29 = t1**2
      t30 = t29*t10
      t31 = t22*SI(4)
      t32 = t3*DIRP(1,1)
      t33 = t5*DIRP(1,2)
      t35 = t32+t33+t7*DIRP(1,3)
      t36 = t3*t35
      t38 = DIRP(1,1)-0.3E1*t36*t10
      t41 = t22*SI(5)
      t42 = t3*DIRP(2,1)
      t43 = t5*DIRP(2,2)
      t45 = t42+t43+t7*DIRP(2,3)
      t46 = t3*t45
      t48 = DIRP(2,1)-0.3E1*t46*t10
      t52 = t26*SI(3)
      t56 = t22*SI(6)
      t60 = t23*t5
      t63 = t5*t35
      t65 = DIRP(1,2)-0.3E1*t63*t10
      t68 = t5*t45
      t70 = DIRP(2,2)-0.3E1*t68*t10
      t80 = t23*t7
      t85 = DIRP(1,3)-0.3E1*t7*t35*t10
      t90 = DIRP(2,3)-0.3E1*t7*t45*t10
      t100 = t9**2
      t101 = 1/t100
      t102 = t2*t101
      t103 = t22*t3
      t104 = 2*XFP-2*XC(1)
      t105 = t103*t104
      t107 = t2*t10
      t108 = t21**2
      t110 = 1/t108/t21
      t111 = t110*t3
      t112 = t111*t104
      t114 = t2*t23
      t116 = t27*t101
      t118 = t27*t10
      t120 = t27*t23
      t122 = t29*t101*t22
      t123 = SI(4)*t38
      t124 = t123*t104
      t126 = t30*t110
      t128 = t35*t10
      t130 = t101*t104
      t132 = -0.3E1*t128-0.3E1*t32*t10+0.3E1*t36*t130
      t135 = SI(5)*t48
      t136 = t135*t104
      t139 = t45*t10
      t142 = -0.3E1*t139-0.3E1*t42*t10+0.3E1*t46*t130
      t146 = t52*t101
      t148 = t52*t10
      t150 = t52*t23
      t151 = SI(5)*t38
      t152 = t151*t104
      t157 = SI(6)*t48
      t158 = t157*t104
      t164 = 2*YFP-2*XC(2)
      t165 = t103*t164
      t167 = t111*t164
      t172 = t123*t164
      t177 = t101*t164
      t179 = -0.3E1*t3*DIRP(1,2)*t10+0.3E1*t36*t177
      t182 = t135*t164
      t188 = -0.3E1*t3*DIRP(2,2)*t10+0.3E1*t46*t177
      t194 = t151*t164
      t199 = t157*t164
      t205 = 2*ZFP-2*XC(3)
      t206 = t103*t205
      t208 = t111*t205
      t213 = t123*t205
      t218 = t101*t205
      t220 = -0.3E1*t3*DIRP(1,3)*t10+0.3E1*t36*t218
      t223 = t135*t205
      t229 = -0.3E1*t3*DIRP(2,3)*t10+0.3E1*t46*t218
      t235 = t151*t205
      t240 = t157*t205
      t246 = t22*t5
      t247 = t246*t164
      t249 = t110*t5
      t250 = t249*t164
      t255 = SI(4)*t65
      t256 = t255*t164
      t261 = -0.3E1*t128-0.3E1*t33*t10+0.3E1*t63*t177
      t264 = SI(5)*t70
      t265 = t264*t164
      t270 = -0.3E1*t139-0.3E1*t43*t10+0.3E1*t68*t177
      t276 = SI(5)*t65
      t277 = t276*t164
      t282 = SI(6)*t70
      t283 = t282*t164
      t289 = t246*t205
      t291 = t249*t205
      t296 = t255*t205
      t302 = -0.3E1*t5*DIRP(1,3)*t10+0.3E1*t63*t218
      t305 = t264*t205
      t311 = -0.3E1*t5*DIRP(2,3)*t10+0.3E1*t68*t218
      t317 = t276*t205
      t322 = t282*t205
      VX = 0.1E1*t2*t24
      VXX = 0.1E1*t27*t24-t30*t31*t38-t30*t41*t48
      VXY = 0.1E1*t52*t24-t30*t41*t38-t30*t56*t48
      VY = 0.1E1*t2*t60
      VYX = 0.1E1*t27*t60-t30*t31*t65-t30*t41*t70
      VYY = 0.1E1*t52*t60-t30*t41*t65-t30*t56*t70
      VZ = 0.1E1*t2*t80
      VZX = 0.1E1*t27*t80-t30*t31*t85-t30*t41*t90
      VZY = 0.1E1*t52*t80-t30*t41*t85-t30*t56*t90
      VXDX = -t102*t105-0.5E0*t107*t112+0.1E1*t114
      VXXDX = -t116*t105-0.5E0*t118*t112+0.1E1*t120+0.1E1*t122*t124+
     #   0.5E0*t126*t124-t30*t31*t132+0.1E1*t122*t136+0.5E0*t126*t136-
     #   t30*t41*t142
      VXYDX = -t146*t105-0.5E0*t148*t112+0.1E1*t150+0.1E1*t122*t152+
     #   0.5E0*t126*t152-t30*t41*t132+0.1E1*t122*t158+0.5E0*t126*t158-
     #   t30*t56*t142
      VXDY = -t102*t165-0.5E0*t107*t167
      VXXDY = -t116*t165-0.5E0*t118*t167+0.1E1*t122*t172+0.5E0*t126*
     #   t172-t30*t31*t179+0.1E1*t122*t182+0.5E0*t126*t182-t30*t41*t188
      VXYDY = -t146*t165-0.5E0*t148*t167+0.1E1*t122*t194+0.5E0*t126*
     #   t194-t30*t41*t179+0.1E1*t122*t199+0.5E0*t126*t199-t30*t56*t188
      VXDZ = -t102*t206-0.5E0*t107*t208
      VXXDZ = -t116*t206-0.5E0*t118*t208+0.1E1*t122*t213+0.5E0*t126*
     #   t213-t30*t31*t220+0.1E1*t122*t223+0.5E0*t126*t223-t30*t41*t229
      VXYDZ = -t146*t206-0.5E0*t148*t208+0.1E1*t122*t235+0.5E0*t126*
     #   t235-t30*t41*t220+0.1E1*t122*t240+0.5E0*t126*t240-t30*t56*t229
      VYDY = -t102*t247-0.5E0*t107*t250+0.1E1*t114
      VYXDY = -t116*t247-0.5E0*t118*t250+0.1E1*t120+0.1E1*t122*t256+
     #   0.5E0*t126*t256-t30*t31*t261+0.1E1*t122*t265+0.5E0*t126*t265-
     #   t30*t41*t270
      VYYDY = -t146*t247-0.5E0*t148*t250+0.1E1*t150+0.1E1*t122*t277+
     #   0.5E0*t126*t277-t30*t41*t261+0.1E1*t122*t283+0.5E0*t126*t283-
     #   t30*t56*t270
      VYDZ = -t102*t289-0.5E0*t107*t291
      VYXDZ = -t116*t289-0.5E0*t118*t291+0.1E1*t122*t296+0.5E0*t126*
     #   t296-t30*t31*t302+0.1E1*t122*t305+0.5E0*t126*t305-t30*t41*t311
      VYYDZ = -t146*t289-0.5E0*t148*t291+0.1E1*t122*t317+0.5E0*t126*
     #   t317-t30*t41*t302+0.1E1*t122*t322+0.5E0*t126*t322-t30*t56*t311

      VZDZ = -VXDX - VYDY
      VYDX = VXDY
      VZDX = VXDZ
      VZDY = VYDZ
      VZXDZ = -VXXDX - VYXDY
      VYXDX = VXXDY
      VZXDX = VXXDZ
      VZXDY = VYXDZ
      VZYDZ = -VXYDX - VYYDY
      VYYDX = VXYDY
      VZYDX = VXYDZ
      VZYDY = VYYDZ

      RETURN
      END


      SUBROUTINE FLATDZ(XC,SI,TLEN,XFP,YFP,ZFP,VXDXZ,VXDYZ,VXDZZ,
     *                  VYDYZ,VYDZZ,VXXDXZ,VXXDYZ,VXXDZZ,VYXDYZ,VYXDZZ,
     *                  VXYDXZ,VXYDYZ,VXYDZZ,VYYDYZ,VYYDZZ,DIRP)

      DIMENSION XC(3),SI(15),DIRP(3,3)

      t1 = TLEN**2
      t2 = t1*SI(1)
      t3 = XFP-XC(1)
      t4 = t3**2
      t5 = YFP-XC(2)
      t6 = t5**2
      t7 = ZFP-XC(3)
      t8 = t7**2
      t9 = t4+t6+t8
      t10 = t9**2
      t12 = 1/t10/t9
      t13 = t2*t12
      t14 = XFP**2
      t16 = XC(1)**2
      t17 = YFP**2
      t19 = XC(2)**2
      t20 = ZFP**2
      t22 = XC(3)**2
      t24 = sqrt(t14-2*XFP*XC(1)+t16+t17-2*YFP*XC(2)+t19+t20 -
     #           2*ZFP*XC(3)+t22)
      t25 = 1/t24
      t26 = t25*t3
      t27 = 2*XFP-2*XC(1)
      t28 = 2*ZFP-2*XC(3)
      t29 = t27*t28
      t30 = t26*t29
      t32 = 1/t10
      t33 = t2*t32
      t34 = t24**2
      t36 = 1/t34/t24
      t37 = t36*t3
      t38 = t37*t29
      t40 = 1/t9
      t41 = t2*t40
      t42 = t34**2
      t44 = 1/t42/t24
      t45 = t44*t3
      t46 = t45*t29
      t48 = t32*t25
      t49 = t48*t28
      t50 = t2*t49
      t51 = t40*t36
      t52 = t51*t28
      t53 = t2*t52
      t55 = 2*YFP-2*XC(2)
      t56 = t55*t28
      t57 = t26*t56
      t59 = t37*t56
      t61 = t45*t56
      t64 = t28**2
      t65 = t26*t64
      t67 = t37*t64
      t69 = t48*t3
      t71 = t45*t64
      t73 = t51*t3
      t76 = t25*t5
      t77 = t76*t56
      t79 = t36*t5
      t80 = t79*t56
      t82 = t44*t5
      t83 = t82*t56
      t86 = t76*t64
      t88 = t79*t64
      t90 = t48*t5
      t92 = t82*t64
      t94 = t51*t5
      t97 = t1*TLEN
      t98 = t97*SI(2)
      t99 = t98*t12
      t101 = t98*t32
      t103 = t98*t40
      t105 = t98*t49
      t106 = t98*t52
      t107 = t1**2
      t109 = t107*t12*t25
      t110 = t3*DIRP(1,1)
      t111 = t5*DIRP(1,2)
      t113 = t110+t111+t7*DIRP(1,3)
      t114 = t3*t113
      t116 = DIRP(1,1)-0.3E1*t114*t40
      t117 = SI(4)*t116
      t118 = t117*t29
      t120 = t107*t32
      t121 = t120*t36
      t123 = t120*t25
      t124 = t3*DIRP(1,3)
      t126 = t32*t28
      t128 = -0.3E1*t124*t40+0.3E1*t114*t126
      t129 = SI(4)*t128
      t130 = t129*t27
      t132 = t107*t40
      t133 = t132*t44
      t135 = t132*t36
      t138 = t113*t40
      t140 = t32*t27
      t142 = -0.3E1*t138-0.3E1*t110*t40+0.3E1*t114*t140
      t144 = SI(4)*t142*t28
      t147 = t25*SI(4)
      t148 = DIRP(1,3)*t40
      t150 = t113*t32*t28
      t154 = t12*t27*t28
      t156 = -0.3E1*t148+0.3E1*t150+0.3E1*t110*t126 +
     #       0.3E1*t124*t140-0.6E1*t114*t154
      t159 = t3*DIRP(2,1)
      t160 = t5*DIRP(2,2)
      t162 = t159+t160+t7*DIRP(2,3)
      t163 = t3*t162
      t165 = DIRP(2,1)-0.3E1*t163*t40
      t166 = SI(5)*t165
      t167 = t166*t29
      t170 = t3*DIRP(2,3)
      t173 = -0.3E1*t170*t40+0.3E1*t163*t126
      t174 = SI(5)*t173
      t175 = t174*t27
      t179 = t162*t40
      t182 = -0.3E1*t179-0.3E1*t159*t40+0.3E1*t163*t140
      t184 = SI(5)*t182*t28
      t187 = t25*SI(5)
      t188 = DIRP(2,3)*t40
      t190 = t162*t32*t28
      t194 = -0.3E1*t188+0.3E1*t190+0.3E1*t159*t126 +
     #       0.3E1*t170*t140-0.6E1*t163*t154
      t197 = 0.1E1*t123*t144+0.5E0*t135*t144-t132*t147*t156-0.2E1*t109*
     #      t167-t121*t167+0.1E1*t123*t175-0.75E0*t133*t167+0.5E0*t135*
     #      t175+0.1E1*t123*t184+0.5E0*t135*t184-t132*t187*t194
      t202 = t117*t56
      t205 = t129*t55
      t209 = t3*DIRP(1,2)
      t211 = t32*t55
      t213 = -0.3E1*t209*t40+0.3E1*t114*t211
      t215 = SI(4)*t213*t28
      t221 = t12*t55*t28
      t223 = 0.3E1*t209*t126+0.3E1*t124*t211-0.6E1*t114*t221
      t226 = t166*t56
      t229 = t174*t55
      t233 = t3*DIRP(2,2)
      t236 = -0.3E1*t233*t40+0.3E1*t163*t211
      t238 = SI(5)*t236*t28
      t244 = 0.3E1*t233*t126+0.3E1*t170*t211-0.6E1*t163*t221
      t247 = 0.2E1*t99*t57+0.1E1*t101*t59+0.75E0*t103*t61-0.2E1*t109*
     #  t202-t121*t202+0.1E1*t123*t205-0.75E0*t133*t202+0.5E0*t135*
     #  t205+0.1E1*t123*t215+0.5E0*t135*t215-t132*t147*t223-0.2E1*t109*
     #  t226-t121*t226+0.1E1*t123*t229-0.75E0*t133*t226+0.5E0*t135*t229
     #  +0.1E1*t123*t238+0.5E0*t135*t238-t132*t187*t244
      t253 = t117*t64
      t256 = t129*t28
      t263 = t36*SI(4)
      t267 = t12*t64
      t270 = 0.6E1*t124*t126-0.6E1*t114*t267+0.6E1*t114*t32
      t273 = t166*t64
      t276 = t174*t28
      t282 = t36*SI(5)
      t288 = 0.6E1*t170*t126-0.6E1*t163*t267+0.6E1*t163*t32
      t291 = 0.1E1*t135*t256+0.1E1*t132*t263*t116-t132*t147*t270-0.2E1*
     #  t109*t273-t121*t273+0.2E1*t123*t276+0.2E1*t120*t187*t165-0.75E0*
     #  t133*t273+0.1E1*t135*t276+0.1E1*t132*t282*t165-t132*t187*t288
      t296 = t5*t113
      t298 = DIRP(1,2)-0.3E1*t296*t40
      t299 = SI(4)*t298
      t300 = t299*t56
      t303 = t5*DIRP(1,3)
      t306 = -0.3E1*t303*t40+0.3E1*t296*t126
      t307 = SI(4)*t306
      t308 = t307*t55
      t315 = -0.3E1*t138-0.3E1*t111*t40+0.3E1*t296*t211
      t317 = SI(4)*t315*t28
      t323 = -0.3E1*t148+0.3E1*t150+0.3E1*t111*t126+0.3E1*t303*t211-
     #  0.6E1*t296*t221
      t326 = t5*t162
      t328 = DIRP(2,2)-0.3E1*t326*t40
      t329 = SI(5)*t328
      t330 = t329*t56
      t333 = t5*DIRP(2,3)
      t336 = -0.3E1*t333*t40+0.3E1*t326*t126
      t337 = SI(5)*t336
      t338 = t337*t55
      t344 = -0.3E1*t179-0.3E1*t160*t40+0.3E1*t326*t211
      t346 = SI(5)*t344*t28
      t352 = -0.3E1*t188+0.3E1*t190+0.3E1*t160*t126+0.3E1*t333*t211-
     #  0.6E1*t326*t221
      t355 = 0.1E1*t123*t317+0.5E0*t135*t317-t132*t147*t323-0.2E1*t109*
     #  t330-t121*t330+0.1E1*t123*t338-0.75E0*t133*t330+0.5E0*t135*
     #  t338+0.1E1*t123*t346+0.5E0*t135*t346-t132*t187*t352
      t360 = t299*t64
      t362 = t307*t28
      t369 = 0.6E1*t303*t126-0.6E1*t296*t267+0.6E1*t296*t32
      t372 = t329*t64
      t374 = t337*t28
      t389 = 0.6E1*t333*t126-0.6E1*t326*t267+0.6E1*t326*t32
      t397 = 0.75E0*t103*t92-t98*t94-0.2E1*t109*t360-t121*t360+0.2E1*
     #  t123*t362+0.1E1*t132*t282*t328-t132*t187*t389-0.2E1*t109*t372-
     #  t121*t372+0.2E1*t123*t374+0.2E1*t120*t187*t328
      t399 = t97*SI(3)
      t400 = t399*t12
      t402 = t399*t32
      t404 = t399*t40
      t406 = t399*t49
      t407 = t399*t52
      t408 = SI(5)*t116
      t409 = t408*t29
      t412 = SI(5)*t128
      t413 = t412*t27
      t419 = SI(5)*t142*t28
      t424 = SI(6)*t165
      t425 = t424*t29
      t428 = SI(6)*t173
      t429 = t428*t27
      t434 = SI(6)*t182*t28
      t437 = t25*SI(6)
      t440 = 0.1E1*t123*t419+0.5E0*t135*t419-t132*t187*t156-0.2E1*t109*
     #  t425-t121*t425+0.1E1*t123*t429-0.75E0*t133*t425+0.5E0*t135*
     #  t429+0.1E1*t123*t434+0.5E0*t135*t434-t132*t437*t194
      t445 = t408*t56
      t448 = t412*t55
      t453 = SI(5)*t213*t28
      t458 = t424*t56
      t461 = t428*t55
      t466 = SI(6)*t236*t28
      t471 = 0.2E1*t400*t57+0.1E1*t402*t59+0.75E0*t404*t61-0.2E1*t109*
     #  t445-t121*t445+0.1E1*t123*t448-0.75E0*t133*t445+0.5E0*t135*
     #  t448+0.1E1*t123*t453+0.5E0*t135*t453-t132*t187*t223-0.2E1*t109*
     #  t458-t121*t458+0.1E1*t123*t461-0.75E0*t133*t458+0.5E0*t135*
     #  t461+0.1E1*t123*t466+0.5E0*t135*t466-t132*t437*t244
      t472 = t424*t64
      t475 = t428*t28
      t477 = t408*t64
      t479 = t412*t28
      t498 = t36*SI(6)
      t505 = -t399*t73-0.2E1*t109*t477-t121*t477+0.2E1*t123*t479+0.2E1*
     #  t120*t187*t116+0.2E1*t120*t437*t165-0.75E0*t133*t472+0.1E1*
     #  t135*t475+0.1E1*t132*t498*t165-t132*t437*t288-t132*t187*t270
      t510 = SI(5)*t298
      t511 = t510*t56
      t514 = SI(5)*t306
      t515 = t514*t55
      t521 = SI(5)*t315*t28
      t526 = SI(6)*t328
      t527 = t526*t56
      t530 = SI(6)*t336
      t531 = t530*t55
      t536 = SI(6)*t344*t28
      t541 = 0.1E1*t123*t521+0.5E0*t135*t521-t132*t187*t323-0.2E1*t109*
     #  t527-t121*t527+0.1E1*t123*t531-0.75E0*t133*t527+0.5E0*t135*
     #  t531+0.1E1*t123*t536+0.5E0*t135*t536-t132*t437*t352
      t545 = t510*t64
      t547 = t526*t64
      t549 = t530*t28
      t556 = t514*t28
      t575 = -0.75E0*t133*t545+0.1E1*t135*t556+0.1E1*t132*t282*t298-
     #  t132*t187*t369-0.2E1*t109*t547-t121*t547+0.2E1*t123*t549+0.2E1*
     #  t120*t437*t328+0.2E1*t400*t86+0.1E1*t402*t88-0.2E1*t399*t90
      VXDXZ = 0.2E1*t13*t30+0.1E1*t33*t38+0.75E0*t41*t46-t50-0.5E0*t53
      VXDYZ = 0.2E1*t13*t57+0.1E1*t33*t59+0.75E0*t41*t61
      VXDZZ = 0.2E1*t13*t65+0.1E1*t33*t67-0.2E1*t2*t69 +
     #        0.75E0*t41*t71-t2*t73
      VYDYZ = 0.2E1*t13*t77+0.1E1*t33*t80+0.75E0*t41*t83-t50-0.5E0*t53
      VYDZZ = 0.2E1*t13*t86+0.1E1*t33*t88-0.2E1*t2*t90 +
     #        0.75E0*t41*t92-t2*t94
      VXXDXZ = 0.2E1*t99*t30+0.1E1*t101*t38+0.75E0*t103*t46-t105-0.5E0*
     #  t106-0.2E1*t109*t118-t121*t118+0.1E1*t123*t130-0.75E0*t133*
     #  t118+0.5E0*t135*t130+t197
      VXXDYZ = t247
      VXXDZZ = 0.2E1*t99*t65+0.1E1*t101*t67-0.2E1*t98*t69+0.75E0*t103*
     #  t71-t98*t73-0.2E1*t109*t253-t121*t253+0.2E1*t123*t256+0.2E1*
     #  t120*t147*t116-0.75E0*t133*t253+t291
      VYXDYZ = 0.2E1*t99*t77+0.1E1*t101*t80+0.75E0*t103*t83-t105-0.5E0*
     #  t106-0.2E1*t109*t300-t121*t300+0.1E1*t123*t308-0.75E0*t133*
     #  t300+0.5E0*t135*t308+t355
      VYXDZZ = 0.2E1*t99*t86+0.2E1*t120*t147*t298-0.75E0*t133*t360+
     #  0.1E1*t135*t362+0.1E1*t132*t263*t298-t132*t147*t369-0.75E0*
     #  t133*t372+0.1E1*t135*t374+0.1E1*t101*t88-0.2E1*t98*t90+t397
      VXYDXZ = 0.2E1*t400*t30+0.1E1*t402*t38+0.75E0*t404*t46-t406-
     #  0.5E0*t407-0.2E1*t109*t409-t121*t409+0.1E1*t123*t413-0.75E0*
     #  t133*t409+0.5E0*t135*t413+t440
      VXYDYZ = t471
      VXYDZZ = -0.2E1*t109*t472-t121*t472+0.2E1*t123*t475-0.75E0*t133*
     #  t477+0.1E1*t135*t479+0.1E1*t132*t282*t116+0.2E1*t400*t65+0.1E1*
     #  t402*t67-0.2E1*t399*t69+0.75E0*t404*t71+t505
      VYYDYZ = 0.2E1*t400*t77+0.1E1*t402*t80+0.75E0*t404*t83-t406-
     #  0.5E0*t407-0.2E1*t109*t511-t121*t511+0.1E1*t123*t515-0.75E0*
     #  t133*t511+0.5E0*t135*t515+t541
      VYYDZZ = 0.75E0*t404*t92-t399*t94-0.2E1*t109*t545-0.75E0*t133*
     #  t547+0.1E1*t135*t549+0.1E1*t132*t498*t328-t132*t437*t389-t121*
     #  t545+0.2E1*t123*t556+0.2E1*t120*t187*t298+t575

      RETURN
      END

